package com.chelvc.framework.base.apidoc;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.time.Duration;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import com.arsframework.apidoc.core.Parameter;
import com.arsframework.apidoc.core.ParameterAnalyser;
import com.chelvc.framework.common.jackson.JacksonEnumerateSerializer;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.common.model.Period;
import com.chelvc.framework.common.model.Region;
import com.chelvc.framework.common.util.ObjectUtils;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.google.common.collect.Lists;

/**
 * 自定义接口文档参数解析实现
 *
 * @author Woody
 * @date 2024/1/30
 */
public class CustomizeParameterAnalyser extends ParameterAnalyser {
    public CustomizeParameterAnalyser(Method method) {
        super(method);
    }

    /**
     * 判断参数是否需要枚举包装
     *
     * @param parameter 参数对象
     * @return true/false
     */
    private boolean isEnumerateWrapping(Parameter parameter) {
        Field field = parameter.getField();
        Class<?> type = ObjectUtils.ifNull(field, Field::getType, parameter::getOriginal);
        JsonSerialize annotation = type.getAnnotation(JsonSerialize.class);
        if (annotation != null && (annotation.using() == JacksonEnumerateSerializer.class
                || annotation.contentUsing() == JacksonEnumerateSerializer.class)) {
            return true;
        }
        return field != null && (annotation = field.getAnnotation(JsonSerialize.class)) != null
                && (annotation.using() == JacksonEnumerateSerializer.class
                || annotation.contentUsing() == JacksonEnumerateSerializer.class);
    }

    /**
     * 初始化枚举包装类型参数
     *
     * @param name        参数名称
     * @param description 参数描述
     * @param options     参数可选项
     * @return 参数对象实例
     */
    private Parameter initializeEnumerateWrappingParameter(String name, String description,
                                                           List<Parameter.Option> options) {
        Parameter.Option example = ObjectUtils.isEmpty(options) ? null : options.get(0);
        return Parameter.builder().input(false).type(String.class).original(String.class)
                .name(name).description(description).options(options)
                .example(ObjectUtils.ifNull(example, Parameter.Option::getKey)).build();
    }

    @Override
    protected void afterInitializeParameter(Parameter parameter) {
        if (parameter.getOriginal() == Paging.class) {
            parameter.setType(Paging.class);
            parameter.setFields(null);
            parameter.setFormat("页码,分页大小");
            parameter.setExample("1,10");
        } else if (parameter.getOriginal() == Period.class) {
            parameter.setType(Period.class);
            parameter.setFields(null);
            parameter.setFormat("开始时间,结束时间");
            parameter.setExample(Period.today().toString());
        } else if (parameter.getOriginal() == Region.class) {
            parameter.setType(Integer.class);
            parameter.setFields(null);
            parameter.setExample("110000");
        } else if (parameter.getOriginal() == Duration.class) {
            parameter.setType(String.class);
            parameter.setFields(null);
            parameter.setExample("PT1S");
        } else if (!parameter.isInput() && this.isEnumerateWrapping(parameter)) {
            parameter.setType(Object.class);
            List<Parameter> fields = Lists.newLinkedList();
            List<Parameter.Option> options = ObjectUtils.ifEmpty(
                    parameter.getOptions(),
                    os -> os.stream().map(option -> Parameter.Option.builder().key(option.getValue())
                            .value(option.getKey()).deprecated(option.isDeprecated()).build())
                            .collect(Collectors.toList()),
                    Collections::emptyList
            );
            Class<?> type = ObjectUtils.ifNull(parameter.getField(), Field::getType, parameter::getOriginal);
            ObjectUtils.foreachClassFields(type, field -> {
                String name = field.getName();
                List<Parameter.Option> _options = JacksonEnumerateSerializer.CODE.equals(name)
                        || JacksonEnumerateSerializer.DESCRIPTION.equals(name) ? options : Collections.emptyList();
                fields.add(this.initializeEnumerateWrappingParameter(name, this.getDescription(field), _options));
            });
            if (fields.stream().noneMatch(field -> JacksonEnumerateSerializer.CODE.equals(field.getName()))) {
                fields.add(this.initializeEnumerateWrappingParameter(
                        JacksonEnumerateSerializer.CODE, "编码", options
                ));
            }
            if (fields.stream().noneMatch(field -> JacksonEnumerateSerializer.DESCRIPTION.equals(field.getName()))) {
                fields.add(this.initializeEnumerateWrappingParameter(
                        JacksonEnumerateSerializer.DESCRIPTION, "描述", options
                ));
            }
            parameter.setFields(fields);
            parameter.setOptions(Collections.emptyList());
        }
    }
}
