package com.chelvc.framework.base.config;

import java.util.List;
import java.util.Map;
import java.util.function.Predicate;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.util.SpringUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.Maps;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.SpringApplicationRunListener;
import org.springframework.boot.autoconfigure.web.servlet.WebMvcRegistrations;
import org.springframework.context.annotation.Import;
import org.springframework.core.io.Resource;
import org.springframework.web.servlet.mvc.method.annotation.RequestMappingHandlerMapping;

/**
 * 多服务部署配置
 *
 * @author Woody
 * @date 2024/4/3
 */
public class MultipleServerConfigurer implements WebMvcRegistrations {
    /**
     * 多服务部署应用监听类
     */
    public static class Listener implements SpringApplicationRunListener {
        /**
         * 是否已初始化
         */
        private static volatile boolean INITIALIZED;

        public Listener(SpringApplication application, String[] args) {
            if (!INITIALIZED) {
                synchronized (Listener.class) {
                    if (!INITIALIZED) {
                        INITIALIZED = true;
                        SpringUtils.setMultipleServer(this.isMultipleServer(application));
                    }
                }
            }
        }

        /**
         * 判断是否是多服务配置
         *
         * @param application Spring应用实例
         * @return true/false
         */
        private boolean isMultipleServer(SpringApplication application) {
            Class<?> main = application.getMainApplicationClass();
            if (main != null) {
                Class<?>[] imports = ObjectUtils.ifNull(main.getAnnotation(Import.class), Import::value);
                if (ObjectUtils.notEmpty(imports)) {
                    for (Class<?> clazz : imports) {
                        if (clazz == MultipleServerConfigurer.class) {
                            return true;
                        }
                    }
                }
            }
            return false;
        }
    }

    @Override
    public RequestMappingHandlerMapping getRequestMappingHandlerMapping() {
        List<Resource> resources = SpringUtils.getPropertyResources();
        Map<String, Predicate<Class<?>>> prefixes = Maps.newHashMapWithExpectedSize(resources.size());
        for (Resource resource : resources) {
            String server = ApplicationContextHolder.getApplicationName(resource);
            if (StringUtils.notEmpty(server)) {
                prefixes.put(server, clazz -> SpringUtils.isResourceClass(resource, clazz));
            }
        }
        RequestMappingHandlerMapping mapping = new RequestMappingHandlerMapping();
        mapping.setPathPrefixes(prefixes);
        return mapping;
    }
}