package com.chelvc.framework.base.context;

import java.lang.ref.WeakReference;
import java.time.Duration;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Objects;
import java.util.Properties;
import java.util.Set;
import java.util.function.Consumer;
import java.util.function.Function;

import com.chelvc.framework.base.util.SpringUtils;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.common.model.Period;
import com.chelvc.framework.common.model.Range;
import com.chelvc.framework.common.model.Region;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.DateUtils;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.fasterxml.jackson.core.type.TypeReference;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.aop.framework.AopProxyUtils;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.NoSuchBeanDefinitionException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.EnvironmentAware;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.core.env.Environment;
import org.springframework.core.io.Resource;
import org.springframework.stereotype.Component;

/**
 * 应用上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@Component
@Order(Ordered.HIGHEST_PRECEDENCE)
public class ApplicationContextHolder implements EnvironmentAware, ApplicationContextAware, PropertyRefreshListener {
    /**
     * 应用名称属性
     */
    public static final String APPLICATION_NAME_PROPERTY = "spring.application.name";

    /**
     * 应用环境属性
     */
    public static final String APPLICATION_PROFILE_PROPERTY = "spring.profiles.active";

    /**
     * 属性名/对象实例映射表
     */
    private static final Map<String, WeakReference<Object>> PROPERTY_OBJECT_MAPPING = Maps.newConcurrentMap();

    /**
     * 应用环境对象
     */
    private static Environment ENVIRONMENT;

    /**
     * 应用上下文对象
     */
    private static ApplicationContext APPLICATION_CONTEXT;

    @Override
    public void setEnvironment(Environment environment) {
        ENVIRONMENT = environment;
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        APPLICATION_CONTEXT = applicationContext;
    }

    @Override
    public void refreshProperty(Set<String> properties) {
        if (ObjectUtils.notEmpty(properties)) {
            properties.forEach(PROPERTY_OBJECT_MAPPING::remove);
        }
    }

    /**
     * 获取环境标识
     *
     * @return 环境标识
     */
    public static String getProfile() {
        return getProfile(false);
    }

    /**
     * 获取环境标识
     *
     * @param required 是否必须
     * @return 环境标识
     */
    public static String getProfile(boolean required) {
        Environment environment = getEnvironment(required);
        return environment == null ? null : getProfile(environment);
    }

    /**
     * 根据 *.yml、*.properties文件资源获取环境标识
     *
     * @param resource *.yml、*.properties文件资源对象实例
     * @return 环境标识
     */
    public static String getProfile(@NonNull Resource resource) {
        Properties properties = SpringUtils.getResourceProperties(resource);
        return properties == null ? null : properties.getProperty(APPLICATION_PROFILE_PROPERTY);
    }

    /**
     * 获取环境标识
     *
     * @param environment 应用环境
     * @return 环境标识
     */
    public static String getProfile(@NonNull Environment environment) {
        return environment.getProperty(APPLICATION_PROFILE_PROPERTY);
    }

    /**
     * 判断是否是生产环境
     *
     * @return true/false
     */
    public static boolean isProduction() {
        Boolean bool = getProperty("environment.production", Boolean.class);
        return Boolean.TRUE.equals(bool) || Objects.equals(getProfile(), "prod");
    }

    /**
     * 获取应用名称
     *
     * @return 应用名称
     */
    public static String getApplicationName() {
        return getApplicationName(false);
    }

    /**
     * 获取应用名称
     *
     * @param required 是否必须
     * @return 应用名称
     */
    public static String getApplicationName(boolean required) {
        Environment environment = getEnvironment(required);
        return environment == null ? null : getApplicationName(environment);
    }

    /**
     * 根据 *.yml、*.properties文件资源获取应用名称
     *
     * @param resource *.yml、*.properties文件资源对象实例
     * @return 应用名称
     */
    public static String getApplicationName(@NonNull Resource resource) {
        Properties properties = SpringUtils.getResourceProperties(resource);
        return properties == null ? null : properties.getProperty(APPLICATION_NAME_PROPERTY);
    }

    /**
     * 获取应用名称
     *
     * @param environment 应用环境
     * @return 应用名称
     */
    public static String getApplicationName(@NonNull Environment environment) {
        return environment.getProperty(APPLICATION_NAME_PROPERTY);
    }

    /**
     * 获取应用环境对象
     *
     * @return 应用环境对象
     */
    public static Environment getEnvironment() {
        return getEnvironment(true);
    }

    /**
     * 获取应用环境对象
     *
     * @param required 是否必须
     * @return 应用环境对象
     */
    public static Environment getEnvironment(boolean required) {
        if (required) {
            AssertUtils.state(ENVIRONMENT != null, () -> "Application environment has not been initialized");
        }
        return ENVIRONMENT;
    }

    /**
     * 获取应用上下文对象
     *
     * @return 应用上下文对象
     */
    public static ApplicationContext getApplicationContext() {
        return getApplicationContext(true);
    }

    /**
     * 获取应用上下文对象
     *
     * @param required 是否必须
     * @return 应用上下文对象
     */
    public static ApplicationContext getApplicationContext(boolean required) {
        if (required) {
            AssertUtils.state(APPLICATION_CONTEXT != null, () -> "Application context has not been initialized");
        }
        return APPLICATION_CONTEXT;
    }

    /**
     * 基于@Order注解对类对象排序
     *
     * @param classes 类对象列表
     */
    public static void sort(List<Class<?>> classes) {
        if (ObjectUtils.notEmpty(classes)) {
            classes.sort(Comparator.nullsFirst(Comparator.comparingInt(clazz -> ObjectUtils.ifNull(
                    clazz.getAnnotation(Order.class), Order::value, () -> 0
            ))));
        }
    }

    /**
     * 基于@Order注解对Bean排序
     *
     * @param beans Bean对象实例集合
     * @param <T>   对象类型
     * @return 排序后对象实例列表
     */
    public static <T> List<T> sort(Collection<? extends T> beans) {
        if (ObjectUtils.isEmpty(beans)) {
            return Collections.emptyList();
        }
        List<T> orders = Lists.newArrayListWithCapacity(beans.size());
        sort(beans, orders::add);
        return orders;
    }

    /**
     * 基于@Order注解对Bean排序
     *
     * @param beans    Bean对象实例集合
     * @param consumer Bean对象实例回调函数
     * @param <T>      对象类型
     */
    public static <T> void sort(Collection<? extends T> beans, @NonNull Consumer<T> consumer) {
        if (ObjectUtils.isEmpty(beans)) {
            return;
        }
        beans.stream().sorted(Comparator.nullsFirst(Comparator.comparingInt(bean -> {
            if (bean instanceof Ordered) {
                return ((Ordered) bean).getOrder();
            }
            Order order = AopProxyUtils.ultimateTargetClass(bean).getAnnotation(Order.class);
            return order == null ? 0 : order.value();
        }))).forEach(consumer);
    }

    /**
     * 获取对象实例
     *
     * @param type 对象类型
     * @param <T>  对象类型泛型
     * @return 对象实例
     */
    public static <T> T getBean(@NonNull Class<T> type) {
        return getBean(type, true);
    }

    /**
     * 获取对象实例
     *
     * @param applicationContext 应用上下文
     * @param type               对象类型
     * @param <T>                对象类型泛型
     * @return 对象实例
     */
    public static <T> T getBean(@NonNull ApplicationContext applicationContext, @NonNull Class<T> type) {
        return getBean(applicationContext, type, true);
    }

    /**
     * 获取对象实例
     *
     * @param type     对象类型
     * @param required 对象实例是否必须存在
     * @param <T>      对象类型泛型
     * @return 对象实例
     */
    public static <T> T getBean(@NonNull Class<T> type, boolean required) {
        ApplicationContext applicationContext = getApplicationContext(required);
        return applicationContext == null ? null : getBean(applicationContext, type, required);
    }

    /**
     * 获取对象实例
     *
     * @param applicationContext 应用上下文
     * @param type               对象类型
     * @param required           对象实例是否必须存在
     * @param <T>                对象类型泛型
     * @return 对象实例
     */
    public static <T> T getBean(@NonNull ApplicationContext applicationContext, @NonNull Class<T> type,
                                boolean required) {
        try {
            return applicationContext.getBean(type);
        } catch (NoSuchBeanDefinitionException e) {
            if (required) {
                throw e;
            }
            return null;
        }
    }

    /**
     * 获取对象实例
     *
     * @param name 对象名称
     * @param <T>  对象类型泛型
     * @return 对象实例
     */
    public static <T> T getBean(@NonNull String name) {
        return getBean(name, true);
    }

    /**
     * 获取对象实例
     *
     * @param applicationContext 应用上下文
     * @param name               对象名称
     * @param <T>                对象类型泛型
     * @return 对象实例
     */
    public static <T> T getBean(@NonNull ApplicationContext applicationContext, @NonNull String name) {
        return getBean(applicationContext, name, true);
    }

    /**
     * 获取对象实例
     *
     * @param name     对象名称
     * @param required 对象实例是否必须存在
     * @param <T>      对象类型泛型
     * @return 对象实例
     */
    public static <T> T getBean(@NonNull String name, boolean required) {
        ApplicationContext applicationContext = getApplicationContext(required);
        return applicationContext == null ? null : getBean(applicationContext, name, required);
    }

    /**
     * 获取对象实例
     *
     * @param applicationContext 应用上下文
     * @param name               对象名称
     * @param required           对象实例是否必须存在
     * @param <T>                对象类型泛型
     * @return 对象实例
     */
    @SuppressWarnings("unchecked")
    public static <T> T getBean(@NonNull ApplicationContext applicationContext, @NonNull String name,
                                boolean required) {
        try {
            return (T) applicationContext.getBean(name);
        } catch (NoSuchBeanDefinitionException e) {
            if (required) {
                throw e;
            }
            return null;
        }
    }

    /**
     * 获取对象名称/实例映射表
     *
     * @param type 对象类型
     * @param <T>  对象类型
     * @return 对象名称/实例映射表
     */
    public static <T> Map<String, T> getBeans(@NonNull Class<T> type) {
        return getBeans(getApplicationContext(), type);
    }

    /**
     * 获取对象名称/实例映射表
     *
     * @param applicationContext 应用上下文
     * @param type               对象类型
     * @param <T>                对象类型
     * @return 对象名称/实例映射表
     */
    public static <T> Map<String, T> getBeans(@NonNull ApplicationContext applicationContext, @NonNull Class<T> type) {
        return applicationContext.getBeansOfType(type);
    }

    /**
     * 根据@Order注解获取排序Bean实例
     *
     * @param type 对象类型
     * @param <T>  对象类型
     * @return Bean对象实例列表
     */
    public static <T> List<T> getSortBeans(@NonNull Class<? extends T> type) {
        return getSortBeans(getApplicationContext(), type);
    }

    /**
     * 根据@Order注解获取排序Bean实例
     *
     * @param applicationContext 应用上下文
     * @param type               对象类型
     * @param <T>                对象类型
     * @return Bean对象实例列表
     */
    public static <T> List<T> getSortBeans(@NonNull ApplicationContext applicationContext,
                                           @NonNull Class<? extends T> type) {
        Map<String, ? extends T> beans = getBeans(applicationContext, type);
        return ObjectUtils.isEmpty(beans) ? Collections.emptyList() : sort(beans.values());
    }

    /**
     * 获取属性值
     *
     * @param key 属性名
     * @return 属性值
     */
    public static String getProperty(@NonNull String key) {
        return getProperty(key, String.class, null);
    }

    /**
     * 获取属性值
     *
     * @param key          属性名
     * @param defaultValue 默认值
     * @return 属性值
     */
    public static String getProperty(@NonNull String key, String defaultValue) {
        return getProperty(key, String.class, defaultValue);
    }

    /**
     * 获取属性值
     *
     * @param key  属性名
     * @param type 值类型
     * @param <T>  数据类型
     * @return 属性值
     */
    public static <T> T getProperty(@NonNull String key, @NonNull Class<T> type) {
        return getProperty(key, type, null);
    }

    /**
     * 获取属性值
     *
     * @param key          属性名
     * @param type         值类型
     * @param defaultValue 默认值
     * @param <T>          数据类型
     * @return 属性值
     */
    @SuppressWarnings("unchecked")
    public static <T> T getProperty(@NonNull String key, @NonNull Class<T> type, T defaultValue) {
        T value = null;
        if (type == String.class || ObjectUtils.isMetaClass(type) || type.isEnum() || type.isArray()
                || Collection.class.isAssignableFrom(type)) {
            Environment environment = getEnvironment(false);
            if (environment != null) {
                try {
                    if (type == String.class) {
                        value = (T) environment.getProperty(key);
                    } else {
                        value = environment.getProperty(key, type);
                    }
                } catch (Exception e) {
                    log.error("Get property value failed: {}", key, e);
                }
            }
        } else if (type == Date.class) {
            value = (T) getProperty(key, DateUtils::parse);
        } else if (type == Range.class) {
            value = (T) getProperty(key, Range::parse);
        } else if (type == Paging.class) {
            value = (T) getProperty(key, Paging::parse);
        } else if (type == Duration.class) {
            value = (T) getProperty(key, Duration::parse);
        } else if (type == Region.class) {
            value = (T) getProperty(key, code -> Region.of(Integer.valueOf(code)));
        } else if (type == Period.class) {
            Environment environment = getEnvironment(false);
            value = environment == null ? null : (T) StringUtils.ifEmpty(environment.getProperty(key), Period::parse);
        } else {
            value = getProperty(key, json -> JacksonUtils.deserialize(json, type));
        }
        return ObjectUtils.ifNull(value, defaultValue);
    }

    /**
     * 获取属性值
     *
     * @param key      属性名
     * @param function 属性值转换回调函数
     * @param <T>      数据类型
     * @return 属性值
     */
    @SuppressWarnings("unchecked")
    public static <T> T getProperty(@NonNull String key, @NonNull Function<String, T> function) {
        Object value = ObjectUtils.ifNull(PROPERTY_OBJECT_MAPPING.get(key), WeakReference::get);
        if (value == null) {
            Environment environment = getEnvironment(false);
            if (environment != null) {
                try {
                    value = function.apply(environment.getProperty(key));
                } catch (Exception e) {
                    log.error("Get property value failed: {}", key, e);
                }
            }
            value = ObjectUtils.ifNull(value, ObjectUtils.EMPTY);
            PROPERTY_OBJECT_MAPPING.put(key, new WeakReference<>(value));
        }
        return value == ObjectUtils.EMPTY ? null : (T) value;
    }

    /**
     * 获取属性值
     *
     * @param key  属性名
     * @param type 值类型
     * @param <T>  数据类型
     * @return 属性值
     */
    public static <T> T getProperty(@NonNull String key, @NonNull TypeReference<T> type) {
        return getProperty(key, type, null);
    }

    /**
     * 获取属性值
     *
     * @param key          属性名
     * @param type         值类型
     * @param defaultValue 默认值
     * @param <T>          数据类型
     * @return 属性值
     */
    public static <T> T getProperty(@NonNull String key, @NonNull TypeReference<T> type, T defaultValue) {
        T value = getProperty(key, json -> JacksonUtils.deserialize(json, type));
        return ObjectUtils.ifNull(value, defaultValue);
    }

    /**
     * 获取国际化消息
     *
     * @param code 消息标识
     * @param args 消息参数
     * @return 消息内容
     */
    public static String getMessage(@NonNull String code, @NonNull Object... args) {
        return getMessage(code, code, args);
    }

    /**
     * 获取国际化消息
     *
     * @param code     消息标识
     * @param defaults 默认消息
     * @param args     消息参数
     * @return 消息内容
     */
    public static String getMessage(@NonNull String code, String defaults, @NonNull Object... args) {
        return getMessage(LocaleContextHolder.getLocale(), code, defaults, args);
    }

    /**
     * 获取国际化消息
     *
     * @param locale 本地信息
     * @param code   消息标识
     * @param args   消息参数
     * @return 消息内容
     */
    public static String getMessage(@NonNull Locale locale, @NonNull String code, @NonNull Object... args) {
        return getMessage(locale, code, code, args);
    }

    /**
     * 获取国际化消息
     *
     * @param locale   本地信息
     * @param code     消息标识
     * @param defaults 默认消息
     * @param args     消息参数
     * @return 消息内容
     */
    public static String getMessage(@NonNull Locale locale, @NonNull String code, String defaults,
                                    @NonNull Object... args) {
        if (Objects.isNull(defaults)) {
            defaults = code;
        }
        ApplicationContext applicationContext = getApplicationContext(false);
        if (applicationContext != null) {
            try {
                return applicationContext.getMessage(code, args, defaults, locale);
            } catch (Exception e) {
                log.warn("Application message get failed [{}]: {}, {}", locale, code, e.getMessage());
            }
        }
        return defaults;
    }
}
