package com.chelvc.framework.base.context;

import java.util.Collections;
import java.util.Enumeration;
import java.util.Set;
import javax.servlet.http.HttpServletRequest;

import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.common.model.Platform;
import com.chelvc.framework.common.model.Terminal;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.Sets;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * Session对象工厂默认实现
 *
 * @author Woody
 * @date 2024/4/18
 */
@Slf4j
public class DefaultSessionFactory implements SessionFactory {
    /**
     * 获取客户端身份标识
     *
     * @param request Http请求对象
     * @return 身份标识
     */
    protected Long getId(@NonNull HttpServletRequest request) {
        try {
            return StringUtils.ifEmpty(request.getHeader(SessionContextHolder.HEADER_ID), Long::parseLong);
        } catch (Exception e) {
            log.warn("Id convert failed: {}", e.getMessage());
        }
        return null;
    }

    /**
     * 获取使用类别
     *
     * @param request Http请求对象
     * @return 使用类别
     */
    protected Using getUsing(@NonNull HttpServletRequest request) {
        Using using = null;
        try {
            using = StringUtils.ifEmpty(request.getHeader(SessionContextHolder.HEADER_USING), Using::valueOf);
        } catch (Exception e) {
            log.warn("Using convert failed: {}", e.getMessage());
        }
        return ObjectUtils.ifNull(using, Using.NORMAL);
    }

    /**
     * 获取客户端主机地址
     *
     * @param request Http请求对象
     * @return 主机地址
     */
    protected String getHost(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(HttpUtils.getHost(request), (String) null);
    }

    /**
     * 获取应用范围
     *
     * @param request Http请求对象
     * @return 应用范围
     */
    protected String getScope(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(SessionContextHolder.HEADER_SCOPE), (String) null);
    }

    /**
     * 获取手机号码
     *
     * @param request Http请求对象
     * @return 手机号码
     */
    protected String getMobile(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(SessionContextHolder.HEADER_MOBILE), (String) null);
    }

    /**
     * 获取客户端设备标识
     *
     * @param request Http请求对象
     * @return 设备标识
     */
    protected String getDevice(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(SessionContextHolder.HEADER_DEVICE), (String) null);
    }

    /**
     * 获取客户端渠道来源
     *
     * @param request Http请求对象
     * @return 渠道来源
     */
    protected String getChannel(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(SessionContextHolder.HEADER_CHANNEL), (String) null);
    }

    /**
     * 获取客户端平台标识
     *
     * @param request Http请求对象
     * @return 平台标识
     */
    protected Platform getPlatform(@NonNull HttpServletRequest request) {
        try {
            return StringUtils.ifEmpty(request.getHeader(SessionContextHolder.HEADER_PLATFORM), Platform::valueOf);
        } catch (Exception e) {
            log.warn("Platform convert failed: {}", e.getMessage());
        }
        return null;
    }

    /**
     * 获取客户端终端
     *
     * @param request Http请求对象
     * @return 终端标识
     */
    protected Terminal getTerminal(@NonNull HttpServletRequest request) {
        try {
            return StringUtils.ifEmpty(request.getHeader(SessionContextHolder.HEADER_TERMINAL), Terminal::valueOf);
        } catch (Exception e) {
            log.warn("Terminal convert failed: {}", e.getMessage());
        }
        return null;
    }

    /**
     * 获取客户端版本号
     *
     * @param request Http请求对象
     * @return 版本号
     */
    protected String getVersion(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(SessionContextHolder.HEADER_VERSION), (String) null);
    }

    /**
     * 获取请求时间戳
     *
     * @param request Http请求对象
     * @return 时间戳
     */
    protected Long getTimestamp(@NonNull HttpServletRequest request) {
        try {
            return StringUtils.ifEmpty(request.getHeader(SessionContextHolder.HEADER_TIMESTAMP), Long::parseLong);
        } catch (Exception e) {
            log.warn("Timestamp convert failed: {}", e.getMessage());
        }
        return null;
    }

    /**
     * 获取注册时间戳
     *
     * @param request Http请求对象
     * @return 时间戳
     */
    protected Long getRegistering(@NonNull HttpServletRequest request) {
        try {
            return StringUtils.ifEmpty(request.getHeader(SessionContextHolder.HEADER_REGISTERING), Long::parseLong);
        } catch (Exception e) {
            log.warn("Registering convert failed: {}", e.getMessage());
        }
        return null;
    }

    /**
     * 获取权限标识集合
     *
     * @param request Http请求对象
     * @return 权限标识集合
     */
    protected Set<String> getAuthorities(@NonNull HttpServletRequest request) {
        Enumeration<String> enumeration = request.getHeaders(SessionContextHolder.HEADER_AUTHORITIES);
        if (enumeration == null) {
            return Collections.emptySet();
        }
        Set<String> authorities = Sets.newHashSetWithExpectedSize(3);
        while (enumeration.hasMoreElements()) {
            authorities.add(enumeration.nextElement());
        }
        return authorities;
    }

    @Override
    public Session build(@NonNull HttpServletRequest request) {
        return Session.builder().id(this.getId(request)).using(this.getUsing(request)).host(this.getHost(request))
                .scope(this.getScope(request)).mobile(this.getMobile(request)).device(this.getDevice(request))
                .channel(this.getChannel(request)).platform(this.getPlatform(request))
                .terminal(this.getTerminal(request)).version(this.getVersion(request))
                .timestamp(this.getTimestamp(request)).registering(this.getRegistering(request))
                .authorities(Collections.unmodifiableSet(this.getAuthorities(request))).build();
    }
}
