package com.chelvc.framework.base.context;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Type;

import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.NonNull;

/**
 * Jackson上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class JacksonContextHolder {
    /**
     * 全局对象序列化处理器实例
     */
    private static volatile ObjectMapper GLOBAL_SERIALIZER;

    private JacksonContextHolder() {
    }

    /**
     * 获取全局序列化处理器实例
     *
     * @return 对象序列化处理器实例
     */
    public static ObjectMapper getGlobalSerializer() {
        if (GLOBAL_SERIALIZER == null) {
            synchronized (ObjectMapper.class) {
                if (GLOBAL_SERIALIZER == null) {
                    GLOBAL_SERIALIZER = ApplicationContextHolder.getBean(ObjectMapper.class, false);
                }
            }
        }
        return ObjectUtils.ifNull(GLOBAL_SERIALIZER, JacksonUtils::getDefaultSerializer);
    }

    /**
     * 将对象写入输出流
     *
     * @param output 输出流对象
     * @param value  对象实例
     * @throws IOException I/O异常
     */
    public static void write(@NonNull OutputStream output, Object value) throws IOException {
        JacksonUtils.write(getGlobalSerializer(), output, value);
    }

    /**
     * 对象序列化
     *
     * @param object 目标对象实例
     * @return JSON字符串
     */
    public static String serialize(Object object) {
        return JacksonUtils.serialize(getGlobalSerializer(), object);
    }

    /**
     * 对象反序列化
     *
     * @param json JSON字符串
     * @return 对象实例
     */
    public static Object deserialize(String json) {
        return JacksonUtils.deserialize(getGlobalSerializer(), json, Object.class);
    }

    /**
     * 对象反序列化
     *
     * @param json JSON字符串
     * @param type 类型对象
     * @return 对象实例
     */
    public static Object deserialize(String json, @NonNull Type type) {
        return JacksonUtils.deserialize(getGlobalSerializer(), json, type);
    }

    /**
     * 对象反序列化
     *
     * @param json JSON字符串
     * @param type 类型对象
     * @param <T>  对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(String json, @NonNull Class<T> type) {
        return JacksonUtils.deserialize(getGlobalSerializer(), json, type);
    }

    /**
     * 对象反序列化
     *
     * @param json JSON字符串
     * @param type 类型引用对象
     * @param <T>  对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(String json, @NonNull TypeReference<T> type) {
        return JacksonUtils.deserialize(getGlobalSerializer(), json, type);
    }

    /**
     * 对象反序列化
     *
     * @param bytes JSON字节数组
     * @return 对象实例
     */
    public static Object deserialize(byte[] bytes) {
        return JacksonUtils.deserialize(getGlobalSerializer(), bytes, Object.class);
    }

    /**
     * 对象反序列化
     *
     * @param bytes JSON字节数组
     * @param type  类型对象
     * @return 对象实例
     */
    public static Object deserialize(byte[] bytes, @NonNull Type type) {
        return JacksonUtils.deserialize(getGlobalSerializer(), bytes, type);
    }

    /**
     * 对象反序列化
     *
     * @param bytes JSON字节数组
     * @param type  类型对象
     * @param <T>   对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(byte[] bytes, @NonNull Class<T> type) {
        return JacksonUtils.deserialize(getGlobalSerializer(), bytes, type);
    }

    /**
     * 对象反序列化
     *
     * @param bytes JSON字节数组
     * @param type  类型引用对象
     * @param <T>   对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(byte[] bytes, @NonNull TypeReference<T> type) {
        return JacksonUtils.deserialize(getGlobalSerializer(), bytes, type);
    }

    /**
     * 对象反序列化
     *
     * @param input JSON字节流
     * @return 对象实例
     * @throws IOException I/O异常
     */
    public static Object deserialize(InputStream input) throws IOException {
        return JacksonUtils.deserialize(getGlobalSerializer(), input, Object.class);
    }

    /**
     * 对象反序列化
     *
     * @param input JSON字节流
     * @param type  类型对象
     * @return 对象实例
     * @throws IOException I/O异常
     */
    public static Object deserialize(InputStream input, @NonNull Type type) throws IOException {
        return JacksonUtils.deserialize(getGlobalSerializer(), input, type);
    }

    /**
     * 对象反序列化
     *
     * @param input JSON字节流
     * @param type  类型对象
     * @param <T>   对象类型
     * @return 对象实例
     * @throws IOException I/O异常
     */
    public static <T> T deserialize(InputStream input, @NonNull Class<T> type) throws IOException {
        return JacksonUtils.deserialize(getGlobalSerializer(), input, type);
    }

    /**
     * 对象反序列化
     *
     * @param input JSON字节流
     * @param type  类型引用对象
     * @param <T>   对象类型
     * @return 对象实例
     * @throws IOException I/O异常
     */
    public static <T> T deserialize(InputStream input, @NonNull TypeReference<T> type) throws IOException {
        return JacksonUtils.deserialize(getGlobalSerializer(), input, type);
    }
}
