package com.chelvc.framework.base.context;

import java.io.Serializable;
import java.util.Objects;

import com.chelvc.framework.common.exception.FrameworkException;
import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.NonNull;
import lombok.ToString;
import org.springframework.http.HttpStatus;

/**
 * 请求结果对象
 *
 * @param <T> 结果数据类型
 * @author Woody
 * @date 2024/1/30
 */
@Getter
@ToString
@EqualsAndHashCode
public class Result<T> implements Serializable {
    /**
     * 成功编码
     */
    public static final String OK = "OK";

    /**
     * 错误编码
     */
    public static final String ERROR = "ERROR";

    /**
     * 结果数据
     */
    private T data;

    /**
     * 状态码
     */
    private String code;

    /**
     * 结果信息
     */
    private String message;

    /**
     * 时间戳
     */
    private final Long timestamp = System.currentTimeMillis();

    /**
     * 构建成功结果
     *
     * @param <T> 结果数据类型
     * @return 结果对象
     */
    public static <T> Result<T> success() {
        return success(null);
    }

    /**
     * 构建成功结果
     *
     * @param data 结果数据
     * @param <T>  结果数据类型
     * @return 结果对象
     */
    public static <T> Result<T> success(T data) {
        return success(data, "Success");
    }

    /**
     * 构建成功结果
     *
     * @param data    结果数据
     * @param message 结果信息
     * @param <T>     结果数据类型
     * @return 结果对象
     */
    public static <T> Result<T> success(T data, String message) {
        return of(OK, data, message);
    }

    /**
     * 构建异常结果
     *
     * @param <T> 结果数据类型
     * @return 结果对象
     */
    public static <T> Result<T> failure() {
        return failure("Failure");
    }

    /**
     * 构建异常结果
     *
     * @param message 异常消息
     * @param <T>     结果数据类型
     * @return 结果对象
     */
    public static <T> Result<T> failure(String message) {
        return of(ERROR, null, message);
    }

    /**
     * 构建异常结果
     *
     * @param data    结果数据
     * @param message 异常消息
     * @param <T>     结果数据类型
     * @return 结果对象
     */
    public static <T> Result<T> failure(T data, String message) {
        return of(ERROR, data, message);
    }

    /**
     * 构建请求结果
     *
     * @param e 异常对象实例
     * @return 结果对象
     */
    public static Result<?> of(@NonNull FrameworkException e) {
        return of(e.getCode(), e.getData(), e.getMessage());
    }

    /**
     * 构建请求结果
     *
     * @param code 状态码
     * @param data 结果数据
     * @param <T>  结果数据类型
     * @return 结果对象
     */
    public static <T> Result<T> of(@NonNull String code, T data) {
        return of(code, data, null);
    }

    /**
     * 构建请求结果
     *
     * @param code    状态码
     * @param data    结果数据
     * @param message 结果信息
     * @param <T>     结果数据类型
     * @return 结果对象
     */
    public static <T> Result<T> of(@NonNull String code, T data, String message) {
        Result<T> result = new Result<>();
        result.code = code;
        result.data = data;
        result.message = message;
        return result;
    }

    /**
     * 获取Http状态码
     *
     * @return Http状态码
     */
    @JsonIgnore
    public HttpStatus status() {
        if (Objects.equals(this.code, OK)) {
            return HttpStatus.OK;
        } else if (Objects.equals(this.code, ERROR)) {
            return HttpStatus.INTERNAL_SERVER_ERROR;
        }
        return HttpStatus.BAD_REQUEST;
    }
}
