package com.chelvc.framework.base.context;

import java.io.Serializable;
import java.util.Collections;
import java.util.Map;
import java.util.Set;

import com.chelvc.framework.common.model.Caps;
import com.chelvc.framework.common.model.Platform;
import com.chelvc.framework.common.model.Terminal;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.google.common.collect.Maps;
import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.NonNull;
import lombok.Setter;
import lombok.ToString;

/**
 * 会话信息
 *
 * @author Woody
 * @date 2024/1/30
 */
@Getter
@Builder
@ToString
@NoArgsConstructor
@AllArgsConstructor
public class Session implements Serializable {
    /**
     * 会话信息命名
     */
    public static final String NAMING = "session";

    /**
     * 主体标识
     */
    private Long id;

    /**
     * 使用类别
     */
    private Using using;

    /**
     * 请求地址
     */
    private String host;

    /**
     * 应用范围
     */
    private String scope;

    /**
     * 手机号码
     */
    private String mobile;

    /**
     * 设备标识
     */
    private String device;

    /**
     * 渠道来源
     */
    private String channel;

    /**
     * 应用平台
     */
    private Platform platform;

    /**
     * 客户终端
     */
    private Terminal terminal;

    /**
     * 终端版本
     */
    private String version;

    /**
     * 请求时间戳
     */
    private Long timestamp;

    /**
     * 注册时间戳
     */
    private Long registering;

    /**
     * 权限标识集合
     */
    private Set<String> authorities;

    /**
     * 分组场景/标识映射表
     */
    @JsonIgnore
    @Builder.Default
    @Getter(AccessLevel.NONE)
    @Setter(AccessLevel.NONE)
    private final Map<String, Caps> groups = Maps.newConcurrentMap();

    /**
     * 获取场景分组
     *
     * @param scene 分组场景
     * @return 分组标识
     */
    public Caps getGroup(@NonNull String scene) {
        return this.groups.get(scene);
    }

    /**
     * 设置场景分组
     *
     * @param scene 分组场景
     * @param group 分组标识
     */
    public void setGroup(@NonNull String scene, @NonNull Caps group) {
        this.groups.put(scene, group);
    }

    /**
     * 初始化会话主体信息
     *
     * @param id          主体标识
     * @param using       使用类别
     * @param scope       应用范围
     * @param mobile      手机号码
     * @param registering 注册时间戳
     * @param authorities 权限标识集合
     */
    void initializePrincipal(@NonNull Long id, @NonNull Using using, @NonNull String scope, String mobile,
                             Long registering, @NonNull Set<String> authorities) {
        this.id = id;
        this.using = using;
        this.scope = scope;
        this.mobile = mobile;
        this.registering = registering;
        this.authorities = Collections.unmodifiableSet(authorities);
    }
}
