package com.chelvc.framework.base.crypto;

import java.util.List;
import java.util.Set;

import com.chelvc.framework.common.util.ObjectUtils;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 加解密处理接口
 *
 * @author Woody
 * @date 2024/12/26
 */
public interface CipherHandler {
    /**
     * 日志处理器实例
     */
    Logger log = LoggerFactory.getLogger(CipherHandler.class);

    /**
     * 数据加密
     *
     * @param bytes 明文字节数组
     * @return 密文字节数组
     */
    byte[] encrypt(byte[] bytes);

    /**
     * 数据加密
     *
     * @param plaintext 明文
     * @return 密文
     */
    String encrypt(String plaintext);

    /**
     * 数据加密
     *
     * @param plaintext 明文
     * @param force     是否忽略异常
     * @return 密文
     */
    default String encrypt(String plaintext, boolean force) {
        try {
            return this.encrypt(plaintext);
        } catch (Exception e) {
            if (!force) {
                throw e;
            }
            log.error("Plaintext encrypt failed: {}, {}", plaintext, e.getMessage());
        }
        return plaintext;
    }

    /**
     * 数据加密
     *
     * @param plaintexts 明文数组
     * @return 密文数组
     */
    default String[] encrypt(String... plaintexts) {
        return this.encrypt(plaintexts, false);
    }

    /**
     * 数据加密
     *
     * @param plaintexts 明文数组
     * @param force      是否忽略异常
     * @return 密文数组
     */
    default String[] encrypt(String[] plaintexts, boolean force) {
        if (ObjectUtils.isEmpty(plaintexts)) {
            return plaintexts;
        }
        String[] ciphertexts = new String[plaintexts.length];
        for (int i = 0; i < plaintexts.length; i++) {
            ciphertexts[i] = this.encrypt(plaintexts[i], force);
        }
        return ciphertexts;
    }

    /**
     * 数据加密
     *
     * @param plaintexts 明文集合
     * @return 密文集合
     */
    default Set<String> encrypt(Set<String> plaintexts) {
        return this.encrypt(plaintexts, false);
    }

    /**
     * 数据加密
     *
     * @param plaintexts 明文集合
     * @param force      是否忽略异常
     * @return 密文集合
     */
    default Set<String> encrypt(Set<String> plaintexts, boolean force) {
        if (ObjectUtils.isEmpty(plaintexts)) {
            return plaintexts;
        }
        Set<String> ciphertexts = Sets.newHashSetWithExpectedSize(plaintexts.size());
        plaintexts.forEach(plaintext -> ciphertexts.add(this.encrypt(plaintext, force)));
        return ciphertexts;
    }

    /**
     * 数据加密
     *
     * @param plaintexts 明文列表
     * @return 密文列表
     */
    default List<String> encrypt(List<String> plaintexts) {
        return this.encrypt(plaintexts, false);
    }

    /**
     * 数据加密
     *
     * @param plaintexts 明文列表
     * @param force      是否忽略异常
     * @return 密文列表
     */
    default List<String> encrypt(List<String> plaintexts, boolean force) {
        if (ObjectUtils.isEmpty(plaintexts)) {
            return plaintexts;
        }
        List<String> ciphertexts = Lists.newArrayListWithCapacity(plaintexts.size());
        plaintexts.forEach(plaintext -> ciphertexts.add(this.encrypt(plaintext, force)));
        return ciphertexts;
    }

    /**
     * 数据解密
     *
     * @param bytes 密文字节数组
     * @return 明文字节数组
     */
    byte[] decrypt(byte[] bytes);

    /**
     * 数据解密
     *
     * @param ciphertext 密文
     * @return 明文
     */
    String decrypt(String ciphertext);

    /**
     * 数据解密
     *
     * @param ciphertext 密文
     * @param force      是否忽略异常
     * @return 明文
     */
    default String decrypt(String ciphertext, boolean force) {
        try {
            return this.decrypt(ciphertext);
        } catch (Exception e) {
            if (!force) {
                throw e;
            }
            log.error("Ciphertext decrypt failed: {}, {}", ciphertext, e.getMessage());
        }
        return ciphertext;
    }

    /**
     * 数据解密
     *
     * @param ciphertexts 密文数组
     * @return 明文数组
     */
    default String[] decrypt(String... ciphertexts) {
        return this.decrypt(ciphertexts, false);
    }

    /**
     * 数据解密
     *
     * @param ciphertexts 密文数组
     * @param force       是否忽略异常
     * @return 明文数组
     */
    default String[] decrypt(String[] ciphertexts, boolean force) {
        if (ObjectUtils.isEmpty(ciphertexts)) {
            return ciphertexts;
        }
        String[] plaintexts = new String[ciphertexts.length];
        for (int i = 0; i < ciphertexts.length; i++) {
            plaintexts[i] = this.decrypt(ciphertexts[i], force);
        }
        return plaintexts;
    }

    /**
     * 数据解密
     *
     * @param ciphertexts 密文集合
     * @return 明文集合
     */
    default Set<String> decrypt(Set<String> ciphertexts) {
        return this.decrypt(ciphertexts, false);
    }

    /**
     * 数据解密
     *
     * @param ciphertexts 密文集合
     * @param force       是否忽略异常
     * @return 明文集合
     */
    default Set<String> decrypt(Set<String> ciphertexts, boolean force) {
        if (ObjectUtils.isEmpty(ciphertexts)) {
            return ciphertexts;
        }
        Set<String> plaintexts = Sets.newHashSetWithExpectedSize(ciphertexts.size());
        ciphertexts.forEach(ciphertext -> plaintexts.add(this.decrypt(ciphertext, force)));
        return plaintexts;
    }

    /**
     * 数据解密
     *
     * @param ciphertexts 密文列表
     * @return 明文列表
     */
    default List<String> decrypt(List<String> ciphertexts) {
        return this.decrypt(ciphertexts, false);
    }

    /**
     * 数据解密
     *
     * @param ciphertexts 密文列表
     * @param force       是否忽略异常
     * @return 明文列表
     */
    default List<String> decrypt(List<String> ciphertexts, boolean force) {
        if (ObjectUtils.isEmpty(ciphertexts)) {
            return ciphertexts;
        }
        List<String> plaintexts = Lists.newArrayListWithCapacity(ciphertexts.size());
        ciphertexts.forEach(ciphertext -> plaintexts.add(this.decrypt(ciphertext, force)));
        return plaintexts;
    }
}
