package com.chelvc.framework.base.interceptor;

import ch.qos.logback.classic.LoggerContext;
import com.chelvc.framework.base.context.LoggingContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.ILoggerFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.web.servlet.error.ErrorController;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.ExceptionHandler;

/**
 * Controller接口访问拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@Aspect
@Component
@Order(Ordered.HIGHEST_PRECEDENCE)
public class ControllerAccessInterceptor {
    /**
     * 获取接口方法Logger实例
     *
     * @param point 接口方法拦截点
     * @return Logger实例
     */
    private Logger getLogger(JoinPoint point) {
        // 排除ErrorController接口
        if (point.getTarget() instanceof ErrorController) {
            return null;
        }

        // 排除@ExceptionHandler注解方法
        Signature signature = point.getSignature();
        if (signature instanceof MethodSignature
                && ((MethodSignature) signature).getMethod().isAnnotationPresent(ExceptionHandler.class)) {
            return null;
        }

        // 获取Logger实例
        ILoggerFactory factory = LoggerFactory.getILoggerFactory();
        String name = signature.getDeclaringTypeName() + "." + signature.getName();
        return factory instanceof LoggerContext ? ((LoggerContext) factory).exists(name) : factory.getLogger(name);
    }

    /**
     * 接口请求日志打印
     *
     * @param point 接口方法拦截点
     * @throws Throwable 接口调用异常
     */
    @Around("@within(org.springframework.stereotype.Controller) " +
            "|| @within(org.springframework.web.bind.annotation.RestController)")
    public Object logging(ProceedingJoinPoint point) throws Throwable {
        Object value = point.proceed();
        if (!(point.getTarget() instanceof ErrorController)) {
            try {
                Logger logger = this.getLogger(point);
                if (logger != null) {
                    LoggingContextHolder.debug(logger, SessionContextHolder.getRequest(), value);
                }
            } catch (Exception e) {
                log.error("Controller access logging failed", e);
            }
        }
        return value;
    }
}
