package com.chelvc.framework.base.interceptor;

import java.util.Objects;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.annotation.ResponseWrapping;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.Result;
import com.chelvc.framework.base.context.SessionContextHolder;
import org.springframework.core.MethodParameter;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.support.HandlerMethodReturnValueHandler;
import org.springframework.web.method.support.ModelAndViewContainer;

/**
 * 响应结果包装处理器实现
 *
 * @author Woody
 * @date 2024/1/30
 */
public class ResponseWrapInterceptor implements HandlerMethodReturnValueHandler {
    private final HandlerMethodReturnValueHandler handler;

    public ResponseWrapInterceptor(HandlerMethodReturnValueHandler handler) {
        this.handler = handler;
    }

    /**
     * 判断目标方法是否使用@ResponseBody注解
     *
     * @param method 目标方法
     * @return true/false
     */
    private boolean isResponseBodyMethod(MethodParameter method) {
        return method.hasMethodAnnotation(ResponseBody.class)
                || method.getDeclaringClass().isAnnotationPresent(ResponseBody.class)
                || method.getDeclaringClass().isAnnotationPresent(RestController.class);
    }

    /**
     * 判断目标方法是否使用了@ResponseWrapping注解
     *
     * @param method 目标方法
     * @return true/false
     */
    private boolean isResponseWrappingMethod(MethodParameter method) {
        ResponseWrapping annotation = method.getMethodAnnotation(ResponseWrapping.class);
        if (annotation == null) {
            annotation = method.getDeclaringClass().getAnnotation(ResponseWrapping.class);
        }
        return annotation != null && annotation.enabled();
    }

    /**
     * 包装请求结果
     *
     * @param request 请求对象
     * @param method  请求方法
     * @param value   结果值
     * @return 结果包装对象实例
     */
    protected Result<?> wrap(NativeWebRequest request, MethodParameter method, Object value) {
        if (value instanceof Result) {
            return (Result<?>) value;
        }
        return Result.success(value, ApplicationContextHolder.getMessage("Success"));
    }

    @Override
    public boolean supportsReturnType(MethodParameter method) {
        return this.isResponseBodyMethod(method) && this.isResponseWrappingMethod(method);
    }

    @Override
    public void handleReturnValue(Object value, MethodParameter method, ModelAndViewContainer container,
                                  NativeWebRequest request) throws Exception {
        Result<?> result = this.wrap(request, method, value);
        if (this.handler == null) {
            container.setRequestHandled(true);
            HttpServletResponse response = (HttpServletResponse) request.getNativeResponse();
            SessionContextHolder.response(Objects.requireNonNull(response), result);
        } else {
            this.handler.handleReturnValue(result, method, container, request);
        }
    }
}
