package com.chelvc.framework.base.interceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.annotation.Version;
import com.chelvc.framework.base.annotation.Versions;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import com.chelvc.framework.common.model.Terminal;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import org.springframework.core.Ordered;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

/**
 * 接口版本控制拦截器
 *
 * @author Woody
 * @date 2024/12/18
 */
@Component
public class VersionControlInterceptor implements HandlerInterceptor, WebMvcConfigurer {
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
            throws Exception {
        if (!(handler instanceof HandlerMethod)) {
            return true;
        }

        Terminal terminal = SessionContextHolder.getTerminal();
        if (terminal == null) {
            return true;
        }

        // 接口方法/类版本注解校验
        HandlerMethod method = (HandlerMethod) handler;
        Version[] versions = ObjectUtils.ifNull(method.getMethodAnnotation(Versions.class), Versions::value);
        if (ObjectUtils.isEmpty(versions)) {
            versions = ObjectUtils.ifNull(method.getBeanType().getAnnotation(Versions.class), Versions::value);
        }
        if (ObjectUtils.notEmpty(versions)) {
            Environment environment = ApplicationContextHolder.getEnvironment(false);
            for (Version version : versions) {
                if (version.terminal() == terminal && StringUtils.notEmpty(version.value())
                        && !SessionContextHolder.isVersion(version)) {
                    String message = version.message();
                    if (environment != null && StringUtils.notEmpty(message)) {
                        message = environment.resolvePlaceholders(message);
                    }
                    throw new ResourceUnavailableException(message);
                }
            }
        }

        // 终端最低版本配置校验
        String minimum = ApplicationContextHolder.getProperty(terminal.name().toLowerCase() + ".version.minimum");
        if (StringUtils.notEmpty(minimum) && !SessionContextHolder.isAfterVersion(terminal, minimum, true)) {
            throw new ResourceUnavailableException(ApplicationContextHolder.getMessage("Version.Minimum.Limit"));
        }
        return true;
    }

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(this).addPathPatterns("/**").order(Ordered.LOWEST_PRECEDENCE);
    }
}
