package com.chelvc.framework.base.logic;

import java.util.Map;
import java.util.function.Supplier;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.Maps;
import lombok.NonNull;

/**
 * 上下文参数变量解析器实现
 *
 * @author Woody
 * @date 2025/2/23
 */
public class ContextVariableParser implements VariableParser {
    /**
     * 请求参数变量前缀
     */
    public static final String REQUEST_PARAM_PREFIX = "request.param.";

    /**
     * 请求头变量前缀
     */
    public static final String REQUEST_HEADER_PREFIX = "request.header.";

    private final Map<String, Supplier<String>> providers = Maps.newHashMap();

    public ContextVariableParser() {
        this.initialize();
    }

    /**
     * 初始化解析器
     */
    protected void initialize() {
        // 设置默认参数变量值提供器
        this.setProvider("request.ip", SessionContextHolder::getHost);
        this.setProvider("request.uri", SessionContextHolder::getUri);
        this.setProvider("request.method", SessionContextHolder::getMethod);
        this.setProvider("request.content-type", SessionContextHolder::getContentType);
        this.setProvider("request.content-length", () -> StringUtils.toString(SessionContextHolder.getContentLength()));
        this.setProvider("request.header.host", () -> SessionContextHolder.getHeader("host"));
        this.setProvider("request.header.referer", () -> SessionContextHolder.getHeader("referer"));
        this.setProvider("request.header.device", SessionContextHolder::getDevice);
        this.setProvider("request.header.channel", SessionContextHolder::getChannel);
        this.setProvider("request.header.version", SessionContextHolder::getVersion);
        this.setProvider("request.header.platform", () -> StringUtils.toString(SessionContextHolder.getPlatform()));
        this.setProvider("request.header.terminal", () -> StringUtils.toString(SessionContextHolder.getTerminal()));
        this.setProvider("request.header.timestamp", () -> StringUtils.toString(SessionContextHolder.getTimestamp()));
        this.setProvider("request.header.user-agent", () -> SessionContextHolder.getHeader("user-agent"));
        this.setProvider("request.header.content-type", SessionContextHolder::getContentType);
        this.setProvider("session.id", () -> StringUtils.toString(SessionContextHolder.getId()));
        this.setProvider("session.scope", SessionContextHolder::getScope);
        this.setProvider("session.using", () -> StringUtils.toString(SessionContextHolder.getUsing()));
        this.setProvider("session.mobile", SessionContextHolder::getMobile);
    }

    /**
     * 设置参数变量值提供器
     *
     * @param variable 参数变量
     * @param provider 参数值提供器
     */
    protected void setProvider(@NonNull String variable, @NonNull Supplier<String> provider) {
        this.providers.put(variable, provider);
    }

    @Override
    public String parse(String variable) {
        if (StringUtils.isEmpty(variable)) {
            return null;
        }

        Supplier<String> provider = this.providers.get(variable);
        if (provider != null) {
            return provider.get();
        } else if (variable.startsWith(REQUEST_PARAM_PREFIX)) {
            // 解析请求参数变量
            return SessionContextHolder.getParameter(variable.substring(REQUEST_PARAM_PREFIX.length()));
        } else if (variable.startsWith(REQUEST_HEADER_PREFIX)) {
            // 解析请求头变量
            return SessionContextHolder.getHeader(variable.substring(REQUEST_HEADER_PREFIX.length()));
        }
        return ApplicationContextHolder.getProperty(variable);
    }
}
