package com.chelvc.framework.base.logic;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import lombok.NonNull;

/**
 * 逻辑判断表达式解析器默认实现
 *
 * @author Woody
 * @date 2025/2/25
 */
public class DefaultJudgeAnalyser implements JudgeAnalyser {
    /**
     * 逻辑判断表达式解析器实例
     */
    private static final DefaultJudgeAnalyser INSTANCE = new DefaultJudgeAnalyser();

    /**
     * 符号正则匹配模式
     */
    private static final Pattern SYMBOL_PATTERN =
            Pattern.compile("( +!=≈| +!≈=| +=≈| +≈=| +!=| +!≈| +>=| +<=| +=| +≈| +>| +<)");

    /**
     * 获取默认逻辑判断表达式解析器实例
     *
     * @return 逻辑判断表达式解析器实例
     */
    public static DefaultJudgeAnalyser getInstance() {
        return INSTANCE;
    }

    /**
     * 构建逻辑判断对象
     *
     * @param left   运算符左表达式
     * @param right  运算符右表达式
     * @param symbol 运算符号
     * @return 逻辑判断对象实例
     */
    protected Judge build(String left, String right, String symbol) {
        switch (symbol) {
            case Eq.SYMBOL:
                return new Eq(left, right);
            case Neq.SYMBOL:
                return new Neq(left, right);
            case In.SYMBOL:
                return new In(left, right);
            case Nin.SYMBOL:
                return new Nin(left, right);
            case St.SYMBOL:
                return new St(left, right);
            case Nst.SYMBOL:
                return new Nst(left, right);
            case En.SYMBOL:
                return new En(left, right);
            case Nen.SYMBOL:
                return new Nen(left, right);
            case Gt.SYMBOL:
                return new Gt(left, right);
            case Ge.SYMBOL:
                return new Ge(left, right);
            case Lt.SYMBOL:
                return new Lt(left, right);
            case Le.SYMBOL:
                return new Le(left, right);
            default:
                return null;
        }
    }

    @Override
    public Judge analyse(@NonNull String expression) {
        Matcher matcher = SYMBOL_PATTERN.matcher(expression = expression.trim());
        if (matcher.find() && (matcher.end() == expression.length()
                || expression.charAt(matcher.end()) <= Expression.SPACE)) {
            String symbol = matcher.group().trim();
            String left = expression.substring(0, matcher.start()).trim();
            String right = expression.substring(matcher.end()).trim();
            Judge judge = this.build(left, right, symbol);
            if (judge != null) {
                return judge;
            }
        }
        throw new IllegalArgumentException("Invalid expression: " + expression);
    }
}
