package com.chelvc.framework.base.logic;

import java.io.Serializable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Objects;
import java.util.Stack;
import java.util.function.BiConsumer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.chelvc.framework.common.model.Pair;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.Lists;
import lombok.NonNull;

/**
 * 逻辑表达式对象
 * <p>
 * 示例：a = 1 && (b = 2 || c = 3) && (e = 4 && (f = 5 || g = 6))
 *
 * @author Woody
 * @date 2025/2/19
 */
public abstract class Expression implements Serializable {
    /**
     * 空格字符
     */
    public static final char SPACE = ' ';

    /**
     * 左括号字符
     */
    public static final char LEFT_BRACKET = '(';

    /**
     * 右括号字符
     */
    public static final char RIGHT_BRACKET = ')';

    /**
     * 逻辑符号正则匹配模式
     */
    private static final Pattern LOGIC_SYMBOL_PATTERN = Pattern.compile("( +\\|\\| *| +&& *)");

    /**
     * 执行逻辑运算
     *
     * @return true/false
     */
    public boolean eval() {
        return this.eval(variable -> variable);
    }

    /**
     * 执行逻辑运算
     *
     * @param parser 参数变量解析函数
     * @return true/false
     */
    public abstract boolean eval(VariableParser parser);

    /**
     * 合并逻辑
     *
     * @param pairs 逻辑列表
     * @return 逻辑实例
     */
    private static Pair<String, Expression> merge(LinkedList<Pair<String, Expression>> pairs) {
        if (ObjectUtils.isEmpty(pairs)) {
            return null;
        }

        // 合并所有且逻辑
        Pair<String, Expression> and = null;
        Iterator<Pair<String, Expression>> iterator = pairs.iterator();
        while (iterator.hasNext()) {
            Pair<String, Expression> pair = iterator.next();
            if (and == null) {
                and = Objects.equals(pair.getKey(), And.SYMBOL) ? pair : null;
            } else {
                and.setValue(new And(and.getValue(), pair.getValue()));
                if (!Objects.equals(pair.getKey(), And.SYMBOL)) {
                    and = null;
                }
                iterator.remove();
            }
        }

        // 合并所有或逻辑
        Pair<String, Expression> first = (iterator = pairs.iterator()).next();
        while (iterator.hasNext()) {
            Pair<String, Expression> pair = iterator.next();
            first.setValue(new Or(first.getValue(), pair.getValue()));
            iterator.remove();
        }
        return first;
    }

    /**
     * 混合解析逻辑表达式
     *
     * @param analyser   判断逻辑解析器
     * @param expression 逻辑表达式
     * @return 逻辑运算符/实例信息
     */
    private static Pair<String, Expression> combine(JudgeAnalyser analyser, String expression) {
        Stack<Character> chars = new Stack<>();
        LinkedList<Pair<String, Expression>> pairs = Lists.newLinkedList();
        for (int i = 0, m = 0, n = 0, size = expression.length(); i < size; i++) {
            char c = chars.push(expression.charAt(i));
            if (c == LEFT_BRACKET && m++ == 0) {
                // 解析"("左边逻辑表达式
                chars.pop();
                popping(analyser, chars, pairs);
            } else if (c == RIGHT_BRACKET && ++n == m) {
                // 解析"()"中逻辑表达式
                StringBuilder buffer = new StringBuilder();
                while (!chars.isEmpty()) {
                    buffer.append(chars.pop());
                }
                String string = StringUtils.trim(buffer.deleteCharAt(0).reverse());
                if (StringUtils.notEmpty(string)) {
                    pairs.add(combine(analyser, string));
                }

                // 重置优先级括号数量
                m = n = 0;
            }
        }

        // 解析未入栈逻辑表达式
        if (!chars.isEmpty()) {
            popping(analyser, chars, pairs);
        }

        // 合并逻辑
        return pairs.isEmpty() ? null : merge(pairs);
    }

    /**
     * 解析逻辑表达式
     *
     * @param analyser   判断逻辑解析器
     * @param expression 逻辑表达式
     * @param consumer   解析回调函数
     */
    private static void analyse(JudgeAnalyser analyser, String expression, BiConsumer<String, Judge> consumer) {
        int mark = 0;
        Matcher matcher = LOGIC_SYMBOL_PATTERN.matcher(expression);
        while (matcher.find()) {
            String symbol = matcher.group().trim();
            Judge judge = analyser.analyse(expression.substring(mark, matcher.start()));
            consumer.accept(symbol, judge);
            mark = matcher.end();
        }
        if (expression.length() > mark) {
            Judge judge = analyser.analyse(expression.substring(mark));
            consumer.accept(null, judge);
        }
    }

    /**
     * 从字符栈中弹出并解析逻辑表达式
     *
     * @param analyser 判断逻辑解析器
     * @param chars    字符栈
     * @param pairs    逻辑运算列表
     */
    private static void popping(JudgeAnalyser analyser, Stack<Character> chars,
                                LinkedList<Pair<String, Expression>> pairs) {
        // 从字符栈中弹出逻辑表达式
        StringBuilder buffer = new StringBuilder();
        while (!chars.isEmpty()) {
            buffer.append(chars.pop());
        }
        if (buffer.length() == 0) {
            return;
        }

        // 解析逻辑表达式
        String string = StringUtils.ltrim(buffer.reverse());
        if (StringUtils.hasRealPrefix(string, "&& ", "|| ")) {
            String symbol = string.substring(0, 3).trim();
            if (ObjectUtils.notEmpty(pairs)) {
                // 更新最近逻辑运算符
                Pair<String, Expression> last = pairs.removeLast();
                pairs.addLast(Pair.of(symbol, last.getValue()));
            }
            string = string.substring(3);
        }
        if (StringUtils.notEmpty(string)) {
            analyse(analyser, string, (symbol, compare) -> pairs.add(Pair.of(symbol, compare)));
        }
    }

    /**
     * 逻辑表达式对象转换
     *
     * @param expression 表达式
     * @return 逻辑对象实例
     */
    public static Expression parse(String expression) {
        return parse(DefaultJudgeAnalyser.getInstance(), expression);
    }

    /**
     * 逻辑表达式对象转换
     *
     * @param analyser   判断逻辑解析器
     * @param expression 表达式
     * @return 逻辑对象实例
     */
    public static Expression parse(@NonNull JudgeAnalyser analyser, String expression) {
        if (StringUtils.isEmpty(expression) || (expression = expression.trim()).isEmpty()) {
            return null;
        }

        Pair<String, Expression> pair = combine(analyser, expression);
        return ObjectUtils.ifNull(pair, Pair::getValue);
    }
}
