package com.chelvc.framework.base.logic;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.chelvc.framework.common.model.Ip;
import com.chelvc.framework.common.model.Pair;
import com.chelvc.framework.common.model.Version;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.Lists;
import lombok.Getter;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 逻辑参数变量对象
 *
 * @author Woody
 * @date 2025/2/25
 */
@Slf4j
public class Variable implements Serializable {
    /**
     * 参数变量正则匹配模式
     */
    private static final Pattern VARIABLE_PATTERN = Pattern.compile("\\{\\{[ \\.\\_\\-0-9a-zA-Z]+\\}\\}");

    @Getter
    private final String expression;
    private final List<Pair<Boolean, CharSequence>> values;

    public Variable(@NonNull String expression) {
        this.expression = expression;

        // 初始化参数列表
        int offset = 0;
        List<Pair<Boolean, CharSequence>> values = Lists.newLinkedList();
        Matcher matcher = VARIABLE_PATTERN.matcher(expression);
        while (matcher.find()) {
            if (matcher.start() > offset) {
                String define = expression.substring(offset, matcher.start());
                values.add(Pair.of(false, define));
            }
            String define = matcher.group().toLowerCase();
            define = define.substring(2, define.length() - 2).trim();
            if (StringUtils.notEmpty(define)) {
                values.add(Pair.of(true, define));
            }
            offset = matcher.end();
        }
        if (expression.length() > offset) {
            values.add(Pair.of(false, expression.substring(offset)));
        }
        this.values = ObjectUtils.isEmpty(values) ? Collections.emptyList() : Lists.newArrayList(values);

        // 重置定制参数
        Pair<Boolean, CharSequence> pair;
        if (this.values.size() == 1 && !Boolean.TRUE.equals((pair = this.values.get(0)).getKey())) {
            if (StringUtils.isIp(pair.getValue())) {
                // IP类型参数
                pair.setValue(new Ip(pair.getValue().toString()));
            } else if (StringUtils.isVersion(pair.getValue())) {
                // 版本号类型参数
                pair.setValue(new Version(pair.getValue().toString()));
            }
        }
    }

    /**
     * 参数解析
     *
     * @param parser 参数变量解析函数
     * @param pair   参数变量关联关系
     * @return 参数值
     */
    private CharSequence parse(VariableParser parser, Pair<Boolean, CharSequence> pair) {
        if (!Boolean.TRUE.equals(pair.getKey())) {
            return pair.getValue();
        }

        String value = null;
        try {
            value = parser.parse(pair.getValue().toString());
        } catch (Exception e) {
            log.error("Variable parse failed: {}", pair.getValue(), e);
        }
        return ObjectUtils.ifNull(StringUtils.trim(value), StringUtils.EMPTY);
    }

    /**
     * 参数解析
     *
     * @return 参数值
     */
    public CharSequence parse() {
        return this.parse(variable -> variable);
    }

    /**
     * 参数解析
     *
     * @param parser 参数变量解析函数
     * @return 参数值
     */
    public CharSequence parse(@NonNull VariableParser parser) {
        if (ObjectUtils.isEmpty(this.values)) {
            return StringUtils.EMPTY;
        } else if (this.values.size() == 1) {
            return this.parse(parser, this.values.get(0));
        }

        StringBuilder buffer = new StringBuilder();
        this.values.forEach(pair -> buffer.append(this.parse(parser, pair)));
        return buffer.length() == 0 ? StringUtils.EMPTY : buffer.toString();
    }

    @Override
    public int hashCode() {
        return this.expression.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        } else if (!(obj instanceof Variable)) {
            return false;
        }
        return Objects.equals(this.expression, ((Variable) obj).expression);
    }

    @Override
    public String toString() {
        return this.expression;
    }
}
