package com.chelvc.framework.base.util;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Properties;
import java.util.Set;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.config.YamlPropertiesFactoryBean;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.core.env.Environment;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.core.io.support.ResourcePatternResolver;
import org.springframework.core.type.classreading.CachingMetadataReaderFactory;
import org.springframework.core.type.classreading.MetadataReader;
import org.springframework.core.type.classreading.MetadataReaderFactory;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.StringHttpMessageConverter;
import org.springframework.http.converter.cbor.MappingJackson2CborHttpMessageConverter;
import org.springframework.http.converter.json.AbstractJackson2HttpMessageConverter;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.http.converter.smile.MappingJackson2SmileHttpMessageConverter;
import org.springframework.http.converter.xml.MappingJackson2XmlHttpMessageConverter;
import org.springframework.stereotype.Controller;
import org.springframework.util.AntPathMatcher;
import org.springframework.util.ClassUtils;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * Spring工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public final class SpringUtils {
    /**
     * Ant风格路径标记
     */
    public static final String ANT_PATH_MARK = "/*";

    /**
     * 是否是多服务部署
     */
    private static Boolean MULTIPLE_SERVER;

    /**
     * 资源路径匹配器
     */
    private static final AntPathMatcher PATH_MATCHER = new AntPathMatcher(File.separator);

    private SpringUtils() {
    }

    /**
     * 判断是否是多服务部署
     *
     * @return true/false
     */
    public static boolean isMultipleServer() {
        return Boolean.TRUE.equals(MULTIPLE_SERVER);
    }

    /**
     * 设置是否是多服务部署
     *
     * @param multiple 是否多服务部署
     */
    public static void setMultipleServer(boolean multiple) {
        if (MULTIPLE_SERVER != null) {
            throw new IllegalStateException("'MULTIPLE_SERVER' has been initialized");
        }
        MULTIPLE_SERVER = multiple;
    }

    /**
     * 判断资源路径是否符合指定匹配模式
     *
     * @param path     资源路径
     * @param patterns 匹配模式数组
     * @return true/false
     */
    public static boolean isPath(String path, @NonNull String... patterns) {
        if (StringUtils.notEmpty(path) && ObjectUtils.notEmpty(patterns)) {
            for (String pattern : patterns) {
                if (StringUtils.notEmpty(pattern) && PATH_MATCHER.match(pattern, path)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 判断资源路径是否符合指定匹配模式
     *
     * @param path     资源路径
     * @param patterns 匹配模式集合
     * @return true/false
     */
    public static boolean isPath(String path, @NonNull Collection<String> patterns) {
        if (StringUtils.notEmpty(path) && ObjectUtils.notEmpty(patterns)) {
            for (String pattern : patterns) {
                if (StringUtils.notEmpty(pattern) && PATH_MATCHER.match(pattern, path)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 获取接口地址
     *
     * @param mapping 请求映射注解实例
     * @return 接口地址列表
     */
    public static List<String> getApis(@NonNull RequestMapping mapping) {
        String api;
        if (!(api = mapping.name().trim()).isEmpty()) {
            return Collections.singletonList(api);
        } else if (mapping.value().length > 0) {
            return Stream.of(mapping.value()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        } else if (mapping.path().length > 0) {
            return Stream.of(mapping.path()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    /**
     * 获取POST接口地址
     *
     * @param mapping POST方法映射注解实例
     * @return 接口地址列表
     */
    public static List<String> getApis(@NonNull PostMapping mapping) {
        String api;
        if (!(api = mapping.name().trim()).isEmpty()) {
            return Collections.singletonList(api);
        } else if (mapping.value().length > 0) {
            return Stream.of(mapping.value()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        } else if (mapping.path().length > 0) {
            return Stream.of(mapping.path()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    /**
     * 获取GET接口地址
     *
     * @param mapping GET方法映射注解实例
     * @return 接口地址列表
     */
    public static List<String> getApis(@NonNull GetMapping mapping) {
        String api;
        if (!(api = mapping.name().trim()).isEmpty()) {
            return Collections.singletonList(api);
        } else if (mapping.value().length > 0) {
            return Stream.of(mapping.value()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        } else if (mapping.path().length > 0) {
            return Stream.of(mapping.path()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    /**
     * 获取PUT接口地址
     *
     * @param mapping PUT方法映射注解实例
     * @return 接口地址列表
     */
    public static List<String> getApis(@NonNull PutMapping mapping) {
        String api;
        if (!(api = mapping.name().trim()).isEmpty()) {
            return Collections.singletonList(api);
        } else if (mapping.value().length > 0) {
            return Stream.of(mapping.value()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        } else if (mapping.path().length > 0) {
            return Stream.of(mapping.path()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    /**
     * 获取DELETE接口地址
     *
     * @param mapping DELETE方法映射注解实例
     * @return 接口地址列表
     */
    public static List<String> getApis(@NonNull DeleteMapping mapping) {
        String api;
        if (!(api = mapping.name().trim()).isEmpty()) {
            return Collections.singletonList(api);
        } else if (mapping.value().length > 0) {
            return Stream.of(mapping.value()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        } else if (mapping.path().length > 0) {
            return Stream.of(mapping.path()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    /**
     * 获取PATCH接口地址
     *
     * @param mapping PATCH方法映射注解实例
     * @return 接口地址列表
     */
    public static List<String> getApis(@NonNull PatchMapping mapping) {
        String api;
        if (!(api = mapping.name().trim()).isEmpty()) {
            return Collections.singletonList(api);
        } else if (mapping.value().length > 0) {
            return Stream.of(mapping.value()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        } else if (mapping.path().length > 0) {
            return Stream.of(mapping.path()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    /**
     * 获取接口地址
     *
     * @param clazz 接口对象
     * @return 接口地址列表
     */
    public static List<String> getApis(@NonNull Class<?> clazz) {
        String mapping;
        Annotation annotation;
        if ((annotation = clazz.getAnnotation(Controller.class)) != null
                && !(mapping = ((Controller) annotation).value().trim()).isEmpty()) {
            return Collections.singletonList(mapping);
        } else if ((annotation = clazz.getAnnotation(RestController.class)) != null
                && !(mapping = ((RestController) annotation).value().trim()).isEmpty()) {
            return Collections.singletonList(mapping);
        } else if ((annotation = clazz.getAnnotation(RequestMapping.class)) != null) {
            return getApis((RequestMapping) annotation);
        }
        return Collections.emptyList();
    }

    /**
     * 获取接口地址
     *
     * @param method 接口方法对象实例
     * @return 接口地址列表
     */
    public static List<String> getApis(@NonNull Method method) {
        Annotation annotation;
        List<String> prefixes = getApis(method.getDeclaringClass());
        if ((annotation = method.getAnnotation(RequestMapping.class)) != null) {
            return HttpUtils.uris(prefixes, getApis((RequestMapping) annotation));
        } else if ((annotation = method.getAnnotation(PostMapping.class)) != null) {
            return HttpUtils.uris(prefixes, getApis((PostMapping) annotation));
        } else if ((annotation = method.getAnnotation(GetMapping.class)) != null) {
            return HttpUtils.uris(prefixes, getApis((GetMapping) annotation));
        } else if ((annotation = method.getAnnotation(PutMapping.class)) != null) {
            return HttpUtils.uris(prefixes, getApis((PutMapping) annotation));
        } else if ((annotation = method.getAnnotation(DeleteMapping.class)) != null) {
            return HttpUtils.uris(prefixes, getApis((DeleteMapping) annotation));
        } else if ((annotation = method.getAnnotation(PatchMapping.class)) != null) {
            return HttpUtils.uris(prefixes, getApis((PatchMapping) annotation));
        }
        return Collections.emptyList();
    }

    /**
     * 获取系统资源
     *
     * @param locations 资源位置数组
     * @return 资源对象列表
     */
    public static List<Resource> getResources(@NonNull String... locations) {
        List<Resource> resources = Lists.newLinkedList();
        ResourcePatternResolver resourcePatternResolver = new PathMatchingResourcePatternResolver();
        for (String location : locations) {
            try {
                resources.addAll(Arrays.asList(resourcePatternResolver.getResources(location)));
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
        return resources.isEmpty() ? Collections.emptyList() : resources;
    }

    /**
     * 判断类对象是否属于指定资源
     *
     * @param resource 资源对象
     * @param clazz    类对象
     * @return true/false
     */
    public static boolean isResourceClass(@NonNull Resource resource, @NonNull Class<?> clazz) {
        String file = clazz.getProtectionDomain().getCodeSource().getLocation().getPath();
        try {
            return resource.getURL().getPath().contains(file);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取类对象所属资源
     *
     * @param clazz     类对象
     * @param resources 资源对象集合
     * @return 资源对象
     */
    public static Resource getClassResource(@NonNull Class<?> clazz, @NonNull Collection<Resource> resources) {
        if (ObjectUtils.isEmpty(resources)) {
            return null;
        }
        return resources.stream().filter(resource -> isResourceClass(resource, clazz)).findAny().orElse(null);
    }

    /**
     * 获取资源配置属性
     *
     * @param resource *.yml、*.properties文件资源对象实例
     * @return 属性配置对象
     */
    public static Properties getResourceProperties(@NonNull Resource resource) {
        String filename = resource.getFilename();
        if (StringUtils.isEmpty(filename) || filename.endsWith(".yml")) {
            // yml
            YamlPropertiesFactoryBean yaml = new YamlPropertiesFactoryBean();
            yaml.setResources(resource);
            return yaml.getObject();
        }
        // properties
        Properties properties = new Properties();
        try (InputStream input = resource.getInputStream()) {
            properties.load(input);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return properties;
    }

    /**
     * 获取配置资源
     *
     * @return 资源对象列表
     */
    public static List<Resource> getPropertyResources() {
        return getResources(
                "classpath*:/bootstrap.yml", "classpath*:/config/bootstrap.yml",
                "classpath*:/bootstrap.properties", "classpath*:/config/bootstrap.properties",
                "classpath*:/application.yml", "classpath*:/config/application.yml",
                "classpath*:/application.properties", "classpath*:/config/application.properties"
        );
    }

    /**
     * 获取配置资源
     *
     * @param environment 应用环境
     * @return 资源对象列表
     */
    public static List<Resource> getPropertyResources(@NonNull Environment environment) {
        String[] profiles = environment.getActiveProfiles();
        if (ObjectUtils.isEmpty(profiles)) {
            return getResources(
                    "classpath*:/bootstrap.yml", "classpath*:/config/bootstrap.yml",
                    "classpath*:/bootstrap.properties", "classpath*:/config/bootstrap.properties"
            );
        }

        int n = 0;
        String[] locations = new String[profiles.length * 4 + 4];
        locations[n++] = "classpath*:/application.yml";
        locations[n++] = "classpath*:/config/application.yml";
        locations[n++] = "classpath*:/application.properties";
        locations[n++] = "classpath*:/config/application.properties";
        for (String profile : profiles) {
            locations[n++] = "classpath*:/application-" + profile + ".yml";
            locations[n++] = "classpath*:/config/application-" + profile + ".yml";
            locations[n++] = "classpath*:/application-" + profile + ".properties";
            locations[n++] = "classpath*:/config/application-" + profile + ".properties";
        }
        return getResources(locations);
    }

    /**
     * 获取类对象
     *
     * @param pack 包路径
     * @return 类对象列表
     */
    public static List<Class<?>> getClasses(String pack) {
        return getClasses(pack, clazz -> true);
    }

    /**
     * 获取类对象
     *
     * @param pack      包路径
     * @param condition 类对象条件判断
     * @return 类对象列表
     */
    public static List<Class<?>> getClasses(String pack, @NonNull Predicate<Class<?>> condition) {
        // 获取class资源文件
        String pattern = ResourcePatternResolver.CLASSPATH_ALL_URL_PREFIX +
                (pack == null || (pack = pack.trim()).isEmpty() ? "**/*.class" :
                        (ClassUtils.convertClassNameToResourcePath(pack) + "/**/*.class"));
        List<Resource> resources = getResources(pattern);

        // 将资源文件转换为类对象
        Stream.Builder<Class<?>> classes = Stream.builder();
        MetadataReaderFactory readerFactory =
                new CachingMetadataReaderFactory(new PathMatchingResourcePatternResolver());
        ClassLoader classLoader = Objects.requireNonNull(ClassUtils.getDefaultClassLoader());
        for (Resource resource : resources) {
            // 加载类对象
            Class<?> clazz = null;
            try {
                MetadataReader reader = readerFactory.getMetadataReader(resource);
                clazz = classLoader.loadClass(reader.getClassMetadata().getClassName());
            } catch (Throwable t) {
                log.warn("Class load failed: {}", t.getMessage());
            }
            if (clazz != null && condition.test(clazz)) {
                classes.add(clazz);
            }
        }
        return classes.build().distinct().collect(Collectors.toList());
    }

    /**
     * 获取类对象
     *
     * @param applicationContext 应用上下文对象
     * @param condition          类对象条件判断
     * @return 类对象列表
     */
    public static List<Class<?>> getClasses(@NonNull ApplicationContext applicationContext,
                                            @NonNull Predicate<Class<?>> condition) {
        Set<String> packages = Sets.newHashSet();
        applicationContext.getBeansWithAnnotation(SpringBootApplication.class).forEach((key, value) -> {
            Class<?> clazz = value.getClass();

            // 加载@ComponentScan包扫描路径配置
            ComponentScan scan = AnnotationUtils.findAnnotation(clazz, ComponentScan.class);
            if (scan != null) {
                packages.addAll(Arrays.asList(scan.basePackages()));
            }

            // 加载@SpringBootApplication包扫描路径配置
            SpringBootApplication application = AnnotationUtils.findAnnotation(clazz, SpringBootApplication.class);
            if (application != null) {
                packages.addAll(Arrays.asList(application.scanBasePackages()));
            }
        });

        // 根据包扫描路径查找类对象
        return packages.stream().flatMap(pack -> getClasses(pack, condition).stream())
                .distinct().collect(Collectors.toList());
    }

    /**
     * 获取接口对象实例
     *
     * @param applicationContext 应用上下文
     * @return 接口对象实例列表
     */
    public static List<Object> getControllers(@NonNull ApplicationContext applicationContext) {
        return Stream.of(applicationContext.getBeansWithAnnotation(Controller.class).values(),
                applicationContext.getBeansWithAnnotation(RestController.class).values())
                .flatMap(Collection::stream).distinct().collect(Collectors.toList());
    }

    /**
     * 自定义消息转换处理器
     *
     * @param converters 消息转换处理器列表
     * @return 消息转换处理器列表
     */
    public static List<HttpMessageConverter<?>> customizeConverters(List<HttpMessageConverter<?>> converters) {
        if (ObjectUtils.isEmpty(converters)) {
            return Collections.emptyList();
        }

        return converters.stream().map(converter -> {
            Class<?> clazz = converter.getClass();
            if (clazz == StringHttpMessageConverter.class) {
                converter = new StringHttpMessageConverter(StandardCharsets.UTF_8);
            } else if (converter instanceof AbstractJackson2HttpMessageConverter) {
                ObjectMapper mapper = JacksonUtils.getDefaultSerializer();
                List<MediaType> types = Lists.newArrayList(converter.getSupportedMediaTypes());
                if (!types.contains(MediaType.TEXT_PLAIN)) {
                    types.add(MediaType.TEXT_PLAIN);
                }
                if (clazz == MappingJackson2HttpMessageConverter.class) {
                    converter = new MappingJackson2HttpMessageConverter(mapper);
                } else if (clazz == MappingJackson2XmlHttpMessageConverter.class) {
                    converter = new MappingJackson2XmlHttpMessageConverter(mapper);
                } else if (clazz == MappingJackson2CborHttpMessageConverter.class) {
                    converter = new MappingJackson2CborHttpMessageConverter(mapper);
                } else if (clazz == MappingJackson2SmileHttpMessageConverter.class) {
                    converter = new MappingJackson2SmileHttpMessageConverter(mapper);
                }
                ((AbstractJackson2HttpMessageConverter) converter).setSupportedMediaTypes(types);
            }
            return converter;
        }).collect(Collectors.toList());
    }
}
