package com.chelvc.framework.common.function;

import java.util.Objects;
import java.util.function.BiPredicate;

import com.chelvc.framework.common.util.ThreadUtils;
import lombok.NonNull;

/**
 * 对象提供方法接口
 *
 * @param <T> 目标对象类型
 * @author Woody
 * @date 2024/1/30
 */
@FunctionalInterface
public interface Provider<T> {
    /**
     * 获取对象
     *
     * @return 对象实例
     * @throws Exception 处理异常
     */
    T get() throws Exception;

    /**
     * 方法处理重试
     *
     * @param provider 方法处理回调函数
     * @param count    重试次数
     * @param interval 时间间隔（毫秒）
     * @param <T>      目标对象类型
     */
    static <T> T retry(@NonNull Provider<T> provider, int count, long interval) {
        return retry(provider, count, interval, (i, e) -> Objects.nonNull(e));
    }

    /**
     * 方法处理重试
     *
     * @param provider    方法处理回调函数
     * @param count       重试次数
     * @param interval    时间间隔（毫秒）
     * @param continuable 判断异常是否可继续重试
     * @param <T>         目标对象类型
     */
    static <T> T retry(@NonNull Provider<T> provider, int count, long interval,
                       @NonNull BiPredicate<Integer, Exception> continuable) {
        if (count < 1) {
            return null;
        }

        // 睡眠指定时间间隔
        if (interval > 0) {
            ThreadUtils.sleep(interval);
        }

        // 重试处理
        Exception exception = null;
        for (int i = 0; i < count; i++) {
            try {
                return provider.get();
            } catch (Exception e) {
                exception = e;

                // 判断是否可继续重试
                if (!continuable.test(i, e)) {
                    break;
                }
            }
        }
        throw new RuntimeException(exception);
    }
}
