package com.chelvc.framework.common.model;

import java.io.Serializable;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import lombok.Getter;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * IP地址对象
 *
 * @author Woody
 * @date 2025/2/24
 */
@Slf4j
@Getter
public class Ip implements CharSequence, Comparable<Ip>, Serializable {
    private final long mask;
    private final long number;
    private final String value;

    public Ip(@NonNull String value) {
        this(value, value.indexOf('/'));
    }

    private Ip(@NonNull String value, int delimiter) {
        this.value = value;
        this.mask = delimiter < 0 ? 0 : getMaskNumber(Integer.parseInt(value.substring(delimiter + 1)));
        this.number = delimiter < 0 ? getAddressNumber(value) :
                (getAddressNumber(value.substring(0, delimiter)) & this.mask);
    }

    /**
     * 将字符串转换成IP对象
     *
     * @param hosts ip字符串
     * @return IP对象列表
     */
    public static List<Ip> host2ips(String hosts) {
        if (StringUtils.isEmpty(hosts)) {
            return Collections.emptyList();
        }
        List<Ip> ips = Stream.of(hosts.split(",")).map(String::trim).filter(StringUtils::notEmpty)
                .map(Ip::new).collect(Collectors.toList());
        return ObjectUtils.isEmpty(ips) ? Collections.emptyList() : ips;
    }

    /**
     * 获取掩码数字
     *
     * @param length 掩码长度
     * @return 掩码数字
     */
    public static long getMaskNumber(int length) {
        return length > 0 ? (-1L << (32 - length)) : 0;
    }

    /**
     * 获取IP地址数字
     *
     * @param address IP地址
     * @return 地址数字
     */
    public static long getAddressNumber(String address) {
        if (StringUtils.isEmpty(address)) {
            return 0;
        }

        byte[] bytes;
        try {
            bytes = InetAddress.getByName(address).getAddress();
        } catch (UnknownHostException e) {
            throw new RuntimeException(e);
        }

        long value = 0;
        for (int i = 0; i < 4; i++) {
            value <<= 8;
            value |= bytes[i] & 0xff;
        }
        return value;
    }

    /**
     * 判断地址是否匹配
     *
     * @param number IP数字
     * @return true/false
     */
    public boolean matches(long number) {
        return number > 0 && ((this.mask == 0 && this.number == number)
                || (this.mask < 0 && this.number == (number & this.mask)));
    }

    /**
     * 判断地址是否匹配
     *
     * @param address IP地址
     * @return true/false
     */
    public boolean matches(CharSequence address) {
        if (StringUtils.isEmpty(address)) {
            return false;
        } else if (this.mask == 0) {
            if (address instanceof Ip) {
                return ((Ip) address).matches(this.number);
            }

            // 普通ip比较
            String value = address.toString();
            int delimiter = value.indexOf('/');
            if (delimiter < 0) {
                return Objects.equals(this.value, value);
            }

            // 网段ip比较
            try {
                return new Ip(value, delimiter).matches(this.number);
            } catch (Exception e) {
                log.error("Address number convert failed", e);
            }
            return false;
        } else if (address instanceof Ip) {
            // 分段ip比较
            Ip ip = (Ip) address;
            if (ip.getMask() == 0) {
                return this.number == (ip.getNumber() & this.mask);
            }
            return this.number == ip.getNumber() && this.mask == ip.getMask();
        }

        // 字符串ip比较
        String value = address.toString();
        int delimiter = value.indexOf('/');
        if (delimiter < 0) {
            try {
                return this.number == (getAddressNumber(value) & this.mask);
            } catch (Exception e) {
                log.error("Address number convert failed", e);
            }
            return false;
        }
        return Objects.equals(this.value, value);
    }

    @Override
    public int length() {
        return this.value.length();
    }

    @Override
    public char charAt(int index) {
        return this.value.charAt(index);
    }

    @Override
    public CharSequence subSequence(int start, int end) {
        return this.value.substring(start, end);
    }

    @Override
    public int hashCode() {
        return this.value.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        } else if (!(obj instanceof Ip)) {
            return false;
        }
        return Objects.equals(this.value, ((Ip) obj).getValue());
    }

    @Override
    public int compareTo(Ip ip) {
        return ip == null ? 1 : this.value.compareTo(ip.getValue());
    }

    @Override
    public String toString() {
        return this.value;
    }
}
