package com.chelvc.framework.common.model;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;
import java.util.function.Function;

import com.chelvc.framework.common.util.ObjectUtils;
import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.NonNull;
import lombok.ToString;
import lombok.experimental.SuperBuilder;

/**
 * 分页信息
 *
 * @author Woody
 * @date 2024/1/30
 */
@Getter
@ToString
@SuperBuilder
@EqualsAndHashCode
@NoArgsConstructor
@AllArgsConstructor
public class Pagination<T> implements Serializable {
    /**
     * 空分页信息
     */
    private static final Pagination<?> EMPTY = Pagination.builder().build();

    /**
     * 总数量
     */
    @Builder.Default
    private final Long total = 0L;

    /**
     * 总页数
     */
    @Builder.Default
    private final Long pages = 0L;

    /**
     * 当页数据列表
     */
    @Builder.Default
    private final List<T> records = Collections.emptyList();

    /**
     * 获取空分页信息
     *
     * @param <T> 分页对象类型
     * @return 分页信息
     */
    @SuppressWarnings("unchecked")
    public static <T> Pagination<T> empty() {
        return (Pagination<T>) EMPTY;
    }

    /**
     * 判断分页记录是否为空
     *
     * @return true/false
     */
    @JsonIgnore
    public boolean isEmpty() {
        return ObjectUtils.isEmpty(this.records);
    }

    /**
     * 数据转换
     *
     * @param type 目标对象类型
     * @param <M>  目标数据类型
     * @return 数据列表分页实例
     */
    public <M> Pagination<M> convert(@NonNull Class<M> type) {
        return this.isEmpty() ? empty() : Pagination.<M>builder().total(this.total).pages(this.pages)
                .records(ObjectUtils.copying(this.records, type)).build();
    }

    /**
     * 数据转换
     *
     * @param records 目标数据列表
     * @param <M>     目标数据类型
     * @return 数据列表分页实例
     */
    public <M> Pagination<M> convert(List<M> records) {
        return Pagination.<M>builder().total(this.total).pages(this.pages).records(records).build();
    }

    /**
     * 数据转换
     *
     * @param adapter 对象转换适配器
     * @param <M>     目标数据类型
     * @return 数据列表分页实例
     */
    public <M> Pagination<M> convert(@NonNull Function<List<T>, List<M>> adapter) {
        return this.isEmpty() ? empty() : Pagination.<M>builder().total(this.total).pages(this.pages)
                .records(adapter.apply(this.records)).build();
    }
}
