package com.chelvc.framework.common.model;

import java.lang.reflect.Field;
import java.util.Objects;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.function.Consumer;
import java.util.function.Supplier;

import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import lombok.NonNull;
import sun.misc.Unsafe;

/**
 * 对象池
 *
 * @param <T> 对象类型
 * @author Woody
 * @date 2024/6/15
 */
public class Pool<T> {
    /**
     * 对象链表
     *
     * @param <T> 对象类型
     */
    static class Node<T> {
        final T item;
        volatile long mark;
        volatile Node<T> next;

        Node(T item, Node<T> next, long mark) {
            this.item = item;
            this.next = next;
            this.mark = mark;
        }
    }

    /**
     * 对象标记
     */
    private static final long MARK;

    /**
     * Unsafe实例
     */
    private static final Unsafe UNSAFE;

    private final long idle;
    private final Node<T> root;
    private final Supplier<T> supplier;
    private final Lock lock = new ReentrantLock();
    private volatile long cleaning = System.currentTimeMillis();

    static {
        try {
            Field field = sun.misc.Unsafe.class.getDeclaredField("theUnsafe");
            UNSAFE = (Unsafe) ObjectUtils.getObjectValue(sun.misc.Unsafe.class, field);
            MARK = UNSAFE.objectFieldOffset(Node.class.getDeclaredField("mark"));
        } catch (Exception e) {
            throw new Error(e);
        }
    }

    public Pool(@NonNull Supplier<T> supplier) {
        this(60000, supplier);
    }

    public Pool(long idle, @NonNull Supplier<T> supplier) {
        AssertUtils.arg(idle > 0, () -> "Pool idle must be greater than 0");
        this.idle = idle;
        this.root = new Node<>((this.supplier = supplier).get(), null, -1);
    }

    /**
     * 获取对象
     *
     * @return 对象节点
     */
    private Node<T> get() {
        // 从链表中获取可用对象
        for (Node<T> node = this.root; node != null; node = node.next) {
            long mark = node.mark;
            if (mark < 0 && UNSAFE.compareAndSwapLong(node, MARK, mark, 1)) {
                return node;
            }
        }

        // 新建对象并将对象加入链表
        T item = this.supplier.get();
        Node<T> root = Objects.requireNonNull(this.root);
        synchronized (this.root) {
            return root.next = new Node<>(item, root.next, 1);
        }
    }

    /**
     * 对象使用
     *
     * @param consumer 对象回调函数
     */
    public void apply(@NonNull Consumer<T> consumer) {
        // 对象使用
        long timestamp;
        Node<T> node = this.get();
        try {
            consumer.accept(node.item);
        } finally {
            node.mark = -(timestamp = System.currentTimeMillis());
        }

        // 清理过期对象
        if (timestamp - this.cleaning > this.idle && this.lock.tryLock()) {
            try {
                for (Node<T> n = this.root.next, p = n; n != null; p = n, n = n.next) {
                    long m = n.mark;
                    if (m < 0 && timestamp + m > this.idle && UNSAFE.compareAndSwapLong(n, MARK, m, 1)) {
                        p.next = n.next;
                    }
                }
                this.cleaning = timestamp;
            } finally {
                this.lock.unlock();
            }
        }
    }
}
