package com.chelvc.framework.common.model;

import java.io.Serializable;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Consumer;
import java.util.stream.Collectors;

import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.common.util.TreeUtils;
import com.google.common.collect.Maps;
import lombok.NonNull;

/**
 * 版本对象
 *
 * @author Woody
 * @date 2024/1/30
 */
public class Version extends Tree.Simple<String, Version> implements CharSequence, Comparable<Version>, Serializable {
    public Version(@NonNull String id) {
        this(id, null, (String) null);
    }

    public Version(@NonNull String id, String name) {
        this(id, name, (String) null);
    }

    public Version(@NonNull String id, Version parent) {
        this(id, null, parent);
    }

    public Version(@NonNull String id, String name, Version parent) {
        this(id, name, ObjectUtils.ifNull(parent, Version::getId));
    }

    public Version(@NonNull String id, String name, String parentId) {
        this.setId(id);
        this.setName(name);
        this.setParentId(parentId);
    }

    /**
     * 版本号比较
     *
     * @param v1 版本号
     * @param v2 版本号
     * @return 比较数字
     */
    public static int compare(String v1, String v2) {
        int s1 = StringUtils.isEmpty(v1) ? 0 : v1.length();
        int s2 = StringUtils.isEmpty(v2) ? 0 : v2.length();
        if (s1 == 0 && s2 == 0) {
            return 0;
        } else if (s1 == 0 || s2 == 0) {
            return s1 - s2;
        }

        int m = 0, n = 0;
        for (int i, k; (i = v1.indexOf('.', m)) > 0 && (k = v2.indexOf('.', n)) > 0; m = i + 1, n = k + 1) {
            if (i != k) {
                return i - k;
            }
            for (int j = m; j < i; j++) {
                int c = v1.charAt(j) - v2.charAt(j);
                if (c != 0) {
                    return c;
                }
            }
        }

        if (s1 == s2) {
            for (; m < s1; m++) {
                int c = v1.charAt(m) - v2.charAt(m);
                if (c != 0) {
                    return c;
                }
            }
        }
        return s1 - s2;
    }

    /**
     * 判断版本号是否在指定版本号之前
     *
     * @param v1 版本号
     * @param v2 版本号
     * @return true/false
     */
    public static boolean isBefore(String v1, String v2) {
        return isBefore(v1, v2, false);
    }

    /**
     * 判断版本号是否在指定版本号之前
     *
     * @param v1       版本号
     * @param v2       版本号
     * @param contains 是否包含指定版本号
     * @return true/false
     */
    public static boolean isBefore(String v1, String v2, boolean contains) {
        int compared = compare(v1, v2);
        return contains ? compared <= 0 : compared < 0;
    }

    /**
     * 判断版本号是否在指定版本号之后
     *
     * @param v1 版本号
     * @param v2 版本号
     * @return true/false
     */
    public static boolean isAfter(String v1, String v2) {
        return isAfter(v1, v2, false);
    }

    /**
     * 判断版本号是否在指定版本号之后
     *
     * @param v1       版本号
     * @param v2       版本号
     * @param contains 是否包含指定版本号
     * @return true/false
     */
    public static boolean isAfter(String v1, String v2, boolean contains) {
        int compared = compare(v1, v2);
        return contains ? compared >= 0 : compared > 0;
    }

    /**
     * 将版本号转换成树列表
     *
     * @param versions 版本号集合
     * @return 版本树列表
     */
    public static List<Version> trees(Collection<String> versions) {
        return trees(versions, null);
    }

    /**
     * 将版本号转换成树列表
     *
     * @param versions 版本号集合
     * @param consumer 版本回调方法
     * @return 版本树列表
     */
    public static List<Version> trees(Collection<String> versions, Consumer<Version> consumer) {
        if (ObjectUtils.isEmpty(versions)) {
            return Collections.emptyList();
        }
        Map<String, Version> caches = Maps.newHashMap();
        versions.forEach(code -> {
            int i = 0;
            Version parent = null;
            do {
                i = code.indexOf('.', i + 1);
                if (i > 0) {
                    Version p = parent;
                    parent = caches.computeIfAbsent(code.substring(0, i), id -> {
                        Version version = new Version(id, id + ".x", p);
                        if (consumer != null) {
                            consumer.accept(version);
                        }
                        return version;
                    });
                } else {
                    Version version = new Version(code, code, parent);
                    if (consumer != null) {
                        consumer.accept(version);
                    }
                    caches.put(code, version);
                }
            } while (i > 0);
        });
        Comparator<Version> comparator = Comparator.nullsFirst(Comparator.comparing(Version::getId, Version::compare));
        Collection<Version> values = caches.values().stream().sorted(comparator).collect(Collectors.toList());
        return TreeUtils.assemble(values);
    }

    @Override
    public int length() {
        return this.getId().length();
    }

    @Override
    public char charAt(int index) {
        return this.getId().charAt(index);
    }

    @Override
    public CharSequence subSequence(int start, int end) {
        return this.getId().substring(start, end);
    }

    @Override
    public int hashCode() {
        return this.getId().hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        } else if (!(obj instanceof Version)) {
            return false;
        }
        return Objects.equals(this.getId(), ((Version) obj).getId());
    }

    @Override
    public int compareTo(Version version) {
        return version == null ? 1 : compare(this.getId(), version.getId());
    }

    @Override
    public String toString() {
        return this.getId();
    }
}
