package com.chelvc.framework.common.util;

import java.nio.charset.StandardCharsets;
import java.security.GeneralSecurityException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.security.Security;
import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

import lombok.NonNull;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

/**
 * AES加解密工具类
 *
 * @author woody
 * @date 2024/1/30
 */
public final class AESUtils {
    /**
     * 密钥长度
     */
    public static final int KEY_SIZE = 128;

    /**
     * AES算法名称
     */
    public static final String ALGORITHM = "AES";

    /**
     * CBC NoPadding模式
     */
    public static final String CBC_NOPADDING = "AES/CBC/NoPadding";

    /**
     * CBC PKCS5Padding模式
     */
    public static final String CBC_PKCS5PADDING = "AES/CBC/PKCS5Padding";

    /**
     * ECB PKCS5Padding模式
     */
    public static final String ECB_PKCS5PADDING = "AES/ECB/PKCS5Padding";

    static {
        Security.addProvider(new BouncyCastleProvider());
        try {
            KeyGenerator.getInstance(ALGORITHM).init(KEY_SIZE);
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        }
    }

    private AESUtils() {
    }

    /**
     * 修正密钥
     *
     * @param key 密钥标识
     * @return 修正后的密钥
     */
    public static String revise(String key) {
        if (key == null) {
            return null;
        }

        // 如果小于16位则左边补0到16位
        if (key.length() < 16) {
            return StringUtils.rjust(key, 16, '0');
        }

        // 如果长度不是8的倍数则左边补0到8的倍数
        int offset = 8 - key.length() % 8;
        return offset == 8 ? key : StringUtils.rjust(key, key.length() + offset, '0');
    }

    /**
     * 获取密码处理器
     *
     * @param name   处理器名称
     * @param mode   加解密模式
     * @param secret 密钥
     * @return 密码处理器实例
     */
    public static Cipher getCipher(@NonNull String name, int mode, @NonNull byte[] secret) {
        try {
            Cipher cipher = Cipher.getInstance(name);
            cipher.init(mode, new SecretKeySpec(secret, ALGORITHM));
            return cipher;
        } catch (GeneralSecurityException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取密码处理器
     *
     * @param name   处理器名称
     * @param mode   加解密模式
     * @param secret 密钥
     * @return 密码处理器实例
     */
    public static Cipher getCipher(@NonNull String name, int mode, @NonNull String secret) {
        return getCipher(name, mode, secret.getBytes(StandardCharsets.UTF_8));
    }

    /**
     * 获取密码处理器
     *
     * @param name   处理器名称
     * @param mode   加解密模式
     * @param secret 密钥
     * @param iv     向量
     * @return 密码处理器实例
     */
    public static Cipher getCipher(@NonNull String name, int mode, @NonNull byte[] secret, @NonNull byte[] iv) {
        try {
            Cipher cipher = Cipher.getInstance(name);
            Key key = new SecretKeySpec(secret, ALGORITHM);
            cipher.init(mode, key, new IvParameterSpec(iv));
            return cipher;
        } catch (GeneralSecurityException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取密码处理器
     *
     * @param name   处理器名称
     * @param mode   加解密模式
     * @param secret 密钥
     * @param iv     向量
     * @return 密码处理器实例
     */
    public static Cipher getCipher(@NonNull String name, int mode, @NonNull String secret, @NonNull String iv) {
        return getCipher(name, mode, secret.getBytes(StandardCharsets.UTF_8), iv.getBytes(StandardCharsets.UTF_8));
    }

    /**
     * 获取加密处理器
     *
     * @param name   处理器名称
     * @param secret 密钥
     * @return 处理器实例
     */
    public static Cipher getEncryptor(@NonNull String name, @NonNull byte[] secret) {
        return getCipher(name, Cipher.ENCRYPT_MODE, secret);
    }

    /**
     * 获取加密处理器
     *
     * @param name   处理器名称
     * @param secret 密钥
     * @return 处理器实例
     */
    public static Cipher getEncryptor(@NonNull String name, @NonNull String secret) {
        return getCipher(name, Cipher.ENCRYPT_MODE, secret);
    }

    /**
     * 获取加密处理器
     *
     * @param name   处理器名称
     * @param secret 密钥
     * @param iv     向量
     * @return 处理器实例
     */
    public static Cipher getEncryptor(@NonNull String name, @NonNull byte[] secret, @NonNull byte[] iv) {
        return getCipher(name, Cipher.ENCRYPT_MODE, secret, iv);
    }

    /**
     * 获取加密处理器
     *
     * @param name   处理器名称
     * @param secret 密钥
     * @param iv     向量
     * @return 处理器实例
     */
    public static Cipher getEncryptor(@NonNull String name, @NonNull String secret, @NonNull String iv) {
        return getCipher(name, Cipher.ENCRYPT_MODE, secret, iv);
    }

    /**
     * 获取解密处理器
     *
     * @param name   处理器名称
     * @param secret 密钥
     * @return 处理器实例
     */
    public static Cipher getDecrypter(@NonNull String name, @NonNull String secret) {
        return getCipher(name, Cipher.DECRYPT_MODE, secret);
    }

    /**
     * 获取解密处理器
     *
     * @param name   处理器名称
     * @param secret 密钥
     * @return 处理器实例
     */
    public static Cipher getDecrypter(@NonNull String name, @NonNull byte[] secret) {
        return getCipher(name, Cipher.DECRYPT_MODE, secret);
    }

    /**
     * 获取解密处理器
     *
     * @param name   处理器名称
     * @param secret 密钥
     * @param iv     向量
     * @return 处理器实例
     */
    public static Cipher getDecrypter(@NonNull String name, @NonNull String secret, @NonNull String iv) {
        return getCipher(name, Cipher.DECRYPT_MODE, secret, iv);
    }

    /**
     * 获取解密处理器
     *
     * @param name   处理器名称
     * @param secret 密钥
     * @param iv     向量
     * @return 处理器实例
     */
    public static Cipher getDecrypter(@NonNull String name, @NonNull byte[] secret, @NonNull byte[] iv) {
        return getCipher(name, Cipher.DECRYPT_MODE, secret, iv);
    }
}
