package com.chelvc.framework.common.util;

import java.math.BigInteger;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Collections;
import java.util.Date;
import java.util.Map;
import java.util.Set;
import java.util.function.Predicate;
import java.util.regex.Matcher;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import lombok.Getter;
import lombok.NonNull;

/**
 * 中文处理工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class ChineseUtils {
    /**
     * 混合数字正则表达式
     */
    public static final String MIXED_NUMBER_REGEX;

    /**
     * 中文日期格式化处理器
     */
    public static final DateTimeFormatter CHINESE_DATE_FORMATTER = DateTimeFormatter.ofPattern("yyyy年M月d日");

    /**
     * 中文日期时间格式化处理器
     */
    public static final DateTimeFormatter CHINESE_DATETIME_FORMATTER =
            DateTimeFormatter.ofPattern("yyyy年M月d日 H时m分s秒");

    /**
     * 中文日期时间格式化处理器（精确到小时）
     */
    public static final DateTimeFormatter CHINESE_DATETIME_HOUR_FORMATTER =
            DateTimeFormatter.ofPattern("yyyy年M月d日 H时");

    /**
     * 中文日期时间格式化处理器（精确到分钟）
     */
    public static final DateTimeFormatter CHINESE_DATETIME_MINUTE_FORMATTER =
            DateTimeFormatter.ofPattern("yyyy年M月d日 H时m分");

    /**
     * 最大长整形大数字对象
     */
    private static final BigInteger MAX_LONG_VALUE = BigInteger.valueOf(Long.MAX_VALUE);

    /**
     * 数字匹配正则表达式
     */
    private static final String NUMBER_REGEX;

    static {
        // 初始化数字匹配模式
        String units = Stream.of(Unit.values()).flatMap(unit -> unit.getAliases().stream())
                .map(Object::toString).collect(Collectors.joining(StringUtils.EMPTY));
        String numbers = Stream.of(Number.values()).flatMap(number -> number.getAliases().stream())
                .map(Object::toString).collect(Collectors.joining(StringUtils.EMPTY));
        NUMBER_REGEX = String.format(
                "((([%s]?[%s][%s]?)+)|([0-9]+[%s]*)|([%s]))", numbers, units, numbers, units, numbers
        );

        // 初始化混合数字正则表达式
        MIXED_NUMBER_REGEX = "[0-9]+[" + units + "]*";
    }

    private ChineseUtils() {
    }

    /**
     * 日期格式化
     *
     * @param date 日期对象实例
     * @return 日期文本
     */
    public static String format(Date date) {
        return date == null ? null : format(DateUtils.convert(date));
    }

    /**
     * 日期格式化
     *
     * @param date     日期对象实例
     * @param temporal 是否格式化时间
     * @return 日期文本
     */
    public static String format(Date date, boolean temporal) {
        return date == null ? null : format(DateUtils.convert(date), temporal);
    }

    /**
     * 日期格式化
     *
     * @param date 日期对象实例
     * @return 日期文本
     */
    public static String format(LocalDate date) {
        return date == null ? null : CHINESE_DATE_FORMATTER.format(date);
    }

    /**
     * 日期时间格式化
     *
     * @param datetime 日期时间对象实例
     * @return 日期文本
     */
    public static String format(LocalDateTime datetime) {
        return datetime == null ? null : format(datetime, DateUtils.isTemporal(datetime));
    }

    /**
     * 日期时间格式化
     *
     * @param datetime 日期时间对象实例
     * @param temporal 是否格式化时间
     * @return 日期文本
     */
    public static String format(LocalDateTime datetime, boolean temporal) {
        if (datetime == null) {
            return null;
        }
        return temporal ? CHINESE_DATETIME_FORMATTER.format(datetime) : CHINESE_DATE_FORMATTER.format(datetime);
    }

    /**
     * 日期格式化（精确到小时）
     *
     * @param date 日期对象
     * @return 日期文本
     */
    public static String hourly(Date date) {
        return date == null ? null : hourly(DateUtils.convert(date));
    }

    /**
     * 日期时间格式化（精确到小时）
     *
     * @param datetime 日期时间对象
     * @return 日期文本
     */
    public static String hourly(LocalDateTime datetime) {
        return datetime == null ? null : CHINESE_DATETIME_HOUR_FORMATTER.format(datetime);
    }

    /**
     * 日期格式化（精确到分钟）
     *
     * @param date 日期对象
     * @return 日期文本
     */
    public static String minutely(Date date) {
        return date == null ? null : minutely(DateUtils.convert(date));
    }

    /**
     * 日期时间格式化（精确到分钟）
     *
     * @param datetime 日期时间对象
     * @return 日期文本
     */
    public static String minutely(LocalDateTime datetime) {
        return datetime == null ? null : CHINESE_DATETIME_MINUTE_FORMATTER.format(datetime);
    }

    /**
     * 判断是否是中文数字
     *
     * @param text 数字文本
     * @return true/false
     */
    public static boolean isChineseNumber(String text) {
        if (StringUtils.notEmpty(text)) {
            for (int i = 0, size = text.length(); i < size; i++) {
                if (text.charAt(i) > 9) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 将中文数字转换成阿拉伯数字
     *
     * @param chinese 中文数字
     * @return 数字
     */
    public static long chinese2number(@NonNull String chinese) {
        long value = 0;
        Unit minimum = null;
        for (int i = chinese.length() - 1; i >= 0; i--) {
            char c = chinese.charAt(i);
            if (Number.valueOf(c) == Number.ZERO
                    || (minimum = ObjectUtils.ifNull(Unit.valueOf(c), Unit::getSmaller)) != null) {
                break;
            }
        }
        minimum = ObjectUtils.ifNull(minimum, Unit.ONE);
        for (int i = chinese.length() - 1; i >= 0; i--) {
            char c = chinese.charAt(i);
            Unit unit = Unit.valueOf(c);
            if (unit == null) {
                Number number = Number.valueOf(c);
                if (number != null && number != Number.ZERO) {
                    value += number.apply(minimum);
                }
            } else if (unit == Unit.TEN_THOUSAND || unit == Unit.BILLION) {
                return value + unit.apply(Math.max(chinese2number(chinese.substring(0, i)), 1));
            } else {
                minimum = unit;
            }
        }
        return value > 0 || minimum == Unit.ONE ? value : minimum.getValue();
    }

    /**
     * 将字符串数字转换成阿拉伯数字
     *
     * @param number 字符串数字
     * @return 阿拉伯数字
     */
    public static Long string2number(@NonNull String number) {
        if (StringUtils.getPattern(MIXED_NUMBER_REGEX).matcher(number).matches()) {
            int watershed = 0;
            for (int i = number.length() - 1; i >= 0; i--) {
                if (number.charAt(i) <= '9') {
                    watershed = i + 1;
                    break;
                }
            }
            BigInteger integer = new BigInteger(watershed > 0 ? number.substring(0, watershed) : number);
            if (integer.compareTo(MAX_LONG_VALUE) > 0) {
                return null;
            }
            long value = integer.longValue();
            if (watershed > 0 && watershed < number.length()) {
                value *= chinese2number(number.substring(watershed));
            }
            return value;
        }
        return chinese2number(number);
    }

    /**
     * 获取文本中的所有数字
     *
     * @param text 文本信息
     * @return 数字集合
     */
    public static Set<Long> numbers(String text) {
        return numbers(text, number -> true);
    }

    /**
     * 获取文本中的所有数字
     *
     * @param text   文本信息
     * @param filter 数字过滤函数
     * @return 数字集合
     */
    public static Set<Long> numbers(String text, @NonNull Predicate<Long> filter) {
        if (StringUtils.isEmpty(text)) {
            return Collections.emptySet();
        }
        Set<Long> numbers = Sets.newHashSet();
        Matcher matcher = StringUtils.getPattern(NUMBER_REGEX).matcher(text);
        while (matcher.find()) {
            Long number = string2number(matcher.group());
            if (number != null && filter.test(number)) {
                numbers.add(number);
            }
        }
        return numbers.isEmpty() ? Collections.emptySet() : numbers;
    }

    /**
     * 中文数字枚举
     */
    @Getter
    public enum Number {
        /**
         * 零
         */
        ZERO(0, '零', '〇'),

        /**
         * 一
         */
        ONE(1, '一', '壹', '弌'),

        /**
         * 二
         */
        TWO(2, '二', '贰', '弍', '两', '俩'),

        /**
         * 三
         */
        THREE(3, '三', '叁', '弎', '仨'),

        /**
         * 四
         */
        FOUR(4, '四', '肆'),

        /**
         * 五
         */
        FIVE(5, '五', '伍'),

        /**
         * 六
         */
        SIX(6, '六', '陆'),

        /**
         * 七
         */
        SEVEN(7, '七', '柒'),

        /**
         * 八
         */
        EIGHT(8, '八', '捌'),

        /**
         * 九
         */
        NINE(9, '九', '玖'),

        /**
         * 十
         */
        TEN(10, '十', '拾'),

        /**
         * 二十
         */
        TWENTY(20, '廿'),

        /**
         * 三十
         */
        THIRTY(30, '卅'),

        /**
         * 四十
         */
        FORTY(40, '卌'),

        /**
         * 二百
         */
        TWO_HUNDRED(200, '皕');

        /**
         * 别名/数值映射表
         */
        private static final Map<Character, Number> ALIAS_NUMBER_MAPPINGS;

        static {
            // 加载别名/数值映射表
            ALIAS_NUMBER_MAPPINGS = Maps.newHashMapWithExpectedSize(
                    Stream.of(Number.values()).mapToInt(number -> number.getAliases().size()).sum()
            );
            for (Number number : Number.values()) {
                number.getAliases().forEach(alias -> ALIAS_NUMBER_MAPPINGS.put(alias, number));
            }
        }

        /**
         * 数值
         */
        private final int value;
        /**
         * 别名集合
         */
        private final Set<Character> aliases;

        Number(int value, Character... aliases) {
            this.value = value;
            this.aliases = Collections.unmodifiableSet(Sets.newHashSet(aliases));
        }

        /**
         * 阿拉伯数字格式化
         *
         * @param text 数字文本
         * @return 阿拉伯数字字符串
         */
        public static String format(String text) {
            if (StringUtils.isEmpty(text)) {
                return text;
            }
            boolean changed = false;
            char[] chars = text.toCharArray();
            for (int i = 0; i < chars.length; i++) {
                Number number = valueOf(chars[i]);
                if (number != null) {
                    chars[i] = Character.forDigit(number.value, 10);
                    changed = true;
                }
            }
            return changed ? new String(chars) : text;
        }

        /**
         * 根据别名获取中文数字枚举
         *
         * @param alias 数字别名
         * @return 数字枚举
         */
        public static Number valueOf(Character alias) {
            return alias == null ? null : ALIAS_NUMBER_MAPPINGS.get(alias);
        }

        /**
         * 数字单位应用
         *
         * @param unit 单位枚举
         * @return 数值
         */
        public long apply(@NonNull Unit unit) {
            return this.value * unit.value;
        }
    }

    /**
     * 中文数字单位
     */
    @Getter
    public enum Unit {
        /**
         * 个
         */
        ONE(1, null),

        /**
         * 十
         */
        TEN(10, ONE, '十', '拾'),

        /**
         * 百
         */
        HUNDRED(100, TEN, '百', '佰', '陌'),

        /**
         * 千
         */
        THOUSAND(1000, HUNDRED, '千', '仟', '阡', 'k', 'K'),

        /**
         * 万
         */
        TEN_THOUSAND(10000, THOUSAND, '万', '萬', 'w', 'W'),

        /**
         * 亿
         */
        BILLION(100000000, null, '亿', '億');

        /**
         * 别名/枚举值映射表
         */
        private static final Map<Character, Unit> ALIAS_UNIT_MAPPINGS;

        static {
            // 加载别名/数字单位映射表
            ALIAS_UNIT_MAPPINGS = Maps.newHashMapWithExpectedSize(
                    Stream.of(Unit.values()).mapToInt(unit -> unit.getAliases().size()).sum()
            );
            for (Unit unit : Unit.values()) {
                unit.getAliases().forEach(alias -> ALIAS_UNIT_MAPPINGS.put(alias, unit));
            }
        }

        /**
         * 数值
         */
        private final int value;
        /**
         * 更小单位
         */
        private final Unit smaller;
        /**
         * 别名集合
         */
        private final Set<Character> aliases;

        Unit(int value, Unit smaller, Character... aliases) {
            this.value = value;
            this.smaller = smaller;
            this.aliases = Collections.unmodifiableSet(Sets.newHashSet(aliases));
        }

        /**
         * 根据别名获取中文数字枚举
         *
         * @param alias 数字别名
         * @return 数字枚举
         */
        public static Unit valueOf(Character alias) {
            return alias == null ? null : ALIAS_UNIT_MAPPINGS.get(alias);
        }

        /**
         * 数字应用
         *
         * @param number 数字
         * @return 数值
         */
        public long apply(long number) {
            return number * this.value;
        }

        /**
         * 单位数字应用
         *
         * @param number 数字枚举
         * @return 数值
         */
        public long apply(@NonNull Number number) {
            return this.apply(number.value);
        }
    }
}
