package com.chelvc.framework.common.util;

import java.nio.charset.StandardCharsets;
import java.security.GeneralSecurityException;
import javax.crypto.Cipher;

import lombok.NonNull;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.codec.digest.MessageDigestAlgorithms;

/**
 * 编解码工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class CodecUtils {
    private CodecUtils() {
    }

    /**
     * MD5加密
     *
     * @param plaintext 明文
     * @return 密文
     */
    public static String md5(String plaintext) {
        return encrypt(plaintext, MessageDigestAlgorithms.MD5);
    }

    /**
     * MD5加密
     *
     * @param plaintext 明文
     * @return 密文
     */
    public static byte[] md5(byte[] plaintext) {
        return encrypt(plaintext, MessageDigestAlgorithms.MD5);
    }

    /**
     * SHA256加密
     *
     * @param plaintext 明文
     * @return 密文
     */
    public static String sha256(String plaintext) {
        return encrypt(plaintext, MessageDigestAlgorithms.SHA_256);
    }

    /**
     * SHA256加密
     *
     * @param plaintext 明文
     * @return 密文
     */
    public static byte[] sha256(byte[] plaintext) {
        return encrypt(plaintext, MessageDigestAlgorithms.SHA_256);
    }

    /**
     * 数据加解密
     *
     * @param cipher 密码处理器实例
     * @param bytes  数据字节数组
     * @return 明文/密文字节数组
     */
    public static byte[] codec(@NonNull Cipher cipher, byte[] bytes) {
        if (bytes == null) {
            return null;
        }
        try {
            return cipher.doFinal(bytes);
        } catch (GeneralSecurityException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 数据加密
     *
     * @param cipher    密码处理器
     * @param plaintext 明文
     * @return 密文（base64）
     */
    public static String encrypt(@NonNull Cipher cipher, String plaintext) {
        return plaintext == null ? null : encrypt(cipher, plaintext.getBytes(StandardCharsets.UTF_8));
    }

    /**
     * 数据加密
     *
     * @param cipher    密码处理器
     * @param plaintext 明文
     * @return 密文（base64）
     */
    public static String encrypt(@NonNull Cipher cipher, byte[] plaintext) {
        return plaintext == null ? null : Base64.encodeBase64String(codec(cipher, plaintext));
    }

    /**
     * 数据加密
     *
     * @param plaintext 明文
     * @param algorithm 加密算法
     * @return 密文
     */
    public static String encrypt(String plaintext, @NonNull String algorithm) {
        if (plaintext == null) {
            return null;
        }
        return Hex.encodeHexString(encrypt(plaintext.getBytes(StandardCharsets.UTF_8), algorithm));
    }

    /**
     * 数据加密
     *
     * @param plaintext 明文
     * @param algorithm 加密算法
     * @return 密文
     */
    public static byte[] encrypt(byte[] plaintext, @NonNull String algorithm) {
        return plaintext == null ? null : DigestUtils.getDigest(algorithm).digest(plaintext);
    }

    /**
     * 数据解密
     *
     * @param cipher     密码处理器
     * @param ciphertext 密文（base64）
     * @return 明文
     */
    public static String decrypt(@NonNull Cipher cipher, String ciphertext) {
        return ciphertext == null ? null : decrypt(cipher, decodeBase64(ciphertext));
    }

    /**
     * 数据解密
     *
     * @param cipher     密码处理器
     * @param ciphertext 密文
     * @return 明文
     */
    public static String decrypt(@NonNull Cipher cipher, byte[] ciphertext) {
        return ciphertext == null ? null : new String(codec(cipher, ciphertext), StandardCharsets.UTF_8);
    }

    /**
     * 解码Base64字符串
     *
     * @param base64 Base64字节数组
     * @return 原始字符串字节数组
     */
    public static byte[] decodeBase64(byte[] base64) {
        byte[] bytes = Base64.decodeBase64(base64);
        if (ObjectUtils.isEmpty(bytes) && ObjectUtils.notEmpty(base64)) {
            throw new IllegalArgumentException("Invalid base64: " + new String(base64, StandardCharsets.UTF_8));
        }
        return bytes;
    }

    /**
     * 解码Base64字符串
     *
     * @param base64 Base64字符串
     * @return 原始字符串字节数组
     */
    public static byte[] decodeBase64(String base64) {
        return decodeBase64(org.apache.commons.codec.binary.StringUtils.getBytesUtf8(base64));
    }
}
