package com.chelvc.framework.common.util;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.io.Writer;
import java.util.function.Consumer;

import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 文件工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public final class FileUtils {
    /**
     * 临时目录
     */
    public static final String TEMPORARY = System.getProperty("java.io.tmpdir");

    /**
     * 文件上传目录
     */
    public static final String UPLOAD = new File(TEMPORARY, "upload").getPath();

    /**
     * 文件下载目录
     */
    public static final String DOWNLOAD = new File(TEMPORARY, "download").getPath();

    static {
        // 初始化文件上传目录
        File upload = new File(UPLOAD);
        if (!upload.exists() && !upload.mkdirs()) {
            log.warn("Make upload dir failed: {}", UPLOAD);
        }

        // 初始化文件下载目录
        File download = new File(DOWNLOAD);
        if (!download.exists() && !download.mkdirs()) {
            log.warn("Make download dir failed: {}", DOWNLOAD);
        }
    }

    private FileUtils() {
    }

    /**
     * 获取文件后缀名
     *
     * @param file 文件对象
     * @return 后缀名
     */
    public static String getSuffix(@NonNull File file) {
        return getSuffix(file.getName());
    }

    /**
     * 获取文件后缀名
     *
     * @param filename 文件名称
     * @return 后缀名
     */
    public static String getSuffix(@NonNull String filename) {
        int separator = filename.lastIndexOf('.');
        if (separator < 0) {
            return null;
        }
        String suffix = StringUtils.ifEmpty(filename.substring(separator + 1), (String) null);
        return StringUtils.ifEmpty(suffix, String::toLowerCase);
    }

    /**
     * 图片压缩
     *
     * @param image 图片对象
     * @return 图片对象
     */
    public static BufferedImage compress(@NonNull BufferedImage image) {
        int width = image.getWidth(null);
        int height = image.getHeight(null);
        BufferedImage compress = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        compress.getGraphics().drawImage(image.getScaledInstance(width, height, Image.SCALE_SMOOTH), 0, 0, null);
        return compress;
    }

    /**
     * 读取文件行
     *
     * @param file     文件对象
     * @param consumer 文件内容消费者
     * @throws IOException I/O异常
     */
    public static void lines(@NonNull File file, @NonNull Consumer<String> consumer) throws IOException {
        lines(new FileReader(file), consumer);
    }

    /**
     * 读取文件行
     *
     * @param in       文件读取处理器
     * @param consumer 文件内容消费者
     * @throws IOException I/O异常
     */
    public static void lines(@NonNull Reader in, @NonNull Consumer<String> consumer) throws IOException {
        try (BufferedReader reader = new BufferedReader(in)) {
            String line;
            while ((line = reader.readLine()) != null) {
                consumer.accept(line);
            }
        }
    }

    /**
     * 读取文件字节数组
     *
     * @param file 文件对象
     * @return 字节数组
     * @throws IOException I/O异常
     */
    public static byte[] getBytes(@NonNull File file) throws IOException {
        return getBytes(new FileInputStream(file));
    }

    /**
     * 读取输入流字节数组
     *
     * @param in 输入流
     * @return 字节数组
     * @throws IOException I/O异常
     */
    public static byte[] getBytes(@NonNull InputStream in) throws IOException {
        int n;
        byte[] buffer = new byte[1024];
        ByteArrayOutputStream out = new ByteArrayOutputStream(Math.max(32, in.available()));
        while ((n = in.read(buffer)) != -1) {
            out.write(buffer, 0, n);
        }
        return out.toByteArray();
    }

    /**
     * 读取文件字符串
     *
     * @param file 文件对象
     * @return 文件字符串
     * @throws IOException I/O异常
     */
    public static String getString(@NonNull File file) throws IOException {
        return getString(new FileReader(file));
    }

    /**
     * 读取文件字符串
     *
     * @param in 文件读取器
     * @return 文件字符串
     * @throws IOException I/O异常
     */
    public static String getString(@NonNull Reader in) throws IOException {
        StringBuilder buffer = new StringBuilder();
        lines(in, line -> {
            if (buffer.length() > 0) {
                buffer.append(StringUtils.ENTER);
            }
            buffer.append(line);
        });
        return buffer.toString();
    }

    /**
     * 写文件
     *
     * @param file    目标文件
     * @param content 文件内容
     * @throws IOException I/O异常
     */
    public static void write(@NonNull File file, @NonNull String content) throws IOException {
        write(new FileWriter(file), content);
    }

    /**
     * 写文件
     *
     * @param out     文件写入处理器
     * @param content 文件内容
     * @throws IOException I/O异常
     */
    public static void write(@NonNull Writer out, @NonNull String content) throws IOException {
        try (BufferedWriter writer = new BufferedWriter(out)) {
            writer.write(content);
        }
    }

    /**
     * 将文件写入文件
     *
     * @param out 输出文件
     * @param in  输入文件
     * @throws IOException I/O异常
     */
    public static void write(@NonNull File out, @NonNull File in) throws IOException {
        try (OutputStream output = new FileOutputStream(out)) {
            write(output, in);
        }
    }

    /**
     * 将输入流写入文件
     *
     * @param file 文件对象
     * @param in   输入流
     * @throws IOException I/O异常
     */
    public static void write(@NonNull File file, @NonNull InputStream in) throws IOException {
        try (OutputStream output = new FileOutputStream(file)) {
            write(output, in);
        }
    }

    /**
     * 将文件写入输入流
     *
     * @param out  输出流
     * @param file 文件对象
     * @throws IOException I/O异常
     */
    public static void write(@NonNull OutputStream out, @NonNull File file) throws IOException {
        try (InputStream input = new FileInputStream(file)) {
            write(out, input);
        }
    }

    /**
     * 将输入流写入输出流
     *
     * @param out 输出流
     * @param in  输入流
     * @throws IOException I/O异常
     */
    public static void write(@NonNull OutputStream out, @NonNull InputStream in) throws IOException {
        int n;
        byte[] buffer = new byte[1024];
        while ((n = in.read(buffer)) > -1) {
            out.write(buffer, 0, n);
        }
    }

    /**
     * 删除文件
     *
     * @param file 文件对象
     */
    public static void delete(@NonNull File file) {
        if (!file.delete()) {
            log.warn("File delete failed: {}", file.getPath());
        }
    }

    /**
     * 获取文件数组
     *
     * @param path 文件目录
     * @return 文件数组
     */
    public static File[] files(@NonNull File path) {
        if (!path.isDirectory()) {
            return new File[0];
        }
        return path.listFiles(file -> file != null && file.isFile());
    }

    /**
     * 获取文件
     *
     * @param path     文件目录
     * @param consumer 文件消费者
     */
    public static void files(@NonNull File path, @NonNull Consumer<File> consumer) {
        if (!path.isDirectory()) {
            return;
        }
        path.listFiles(file -> {
            if (file != null && file.isFile()) {
                consumer.accept(file);
            }
            return false;
        });
    }

    /**
     * 获取文件数组
     *
     * @param path 文件目录
     * @return 文件数组
     */
    public static File[] files(@NonNull String path) {
        return files(new File(path));
    }

    /**
     * 获取文件
     *
     * @param path     文件目录
     * @param consumer 文件消费者
     */
    public static void files(@NonNull String path, @NonNull Consumer<File> consumer) {
        files(new File(path), consumer);
    }

    /**
     * 获取文件目录数组
     *
     * @param path 文件目录
     * @return 文件目录数组
     */
    public static File[] directories(@NonNull File path) {
        if (!path.isDirectory()) {
            return new File[0];
        }
        return path.listFiles(file -> file != null && file.isFile());
    }

    /**
     * 获取文件目录
     *
     * @param path     文件目录
     * @param consumer 文件目录消费者
     */
    public static void directories(@NonNull File path, @NonNull Consumer<File> consumer) {
        if (!path.isDirectory()) {
            return;
        }
        path.listFiles(file -> {
            if (file != null && file.isFile()) {
                consumer.accept(file);
            }
            return false;
        });
    }

    /**
     * 获取文件目录数组
     *
     * @param path 文件目录
     * @return 文件目录数组
     */
    public static File[] directories(@NonNull String path) {
        return directories(new File(path));
    }

    /**
     * 获取文件目录
     *
     * @param path     文件目录
     * @param consumer 文件目录消费者
     */
    public static void directories(@NonNull String path, @NonNull Consumer<File> consumer) {
        directories(new File(path), consumer);
    }
}
