package com.chelvc.framework.common.util;

import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.google.i18n.phonenumbers.NumberParseException;
import com.google.i18n.phonenumbers.PhoneNumberUtil;
import com.google.i18n.phonenumbers.Phonenumber;
import com.google.i18n.phonenumbers.geocoding.PhoneNumberOfflineGeocoder;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 手机号工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public final class MobileUtils {
    /**
     * 默认手机号脱敏模式表达式
     */
    public static final String DEFAULT_DESENSITIZE_EXPRESSION = "---****-?";

    /**
     * 默认手机号脱敏数字
     */
    private static final String DEFAULT_DESENSITIZE_NUMBER =
            "0-9０-９⁰-⁹₀-₉⓿-❾⓪①②③④⑤⑥⑦⑧⑨〇零一壹二贰弍三叁弎仨四肆五伍六陆七柒八捌九玖";

    /**
     * 脱敏手机号默认匹配模式
     */
    private static final Pattern DEFAULT_DESENSITIZE_PATTERN = Pattern.compile(String.format(
            "[1１¹₁❶①一壹]([^%s]*[%s]){10}", DEFAULT_DESENSITIZE_NUMBER, DEFAULT_DESENSITIZE_NUMBER
    ));

    /**
     * 手机号处理工具实例
     */
    private static final PhoneNumberUtil UTIL = PhoneNumberUtil.getInstance();

    /**
     * 地理编码器实例
     */
    private static final PhoneNumberOfflineGeocoder GEOCODER = PhoneNumberOfflineGeocoder.getInstance();

    private MobileUtils() {
    }

    /**
     * 解析手机号
     *
     * @param mobile 手机号文本
     * @return 手机号对象实例
     */
    public static Phonenumber.PhoneNumber parse(String mobile) {
        return parse(mobile, Locale.CHINA);
    }

    /**
     * 解析手机号
     *
     * @param mobile 手机号文本
     * @param locale 地区信息
     * @return 手机号对象实例
     */
    public static Phonenumber.PhoneNumber parse(String mobile, @NonNull Locale locale) {
        try {
            return StringUtils.isEmpty(mobile) ? null : UTIL.parse(mobile, locale.getCountry());
        } catch (NumberParseException e) {
            log.warn("Invalid mobile[{}]: {}", locale.getCountry(), mobile);
        }
        return null;
    }

    /**
     * 判断是否是有效的手机号
     *
     * @param number 手机号对象
     * @return true/false
     */
    public static boolean isMobile(Phonenumber.PhoneNumber number) {
        return number != null && UTIL.isValidNumber(number);
    }

    /**
     * 判断是否是有效的手机号
     *
     * @param mobile 手机号文本
     * @return true/false
     */
    public static boolean isMobile(String mobile) {
        return isMobile(mobile, Locale.CHINA);
    }

    /**
     * 判断是否是有效的手机号
     *
     * @param mobile 手机号对象
     * @param locale 地区信息
     * @return true/false
     */
    public static boolean isMobile(String mobile, @NonNull Locale locale) {
        return isMobile(parse(mobile, locale));
    }

    /**
     * 获取手机号地址信息
     *
     * @param number 手机号对象
     * @return 地址信息
     */
    public static String mobile2address(Phonenumber.PhoneNumber number) {
        return mobile2address(number, Locale.CHINA);
    }

    /**
     * 获取手机号地址信息
     *
     * @param number 手机号对象
     * @param locale 地区信息
     * @return 地址信息
     */
    public static String mobile2address(Phonenumber.PhoneNumber number, @NonNull Locale locale) {
        return number == null ? null : GEOCODER.getDescriptionForNumber(number, locale);
    }

    /**
     * 获取手机号地址信息
     *
     * @param mobile 手机号文本
     * @return 地址信息
     */
    public static String mobile2address(String mobile) {
        return mobile2address(mobile, Locale.CHINA);
    }

    /**
     * 获取手机号地址信息
     *
     * @param mobile 手机号文本
     * @param locale 地区信息
     * @return 地址信息
     */
    public static String mobile2address(String mobile, @NonNull Locale locale) {
        return mobile2address(parse(mobile, locale), locale);
    }

    /**
     * 脱敏手机号
     *
     * @param text 文本内容
     * @return 文本内容
     */
    public static String desensitize(String text) {
        return desensitize(text, DEFAULT_DESENSITIZE_EXPRESSION);
    }

    /**
     * 脱敏手机号
     *
     * @param text    文本内容
     * @param pattern 手机号匹配模式
     * @return 文本内容
     */
    public static String desensitize(String text, @NonNull Pattern pattern) {
        return desensitize(text, pattern, DEFAULT_DESENSITIZE_EXPRESSION);
    }

    /**
     * 脱敏手机号
     *
     * @param text       文本内容
     * @param expression 手机号脱敏模式表达式
     * @return 文本内容
     */
    public static String desensitize(String text, @NonNull String expression) {
        return desensitize(text, DEFAULT_DESENSITIZE_PATTERN, expression);
    }

    /**
     * 脱敏手机号
     *
     * @param text       文本内容
     * @param pattern    手机号匹配模式
     * @param expression 手机号脱敏模式表达式
     * @return 文本内容
     */
    public static String desensitize(String text, @NonNull Pattern pattern, @NonNull String expression) {
        if (StringUtils.isEmpty(text)) {
            return text;
        }

        int offset = 0;
        StringBuilder buffer = null;
        DesensitizeUtils.Mode mode = null;
        Matcher matcher = pattern.matcher(text);
        while (matcher.find()) {
            if (buffer == null) {
                buffer = new StringBuilder();
            }
            if (mode == null) {
                mode = DesensitizeUtils.analyse(expression);
            }
            int start = matcher.start();
            if (start > 0) {
                buffer.append(text, offset, start);
            }
            String mobile = matcher.group();
            if (mobile.length() > 11) {
                mobile = StringUtils.clear(mobile);
            }
            if (ChineseUtils.isChineseNumber(mobile)) {
                mobile = ChineseUtils.Number.format(mobile);
            }
            if (StringUtils.notEmpty(mobile)) {
                buffer.append(mode.desensitize(mobile));
            }
            offset = matcher.end();
        }
        if (buffer == null) {
            return text;
        } else if (offset < text.length()) {
            buffer.append(text, offset, text.length());
        }
        return buffer.toString();
    }
}
