package com.chelvc.framework.common.util;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.math.RoundingMode;

import lombok.NonNull;

/**
 * 数字处理工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class NumberUtils {
    /**
     * 默认小数保留位数
     */
    public static final int DEFAULT_DECIMAL_SCALE = 8;

    /**
     * 数字缩/放字符数组
     */
    private static final char[] REDUCE_ENLARGE_CHARS = {
            '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
            'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
            'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
            'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
            'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'
    };

    /**
     * 默认小数四舍五入模式
     */
    public static final RoundingMode DEFAULT_ROUND_MODE = RoundingMode.HALF_UP;

    /**
     * 100小数
     */
    private static final BigDecimal HUNDRED_DECIMAL = new BigDecimal(100);

    private NumberUtils() {
    }

    /**
     * 判断数字是否等于0
     *
     * @param number 数字
     * @return true/false
     */
    public static boolean isZero(Number number) {
        if (number instanceof BigInteger) {
            return number.equals(BigInteger.ZERO);
        } else if (number instanceof BigDecimal) {
            return number.equals(BigDecimal.ZERO);
        }
        return number != null && number.doubleValue() == 0;
    }

    /**
     * 判断数字是否小于0
     *
     * @param number 数字
     * @return true/false
     */
    public static boolean isLessZero(Number number) {
        if (number instanceof BigInteger) {
            return ((BigInteger) number).compareTo(BigInteger.ZERO) < 0;
        } else if (number instanceof BigDecimal) {
            return ((BigDecimal) number).compareTo(BigDecimal.ZERO) < 0;
        }
        return number == null || number.doubleValue() < 0;
    }

    /**
     * 判断数字是否大于0
     *
     * @param number 数字
     * @return true/false
     */
    public static boolean isGreaterZero(Number number) {
        if (number instanceof BigInteger) {
            return ((BigInteger) number).compareTo(BigInteger.ZERO) > 0;
        } else if (number instanceof BigDecimal) {
            return ((BigDecimal) number).compareTo(BigDecimal.ZERO) > 0;
        }
        return number != null && number.doubleValue() > 0;
    }

    /**
     * 将数字转换成小数
     *
     * @param number 数字
     * @return 大数字小数
     */
    public static BigDecimal number2decimal(Number number) {
        if (number == null) {
            return null;
        } else if (number instanceof BigDecimal) {
            return (BigDecimal) number;
        } else if (number instanceof BigInteger) {
            return new BigDecimal((BigInteger) number);
        }
        return new BigDecimal(number.toString());
    }

    /**
     * 乘以10
     *
     * @param number 数字
     * @return 结果
     */
    public static BigDecimal multiply10(Number number) {
        return number == null || isZero(number) ? BigDecimal.ZERO : number2decimal(number).multiply(BigDecimal.TEN);
    }

    /**
     * 乘以100
     *
     * @param number 数字
     * @return 结果
     */
    public static BigDecimal multiply100(Number number) {
        return number == null || isZero(number) ? BigDecimal.ZERO : number2decimal(number).multiply(HUNDRED_DECIMAL);
    }

    /**
     * 除以10
     *
     * @param number 被除数
     * @return 结果
     */
    public static BigDecimal divide10(Number number) {
        return divide(number, BigDecimal.TEN);
    }

    /**
     * 除以10
     *
     * @param number 被除数
     * @param scale  小数位数
     * @return 结果
     */
    public static BigDecimal divide10(Number number, int scale) {
        return divide(number, BigDecimal.TEN, scale);
    }

    /**
     * 除以10
     *
     * @param number 被除数
     * @param mode   四舍五入模式
     * @return 结果
     */
    public static BigDecimal divide10(Number number, @NonNull RoundingMode mode) {
        return divide(number, BigDecimal.TEN, mode);
    }

    /**
     * 除以10
     *
     * @param number 被除数
     * @param scale  小数位数
     * @param mode   四舍五入模式
     * @return 结果
     */
    public static BigDecimal divide10(Number number, int scale, @NonNull RoundingMode mode) {
        return divide(number, BigDecimal.TEN, scale, mode);
    }

    /**
     * 除以100
     *
     * @param number 被除数
     * @return 结果
     */
    public static BigDecimal divide100(Number number) {
        return divide(number, HUNDRED_DECIMAL);
    }

    /**
     * 除以100
     *
     * @param number 被除数
     * @param scale  小数位数
     * @return 结果
     */
    public static BigDecimal divide100(Number number, int scale) {
        return divide(number, HUNDRED_DECIMAL, scale);
    }

    /**
     * 除以100
     *
     * @param number 被除数
     * @param mode   四舍五入模式
     * @return 结果
     */
    public static BigDecimal divide100(Number number, @NonNull RoundingMode mode) {
        return divide(number, HUNDRED_DECIMAL, mode);
    }

    /**
     * 除以100
     *
     * @param number 被除数
     * @param scale  小数位数
     * @param mode   四舍五入模式
     * @return 结果
     */
    public static BigDecimal divide100(Number number, int scale, @NonNull RoundingMode mode) {
        return divide(number, HUNDRED_DECIMAL, scale, mode);
    }

    /**
     * 除法
     *
     * @param dividend 被除数
     * @param divisor  除数
     * @return 结果
     */
    public static BigDecimal divide(Number dividend, Number divisor) {
        return divide(dividend, divisor, DEFAULT_DECIMAL_SCALE);
    }

    /**
     * 除法
     *
     * @param dividend 被除数
     * @param divisor  除数
     * @param scale    小数位数
     * @return 结果
     */
    public static BigDecimal divide(Number dividend, Number divisor, int scale) {
        return divide(dividend, divisor, scale, DEFAULT_ROUND_MODE);
    }

    /**
     * 除法
     *
     * @param dividend 被除数
     * @param divisor  除数
     * @param mode     四舍五入模式
     * @return 结果
     */
    public static BigDecimal divide(Number dividend, Number divisor, @NonNull RoundingMode mode) {
        return divide(dividend, divisor, DEFAULT_DECIMAL_SCALE, mode);
    }

    /**
     * 除法
     *
     * @param dividend 被除数
     * @param divisor  除数
     * @param scale    小数位数
     * @param mode     四舍五入模式
     * @return 结果
     */
    public static BigDecimal divide(Number dividend, Number divisor, int scale, @NonNull RoundingMode mode) {
        if (dividend == null || divisor == null || isZero(dividend) || isZero(divisor)) {
            return BigDecimal.ZERO.setScale(scale, mode);
        }
        return number2decimal(dividend).divide(number2decimal(divisor), scale, mode);
    }

    /**
     * 格式化小数
     *
     * @param decimal 小数
     * @return 小数字符串
     */
    public static String format(BigDecimal decimal) {
        return format(decimal, false);
    }

    /**
     * 格式化小数
     *
     * @param decimal 小数
     * @param scale   小数保留位数
     * @return 小数字符串
     */
    public static String format(BigDecimal decimal, int scale) {
        return format(decimal, scale, RoundingMode.HALF_UP, false);
    }

    /**
     * 格式化小数
     *
     * @param decimal 小数
     * @param mode    小数舍弃模式
     * @return 小数字符串
     */
    public static String format(BigDecimal decimal, @NonNull RoundingMode mode) {
        return format(decimal, DEFAULT_DECIMAL_SCALE, mode, false);
    }

    /**
     * 格式化小数
     *
     * @param decimal   小数
     * @param trimZeros 是否去掉多余的0
     * @return 小数字符串
     */
    public static String format(BigDecimal decimal, boolean trimZeros) {
        return format(decimal, DEFAULT_DECIMAL_SCALE, RoundingMode.HALF_UP, trimZeros);
    }

    /**
     * 格式化小数
     *
     * @param decimal 小数
     * @param scale   小数保留位数
     * @param mode    小数舍弃模式
     * @return 小数字符串
     */
    public static String format(BigDecimal decimal, int scale, @NonNull RoundingMode mode) {
        return format(decimal, scale, mode, false);
    }

    /**
     * 格式化小数
     *
     * @param decimal   小数
     * @param scale     小数保留位数
     * @param trimZeros 是否去掉多余的0
     * @return 小数字符串
     */
    public static String format(BigDecimal decimal, int scale, boolean trimZeros) {
        return format(decimal, scale, DEFAULT_ROUND_MODE, trimZeros);
    }

    /**
     * 格式化小数
     *
     * @param decimal   小数
     * @param mode      小数舍弃模式
     * @param trimZeros 是否去掉多余的0
     * @return 小数字符串
     */
    public static String format(BigDecimal decimal, @NonNull RoundingMode mode, boolean trimZeros) {
        return format(decimal, DEFAULT_DECIMAL_SCALE, mode, trimZeros);
    }

    /**
     * 格式化小数
     *
     * @param decimal   小数
     * @param scale     小数保留位数
     * @param mode      小数舍弃模式
     * @param trimZeros 是否去掉多余的0
     * @return 小数字符串
     */
    public static String format(BigDecimal decimal, int scale, @NonNull RoundingMode mode, boolean trimZeros) {
        if (decimal == null) {
            return null;
        }
        decimal = decimal.setScale(scale, mode);
        if (trimZeros) {
            decimal = decimal.stripTrailingZeros();
        }
        return decimal.toPlainString();
    }

    /**
     * 将数字缩小为字符串
     *
     * @param number 数字
     * @return 字符串
     */
    public static String reduce(long number) {
        if (number < 0) {
            number = ((long) 2 * 0x7fffffff) + number + 2;
        }
        int size = 32, pos = size;
        int seed = REDUCE_ENLARGE_CHARS.length;
        char[] buffer = new char[size];
        while ((number / seed) > 0) {
            buffer[--pos] = REDUCE_ENLARGE_CHARS[(int) (number % seed)];
            number /= seed;
        }
        buffer[--pos] = REDUCE_ENLARGE_CHARS[(int) (number % seed)];
        return new String(buffer, pos, (size - pos));
    }

    /**
     * 将字符串放大为数字
     *
     * @param reduce 数字缩小字符串
     * @return 数字
     */
    public static long enlarge(String reduce) {
        if (StringUtils.isEmpty(reduce)) {
            return 0;
        }

        long number = 0, base = 1;
        char[] chars = reduce.toCharArray();
        for (int i = chars.length - 1; i >= 0; i--) {
            int index = 0;
            for (int j = 0, length = REDUCE_ENLARGE_CHARS.length; j < length; j++) {
                if (REDUCE_ENLARGE_CHARS[j] == chars[i]) {
                    index = j;
                }
            }
            number += index * base;
            base *= REDUCE_ENLARGE_CHARS.length;
        }
        return number;
    }
}
