package com.chelvc.framework.common.util;

import java.lang.ref.WeakReference;
import java.util.Collection;
import java.util.Comparator;
import java.util.Map;
import java.util.Objects;
import java.util.UUID;
import java.util.concurrent.ThreadLocalRandom;
import java.util.function.BiFunction;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.function.Supplier;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import java.util.zip.CRC32;

import com.chelvc.framework.common.function.MultipleFunction;
import com.fasterxml.jackson.databind.PropertyNamingStrategy;
import com.google.common.collect.Maps;
import lombok.NonNull;

/**
 * 字符串工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class StringUtils {
    /**
     * 空字符串
     */
    public static final String EMPTY = "";

    /**
     * 空格字符串
     */
    public static final String SPACE = " ";

    /**
     * 等号字符串
     */
    public static final String EQUAL = "=";

    /**
     * 换行符
     */
    public static final String ENTER = "\n";

    /**
     * 布尔真
     */
    public static final String TRUE = "true";

    /**
     * 布尔假
     */
    public static final String FALSE = "false";

    /**
     * 下划线
     */
    public static final String UNDERLINE = "_";

    /**
     * 横线
     */
    public static final String HORIZONTAL = "-";

    /**
     * 空字符串数组
     */
    public static final String[] EMPTY_ARRAY = new String[0];

    /**
     * 资源分隔符
     */
    public static final String URI_SEPARATOR = "/";

    /**
     * 表情符号正则表达式
     */
    public static final String EMOJI_REGEX = "\\[[^\\[\\]]+\\]";

    /**
     * 英文字母正则表达式
     */
    public static final String LETTER_REGEX = "[a-zA-Z]+";

    /**
     * 逗号拆分分隔符
     */
    public static final String COMMA_SPLIT_DELIMITER = "[ ,]";

    /**
     * 手机号正则表达式
     */
    public static final String MOBILE_REGEX = "1[3-9][0-9]{9}";

    /**
     * 纯文本正则表达式
     */
    public static final String PLAINTEXT_REGEX = "[^\\w\u4e00-\u9fa5\\.\\-—~～_\\+]";

    /**
     * 链接符号正则表达式
     */
    public static final String LINK_REGEX = "[\\.\\-—~～_·]";

    /**
     * 非数字符号正则表达式
     */
    public static final String NON_NUMBER_SYMBOL_REGEX = "[\\.\\-—~～_](?![0-9])";

    /**
     * 数字链接符号正则表达式
     */
    public static final String NUMBER_LINK_REGEX = "[0-9]*([\\.\\-—~～_·][0-9]+){2,}";

    /**
     * 数字正则表达式
     */
    public static final String NUMBER_REGEX = "[\\+-]?(\\.?[0-9]+|[0-9]+\\.?[0-9]+|[0-9]+\\.?)";

    /**
     * IP正则表达式
     */
    public static final String IP_REGEX = "[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}(/[1-9]{1,2})?";

    /**
     * URL正则表达式
     */
    public static final String URL_REGEX = "(?i)(http://|https://|ftp://|file://|www\\.)" +
            "[-A-Za-z0-9+&@#/%?=~_|!:,.;]+[-A-Za-z0-9+&@#/%=~_|]";

    /**
     * 图片标签正则表达式
     */
    public static final String IMAGE_REGEX = "(<[Ii][Mm][Gg]\\s*[^>]*?\\s+[Ss][Rr][Cc]\\s*=\\s*[\"'])([^\"']+?)([\"'])";

    /**
     * 邮箱正则表达式
     */
    public static final String EMAIL_REGEX = "\\w+([-+.]\\w+)*@\\w+([-.]\\w+)*\\.\\w+([-.]\\w+)*";

    /**
     * 版本号正则表达式
     */
    public static final String VERSION_REGEX = "[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}";

    /**
     * 特殊字符正则表达式
     */
    public static final String SPECIAL_REGEX =
            "[^\\`\\~\\·\\～\\!\\！\\@\\#\\$\\¥\\%\\^\\…\\&\\*\\(\\)\\（\\）\\-\\_\\—\\=\\+\\[\\]\\{\\}\\【\\】\\「\\」" +
                    "\\|\\、\\｜\\;\\:\\；\\：\\'\"\\,\\.\\，\\。\\<\\>\\《\\》\\/\\?\\？A-Za-z0-9\u4e00-\u9fa5\n ]";

    /**
     * 默认随机字符串长度
     */
    public static final int DEFAULT_RANDOM_LENGTH = 4;

    /**
     * 默认密钥字符串长度
     */
    public static final int DEFAULT_SECRET_LENGTH = 16;

    /**
     * 基础的随机串字符
     */
    private static final char[] BASIC_RANDOM_CHARS = new char[]{
            'V', 'u', 's', 'r', 'l', 'g', 'T', 'z', 'H', 'v', '5', '0', 'P', 'm', '9', 'j', 'x', 'X', 'K', 'w', '8',
            'M', 'd', 'S', 'F', 'e', 'W', '7', 'G', '4', 'y', 'B', 'R', 'Z', 'h', 'U', 'D', 'L', 'A', 'a', 'f', 'N',
            'k', '6', '2', 'C', 'c', '3', 'p', 'Y', 'Q', 'E', 'q', 't', 'b', '1', 'n', 'J'
    };

    /**
     * 标准的随机串字符
     */
    private static final char[] STANDARD_RANDOM_CHARS = {
            'd', 'j', '9', '0', 'e', 'n', 'x', 'A', 'h', 'Y', 'B', '3', '2', 's', 'Q', 'R', 'w', 'c', 'm', '4', '^',
            'V', 'F', 'v', 'U', 'l', 'H', 'W', 'L', '&', 'G', '@', '1', 'J', '#', 'a', 'C', '7', 'N', '|', 'g', 'K',
            'T', 'X', 'u', 'b', 'k', 'P', 'q', 'Z', 'f', '%', 't', 'M', 'y', '*', 'z', 'E', 'S', 'r', '8', 'p', '~',
            'D', '6', '!', '$', '5'
    };

    /**
     * 正则表达式/匹配模式对象映射表
     */
    private static final Map<String, WeakReference<Pattern>> REGEX_PATTERN_MAPPING = Maps.newConcurrentMap();

    private StringUtils() {
    }

    /**
     * 判断对象是否为空
     *
     * @param object 对象实例
     * @return true/false
     */
    public static boolean isEmpty(Object object) {
        return object == null || (object instanceof CharSequence && ((CharSequence) object).length() == 0);
    }

    /**
     * 判断对象是否为非空
     *
     * @param object 对象实例
     * @return true/false
     */
    public static boolean notEmpty(Object object) {
        return !isEmpty(object);
    }

    /**
     * 比较两个字符串中第一个是否为空，如果是则返回第二个字符串，否则返回第一个字符串
     *
     * @param first  第一个字符串
     * @param second 第二个字符串
     * @param <T>    字符串类型
     * @return 非空字符串
     */
    public static <T> T ifEmpty(T first, T second) {
        return isEmpty(first) ? second : first;
    }

    /**
     * 如果原始字符串不为空，则对原始字符串做对象适配
     *
     * @param original 原始字符串
     * @param adapter  对象适配函数
     * @param <T>      原始字符串类型
     * @param <R>      适配目标对象类型
     * @return 适配目标对象实例
     */
    public static <T, R> R ifEmpty(T original, @NonNull Function<T, R> adapter) {
        return isEmpty(original) ? null : adapter.apply(original);
    }

    /**
     * 如果原始字符串不为空，则对原始字符串做对象适配，如果目标适配对象为空则返回默认值
     *
     * @param original     原始字符串
     * @param adapter      对象适配函数
     * @param defaultValue 默认值
     * @param <T>          原始字符串类型
     * @param <R>          适配目标对象类型
     * @return 适配目标对象实例
     */
    public static <T, R> R ifEmpty(T original, @NonNull Function<T, R> adapter, R defaultValue) {
        R value = ifEmpty(original, adapter);
        return isEmpty(value) ? defaultValue : value;
    }

    /**
     * 如果原始字符串不为空，则对原始字符串做对象适配，如果目标适配对象为空则返回默认值
     *
     * @param original  原始字符串
     * @param adapter   对象适配函数
     * @param defaulter 默认值提供者
     * @param <T>       原始字符串类型
     * @param <R>       适配目标对象类型
     * @return 适配目标对象实例
     */
    public static <T, R> R ifEmpty(T original, @NonNull Function<T, R> adapter, @NonNull Supplier<R> defaulter) {
        R value = ifEmpty(original, adapter);
        return isEmpty(value) ? defaulter.get() : value;
    }

    /**
     * 判断字符串是否为空白
     *
     * @param sequence 字符串
     * @param <T>      字符串类型
     * @return true/false
     */
    public static <T extends CharSequence> boolean isBlank(T sequence) {
        return org.apache.commons.lang3.StringUtils.isBlank(sequence);
    }

    /**
     * 判断字符串是否为非空白
     *
     * @param sequence 字符串
     * @param <T>      字符串类型
     * @return true/false
     */
    public static <T extends CharSequence> boolean notBlank(T sequence) {
        return !isBlank(sequence);
    }

    /**
     * 比较两个字符串中第一个是否为空白，如果是则返回第二个字符串，否则返回第一个字符串
     *
     * @param first  第一个字符串
     * @param second 第二个字符串
     * @param <T>    字符串类型
     * @return 非空白字符串
     */
    public static <T extends CharSequence> T ifBlank(T first, T second) {
        return isBlank(first) ? second : first;
    }

    /**
     * 判断字符串是否有效，字符串形式的null为无效字符串
     *
     * @param string 字符串
     * @return true/false
     */
    public static boolean isValid(String string) {
        return notEmpty(string) && !string.equalsIgnoreCase("null");
    }

    /**
     * 判断字符串是否无效，字符串形式的null为无效字符串
     *
     * @param string 字符串
     * @return true/false
     */
    public static boolean isInvalid(String string) {
        return !isValid(string);
    }

    /**
     * 比较两个字符串中第一个是否无效，如果是则返回第二个字符串，否则返回第一个字符串
     *
     * @param first  第一个字符串
     * @param second 第二个字符串
     * @return 非空字符串
     */
    public static String ifInvalid(String first, String second) {
        return isInvalid(first) ? second : first;
    }

    /**
     * 获取正则表达式匹配模式
     *
     * @param regex 正则表达式
     * @return 匹配模式对象
     */
    public static Pattern getPattern(@NonNull String regex) {
        Pattern pattern = ObjectUtils.ifNull(REGEX_PATTERN_MAPPING.get(regex), WeakReference::get);
        if (pattern == null) {
            pattern = Pattern.compile(regex);
            REGEX_PATTERN_MAPPING.put(regex, new WeakReference<>(pattern));
        }
        return pattern;
    }

    /**
     * 正则表达式匹配
     *
     * @param source 源字符串
     * @param regex  正则表达式
     * @param <T>    字符串类型
     * @return true/false
     */
    public static <T extends CharSequence> boolean matches(T source, String regex) {
        return notEmpty(source) && getPattern(regex).matcher(source).matches();
    }

    /**
     * 判断字符串是否是表情符号
     *
     * @param source 字符串
     * @param <T>    字符串类型
     * @return true/false
     */
    public static <T extends CharSequence> boolean isEmoji(T source) {
        return notEmpty(source) && getPattern(EMOJI_REGEX).matcher(source).matches();
    }

    /**
     * 判断字符串是否是英文字母
     *
     * @param source 字符串
     * @param <T>    字符串类型
     * @return true/false
     */
    public static <T extends CharSequence> boolean isLetter(T source) {
        return notEmpty(source) && getPattern(LETTER_REGEX).matcher(source).matches();
    }

    /**
     * 判断字符串是否是IP
     *
     * @param source 字符串
     * @param <T>    字符串类型
     * @return true/false
     */
    public static <T extends CharSequence> boolean isIp(T source) {
        return notEmpty(source) && getPattern(IP_REGEX).matcher(source).matches();
    }

    /**
     * 判断字符串是否是URL
     *
     * @param source 字符串
     * @param <T>    字符串类型
     * @return true/false
     */
    public static <T extends CharSequence> boolean isUrl(T source) {
        return notEmpty(source) && getPattern(URL_REGEX).matcher(source).matches();
    }

    /**
     * 判断字符串是否是图片地址
     *
     * @param source 字符串
     * @param <T>    字符串类型
     * @return true/false
     */
    public static <T extends CharSequence> boolean isImage(T source) {
        return notEmpty(source) && getPattern(IMAGE_REGEX).matcher(source).matches();
    }

    /**
     * 判断字符串是否是邮箱地址
     *
     * @param source 字符串
     * @param <T>    字符串类型
     * @return true/false
     */
    public static <T extends CharSequence> boolean isEmail(T source) {
        return notEmpty(source) && getPattern(EMAIL_REGEX).matcher(source).matches();
    }

    /**
     * 判断字符串是否是版本号
     *
     * @param source 字符串
     * @param <T>    字符串类型
     * @return true/false
     */
    public static <T extends CharSequence> boolean isVersion(T source) {
        return notEmpty(source) && getPattern(VERSION_REGEX).matcher(source).matches();
    }

    /**
     * 判断字符串是否是特殊字符
     *
     * @param source 字符串
     * @param <T>    字符串类型
     * @return true/false
     */
    public static <T extends CharSequence> boolean isSpecial(T source) {
        return notEmpty(source) && getPattern(SPECIAL_REGEX).matcher(source).matches();
    }

    /**
     * 判断字符串是否是整数
     *
     * @param source 字符串
     * @param <T>    字符串类型
     * @return true/false
     */
    public static <T extends CharSequence> boolean isInteger(T source) {
        if (isEmpty(source)) {
            return false;
        }
        char first = source.charAt(0);
        int i = first == '+' || first == '-' ? 1 : 0;
        for (int length = source.length(); i < length; i++) {
            if (!Character.isDigit(source.charAt(i))) {
                return false;
            }
        }
        return true;
    }

    /**
     * 判断字符串是否是手机号
     *
     * @param source 字符串
     * @param <T>    字符串类型
     * @return true/false
     */
    public static <T extends CharSequence> boolean isMobile(T source) {
        return notEmpty(source) && source.length() == 11 && getPattern(MOBILE_REGEX).matcher(source).matches();
    }

    /**
     * 判断字符串是否是数字
     *
     * @param source 字符串
     * @param <T>    字符串类型
     * @return true/false
     */
    public static <T extends CharSequence> boolean isNumber(T source) {
        return notEmpty(source) && getPattern(NUMBER_REGEX).matcher(source).matches();
    }

    /**
     * 字符替换
     *
     * @param source      源字符串
     * @param original    被替换字符
     * @param replacement 替换目标字符
     * @return 替换后字符串
     */
    public static String replace(String source, char original, char replacement) {
        if (source == null || source.isEmpty() || original == replacement) {
            return source;
        }
        return replace(source, c -> c == original ? replacement : c);
    }

    /**
     * 字符替换
     *
     * @param source   源字符串
     * @param function 字符替换函数
     * @return 替换后字符串
     */
    public static String replace(String source, @NonNull Function<Character, Character> function) {
        if (source == null || source.isEmpty()) {
            return source;
        }

        char[] chars = null;
        for (int i = 0, size = source.length(); i < size; i++) {
            char original = source.charAt(i);
            char replacement = function.apply(original);
            if (original != replacement) {
                if (chars == null) {
                    chars = source.toCharArray();
                }
                chars[i] = replacement;
            }
        }
        return chars == null ? source : new String(chars);
    }

    /**
     * 正则表达式替换
     *
     * @param source      源字符串
     * @param regex       正则表达式
     * @param replacement 替换字符串
     * @param <T>         字符串类型
     * @return 替换后字符串
     */
    public static <T extends CharSequence> String replace(T source, String regex, String replacement) {
        return source == null ? null : replacement == null ? source.toString() :
                getPattern(regex).matcher(source).replaceAll(replacement);
    }

    /**
     * 字符串拆分
     *
     * @param source 被拆分字符串
     * @param regex  分割符正则表达式
     * @param <T>    字符串类型
     * @return 拆分字符串数组
     */
    public static <T extends CharSequence> String[] split(T source, String regex) {
        return source == null ? null : getPattern(regex).split(source);
    }

    /**
     * 字符串拆分
     *
     * @param source 被拆分字符串
     * @param regex  分割符正则表达式
     * @param limit  开始下标
     * @param <T>    字符串类型
     * @return 拆分字符串数组
     */
    public static <T extends CharSequence> String[] split(T source, String regex, int limit) {
        return source == null ? null : getPattern(regex).split(source, limit);
    }

    /**
     * 拆分有效字符串（默认使用","号拆分）
     *
     * @param source 被拆分字符串
     * @param <T>    字符串类型
     * @return 字符串拆分数组
     */
    public static <T extends CharSequence> String[] splitActives(T source) {
        return splitActives(source, COMMA_SPLIT_DELIMITER);
    }

    /**
     * 拆分有效字符串（忽略空格）
     *
     * @param regex  拆分符号正则表达式
     * @param source 被拆分字符串
     * @param <T>    字符串类型
     * @return 字符串拆分数组
     */
    public static <T extends CharSequence> String[] splitActives(T source, String regex) {
        return source == null ? null :
                Stream.of(split(source, regex)).filter(StringUtils::notEmpty).toArray(String[]::new);
    }

    /**
     * 截取字符串
     *
     * @param original 原始字符串
     * @param index    开始下标（负数标识从字符串末尾开始）
     * @return 子字符串
     */
    public static String substring(CharSequence original, int index) {
        if (isEmpty(original)) {
            return original == null ? null : EMPTY;
        } else if (index < 0) {
            return original.subSequence(original.length() + index, original.length()).toString();
        }
        return original.subSequence(index, original.length()).toString();
    }

    /**
     * 截取字符串
     *
     * @param original 原始字符串
     * @param index    开始下标（负数标识从字符串末尾开始）
     * @param length   截取长度
     * @return 子字符串
     */
    public static String substring(CharSequence original, int index, int length) {
        AssertUtils.arg(length > -1, () -> "length must be greater than -1");
        if (isEmpty(original)) {
            return original == null ? null : EMPTY;
        } else if (index < 0) {
            index += original.length();
        }
        return original.subSequence(index, Math.min(original.length(), index + length)).toString();
    }

    /**
     * 去掉字符串左右空格
     *
     * @param original 原始字符串
     * @return 字符串
     */
    public static String trim(CharSequence original) {
        if (isEmpty(original)) {
            return original == null ? null : EMPTY;
        } else if (original instanceof String) {
            return ((String) original).trim();
        }

        int i = 0, size = original.length();
        while (i < size && original.charAt(i) <= ' ') {
            i++;
        }
        while (i < size && original.charAt(size - 1) <= ' ') {
            size--;
        }
        if (i > 0 || size < original.length()) {
            return original.subSequence(i, size).toString();
        }
        return original.toString();
    }

    /**
     * 去掉字符串右边空格
     *
     * @param original 原始字符串
     * @return 字符串
     */
    public static String rtrim(CharSequence original) {
        if (isEmpty(original)) {
            return original == null ? null : EMPTY;
        }

        int size = original.length();
        while (size > 0 && original.charAt(size - 1) <= ' ') {
            size--;
        }
        return size < original.length() ? original.subSequence(0, size).toString() : original.toString();
    }

    /**
     * 去掉字符串左边空格
     *
     * @param original 原始字符串
     * @return 字符串
     */
    public static String ltrim(CharSequence original) {
        if (isEmpty(original)) {
            return original == null ? null : EMPTY;
        }

        int i = 0, size = original.length();
        while (i < size && original.charAt(i) <= ' ') {
            i++;
        }
        return i > 0 ? original.subSequence(i, size).toString() : original.toString();
    }

    /**
     * 字符串左对齐
     *
     * @param original 原始字符串
     * @param length   字符串总长度
     * @return 对齐后的字符串
     */
    public static String ljust(CharSequence original, int length) {
        return ljust(original, length, ' ');
    }

    /**
     * 字符串左对齐
     *
     * @param original 原始字符串
     * @param length   字符串总长度
     * @param c        补齐字符
     * @return 对齐后的字符串
     */
    public static String ljust(CharSequence original, int length, char c) {
        return just(original, length, c, false);
    }

    /**
     * 字符串右对齐
     *
     * @param original 原始字符串
     * @param length   字符串总长度
     * @return 对齐后的字符串
     */
    public static String rjust(CharSequence original, int length) {
        return rjust(original, length, ' ');
    }

    /**
     * 字符串右对齐
     *
     * @param original 原始字符串
     * @param length   字符串总长度
     * @param c        补齐字符
     * @return 对齐后的字符串
     */
    public static String rjust(CharSequence original, int length, char c) {
        return just(original, length, c, true);
    }

    /**
     * 字符串对齐
     *
     * @param original 原始字符串
     * @param length   字符串总长度
     * @param c        补齐字符
     * @param right    是否是右对齐
     * @return 对齐后的字符串
     */
    private static String just(CharSequence original, int length, char c, boolean right) {
        AssertUtils.arg(length > -1, () -> "length must be greater than -1");
        if (length == 0 || (notEmpty(original) && length <= original.length())) {
            return ObjectUtils.ifNull(original, Object::toString);
        }
        StringBuilder builder = new StringBuilder();

        // 左对齐
        if (!right && notEmpty(original)) {
            builder.append(original);
        }
        for (int i = 0, size = length - ObjectUtils.ifNull(original, CharSequence::length, () -> 0); i < size; i++) {
            builder.append(c);
        }

        // 右对齐
        if (right && notEmpty(original)) {
            builder.append(original);
        }
        return builder.toString();
    }

    /**
     * 以"_"或","号作为分割符拆分字符串，并对拆分后的字符串进行处理
     *
     * @param text    字符串文本
     * @param builder 对象构建器
     * @param parser  分隔符左右字符串解析器
     * @param <T>     分隔符左右数据类型
     * @param <R>     返回对象类型
     * @return 对象实例
     */
    public static <T, R> R decompose(String text, @NonNull MultipleFunction<T, T, Integer, R> builder,
                                     @NonNull Function<String, T> parser) {
        return decompose(text, builder, parser, (item, first) -> parser.apply(item));
    }

    /**
     * 以"_"或","号作为分割符拆分字符串，并对拆分后的字符串进行处理
     *
     * @param text        字符串文本
     * @param builder     对象构建器
     * @param leftParser  分隔符左边字符串解析器
     * @param rightParser 分隔符右边字符串解析器
     * @param <T>         分隔符左边数据类型
     * @param <U>         分隔符右边数据类型
     * @param <R>         返回对象类型
     * @return 对象实例
     */
    public static <T, U, R> R decompose(String text, @NonNull MultipleFunction<T, U, Integer, R> builder,
                                        @NonNull Function<String, T> leftParser,
                                        @NonNull BiFunction<String, T, U> rightParser) {
        if (isEmpty(text) || (text = text.trim()).isEmpty()
                || (text.charAt(0) == '(' && text.charAt(text.length() - 1) == ')'
                && (text = text.substring(1, text.length() - 1).trim()).isEmpty())) {
            return null;
        }

        // 获取分隔符下标位置
        int delimiter = -1;
        for (int i = 0, max = text.length(); i < max; i++) {
            char c = text.charAt(i);
            if (c == '_' || c == ',') {
                delimiter = i;
                break;
            }
        }

        // 字符串解析
        String item;
        T left = null;
        U right = null;
        if (delimiter < 0) {
            left = leftParser.apply(text);
        } else if (delimiter == 0 && !(item = text.substring(1).trim()).isEmpty()) {
            right = rightParser.apply(item, null);
        } else if (delimiter == text.length() - 1 && !(item = text.substring(0, text.length() - 1).trim()).isEmpty()) {
            left = leftParser.apply(item);
        } else if (delimiter > 0 && delimiter < text.length() - 1) {
            if (!(item = text.substring(0, delimiter).trim()).isEmpty()) {
                left = leftParser.apply(item);
            }
            if (!(item = text.substring(delimiter + 1).trim()).isEmpty()) {
                right = rightParser.apply(item, left);
            }
        }
        return builder.apply(left, right, delimiter);
    }

    /**
     * 字符串拼接
     *
     * @param values 对象数组
     * @param <T>    对象类型
     * @return 字符串
     */
    public static <T> String join(@NonNull T[] values) {
        return join(Stream.of(values), EMPTY, Object::toString);
    }

    /**
     * 字符串拼接
     *
     * @param values     对象数组
     * @param comparator 排序处理回调接口
     * @param <T>        对象类型
     * @return 字符串
     */
    public static <T> String join(@NonNull T[] values, @NonNull Comparator<String> comparator) {
        return join(Stream.of(values), EMPTY, Object::toString, comparator);
    }

    /**
     * 字符串拼接
     *
     * @param values    对象数组
     * @param delimiter 字符串链接符号
     * @param <T>       对象类型
     * @return 字符串
     */
    public static <T> String join(@NonNull T[] values, @NonNull CharSequence delimiter) {
        return join(Stream.of(values), delimiter, Object::toString);
    }

    /**
     * 字符串拼接
     *
     * @param values     对象数组
     * @param delimiter  字符串链接符号
     * @param comparator 排序处理回调接口
     * @param <T>        对象类型
     * @return 字符串
     */
    public static <T> String join(@NonNull T[] values, @NonNull CharSequence delimiter,
                                  @NonNull Comparator<String> comparator) {
        return join(Stream.of(values), delimiter, Object::toString, comparator);
    }

    /**
     * 字符串拼接
     *
     * @param values  对象数组
     * @param adapter 对象字符串适配函数
     * @param <T>     对象类型
     * @return 字符串
     */
    public static <T> String join(@NonNull T[] values, @NonNull Function<T, String> adapter) {
        return join(Stream.of(values), EMPTY, adapter);
    }

    /**
     * 字符串拼接
     *
     * @param values     对象数组
     * @param adapter    对象字符串适配函数
     * @param comparator 排序处理回调接口
     * @param <T>        对象类型
     * @return 字符串
     */
    public static <T> String join(@NonNull T[] values, @NonNull Function<T, String> adapter,
                                  @NonNull Comparator<String> comparator) {
        return join(Stream.of(values), EMPTY, adapter, comparator);
    }

    /**
     * 字符串拼接
     *
     * @param values    对象数组
     * @param delimiter 字符串链接符号
     * @param adapter   对象字符串适配函数
     * @param <T>       对象类型
     * @return 字符串
     */
    public static <T> String join(@NonNull T[] values, @NonNull CharSequence delimiter,
                                  @NonNull Function<T, String> adapter) {
        return join(Stream.of(values), delimiter, adapter);
    }

    /**
     * 字符串拼接
     *
     * @param values     对象数组
     * @param delimiter  字符串链接符号
     * @param adapter    对象字符串适配函数
     * @param comparator 排序处理回调接口
     * @param <T>        对象类型
     * @return 字符串
     */
    public static <T> String join(@NonNull T[] values, @NonNull CharSequence delimiter,
                                  @NonNull Function<T, String> adapter, @NonNull Comparator<String> comparator) {
        return join(Stream.of(values), delimiter, adapter, comparator);
    }

    /**
     * 字符串拼接
     *
     * @param values 对象集合
     * @return 字符串
     */
    public static String join(@NonNull Collection<?> values) {
        return join(values.stream(), EMPTY, Object::toString);
    }

    /**
     * 字符串拼接
     *
     * @param values     对象集合
     * @param comparator 排序处理回调接口
     * @return 字符串
     */
    public static String join(@NonNull Collection<?> values, @NonNull Comparator<String> comparator) {
        return join(values.stream(), EMPTY, Object::toString, comparator);
    }

    /**
     * 字符串拼接
     *
     * @param values    对象集合
     * @param delimiter 字符串链接符号
     * @return 字符串
     */
    public static String join(@NonNull Collection<?> values, @NonNull CharSequence delimiter) {
        return join(values.stream(), delimiter, Object::toString);
    }

    /**
     * 字符串拼接
     *
     * @param values     对象集合
     * @param delimiter  字符串链接符号
     * @param comparator 排序处理回调接口
     * @return 字符串
     */
    public static String join(@NonNull Collection<?> values, @NonNull CharSequence delimiter,
                              @NonNull Comparator<String> comparator) {
        return join(values.stream(), delimiter, Object::toString, comparator);
    }

    /**
     * 字符串拼接
     *
     * @param values  对象集合
     * @param adapter 对象字符串适配函数
     * @param <T>     对象类型
     * @return 字符串
     */
    public static <T> String join(@NonNull Collection<T> values, @NonNull Function<T, String> adapter) {
        return join(values.stream(), EMPTY, adapter);
    }

    /**
     * 字符串拼接
     *
     * @param values     对象集合
     * @param adapter    对象字符串适配函数
     * @param comparator 排序处理回调接口
     * @param <T>        对象类型
     * @return 字符串
     */
    public static <T> String join(@NonNull Collection<T> values, @NonNull Function<T, String> adapter,
                                  @NonNull Comparator<String> comparator) {
        return join(values.stream(), EMPTY, adapter, comparator);
    }

    /**
     * 字符串拼接
     *
     * @param values    对象集合
     * @param delimiter 字符串链接符号
     * @param adapter   对象字符串适配函数
     * @param <T>       对象类型
     * @return 字符串
     */
    public static <T> String join(@NonNull Collection<T> values, @NonNull CharSequence delimiter,
                                  @NonNull Function<T, String> adapter) {
        return join(values.stream(), delimiter, adapter);
    }

    /**
     * 字符串拼接
     *
     * @param values     对象集合
     * @param delimiter  字符串链接符号
     * @param adapter    对象字符串适配函数
     * @param comparator 排序处理回调接口
     * @param <T>        对象类型
     * @return 字符串
     */
    public static <T> String join(@NonNull Collection<T> values, @NonNull CharSequence delimiter,
                                  @NonNull Function<T, String> adapter, @NonNull Comparator<String> comparator) {
        return join(values.stream(), delimiter, adapter, comparator);
    }

    /**
     * 字符串拼接
     *
     * @param stream    对象数据流
     * @param delimiter 字符串链接符号
     * @param adapter   对象字符串适配函数
     * @param <T>       对象类型
     * @return 字符串
     */
    private static <T> String join(@NonNull Stream<T> stream, @NonNull CharSequence delimiter,
                                   @NonNull Function<T, String> adapter) {
        return stream.filter(StringUtils::notEmpty).map(adapter).filter(Objects::nonNull)
                .collect(Collectors.joining(delimiter));
    }

    /**
     * 字符串拼接
     *
     * @param stream     对象数据流
     * @param delimiter  字符串链接符号
     * @param adapter    对象字符串适配函数
     * @param comparator 排序处理回调接口
     * @param <T>        对象类型
     * @return 字符串
     */
    private static <T> String join(@NonNull Stream<T> stream, @NonNull CharSequence delimiter,
                                   @NonNull Function<T, String> adapter, @NonNull Comparator<String> comparator) {
        return stream.filter(StringUtils::notEmpty).map(adapter).filter(Objects::nonNull).sorted(comparator)
                .collect(Collectors.joining(delimiter));
    }

    /**
     * 字符串拼接
     *
     * @param mapping 键/值映射表
     * @param <K>     键类型
     * @param <V>     值类型
     * @return 字符串
     */
    public static <K extends Comparable<? super K>, V> String join(@NonNull Map<K, V> mapping) {
        return join(mapping, EMPTY);
    }

    /**
     * 字符串拼接
     *
     * @param mapping   键/值映射表
     * @param delimiter 字符串链接符号
     * @param <K>       键类型
     * @param <V>       值类型
     * @return 字符串
     */
    public static <K extends Comparable<? super K>, V> String join(@NonNull Map<K, V> mapping,
                                                                   @NonNull CharSequence delimiter) {
        return join(mapping, delimiter, (BiFunction<K, V, String>) (key, value) -> key + EQUAL + ifEmpty(value, EMPTY));
    }

    /**
     * 字符串拼接
     *
     * @param mapping 键/值映射表
     * @param adapter 对象字符串适配函数
     * @param <K>     键类型
     * @param <V>     值类型
     * @return 字符串
     */
    public static <K extends Comparable<? super K>, V> String join(@NonNull Map<K, V> mapping,
                                                                   @NonNull BiFunction<K, V, String> adapter) {
        return join(mapping, EMPTY, adapter);
    }

    /**
     * 字符串拼接
     *
     * @param mapping    键/值映射表
     * @param comparator 排序处理回调接口
     * @param <K>        键类型
     * @param <V>        值类型
     * @return 字符串
     */
    public static <K extends Comparable<? super K>, V> String join(@NonNull Map<K, V> mapping,
                                                                   @NonNull Comparator<? super K> comparator) {
        return join(mapping, EMPTY, comparator);
    }

    /**
     * 字符串拼接
     *
     * @param mapping    键/值映射表
     * @param adapter    对象字符串适配函数
     * @param comparator 排序处理回调接口
     * @param <K>        键类型
     * @param <V>        值类型
     * @return 字符串
     */
    public static <K extends Comparable<? super K>, V> String join(@NonNull Map<K, V> mapping,
                                                                   @NonNull BiFunction<K, V, String> adapter,
                                                                   @NonNull Comparator<? super K> comparator) {
        return join(mapping, EMPTY, adapter, comparator);
    }

    /**
     * 字符串拼接
     *
     * @param mapping   键/值映射表
     * @param delimiter 字符串链接符号
     * @param adapter   对象字符串适配函数
     * @param <K>       键类型
     * @param <V>       值类型
     * @return 字符串
     */
    public static <K extends Comparable<? super K>, V> String join(@NonNull Map<K, V> mapping,
                                                                   @NonNull CharSequence delimiter,
                                                                   @NonNull BiFunction<K, V, String> adapter) {
        if (ObjectUtils.isEmpty(mapping)) {
            return EMPTY;
        }
        return mapping.entrySet().stream().filter(entry -> notEmpty(entry.getKey()))
                .map(entry -> adapter.apply(entry.getKey(), entry.getValue())).collect(Collectors.joining(delimiter));
    }

    /**
     * 字符串拼接
     *
     * @param mapping    键/值映射表
     * @param delimiter  字符串链接符号
     * @param comparator 排序处理回调接口
     * @param <K>        键类型
     * @param <V>        值类型
     * @return 字符串
     */
    public static <K extends Comparable<? super K>, V> String join(@NonNull Map<K, V> mapping,
                                                                   @NonNull CharSequence delimiter,
                                                                   @NonNull Comparator<? super K> comparator) {
        return join(mapping, delimiter, (key, value) -> key + EQUAL + ifEmpty(value, EMPTY), comparator);
    }

    /**
     * 字符串拼接
     *
     * @param mapping    键/值映射表
     * @param delimiter  字符串链接符号
     * @param adapter    对象字符串适配函数
     * @param comparator 排序处理回调接口
     * @param <K>        键类型
     * @param <V>        值类型
     * @return 字符串
     */
    public static <K extends Comparable<? super K>, V> String join(@NonNull Map<K, V> mapping,
                                                                   @NonNull CharSequence delimiter,
                                                                   @NonNull BiFunction<K, V, String> adapter,
                                                                   @NonNull Comparator<? super K> comparator) {
        if (ObjectUtils.isEmpty(mapping)) {
            return EMPTY;
        }
        return mapping.keySet().stream().filter(StringUtils::notEmpty).sorted(comparator)
                .map(key -> adapter.apply(key, mapping.get(key))).collect(Collectors.joining(delimiter));
    }

    /**
     * 根据正则表达式从文本中查找内容
     *
     * @param text  文本内容
     * @param regex 正则表达式
     * @return 查找内容
     */
    public static String find(String text, @NonNull String regex) {
        if (isEmpty(text)) {
            return null;
        }
        Matcher matcher = getPattern(regex).matcher(text);
        return matcher.find() ? matcher.group() : null;
    }

    /**
     * 根据正则表达式查找内容
     *
     * @param text     文本内容
     * @param regex    正则表达式
     * @param consumer 查找结果处理函数
     */
    public static void find(String text, @NonNull String regex, @NonNull Consumer<String> consumer) {
        if (notEmpty(text)) {
            Matcher matcher = getPattern(regex).matcher(text);
            while (matcher.find()) {
                consumer.accept(matcher.group());
            }
        }
    }

    /**
     * 根据正则表达式查找内容
     *
     * @param text     文本内容
     * @param regex    正则表达式
     * @param consumer 查找结果处理函数，返回true则中断循环
     */
    public static void find(String text, @NonNull String regex, @NonNull Predicate<String> consumer) {
        if (notEmpty(text)) {
            Matcher matcher = getPattern(regex).matcher(text);
            while (matcher.find()) {
                if (consumer.test(matcher.group())) {
                    break;
                }
            }
        }
    }

    /**
     * 清理数字
     *
     * @param original 原始信息
     * @return 清理后的信息
     */
    private static String clearNumber(String original) {
        if (isEmpty(original)) {
            return original;
        }

        Matcher matcher = getPattern(NUMBER_LINK_REGEX).matcher(original);
        while (matcher.find()) {
            String group = matcher.group();
            original = getPattern(group).matcher(original).replaceAll(clear(group, LINK_REGEX));
        }
        return original;
    }

    /**
     * 清理文本
     *
     * @param original 原始信息
     * @return 清理后的信息
     */
    public static String clear(String original) {
        if (isEmpty(original)) {
            return original;
        }

        // 过滤表情符号
        original = clear(original, EMOJI_REGEX);

        // 过滤特殊符号
        original = clear(original, PLAINTEXT_REGEX);

        // 过滤非数字符号
        original = clear(original, NON_NUMBER_SYMBOL_REGEX);

        // 过滤数字链接符号
        try {
            return clearNumber(original);
        } catch (StackOverflowError e) {
            // 堆栈溢出后分段重试，当设置 -Xss160K时，文本长度超过600堆栈溢出，所以取值500
            int size = 500, max = original.length();
            StringBuilder buffer = new StringBuilder(max);
            int group = (int) Math.ceil((double) max / size);
            for (int i = 0; i < group; i++) {
                int begin = i * size, end = Math.min((i + 1) * size, max);
                buffer.append(clearNumber(original.substring(begin, end)));
            }
            return buffer.toString();
        }
    }

    /**
     * 清理文本
     *
     * @param original 原始信息
     * @param regex    清理目标字符串正则表达式
     * @return 清理后的信息
     */
    public static String clear(String original, @NonNull String regex) {
        return isEmpty(original) ? original : getPattern(regex).matcher(original).replaceAll(EMPTY);
    }

    /**
     * 字符串转驼峰格式
     *
     * @param original 原始字符串
     * @return 转换后字符串
     */
    public static String hump(String original) {
        if (isEmpty(original)) {
            return original;
        }
        char c = original.charAt(0);
        if (Character.isLowerCase(c)) {
            return original;
        } else if (original.length() == 1) {
            return original.toLowerCase();
        }
        return Character.toLowerCase(c) + original.substring(1);
    }

    /**
     * 驼峰格式转下划线格式
     *
     * @param original 原始字符串
     * @return 下划线格式字符串
     */
    public static String hump2underscore(String original) {
        if (isEmpty(original)) {
            return original;
        }
        PropertyNamingStrategy.SnakeCaseStrategy strategy =
                (PropertyNamingStrategy.SnakeCaseStrategy) PropertyNamingStrategy.SNAKE_CASE;
        return strategy.translate(original);
    }

    /**
     * 下划线格式转驼峰格式
     *
     * @param original 原始字符串
     * @return 驼峰格式字符串
     */
    public static String underscore2hump(String original) {
        if (isEmpty(original)) {
            return original;
        }

        // 获取第一个下划线字符位置
        int delimiter = original.indexOf('_');
        if (delimiter < 0) {
            return original;
        }

        // 计算驼峰字符串开始、结束字符位置
        int begin = 0, end = original.length();
        if (delimiter == 0) {
            // 过滤左边连续的下划线字符
            for (begin = 1; begin < end; begin++) {
                if (original.charAt(begin) != '_') {
                    break;
                }
            }
        }
        // 过滤右边连续的下划线字符
        for (; ; end--) {
            if (original.charAt(end - 1) != '_') {
                break;
            }
        }

        // 拼接驼峰格式字符串
        StringBuilder buffer = new StringBuilder(end - begin);
        buffer.append(Character.toLowerCase(original.charAt(begin++)));
        while ((delimiter = original.indexOf('_', begin)) > 0 && delimiter < end) {
            buffer.append(original, begin, delimiter);
            buffer.append(Character.toUpperCase(original.charAt(begin = delimiter + 1)));
            begin++;
        }
        if (begin < end) {
            buffer.append(original, begin, end);
        }
        return buffer.toString();
    }

    /**
     * 省略字符串多余的字符
     *
     * @param original 原始字符串
     * @param length   字符串最大长度
     * @return 省略后的字符串
     */
    public static String ellipsis(String original, int length) {
        if (isEmpty(original) || length < 1 || original.length() <= length) {
            return original;
        }
        return original.substring(0, length) + "...";
    }

    /**
     * 生成随机字符串
     *
     * @return 随机字符串
     */
    public static String random() {
        return random(DEFAULT_RANDOM_LENGTH);
    }

    /**
     * 生成随机字符串
     *
     * @param length 随机字符串长度
     * @return 随机字符串
     */
    public static String random(int length) {
        return random(BASIC_RANDOM_CHARS, length);
    }

    /**
     * 生成随机字符串
     *
     * @param chars 随机串字符数组
     * @return 随机字符串
     */
    public static String random(@NonNull char[] chars) {
        return random(chars, DEFAULT_RANDOM_LENGTH);
    }

    /**
     * 生成随机字符串
     *
     * @param chars  随机串字符数组
     * @param length 随机字符串长度
     * @return 随机字符串
     */
    public static String random(@NonNull char[] chars, int length) {
        AssertUtils.arg(chars.length > 0, () -> "chars must not be empty");
        AssertUtils.arg(length > 0, () -> "length must be greater than 0");
        char[] array = new char[length];
        ThreadLocalRandom random = ThreadLocalRandom.current();
        for (int i = 0; i < length; i++) {
            array[i] = chars[random.nextInt(chars.length)];
        }
        return new String(array);
    }

    /**
     * 生成密钥字符串
     *
     * @return 密钥字符串
     */
    public static String secret() {
        return secret(DEFAULT_SECRET_LENGTH);
    }

    /**
     * 生成密钥字符串
     *
     * @param length 密钥长度
     * @return 密钥字符串
     */
    public static String secret(int length) {
        return random(STANDARD_RANDOM_CHARS, length);
    }

    /**
     * 生成密钥字符串
     *
     * @param chars 密钥字符数组
     * @return 密钥字符串
     */
    public static String secret(@NonNull char[] chars) {
        return random(chars, DEFAULT_SECRET_LENGTH);
    }

    /**
     * 获取UUID字符串
     *
     * @return UUID字符串
     */
    public static String uuid() {
        return uuid(false);
    }

    /**
     * 获取UUID字符串
     *
     * @param horizontal 是否包含横线
     * @return UUID字符串
     */
    public static String uuid(boolean horizontal) {
        String uuid = UUID.randomUUID().toString();
        if (horizontal) {
            return uuid;
        }
        char skip = HORIZONTAL.charAt(0);
        char[] chars = new char[uuid.length() - 4];
        for (int i = 0, n = 0, size = uuid.length(); i < size; i++) {
            char c = uuid.charAt(i);
            if (c != skip) {
                chars[n++] = c;
            }
        }
        return new String(chars);
    }

    /**
     * 生成随机序列号
     *
     * @param length 序列号长度
     * @return 随机序列号
     */
    public static String sequence(int length) {
        AssertUtils.arg(length > 0, () -> "length must be greater than 0");
        char[] sequence = new char[length];
        for (int i = 0, count = ObjectUtils.group(length, 8); i < count; i++) {
            String uuid = uuid();
            for (int j = 0, n = i * 8 + j; j < 8 && n < length; j++, n++) {
                int number = Integer.parseInt(uuid.substring(j * 4, j * 4 + 4), 16);
                sequence[n] = BASIC_RANDOM_CHARS[number % BASIC_RANDOM_CHARS.length];
            }
        }
        return new String(sequence);
    }

    /**
     * 计算字符串CRC值
     *
     * @param source 源字符串
     * @return CRC值
     */
    public static long crc32(String source) {
        if (isEmpty(source)) {
            return 0;
        }
        CRC32 crc = new CRC32();
        crc.update(source.getBytes());
        return crc.getValue();
    }

    /**
     * 判断字符串是否匹配真实前缀（忽略空格）
     *
     * @param sequence 字符串
     * @param prefix   匹配前缀
     * @return true/false
     */
    public static boolean isRealPrefix(CharSequence sequence, @NonNull String prefix) {
        if (sequence == null) {
            return false;
        } else if (isBlank(prefix)) {
            return true;
        }

        int i = 0, size = sequence.length();
        while (i < size && sequence.charAt(i) <= ' ') {
            i++;
        }

        if (size - i < prefix.length()) {
            return false;
        }

        for (int k = 0, c = prefix.length(); k < c; k++) {
            if (sequence.charAt(i + k) != prefix.charAt(k)) {
                return false;
            }
        }
        return true;
    }

    /**
     * 判断字符串是否包含真实前缀（忽略空格）
     *
     * @param sequence 字符串
     * @param prefixes 前缀数组
     * @return true/false
     */
    public static boolean hasRealPrefix(CharSequence sequence, @NonNull String... prefixes) {
        if (sequence != null && ObjectUtils.notEmpty(prefixes)) {
            for (String prefix : prefixes) {
                if (isRealPrefix(sequence, prefix)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 判断字符串是否匹配真实后缀（忽略空格）
     *
     * @param sequence 字符串
     * @param suffix   匹配后缀
     * @return true/false
     */
    public static boolean isRealSuffix(CharSequence sequence, @NonNull String suffix) {
        if (sequence == null) {
            return false;
        } else if (isBlank(suffix)) {
            return true;
        }

        int size = sequence.length();
        while (size > 0 && sequence.charAt(size - 1) <= ' ') {
            size--;
        }

        if (size < suffix.length()) {
            return false;
        }

        for (int i = size - 1, k = suffix.length() - 1; k >= 0; k--, i--) {
            if (sequence.charAt(i) != suffix.charAt(k)) {
                return false;
            }
        }
        return true;
    }

    /**
     * 判断字符串是否包含真实后缀（忽略空格）
     *
     * @param sequence 字符串
     * @param suffixes 后缀数组
     * @return true/false
     */
    public static boolean hasRealSuffix(CharSequence sequence, @NonNull String... suffixes) {
        if (sequence != null && ObjectUtils.notEmpty(suffixes)) {
            for (String suffix : suffixes) {
                if (isRealSuffix(sequence, suffix)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 将对象转换成字符串
     *
     * @param object 对象实例
     * @return 字符串
     */
    public static String toString(Object object) {
        return object == null ? null : object.toString();
    }
}
