package com.chelvc.framework.common.util;

import java.io.Serializable;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Predicate;
import java.util.stream.Collectors;

import com.chelvc.framework.common.model.Tree;
import com.google.common.collect.Lists;
import lombok.NonNull;

/**
 * 树工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class TreeUtils {
    private TreeUtils() {
    }

    /**
     * 树节点组装，自动关联树节点并返回根节点列表
     *
     * @param trees 树对象实例列表
     * @param <T>   树对象类型
     * @return 树根节点列表
     */
    public static <T extends Tree<?, T>> List<T> assemble(Collection<T> trees) {
        if (ObjectUtils.isEmpty(trees)) {
            return Collections.emptyList();
        }

        // 构建树主键/实例映射表，并初始化树的子节点集合
        Map<?, T> mapping = trees.stream().peek(tree -> tree.setChildren(new LinkedList<>()))
                .collect(Collectors.toMap(Tree::getId, t -> t, (o, n) -> n));

        // 查找并关联树节点，返回所有没有父节点的树
        return trees.stream().filter(tree -> {
            T parent = ObjectUtils.ifNull(tree.getParentId(), mapping::get);
            if (parent != null) {
                parent.getChildren().add(tree);
            }
            return Objects.isNull(parent);
        }).collect(Collectors.toList());
    }

    /**
     * 分散树节点
     *
     * @param trees 树对象实例列表
     * @param <T>   树对象类型
     * @return 树根节点列表
     */
    public static <T extends Tree<?, T>> List<T> disperse(Collection<T> trees) {
        if (ObjectUtils.isEmpty(trees)) {
            return Collections.emptyList();
        }
        List<T> disperses = Lists.newLinkedList();
        iterate(trees, (Consumer<T>) disperses::add);
        return disperses;
    }

    /**
     * 遍历树节点
     *
     * @param trees    树对象列表
     * @param consumer 树节点处理回调方法
     * @param <T>      树对象类型
     */
    public static <T extends Tree<?, T>> void iterate(Collection<T> trees, @NonNull Consumer<T> consumer) {
        if (ObjectUtils.notEmpty(trees)) {
            trees.forEach(tree -> {
                consumer.accept(tree);
                iterate(ObjectUtils.ifNull(tree, Tree::getChildren), consumer);
            });
        }
    }

    /**
     * 遍历树节点
     *
     * @param trees    树对象列表
     * @param consumer 树节点处理回调方法
     * @param <T>      树对象类型
     */
    public static <T extends Tree<?, T>> void iterate(Collection<T> trees,
                                                      @NonNull BiConsumer<T, LinkedList<T>> consumer) {
        if (ObjectUtils.notEmpty(trees)) {
            iterate(trees, Lists.newLinkedList(), consumer);
        }
    }

    /**
     * 遍历树节点
     *
     * @param trees    树对象列表
     * @param parents  上级节点列表
     * @param consumer 树节点处理回调方法
     * @param <T>      树对象类型
     */
    private static <T extends Tree<?, T>> void iterate(@NonNull Collection<T> trees, @NonNull LinkedList<T> parents,
                                                       @NonNull BiConsumer<T, LinkedList<T>> consumer) {
        trees.forEach(tree -> {
            consumer.accept(tree, parents);
            List<T> children = ObjectUtils.ifNull(tree, Tree::getChildren);
            if (ObjectUtils.notEmpty(children)) {
                parents.add(tree);
                iterate(children, parents, consumer);
                parents.removeLast();
            }
        });
    }

    /**
     * 查找树节点
     *
     * @param tree 树对象实例
     * @param id   树节点ID
     * @param <ID> ID类型
     * @param <T>  树节点类型
     * @return 树节点
     */
    public static <ID extends Serializable, T extends Tree<ID, T>> T lookup(T tree, @NonNull ID id) {
        return lookup(tree, (Predicate<T>) t -> Objects.equals(t.getId(), id));
    }

    /**
     * 查找树节点
     *
     * @param trees 树对象实例集合
     * @param id    树节点ID
     * @param <ID>  ID类型
     * @param <T>   树节点类型
     * @return 树节点
     */
    public static <ID extends Serializable, T extends Tree<ID, T>> T lookup(Collection<T> trees, @NonNull ID id) {
        return lookup(trees, (Predicate<T>) t -> Objects.equals(t.getId(), id));
    }

    /**
     * 查找树节点
     *
     * @param tree   树对象实例
     * @param filter 树过滤函数
     * @param <T>    树节点类型
     * @return 树节点
     */
    public static <T extends Tree<?, T>> T lookup(T tree, @NonNull Predicate<T> filter) {
        return tree == null || filter.test(tree) ? tree : lookup(tree.getChildren(), filter);
    }

    /**
     * 查找树节点
     *
     * @param trees  树对象实例集合
     * @param filter 树过滤函数
     * @param <T>    树节点类型
     * @return 树节点
     */
    public static <T extends Tree<?, T>> T lookup(Collection<T> trees, @NonNull Predicate<T> filter) {
        if (ObjectUtils.notEmpty(trees)) {
            for (T tree : trees) {
                if (filter.test(tree)) {
                    return tree;
                }
                T t = lookup(tree.getChildren(), filter);
                if (Objects.nonNull(t)) {
                    return t;
                }
            }
        }
        return null;
    }
}
