package com.chelvc.framework.database.config;

import java.io.InputStream;
import java.io.Reader;
import java.lang.reflect.Constructor;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.time.Instant;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.Month;
import java.time.OffsetDateTime;
import java.time.OffsetTime;
import java.time.Year;
import java.time.YearMonth;
import java.time.ZonedDateTime;
import java.time.chrono.JapaneseDate;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import com.baomidou.mybatisplus.core.config.GlobalConfig;
import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.util.SpringUtils;
import com.chelvc.framework.common.model.File;
import com.chelvc.framework.common.model.Modification;
import com.chelvc.framework.common.model.Period;
import com.chelvc.framework.common.model.Region;
import com.chelvc.framework.common.util.ObjectUtils;
import com.google.common.collect.Lists;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtMethod;
import javassist.CtNewMethod;
import javassist.LoaderClassPath;
import javassist.NotFoundException;
import lombok.NonNull;
import org.apache.ibatis.parsing.XNode;
import org.apache.ibatis.session.Configuration;
import org.apache.ibatis.type.UnknownTypeHandler;

/**
 * Mybatis配置类
 *
 * @author Woody
 * @date 2024/4/6
 */
public abstract class MybatisConfigurer {
    /**
     * 拦截器实例列表
     */
    private static final ArrayList<MybatisConfigurer> INSTANCES = Lists.newArrayList();

    protected final Configuration configuration;

    public MybatisConfigurer(@NonNull Configuration configuration) {
        this.configuration = configuration;
    }

    /**
     * 初始化配置拦截器
     *
     * @param configuration MyBatis配置信息
     */
    public static void initialize(Configuration configuration) {
        try {
            ClassPool pool = ClassPool.getDefault();
            pool.appendClassPath(new LoaderClassPath(Thread.currentThread().getContextClassLoader()));
            initializeMapperListener(pool);
            initializeTableInfoListener(pool);
            initializeUnknownTypeHandler(pool);
            INSTANCES.addAll(initializeInterceptorInstances(configuration));
            INSTANCES.trimToSize();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 判断对象类型处理器是否已注册
     *
     * @param type 对象类型
     * @return true/false
     */
    public static boolean isTypeHandlerRegistered(Class<?> type) {
        return type == byte.class || type == Byte.class || type == byte[].class || type == Byte[].class
                || type == char.class || type == Character.class || type == short.class || type == Short.class
                || type == int.class || type == Integer.class || type == long.class || type == Long.class
                || type == float.class || type == Float.class || type == double.class || type == Double.class
                || type == boolean.class || type == Boolean.class || type == String.class || type == Object.class
                || type == BigInteger.class || type == BigDecimal.class || type == Reader.class
                || type == InputStream.class || type == Date.class || type == java.sql.Date.class
                || type == java.sql.Time.class || type == java.sql.Timestamp.class || type == Instant.class
                || type == LocalDate.class || type == LocalTime.class || type == LocalDateTime.class
                || type == OffsetTime.class || type == OffsetDateTime.class || type == ZonedDateTime.class
                || type == Year.class || type == Month.class || type == YearMonth.class || type == JapaneseDate.class
                || (type != null && type.isEnum()) || type == File.class || type == Period.class
                || type == Region.class || type == Modification.class;
    }

    /**
     * 初始化Mapper解析监听器
     *
     * @param pool 类对象池
     * @throws Exception 初始化异常
     */
    private static void initializeMapperListener(ClassPool pool) throws Exception {
        // 替换XMLMapperBuilder.parse方法逻辑，加入Mapper解析监听回调逻辑
        CtClass clazz = pool.get("org.apache.ibatis.builder.xml.XMLMapperBuilder");
        CtMethod method = clazz.getDeclaredMethod("parse");
        method.setBody(String.format("{\n" +
                "if (!this.configuration.isResourceLoaded(this.resource)) {\n" +
                "   %s node = this.parser.evalNode(\"/mapper\");\n" +
                "   %s.configure(node);\n" +
                "   this.configurationElement(node);\n" +
                "   this.configuration.addLoadedResource(this.resource);\n" +
                "   this.bindMapperForNamespace();\n" +
                "}\n" +
                "this.parsePendingResultMaps();\n" +
                "this.parsePendingCacheRefs();\n" +
                "this.parsePendingStatements();\n" +
                "}", XNode.class.getName(), MybatisConfigurer.class.getName()
        ));
        clazz.toClass();
    }

    /**
     * 初始化数据模型表信息监听器
     *
     * @param pool 类对象池
     * @throws Exception 初始化异常
     */
    private static void initializeTableInfoListener(ClassPool pool) throws Exception {
        // 获取原始initTableFields方法
        CtMethod method = null;
        CtClass clazz = pool.get("com.baomidou.mybatisplus.core.metadata.TableInfoHelper");
        for (CtMethod m : clazz.getDeclaredMethods()) {
            if (m.getName().equals("initTableFields")) {
                method = m;
                break;
            }
        }
        if (method == null) {
            throw new NotFoundException("initTableFields");
        }

        // 重命名initTableFields方法名
        CtMethod copy = CtNewMethod.copy(method, clazz, null);
        copy.setName("doInitTableFields");
        clazz.removeMethod(method);
        clazz.addMethod(copy);

        // 替换initTableFields方法，加入表信息初始化回调逻辑
        String replace = String.format("private static void initTableFields(Class clazz, %s config, %s table, " +
                        "java.util.List excludes) {\n doInitTableFields(clazz, config, table, excludes);\n" +
                        "%s.configure(table);\n}", GlobalConfig.class.getName(), TableInfo.class.getName(),
                MybatisConfigurer.class.getName());
        clazz.addMethod(CtNewMethod.make(replace, clazz));
        clazz.toClass();
    }

    /**
     * 初始化对象未知类型处理器实例
     *
     * @param pool 类对象池
     * @throws Exception 初始化异常
     */
    private static void initializeUnknownTypeHandler(ClassPool pool) throws Exception {
        CtClass clazz = pool.get("org.apache.ibatis.mapping.ParameterMapping");
        CtClass inner = clazz.getNestedClasses()[1];
        CtMethod method = inner.getDeclaredMethod("build");
        method.setBody(String.format("{\n" +
                "this.resolveTypeHandler();\n" +
                "if (this.parameterMapping.typeHandler == null && this.parameterMapping.javaType != null " +
                "&& !%s.isTypeHandlerRegistered(this.parameterMapping.javaType)) {\n" +
                "this.parameterMapping.typeHandler = new %s(this.parameterMapping.configuration);\n" +
                "}\n " +
                "this.validate();\n" +
                "return this.parameterMapping;\n" +
                "}", MybatisConfigurer.class.getName(), UnknownTypeHandler.class.getName()));
        inner.toClass();
        clazz.toClass();
    }

    /**
     * 初始化Mybatis配置拦截器实例
     *
     * @param configuration MyBatis配置信息
     * @return Mybatis配置拦截器列表
     * @throws Exception 初始化异常
     */
    private static List<MybatisConfigurer> initializeInterceptorInstances(Configuration configuration)
            throws Exception {
        // 查找当前包目录下MybatisConfigureInterceptor实现
        List<Class<?>> classes = SpringUtils.getClasses(
                MybatisConfigurer.class.getPackage().getName(),
                clazz -> clazz != MybatisConfigurer.class && MybatisConfigurer.class.isAssignableFrom(clazz)
        );
        if (ObjectUtils.isEmpty(classes)) {
            return Collections.emptyList();
        }

        // 基于@Order注解对拦截器对象排序
        ApplicationContextHolder.sort(classes);

        // 初始化对象实例
        List<MybatisConfigurer> interceptors = Lists.newArrayListWithCapacity(classes.size());
        for (Class<?> clazz : classes) {
            Constructor<?> constructor = clazz.getConstructor(Configuration.class);
            interceptors.add((MybatisConfigurer) constructor.newInstance(configuration));
        }
        return interceptors;
    }

    /**
     * 配置Mapper XML节点
     *
     * @param mapper Mapper XML节点
     */
    public static void configure(@NonNull XNode mapper) {
        INSTANCES.forEach(interceptor -> interceptor.intercept(mapper));
    }

    /**
     * 配置数据模型表信息
     *
     * @param table 数据模型表信息
     */
    public static void configure(@NonNull TableInfo table) {
        INSTANCES.forEach(interceptor -> interceptor.intercept(table));
    }

    /**
     * 拦截Mapper配置
     *
     * @param mapper Mapper XML节点
     */
    public void intercept(XNode mapper) {
    }

    /**
     * 拦截数据模型配置
     *
     * @param table 数据模型表信息
     */
    public void intercept(TableInfo table) {
    }
}
