package com.chelvc.framework.database.context;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collection;
import java.util.List;
import java.util.function.Function;
import java.util.function.Supplier;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.function.Executor;
import com.chelvc.framework.common.function.Handler;
import com.chelvc.framework.common.function.Provider;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.database.crypto.DatabaseCipherHandler;
import lombok.NonNull;
import org.mybatis.spring.SqlSessionTemplate;
import org.springframework.dao.DuplicateKeyException;

/**
 * 数据库操作相关上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class DatabaseContextHolder {
    /**
     * 事务处理器
     */
    private static volatile Transactor TRANSACTOR;

    /**
     * 全局SqlSessionTemplate实例
     */
    private static volatile SqlSessionTemplate SESSION_TEMPLATE;

    /**
     * 数据库加解密处理器
     */
    private static volatile DatabaseCipherHandler DATABASE_CIPHER_HANDLER;

    private DatabaseContextHolder() {
    }

    /**
     * 获取事务处理器
     *
     * @return 事务处理器
     */
    public static Transactor getTransactor() {
        if (TRANSACTOR == null) {
            synchronized (Transactor.class) {
                if (TRANSACTOR == null) {
                    TRANSACTOR = ApplicationContextHolder.getBean(Transactor.class);
                }
            }
        }
        return TRANSACTOR;
    }

    /**
     * 获取SqlSessionTemplate
     *
     * @return SqlSessionTemplate实例
     */
    public static SqlSessionTemplate getSessionTemplate() {
        if (SESSION_TEMPLATE == null) {
            synchronized (SqlSessionTemplate.class) {
                if (SESSION_TEMPLATE == null) {
                    SESSION_TEMPLATE = ApplicationContextHolder.getBean(SqlSessionTemplate.class);
                }
            }
        }
        return SESSION_TEMPLATE;
    }

    /**
     * 获取数据库加解密处理器
     *
     * @return 数据库加解密处理器实例
     */
    public static DatabaseCipherHandler getDatabaseCipherHandler() {
        if (DATABASE_CIPHER_HANDLER == null) {
            synchronized (DatabaseCipherHandler.class) {
                if (DATABASE_CIPHER_HANDLER == null) {
                    DATABASE_CIPHER_HANDLER = ApplicationContextHolder.getBean(DatabaseCipherHandler.class);
                }
            }
        }
        return DATABASE_CIPHER_HANDLER;
    }

    /**
     * 获取原生的数据库连接
     *
     * @return 数据库连接对象
     */
    public static Connection getConnection() {
        return getSessionTemplate().getConnection();
    }

    /**
     * 执行原生SQL
     *
     * @param sql SQL语句
     * @throws SQLException SQL执行异常
     */
    public static void execute(@NonNull String sql) throws SQLException {
        try (Statement statement = getConnection().createStatement()) {
            statement.execute(sql);
        }
    }

    /**
     * 批量执行原生SQL
     *
     * @param sqls SQL语句集合
     * @throws SQLException SQL执行异常
     */
    public static void execute(@NonNull Collection<String> sqls) throws SQLException {
        if (ObjectUtils.isEmpty(sqls)) {
            return;
        }
        try (Statement statement = getConnection().createStatement()) {
            for (String sql : sqls) {
                statement.addBatch(sql);
            }
            statement.executeBatch();
        }
    }

    /**
     * 执行原生查询SQL
     *
     * @param sql SQL语句
     * @return 查询结果集
     * @throws SQLException SQL执行异常
     */
    public static ResultSet query(@NonNull String sql) throws SQLException {
        try (Statement statement = getConnection().createStatement()) {
            return statement.executeQuery(sql);
        }
    }

    /**
     * 事务执行方法
     *
     * @param executor 回调方法
     */
    public static void transactional(@NonNull Executor executor) {
        getTransactor().execute(executor);
    }

    /**
     * 事务执行方法
     *
     * @param executor 回调方法
     * @param <T>      对象类型
     * @return 执行结果
     */
    public static <T> T transactional(@NonNull Supplier<T> executor) {
        return getTransactor().execute(executor);
    }

    /**
     * 事务执行方法
     *
     * @param executor 回调方法
     */
    public static void transactional(@NonNull Handler executor) throws Exception {
        getTransactor().execute(executor);
    }

    /**
     * 事务执行方法
     *
     * @param executor 回调方法
     * @param <T>      对象类型
     * @return 执行结果
     * @throws Exception 执行异常
     */
    public static <T> T transactional(@NonNull Provider<T> executor) throws Exception {
        return getTransactor().execute(executor);
    }

    /**
     * 数据更新或忽略
     *
     * @param function 数据更新执行函数
     * @return true/false
     */
    public static boolean modifyIgnore(@NonNull Executor function) {
        try {
            function.execute();
        } catch (DuplicateKeyException e) {
            return false;
        }
        return true;
    }

    /**
     * 数据更新或忽略
     *
     * @param function 数据更新执行函数
     * @return true/false
     */
    public static boolean modifyIgnore(@NonNull Supplier<Boolean> function) {
        try {
            return Boolean.TRUE.equals(function.get());
        } catch (DuplicateKeyException e) {
            return false;
        }
    }

    /**
     * 将分页请求参数转换成MyBatis Plus分页参数
     *
     * @param paging 分页请求参数
     * @param <T>    目标对象类型泛型
     * @return MyBatis Plus分页参数
     */
    public static <T> Page<T> page(Paging paging) {
        return paging == null ? null : page(paging.getNumber(), paging.getSize(), paging.isCounting());
    }

    /**
     * 将分页请求参数转换成MyBatis Plus分页参数
     *
     * @param paging   分页请求参数
     * @param counting 是否获取总数
     * @param <T>      目标对象类型泛型
     * @return MyBatis Plus分页参数
     */
    public static <T> Page<T> page(Paging paging, boolean counting) {
        return paging == null ? null : page(paging.getNumber(), paging.getSize(), counting);
    }

    /**
     * 构建分页信息
     *
     * @param number 页码
     * @param size   页面大小
     * @param <T>    数据类型
     * @return 分页信息
     */
    public static <T> Page<T> page(long number, long size) {
        return page(number, size, true);
    }

    /**
     * 构建分页信息
     *
     * @param number   页码
     * @param size     页面大小
     * @param counting 是否获取总数
     * @param <T>      数据类型
     * @return 分页信息
     */
    public static <T> Page<T> page(long number, long size, boolean counting) {
        return new Page<>(number, size, counting);
    }

    /**
     * 数据分页处理
     *
     * @param paging   分页信息
     * @param function 分页处理函数
     * @param <T>      数据类型
     * @return 分页信息
     */
    public static <T> Pagination<T> paging(@NonNull Paging paging, @NonNull Function<IPage<T>, IPage<T>> function) {
        IPage<T> page = function.apply(page(paging));
        return page == null || ObjectUtils.isEmpty(page.getRecords()) ? Pagination.empty() : pagination(page);
    }

    /**
     * 数据分页处理
     *
     * @param paging   分页信息
     * @param function 分页处理函数
     * @param adapter  数据转换适配器
     * @param <T>      原始数据类型
     * @param <R>      目标数据类型
     * @return 分页信息
     */
    public static <T, R> Pagination<R> paging(@NonNull Paging paging, @NonNull Function<IPage<T>, IPage<T>> function,
                                              @NonNull Function<List<T>, List<R>> adapter) {
        IPage<T> page = function.apply(page(paging));
        return page == null || ObjectUtils.isEmpty(page.getRecords()) ? Pagination.empty() : pagination(page, adapter);
    }

    /**
     * 将MyBatis Plus分页参数转换成分页返回参数
     *
     * @param page MyBatis Plus分页参数
     * @param <T>  目标对象类型泛型
     * @return 分页返回参数
     */
    public static <T> Pagination<T> pagination(IPage<T> page) {
        return pagination(page, (List<T>) ObjectUtils.ifNull(page, IPage::getRecords));
    }

    /**
     * 将MyBatis Plus分页参数转换成分页返回参数
     *
     * @param page MyBatis Plus分页参数
     * @param type 目标类型
     * @param <T>  目标对象类型泛型
     * @return 分页返回参数
     */
    public static <T> Pagination<T> pagination(IPage<?> page, @NonNull Class<T> type) {
        return pagination(page, ObjectUtils.copying((List<?>) ObjectUtils.ifNull(page, IPage::getRecords), type));
    }

    /**
     * 将MyBatis Plus分页参数转换成分页返回参数
     *
     * @param page    MyBatis Plus分页参数
     * @param adapter 对象适配函数
     * @param <T>     原始对象类型泛型
     * @param <R>     目标对象类型泛型
     * @return 分页返回参数
     */
    public static <T, R> Pagination<R> pagination(IPage<T> page, @NonNull Function<List<T>, List<R>> adapter) {
        List<T> records = ObjectUtils.ifNull(page, IPage::getRecords);
        return ObjectUtils.isEmpty(records) ? Pagination.empty() : pagination(page, adapter.apply(records));
    }

    /**
     * 将MyBatis Plus分页参数转换成分页返回参数
     *
     * @param page    MyBatis Plus分页参数
     * @param records 分页数据列表
     * @param <T>     目标对象类型泛型
     * @return 分页返回参数
     */
    public static <T> Pagination<T> pagination(IPage<?> page, List<T> records) {
        if (page == null || ObjectUtils.isEmpty(records)) {
            return Pagination.empty();
        }
        return Pagination.<T>builder().total(page.getTotal()).pages(page.getPages()).records(records).build();
    }
}