package com.chelvc.framework.database.interceptor;

import java.util.List;

import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.DynamicDataSourceAutoConfiguration;
import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.DynamicDataSourceProperties;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.beust.jcommander.internal.Lists;
import com.chelvc.framework.common.util.ObjectUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.aop.framework.AopProxyUtils;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;

/**
 * 动态数据源拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Aspect
@Order(Ordered.HIGHEST_PRECEDENCE)
@ConditionalOnClass(DynamicDataSourceAutoConfiguration.class)
public class DynamicDatasourceInterceptor {
    private final List<String> routes;

    public DynamicDatasourceInterceptor(DynamicDataSourceProperties properties) {
        this.routes = Lists.newArrayList(properties.getDatasource().keySet());
    }

    /**
     * 查找数据源标识
     *
     * @param point 方法拦截点
     * @return 数据源标识
     */
    private String lookupDatasourceKey(ProceedingJoinPoint point) {
        Object target = point.getTarget();

        // 根据目标类对象查找数据源标识
        String className = AopProxyUtils.ultimateTargetClass(target).getName();
        for (String route : this.routes) {
            if (className.startsWith(route)) {
                return route;
            }
        }

        // 根据目标接口对象查找数据源标识
        Class<?>[] interfaces = AopProxyUtils.proxiedUserInterfaces(target);
        if (ObjectUtils.notEmpty(interfaces)) {
            for (Class<?> clazz : interfaces) {
                String interfaceName = clazz.getName();
                for (String route : this.routes) {
                    if (interfaceName.startsWith(route)) {
                        return route;
                    }
                }
            }
        }
        return null;
    }

    /**
     * 数据源路由
     *
     * @param point 方法拦截点
     * @return 方法调用结果
     * @throws Throwable 方法调用异常
     */
    @Around("@within(org.springframework.stereotype.Service) " +
            "|| @within(org.apache.ibatis.annotations.Mapper) " +
            "|| this(com.baomidou.mybatisplus.core.mapper.BaseMapper) " +
            "|| this(com.chelvc.framework.database.support.EnhanceMapper)")
    public Object routing(ProceedingJoinPoint point) throws Throwable {
        DynamicDataSourceContextHolder.push(this.lookupDatasourceKey(point));
        try {
            return point.proceed();
        } finally {
            DynamicDataSourceContextHolder.poll();
        }
    }
}