package com.chelvc.framework.database.interceptor;

import java.sql.Connection;
import java.time.temporal.Temporal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Consumer;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.core.plugins.InterceptorIgnoreHelper;
import com.baomidou.mybatisplus.core.toolkit.PluginUtils;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.database.config.DatabaseProperties;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import com.chelvc.framework.database.entity.Creatable;
import com.chelvc.framework.database.entity.Deletable;
import com.chelvc.framework.database.entity.Environmental;
import com.chelvc.framework.database.entity.Updatable;
import com.chelvc.framework.database.util.SQLUtils;
import com.google.common.collect.Lists;
import lombok.RequiredArgsConstructor;
import net.sf.jsqlparser.JSQLParserException;
import net.sf.jsqlparser.Model;
import net.sf.jsqlparser.expression.Alias;
import net.sf.jsqlparser.expression.BinaryExpression;
import net.sf.jsqlparser.expression.Expression;
import net.sf.jsqlparser.expression.Function;
import net.sf.jsqlparser.expression.JdbcParameter;
import net.sf.jsqlparser.expression.NullValue;
import net.sf.jsqlparser.expression.Parenthesis;
import net.sf.jsqlparser.expression.StringValue;
import net.sf.jsqlparser.expression.operators.conditional.AndExpression;
import net.sf.jsqlparser.expression.operators.conditional.OrExpression;
import net.sf.jsqlparser.expression.operators.relational.ComparisonOperator;
import net.sf.jsqlparser.expression.operators.relational.EqualsTo;
import net.sf.jsqlparser.expression.operators.relational.ExpressionList;
import net.sf.jsqlparser.expression.operators.relational.InExpression;
import net.sf.jsqlparser.expression.operators.relational.IsNullExpression;
import net.sf.jsqlparser.expression.operators.relational.ItemsList;
import net.sf.jsqlparser.expression.operators.relational.MultiExpressionList;
import net.sf.jsqlparser.expression.operators.relational.NotEqualsTo;
import net.sf.jsqlparser.parser.CCJSqlParserUtil;
import net.sf.jsqlparser.schema.Column;
import net.sf.jsqlparser.schema.Table;
import net.sf.jsqlparser.statement.Statement;
import net.sf.jsqlparser.statement.Statements;
import net.sf.jsqlparser.statement.insert.Insert;
import net.sf.jsqlparser.statement.select.AllColumns;
import net.sf.jsqlparser.statement.select.AllTableColumns;
import net.sf.jsqlparser.statement.select.FromItem;
import net.sf.jsqlparser.statement.select.Join;
import net.sf.jsqlparser.statement.select.LateralSubSelect;
import net.sf.jsqlparser.statement.select.PlainSelect;
import net.sf.jsqlparser.statement.select.Select;
import net.sf.jsqlparser.statement.select.SelectBody;
import net.sf.jsqlparser.statement.select.SelectExpressionItem;
import net.sf.jsqlparser.statement.select.SelectItem;
import net.sf.jsqlparser.statement.select.SetOperationList;
import net.sf.jsqlparser.statement.select.SubJoin;
import net.sf.jsqlparser.statement.select.SubSelect;
import net.sf.jsqlparser.statement.select.WithItem;
import net.sf.jsqlparser.statement.update.Update;
import org.apache.commons.lang3.tuple.Pair;
import org.apache.ibatis.executor.statement.StatementHandler;
import org.apache.ibatis.mapping.BoundSql;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.mapping.ParameterMapping;
import org.apache.ibatis.plugin.Interceptor;
import org.apache.ibatis.plugin.Intercepts;
import org.apache.ibatis.plugin.Invocation;
import org.apache.ibatis.plugin.Plugin;
import org.apache.ibatis.plugin.Signature;
import org.apache.ibatis.reflection.MetaObject;
import org.apache.ibatis.reflection.SystemMetaObject;
import org.apache.ibatis.type.TypeHandler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * SQL执行动态处理拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Component
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Intercepts(@Signature(type = StatementHandler.class, method = "prepare", args = {Connection.class, Integer.class}))
public class DynamicInvokeInterceptor implements Interceptor {
    /**
     * JdbcParameter类型参数查找结果回调接口
     */
    private interface JdbcParameterConsumer {
        /**
         * 结果信息处理，并返回SQL结构是否变更
         *
         * @param table     关联表信息
         * @param column    关联字段信息
         * @param parameter 参数表达式
         * @param condition 条件表达式
         * @param changing  表达式变更回调函数
         * @return true/false
         */
        boolean consume(Table table, Column column, JdbcParameter parameter, Expression condition,
                        Consumer<Expression> changing);
    }

    private final DatabaseProperties properties;

    /**
     * 判断字段别名与指定别名是否相同
     *
     * @param column 字段信息
     * @param alias  别名信息
     * @return true/false
     */
    private boolean isAlias(Column column, Alias alias) {
        Table table = column.getTable();
        if (alias == null) {
            return table == null;
        }
        return table == null || Objects.equals(table.getName(), alias.getName());
    }

    /**
     * 判断是否是元类型参数
     *
     * @param parameter 参数
     * @return true/false
     */
    private boolean isMetaParameter(Object parameter) {
        return parameter == null || parameter instanceof Enum || parameter instanceof Number
                || parameter instanceof String || parameter instanceof Boolean || parameter instanceof Character
                || parameter instanceof Date || parameter instanceof Temporal;
    }

    /**
     * 获取参数值
     *
     * @param bound     绑定SQL
     * @param parameter 参数值表达式
     * @return 参数值
     */
    private Object getParameterValue(BoundSql bound, JdbcParameter parameter) {
        Object object = bound.getParameterObject();
        if (this.isMetaParameter(object)) {
            return object;
        }
        List<ParameterMapping> mappings = bound.getParameterMappings();
        if (ObjectUtils.isEmpty(mappings)) {
            return null;
        }
        String property = mappings.get(parameter.getIndex() - 1).getProperty();
        if (bound.hasAdditionalParameter(property)) {
            return bound.getAdditionalParameter(property);
        }
        return SystemMetaObject.forObject(object).getValue(property);
    }

    /**
     * 查找字段的下标位置，如果不存在则返回 -1
     *
     * @param table   表信息
     * @param column  字段信息
     * @param columns 字段信息列表
     * @return 下标数字
     */
    private int lookupColumnIndex(Table table, Column column, List<Column> columns) {
        for (int i = 0, size = columns.size(); i < size; i++) {
            Column c = columns.get(i);
            if (this.isAlias(c, table.getAlias()) && Objects.equals(c.getColumnName(), column.getColumnName())) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 查找更新字段关联表
     *
     * @param update 数据修改信息
     * @param column 更新字段信息
     * @return 表信息
     */
    private Table lookupColumnTable(Update update, Column column) {
        if (this.isAlias(column, update.getTable().getAlias())) {
            return update.getTable();
        } else if (ObjectUtils.notEmpty(update.getStartJoins())) {
            for (Join join : update.getStartJoins()) {
                FromItem from = join.getRightItem();
                if (from instanceof Table && this.isAlias(column, from.getAlias())) {
                    return (Table) from;
                }
            }
        }
        return null;
    }

    /**
     * 查找查询字段
     *
     * @param plain  查询信息
     * @param column 字段信息
     * @return 字段信息
     */
    private Column lookupSelectColumn(PlainSelect plain, Column column) {
        if (ObjectUtils.notEmpty(plain.getSelectItems())) {
            for (SelectItem item : plain.getSelectItems()) {
                if (item instanceof AllColumns || item instanceof AllTableColumns) {
                    return column.getTable() == null ? column : new Column(column.getColumnName());
                }
                SelectExpressionItem select = (SelectExpressionItem) item;
                Expression expression = select.getExpression();
                if (expression instanceof Column) {
                    String name = ((Column) expression).getColumnName();
                    String alias = ObjectUtils.ifNull(select.getAlias(), Alias::getName);
                    if (Objects.equals(column.getColumnName(), name) || Objects.equals(column.getColumnName(), alias)) {
                        return (Column) expression;
                    }
                }
            }
        }
        return null;
    }

    /**
     * 查找表字段信息
     *
     * @param join   关联信息
     * @param column 字段信息
     * @return 表/字段信息
     */
    private Pair<Table, Column> lookupTableColumn(Join join, Column column) {
        return join.getRightItem() == null ? null : this.lookupTableColumn(join.getRightItem(), column);
    }

    /**
     * 查找表字段信息
     *
     * @param joins  关联信息列表
     * @param column 字段信息
     * @return 表/字段信息
     */
    private Pair<Table, Column> lookupTableColumn(List<Join> joins, Column column) {
        Pair<Table, Column> pair = null;
        for (Join join : joins) {
            if ((pair = this.lookupTableColumn(join, column)) != null) {
                break;
            }
        }
        return pair;
    }

    /**
     * 查找表字段信息
     *
     * @param from   查询信息
     * @param column 字段信息
     * @return 表/字段信息
     */
    private Pair<Table, Column> lookupTableColumn(FromItem from, Column column) {
        Pair<Table, Column> pair = null;
        if (from instanceof Table) {
            Table table = (Table) from;
            if (this.isAlias(column, table.getAlias())) {
                pair = Pair.of(table, column);
            }
        } else if (from instanceof SubJoin) {
            SubJoin sub = (SubJoin) from;
            List<Join> joins = sub.getJoinList();
            if (ObjectUtils.notEmpty(joins) && this.isAlias(column, sub.getAlias())) {
                pair = this.lookupTableColumn(joins, column);
            }
            if (pair == null && sub.getLeft() != null) {
                pair = this.lookupTableColumn(sub.getLeft(), column);
            }
        } else if (from instanceof SubSelect) {
            SubSelect sub = (SubSelect) from;
            SelectBody body = sub.getSelectBody();
            if (body != null) {
                if (this.isAlias(column, sub.getAlias()) && body instanceof PlainSelect) {
                    column = ObjectUtils.ifNull(this.lookupSelectColumn((PlainSelect) body, column), column);
                }
                pair = this.lookupTableColumn(body, column);
            }
            if (pair == null && ObjectUtils.notEmpty(sub.getWithItemsList())) {
                for (WithItem with : sub.getWithItemsList()) {
                    if ((pair = this.lookupTableColumn(with, column)) != null) {
                        break;
                    }
                }
            }
        } else if (from instanceof LateralSubSelect) {
            SubSelect sub = ((LateralSubSelect) from).getSubSelect();
            SelectBody body = ObjectUtils.ifNull(sub, SubSelect::getSelectBody);
            if (body != null) {
                if (this.isAlias(column, sub.getAlias()) && body instanceof PlainSelect) {
                    column = ObjectUtils.ifNull(this.lookupSelectColumn((PlainSelect) body, column), column);
                }
                pair = this.lookupTableColumn(body, column);
            }
        }
        return pair;
    }

    /**
     * 查找表字段信息
     *
     * @param select 查询信息
     * @param column 字段信息
     * @return 表/字段信息
     */
    private Pair<Table, Column> lookupTableColumn(SelectBody select, Column column) {
        Pair<Table, Column> pair = null;
        if (select instanceof PlainSelect) {
            PlainSelect plain = (PlainSelect) select;
            pair = this.lookupTableColumn(plain.getFromItem(), column);
            if (pair == null && ObjectUtils.notEmpty(plain.getJoins())) {
                pair = this.lookupTableColumn(plain.getJoins(), column);
            }
        } else if (select instanceof WithItem) {
            WithItem with = (WithItem) select;
            if (with.getSubSelect() != null) {
                pair = this.lookupTableColumn(with.getSubSelect(), column);
            }
        } else if (select instanceof SetOperationList) {
            List<SelectBody> selects = ((SetOperationList) select).getSelects();
            if (ObjectUtils.notEmpty(selects)) {
                for (SelectBody body : selects) {
                    if ((pair = this.lookupTableColumn(body, column)) != null) {
                        break;
                    }
                }
            }
        }
        return pair;
    }

    /**
     * 查找表字段信息
     *
     * @param update 修改信息
     * @param column 字段信息
     * @return 表/字段信息
     */
    private Pair<Table, Column> lookupTableColumn(Update update, Column column) {
        Pair<Table, Column> pair = null;
        Table table = update.getTable();
        if (this.isAlias(column, table.getAlias())) {
            pair = Pair.of(table, column);
        }
        if (pair == null && ObjectUtils.notEmpty(update.getJoins())) {
            pair = this.lookupTableColumn(update.getJoins(), column);
        }
        if (pair == null && ObjectUtils.notEmpty(update.getStartJoins())) {
            pair = this.lookupTableColumn(update.getStartJoins(), column);
        }
        if (pair == null && update.getFromItem() != null) {
            pair = this.lookupTableColumn(update.getFromItem(), column);
        }
        if (pair == null && ObjectUtils.notEmpty(update.getWithItemsList())) {
            for (WithItem with : update.getWithItemsList()) {
                if ((pair = this.lookupTableColumn(with, column)) != null) {
                    break;
                }
            }
        }
        return pair;
    }

    /**
     * 查找表字段信息
     *
     * @param access 访问域
     * @param column 字段信息
     * @return 表/字段信息
     */
    private Pair<Table, Column> lookupTableColumn(Object access, Column column) {
        if (access instanceof Join) {
            return this.lookupTableColumn((Join) access, column);
        } else if (access instanceof FromItem) {
            return this.lookupTableColumn((FromItem) access, column);
        } else if (access instanceof SelectBody) {
            return this.lookupTableColumn((SelectBody) access, column);
        } else if (access instanceof Update) {
            return this.lookupTableColumn((Update) access, column);
        }
        return null;
    }

    /**
     * 预处理JdbcParameter类型参数，并返回SQL结构是否变更
     *
     * @param access   访问域
     * @param model    SQL模型
     * @param consumer 结果回调接口
     * @return true/false
     */
    private boolean prepareJdbcParameter(Object access, Model model, JdbcParameterConsumer consumer) {
        return this.prepareJdbcParameter(access, model, consumer, null);
    }

    /**
     * 预处理JdbcParameter类型参数，并返回SQL结构是否变更
     *
     * @param access   访问域
     * @param joins    关联信息列表
     * @param consumer 结果回调接口
     * @return true/false
     */
    private boolean prepareJdbcParameter(Object access, List<Join> joins, JdbcParameterConsumer consumer) {
        boolean changed = false;
        for (Join join : joins) {
            if (join.getOnExpression() != null) {
                changed |= this.prepareJdbcParameter(access, join.getOnExpression(), consumer, join::setOnExpression);
            }
            if (join.getRightItem() != null) {
                changed |= this.prepareJdbcParameter(access, join.getRightItem(), consumer, null);
            }
        }
        return changed;
    }

    /**
     * 预处理JdbcParameter类型参数，并返回SQL结构是否变更
     *
     * @param access   访问域
     * @param model    SQL模型
     * @param consumer 结果回调接口
     * @param changing 表达式变更回调函数
     * @return true/false
     */
    private boolean prepareJdbcParameter(Object access, Model model, JdbcParameterConsumer consumer,
                                         Consumer<Expression> changing) {
        boolean changed = false;
        if (model instanceof SubJoin) {
            SubJoin sub = (SubJoin) model;
            if (sub.getLeft() != null) {
                changed = this.prepareJdbcParameter(access, sub.getLeft(), consumer, changing);
            }
            if (ObjectUtils.notEmpty(sub.getJoinList())) {
                changed |= this.prepareJdbcParameter(access, sub.getJoinList(), consumer);
            }
        } else if (model instanceof WithItem) {
            WithItem with = (WithItem) model;
            if (with.getSubSelect() != null) {
                changed = this.prepareJdbcParameter(access, with.getSubSelect(), consumer, changing);
            }
        } else if (model instanceof SubSelect) {
            SubSelect sub = (SubSelect) model;
            if (sub.getSelectBody() != null) {
                changed = this.prepareJdbcParameter(access, sub.getSelectBody(), consumer, changing);
            }
            if (ObjectUtils.notEmpty(sub.getWithItemsList())) {
                for (WithItem with : sub.getWithItemsList()) {
                    changed |= this.prepareJdbcParameter(access, with, consumer, changing);
                }
            }
        } else if (model instanceof PlainSelect) {
            PlainSelect plain = (PlainSelect) model;
            if (plain.getWhere() != null) {
                changed = this.prepareJdbcParameter(plain, plain.getWhere(), consumer, plain::setWhere);
            }
            if (ObjectUtils.notEmpty(plain.getJoins())) {
                changed |= this.prepareJdbcParameter(plain, plain.getJoins(), consumer);
            }
            if (plain.getFromItem() != null && !(plain.getFromItem() instanceof Table)) {
                changed |= this.prepareJdbcParameter(plain, plain.getFromItem(), consumer);
            }
        } else if (model instanceof LateralSubSelect) {
            LateralSubSelect lateral = (LateralSubSelect) model;
            if (lateral.getSubSelect() != null) {
                changed = this.prepareJdbcParameter(access, lateral.getSubSelect(), consumer, changing);
            }
        } else if (model instanceof SetOperationList) {
            List<SelectBody> selects = ((SetOperationList) model).getSelects();
            if (ObjectUtils.notEmpty(selects)) {
                for (SelectBody body : selects) {
                    changed |= this.prepareJdbcParameter(access, body, consumer, changing);
                }
            }
        } else if (model instanceof Parenthesis) {
            Parenthesis parenthesis = (Parenthesis) model;
            if (parenthesis.getExpression() != null) {
                changed = this.prepareJdbcParameter(access, parenthesis.getExpression(), consumer, changing);
            }
        } else if (model instanceof InExpression) {
            InExpression in = (InExpression) model;
            ItemsList items = in.getRightItemsList();
            if (in.getLeftExpression() instanceof Column && items instanceof ExpressionList
                    && ObjectUtils.notEmpty(((ExpressionList) items).getExpressions())) {
                Pair<Table, Column> pair;
                Column column = (Column) in.getLeftExpression();

                // 重新创建值集合，避免出现集合遍历过程中元素被修改导致的异常。
                for (Expression value : Lists.newArrayList(((ExpressionList) items).getExpressions())) {
                    if (value instanceof JdbcParameter && (pair = this.lookupTableColumn(access, column)) != null) {
                        JdbcParameter parameter = (JdbcParameter) value;
                        changed |= consumer.consume(pair.getLeft(), pair.getRight(), parameter, in, expression -> {
                            // 不支持参数值修改回调
                            throw new UnsupportedOperationException();
                        });
                    }
                }
            } else if (items instanceof SubSelect) {
                changed = this.prepareJdbcParameter(in, (FromItem) items, consumer);
            }
        } else if (model instanceof BinaryExpression) {
            BinaryExpression binary = (BinaryExpression) model;
            Expression left = binary.getLeftExpression(), right = binary.getRightExpression();
            Column column = left instanceof Column ? (Column) left : right instanceof Column ? (Column) right : null;
            JdbcParameter parameter = left instanceof JdbcParameter ? (JdbcParameter) left
                    : right instanceof JdbcParameter ? (JdbcParameter) right : null;
            if (column == null || parameter == null) {
                changed = this.prepareJdbcParameter(access, left, consumer, binary::setLeftExpression)
                        | this.prepareJdbcParameter(access, right, consumer, binary::setRightExpression);
            } else {
                Pair<Table, Column> pair = this.lookupTableColumn(access, column);
                if (pair != null) {
                    changed = consumer.consume(pair.getLeft(), pair.getRight(), parameter, binary, changing);
                }
            }
        }
        return changed;
    }

    /**
     * 查询条件查找结果回调接口
     */
    private interface SelectConditionConsumer {
        /**
         * 结果信息处理，并返回SQL结构是否变更
         *
         * @param table     表信息
         * @param condition 条件表达式
         * @param changing  表达式变更回调函数
         * @return true/false
         */
        boolean consume(Table table, Expression condition, Consumer<Expression> changing);
    }

    /**
     * 预处理查询条件，并返回SQL结构是否变更
     *
     * @param model    SQL模型
     * @param consumer 结果回调接口
     * @return true/false
     */
    private boolean prepareSelectCondition(Model model, SelectConditionConsumer consumer) {
        boolean changed = false;
        if (model instanceof Select) {
            Select select = (Select) model;
            if (select.getSelectBody() != null) {
                changed = this.prepareSelectCondition(select.getSelectBody(), consumer);
            }
            if (ObjectUtils.notEmpty(select.getWithItemsList())) {
                for (WithItem with : select.getWithItemsList()) {
                    if (with.getSubSelect() != null) {
                        changed |= this.prepareSelectCondition(with.getSubSelect(), consumer);
                    }
                }
            }
        } else if (model instanceof SubJoin) {
            SubJoin sub = (SubJoin) model;
            if (sub.getLeft() != null) {
                changed = this.prepareSelectCondition(sub.getLeft(), consumer);
            }
            if (ObjectUtils.notEmpty(sub.getJoinList())) {
                for (Join join : sub.getJoinList()) {
                    if (!(join.getRightItem() instanceof Table)) {
                        changed |= this.prepareSelectCondition(join.getRightItem(), consumer);
                    }
                }
            }
        } else if (model instanceof WithItem) {
            WithItem with = (WithItem) model;
            if (with.getSubSelect() != null) {
                changed = this.prepareSelectCondition(with.getSubSelect(), consumer);
            }
        } else if (model instanceof SubSelect) {
            SubSelect sub = (SubSelect) model;
            if (sub.getSelectBody() != null) {
                changed = this.prepareSelectCondition(sub.getSelectBody(), consumer);
            }
            if (ObjectUtils.notEmpty(sub.getWithItemsList())) {
                for (WithItem with : sub.getWithItemsList()) {
                    changed |= this.prepareSelectCondition(with, consumer);
                }
            }
        } else if (model instanceof PlainSelect) {
            PlainSelect plain = (PlainSelect) model;
            FromItem from = plain.getFromItem();
            if (from instanceof Table) {
                changed = consumer.consume((Table) from, plain.getWhere(), plain::setWhere);
            } else {
                changed = this.prepareSelectCondition(from, consumer);
            }
            if (ObjectUtils.notEmpty(plain.getJoins())) {
                for (Join join : plain.getJoins()) {
                    if (join.getRightItem() instanceof Table) {
                        changed |= consumer.consume((Table) join.getRightItem(), plain.getWhere(), plain::setWhere);
                    } else {
                        changed |= this.prepareSelectCondition(join.getRightItem(), consumer);
                    }
                }
            }
        } else if (model instanceof LateralSubSelect) {
            LateralSubSelect lateral = (LateralSubSelect) model;
            if (lateral.getSubSelect() != null) {
                changed = this.prepareSelectCondition(lateral.getSubSelect(), consumer);
            }
        } else if (model instanceof SetOperationList) {
            List<SelectBody> selects = ((SetOperationList) model).getSelects();
            if (ObjectUtils.notEmpty(selects)) {
                for (SelectBody body : selects) {
                    changed |= this.prepareSelectCondition(body, consumer);
                }
            }
        }
        return changed;
    }

    /**
     * 绑定参数TypeHandler
     *
     * @param bound     绑定SQL
     * @param table     表信息
     * @param column    字段信息
     * @param parameter 参数表达式
     */
    private void bindTypeHandler(BoundSql bound, Table table, Column column, JdbcParameter parameter) {
        Tables.Field field = Tables.getTableField(table.getName(), column.getColumnName());
        TypeHandler<?> handler = ObjectUtils.ifNull(field, Tables.Field::getHandler);
        if (handler != null) {
            ParameterMapping mapping = bound.getParameterMappings().get(parameter.getIndex() - 1);
            SystemMetaObject.forObject(mapping).setValue("typeHandler", handler);
        }
    }

    /**
     * 绑定参数TypeHandler
     *
     * @param bound      绑定SQL
     * @param table      表信息
     * @param columns    字段列表
     * @param parameters 参数表达式列表
     */
    private void bindTypeHandler(BoundSql bound, Table table, List<Column> columns, List<Expression> parameters) {
        for (int i = 0, size = parameters.size(); i < size; i++) {
            Expression expression = parameters.get(i);
            if (expression instanceof JdbcParameter) {
                this.bindTypeHandler(bound, table, columns.get(i), (JdbcParameter) expression);
            }
        }
    }

    /**
     * 初始化字段默认值，并返回是否需要重构SQL语句
     *
     * @param bound       绑定SQL
     * @param expressions 字段值表达式列表
     * @param index       字段下标
     * @param value       字段值
     * @param cover       是否直接覆盖参数值
     * @return true/false
     */
    @SuppressWarnings("unchecked")
    private boolean initializeDefaultValue(BoundSql bound, List<?> expressions, int index, Pair<?, Expression> value,
                                           boolean cover) {
        Object original = expressions.get(index);
        Object expression = original;
        if (expression instanceof SelectExpressionItem) {
            expression = ((SelectExpressionItem) expression).getExpression();
        }
        if (expression instanceof Function) {
            return false;
        } else if (expression instanceof JdbcParameter) {
            // 如果是占位符参数，则更新关联参数
            int i = ((JdbcParameter) expression).getIndex() - 1;
            String property = bound.getParameterMappings().get(i).getProperty();
            if (bound.hasAdditionalParameter(property)) {
                if (cover || Objects.isNull(bound.getAdditionalParameter(property))) {
                    bound.setAdditionalParameter(property, value.getLeft());
                }
            } else {
                MetaObject parameter = SystemMetaObject.forObject(bound.getParameterObject());
                if (cover || Objects.isNull(parameter.getValue(property))) {
                    parameter.setValue(property, value.getLeft());
                }
            }
        } else if (cover || expression instanceof NullValue) {
            // 直接更新参数表达式
            if (original instanceof SelectExpressionItem) {
                ((SelectExpressionItem) original).setExpression(value.getRight());
            } else {
                ((List<Object>) expressions).set(index, value.getRight());
            }
            return true;
        }
        return false;
    }

    /**
     * 初始化字段默认值，并返回是否需要重构SQL语句
     *
     * @param bound  绑定SQL
     * @param insert 数据插入对象实例
     * @param column 更新字段对象实例
     * @param value  更新字段值
     * @return true/false
     */
    private boolean initializeDefaultValue(BoundSql bound, Insert insert, Column column, Pair<?, Expression> value) {
        int index = this.lookupColumnIndex(insert.getTable(), column, insert.getColumns());
        if (index < 0) {
            // 如果不存在目标字段则直接增加字段及参数值
            ItemsList items = insert.getItemsList();
            if (items instanceof MultiExpressionList) {
                // 多行插入
                insert.addColumns(column);
                ((MultiExpressionList) items).getExpressionLists()
                        .forEach(expressions -> expressions.getExpressions().add(value.getRight()));
                return true;
            } else if (items instanceof ExpressionList) {
                // 单行插入
                insert.addColumns(column);
                return ((ExpressionList) items).getExpressions().add(value.getRight());
            } else if (insert.getSelect() != null && insert.getSelect().getSelectBody() instanceof PlainSelect) {
                PlainSelect select = (PlainSelect) insert.getSelect().getSelectBody();
                insert.addColumns(column);
                return (select.getSelectItems()).add(new SelectExpressionItem(value.getRight()));
            }
            return false;
        }

        // 动态查找参数值并替换成目标参数值
        ItemsList items = insert.getItemsList();
        if (items instanceof MultiExpressionList) {
            boolean rebuild = false;
            for (ExpressionList expressions : ((MultiExpressionList) items).getExpressionLists()) {
                rebuild |= this.initializeDefaultValue(bound, expressions.getExpressions(), index, value, false);
            }
            return rebuild;
        } else if (items instanceof ExpressionList) {
            return this.initializeDefaultValue(bound, ((ExpressionList) items).getExpressions(), index, value, false);
        } else if (insert.getSelect() != null && insert.getSelect().getSelectBody() instanceof PlainSelect) {
            PlainSelect select = (PlainSelect) insert.getSelect().getSelectBody();
            return this.initializeDefaultValue(bound, select.getSelectItems(), index, value, false);
        }
        return false;
    }

    /**
     * 初始化字段默认值，并返回是否需要重构SQL语句
     *
     * @param bound  绑定SQL
     * @param update 数据修改对象实例
     * @param table  表信息
     * @param column 更新字段对象实例
     * @param value  更新字段值
     * @return true/false
     */
    private boolean initializeDefaultValue(BoundSql bound, Update update, Table table, Column column,
                                           Pair<?, Expression> value) {
        int index = this.lookupColumnIndex(table, column, update.getColumns());
        if (index < 0) {
            if (Objects.isNull(table.getAlias())) {
                update.addColumns(column);
            } else {
                update.addColumns(new Column(table, column.getColumnName()));
            }
            update.addExpressions(value.getRight());
            return true;
        }
        return this.initializeDefaultValue(bound, update.getExpressions(), index, value, true);
    }

    /**
     * 初始化字段默认值，并返回是否需要重构SQL语句
     *
     * @param bound    绑定SQL
     * @param update   数据修改对象实例
     * @param table    表信息
     * @param operator 操作人信息
     * @param datetime 更新时间信息
     * @return true/false
     */
    private boolean initializeDefaultValue(BoundSql bound, Update update, Table table, Pair<?, Expression> operator,
                                           Pair<?, Expression> datetime) {
        Class<?> model = Tables.getTableClass(table.getName());
        if (model == null) {
            return false;
        }
        boolean rebuild = false;
        if (Creatable.class.isAssignableFrom(model)) {
            int index = this.lookupColumnIndex(table, Expressions.CREATOR_COLUMN, update.getColumns());
            if (index >= 0) {
                rebuild = this.initializeDefaultValue(bound, update.getExpressions(), index, operator, false);
            }
            if ((index = this.lookupColumnIndex(table, Expressions.CREATE_TIME_COLUMN, update.getColumns())) >= 0) {
                rebuild |= this.initializeDefaultValue(bound, update.getExpressions(), index, datetime, false);
            }
        }
        if (Updatable.class.isAssignableFrom(model)) {
            rebuild |= this.initializeDefaultValue(bound, update, table, Expressions.UPDATER_COLUMN, operator);
            rebuild |= this.initializeDefaultValue(bound, update, table, Expressions.UPDATE_TIME_COLUMN, datetime);
        }
        if (Environmental.class.isAssignableFrom(model) && Expressions.env().getLeft() != null) {
            int index = this.lookupColumnIndex(table, Expressions.ENV_COLUMN, update.getColumns());
            if (index >= 0) {
                rebuild |= this.initializeDefaultValue(bound, update.getExpressions(), index, Expressions.env(), false);
            }
        }
        if (Deletable.class.isAssignableFrom(model)) {
            int index = this.lookupColumnIndex(table, Expressions.DELETED_COLUMN, update.getColumns());
            if (index >= 0) {
                rebuild |= this.initializeDefaultValue(bound, update.getExpressions(), index, Expressions.FALSE, false);
            }
        }
        return rebuild;
    }

    /**
     * 初始化敏感字段参数，并返回是否需要重构SQL语句
     *
     * @param bound     绑定SQL
     * @param table     表信息
     * @param column    字段信息
     * @param parameter 参数表达式
     * @param condition 条件表达式
     * @param changing  表达式变更回调函数
     * @return true/false
     */
    private boolean initializeSensitiveParameter(BoundSql bound, Table table, Column column, JdbcParameter parameter,
                                                 Expression condition, Consumer<Expression> changing) {
        // 针对敏感字段数据混合模式下参数精确匹配场景，将自动添加混合查询条件
        if (!this.properties.getCrypto().isMixed() || !(condition instanceof EqualsTo
                || condition instanceof NotEqualsTo || condition instanceof InExpression)) {
            return false;
        }

        // 获取字段上下文信息
        Tables.Field field = Tables.getTableField(table.getName(), column.getColumnName());
        if (field == null || !field.isSensitive()) {
            return false;
        }

        // 获取绑定参数值（明文）
        Object value = this.getParameterValue(bound, parameter);
        if (!(value instanceof String)) {
            return false;
        }

        // 追加混合查询明文/密文附加参数
        if (!this.properties.getCrypto().isWritable()) {
            value = DatabaseContextHolder.getDatabaseCipherHandler().encrypt((String) value, true);
        }
        StringValue extra = new StringValue(SQLUtils.escape((String) value));
        if (condition instanceof InExpression) {
            InExpression in = (InExpression) condition;
            ((ExpressionList) in.getRightItemsList()).getExpressions().add(extra);
        } else {
            ComparisonOperator comparison = (ComparisonOperator) condition;
            Expression left = comparison.getLeftExpression(), right = comparison.getRightExpression();
            Column _column = left instanceof Column ? (Column) left : (Column) right;
            InExpression in = new InExpression(_column, new ExpressionList(parameter, extra));
            in.setNot(condition instanceof NotEqualsTo);
            changing.accept(in);
        }
        return true;
    }

    /**
     * SQL预处理，并返回是否需要重构SQL语句
     *
     * @param bound  绑定SQL
     * @param access 访问域
     * @param model  SQL模型
     * @return true/false
     */
    private boolean prepare(BoundSql bound, Object access, Model model) {
        // 查找JdbcParameter类型参数
        return this.prepareJdbcParameter(access, model, (table, column, parameter, condition, changing) -> {
            // 绑定TypeHandler
            this.bindTypeHandler(bound, table, column, parameter);

            // 初始化敏感字段查询参数
            return this.initializeSensitiveParameter(bound, table, column, parameter, condition, changing);
        });
    }

    /**
     * SQL预处理，并返回是否需要重构SQL语句
     *
     * @param bound  绑定SQL
     * @param access 访问域
     * @param joins  关联信息列表
     * @return true/false
     */
    private boolean prepare(BoundSql bound, Object access, List<Join> joins) {
        // 查找JdbcParameter类型参数
        return this.prepareJdbcParameter(access, joins, (table, column, parameter, condition, changing) -> {
            // 绑定TypeHandler
            this.bindTypeHandler(bound, table, column, parameter);

            // 初始化敏感字段查询参数
            return this.initializeSensitiveParameter(bound, table, column, parameter, condition, changing);
        });
    }

    /**
     * 预处理插入SQL，并返回是否需要重构SQL语句
     *
     * @param bound  绑定SQL
     * @param insert 数据插入对象实例
     * @return true/false
     */
    private boolean prepare(BoundSql bound, Insert insert) {
        boolean rebuild = false;
        ItemsList items = insert.getItemsList();
        List<Column> columns = insert.getColumns();
        if (items instanceof MultiExpressionList) {
            for (ExpressionList list : ((MultiExpressionList) items).getExpressionLists()) {
                this.bindTypeHandler(bound, insert.getTable(), columns, list.getExpressions());
            }
        } else if (items instanceof ExpressionList) {
            this.bindTypeHandler(bound, insert.getTable(), columns, ((ExpressionList) items).getExpressions());
        } else if (insert.getSelect() != null) {
            rebuild = this.prepare(bound, insert.getSelect());
        }

        // 初始化参数默认值
        Class<?> model = Tables.getTableClass(insert.getTable().getName());
        if (model == null) {
            return rebuild;
        }
        Pair<?, Expression> operator = null, datetime = null;
        if (Creatable.class.isAssignableFrom(model)) {
            operator = Expressions.operator();
            datetime = Expressions.datetime();
            rebuild = this.initializeDefaultValue(bound, insert, Expressions.CREATOR_COLUMN, operator);
            rebuild |= this.initializeDefaultValue(bound, insert, Expressions.CREATE_TIME_COLUMN, datetime);
        }
        if (Updatable.class.isAssignableFrom(model)) {
            operator = ObjectUtils.ifNull(operator, Expressions::operator);
            datetime = ObjectUtils.ifNull(datetime, Expressions::datetime);
            rebuild |= this.initializeDefaultValue(bound, insert, Expressions.UPDATER_COLUMN, operator);
            rebuild |= this.initializeDefaultValue(bound, insert, Expressions.UPDATE_TIME_COLUMN, datetime);
        }
        if (Environmental.class.isAssignableFrom(model) && Expressions.env().getLeft() != null) {
            rebuild |= this.initializeDefaultValue(bound, insert, Expressions.ENV_COLUMN, Expressions.env());
        }
        if (Deletable.class.isAssignableFrom(model)) {
            rebuild |= this.initializeDefaultValue(bound, insert, Expressions.DELETED_COLUMN, Expressions.FALSE);
        }
        return rebuild;
    }

    /**
     * 预处理修改SQL，并返回是否需要重构SQL语句
     *
     * @param bound  绑定SQL
     * @param update 数据修改对象实例
     * @return true/false
     */
    private boolean prepare(BoundSql bound, Update update) {
        // 绑定参数TypeHandler
        List<Column> columns = update.getColumns();
        List<Expression> expressions = update.getExpressions();
        if (ObjectUtils.notEmpty(expressions)) {
            for (int i = 0, size = expressions.size(); i < size; i++) {
                Expression expression = expressions.get(i);
                if (expression instanceof JdbcParameter) {
                    Column column = columns.get(i);
                    Table table = this.lookupColumnTable(update, column);
                    if (table != null) {
                        this.bindTypeHandler(bound, table, column, (JdbcParameter) expression);
                    }
                }
            }
        }

        // 处理where语句块
        boolean rebuild = false;
        Expression where = update.getWhere();
        if (where != null) {
            rebuild = this.prepareJdbcParameter(update, where, (table, column, parameter, condition, changing) -> {
                // 绑定TypeHandler
                this.bindTypeHandler(bound, table, column, parameter);

                // 初始化敏感字段查询参数
                return this.initializeSensitiveParameter(bound, table, column, parameter, condition, changing);
            }, update::setWhere);
        }

        // 处理join语句块
        if (ObjectUtils.notEmpty(update.getJoins())) {
            rebuild |= this.prepare(bound, update, update.getJoins());
        }
        if (ObjectUtils.notEmpty(update.getStartJoins())) {
            rebuild |= this.prepare(bound, update, update.getStartJoins());
        }

        // 处理from语句块
        if (update.getFromItem() != null) {
            rebuild |= this.prepare(bound, update, update.getFromItem());
        }

        // 处理with语句块
        if (ObjectUtils.notEmpty(update.getWithItemsList())) {
            for (WithItem with : update.getWithItemsList()) {
                SelectBody select = ObjectUtils.ifNull(with.getSubSelect(), SubSelect::getSelectBody);
                rebuild |= select != null && this.prepare(bound, update, select);
            }
        }

        // 初始化主表字段默认值
        Pair<?, Expression> operator = Expressions.operator(), datetime = Expressions.datetime();
        rebuild |= this.initializeDefaultValue(bound, update, update.getTable(), operator, datetime);

        // 初始化关联表字段默认值
        if (ObjectUtils.notEmpty(update.getStartJoins())) {
            for (Join join : update.getStartJoins()) {
                if (join.getRightItem() instanceof Table) {
                    // 初始化关联表字段默认值
                    Table table = (Table) join.getRightItem();
                    if (columns.stream().anyMatch(column -> this.isAlias(column, table.getAlias()))) {
                        rebuild |= this.initializeDefaultValue(bound, update, table, operator, datetime);
                    }
                }
            }
        }
        return rebuild;
    }

    /**
     * 预处理查询SQL，并返回是否需要重构SQL语句
     *
     * @param bound  绑定SQL
     * @param select 数据查询对象实例
     * @return true/false
     */
    private boolean prepare(BoundSql bound, Select select) {
        // 处理查询主体语句块
        SelectBody body = select.getSelectBody();
        boolean rebuild = body != null && this.prepare(bound, body, body);

        // 预处理查询条件
        Map<String, Expressions.Provider> providers = Expressions.getExpressionProviders();
        if (ObjectUtils.notEmpty(providers)) {
            rebuild |= this.prepareSelectCondition(select, (table, condition, changing) -> {
                Tables.Table tb = Tables.getTable(table.getName());
                if (tb == null) {
                    return false;
                }

                String schema = Tables.getContext().getSchema();
                String unquote = SQLUtils.unquote(table.getName());
                Expressions.Provider provider = providers.get(schema + StringPool.DOT + unquote);
                Map<String, ItemsList> expressions = ObjectUtils.ifNull(provider, p -> p.get(tb));
                if (ObjectUtils.isEmpty(expressions)) {
                    return false;
                }

                Alias alias = table.getAlias();
                for (Map.Entry<String, ItemsList> entry : expressions.entrySet()) {
                    Column column = alias == null ? new Column(entry.getKey()) : new Column(table, entry.getKey());
                    InExpression in = new InExpression(column, entry.getValue());
                    IsNullExpression isnull = new IsNullExpression().withLeftExpression(column);
                    Expression expression = new Parenthesis(new OrExpression(in, isnull));
                    condition = condition == null ? expression : new AndExpression(condition, expression);
                }
                changing.accept(condition);
                return true;
            });
        }
        return rebuild;
    }

    /**
     * SQL预处理拦截
     *
     * @param invocation 方法调用信息
     * @return 执行结果
     * @throws Throwable 操作异常
     */
    private Object prepare(Invocation invocation) throws Throwable {
        Statements statements;
        BoundSql bound = ((StatementHandler) invocation.getTarget()).getBoundSql();
        try {
            statements = CCJSqlParserUtil.parseStatements(bound.getSql());
        } catch (JSQLParserException e) {
            throw new RuntimeException(e);
        }

        // 重置SQL语句
        boolean rebuild = false;
        for (Statement statement : statements.getStatements()) {
            if (statement instanceof Insert) {
                rebuild |= this.prepare(bound, (Insert) statement);
            } else if (statement instanceof Update) {
                rebuild |= this.prepare(bound, (Update) statement);
            } else if (statement instanceof Select) {
                rebuild |= this.prepare(bound, (Select) statement);
            }
        }
        if (rebuild) {
            String sql = statements.getStatements().stream().map(Object::toString)
                    .collect(Collectors.joining(StringPool.SEMICOLON));
            SystemMetaObject.forObject(bound).setValue("sql", sql);
        }
        return invocation.proceed();
    }

    /**
     * 判断是否忽略SQL调用拦截处理
     *
     * @param invocation 调用信息
     * @return true/false
     */
    private boolean isIgnore(Invocation invocation) {
        if (!(invocation.getTarget() instanceof StatementHandler)) {
            return true;
        }
        StatementHandler handler = (StatementHandler) invocation.getTarget();
        MappedStatement statement = PluginUtils.mpStatementHandler(handler).mappedStatement();
        return InterceptorIgnoreHelper.willIgnoreTenantLine(statement.getId());
    }

    @Override
    public Object intercept(Invocation invocation) throws Throwable {
        return this.isIgnore(invocation) ? invocation.proceed() : this.prepare(invocation);
    }

    @Override
    public Object plugin(Object target) {
        return target instanceof StatementHandler ? Plugin.wrap(target, this) : target;
    }
}
