package com.chelvc.framework.database.interceptor;

import java.lang.reflect.Field;
import java.math.BigInteger;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.Map;
import java.util.Set;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.DateUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import net.sf.jsqlparser.expression.Expression;
import net.sf.jsqlparser.expression.LongValue;
import net.sf.jsqlparser.expression.StringValue;
import net.sf.jsqlparser.expression.operators.relational.ExpressionList;
import net.sf.jsqlparser.expression.operators.relational.ItemsList;
import net.sf.jsqlparser.schema.Column;
import org.apache.commons.lang3.tuple.Pair;

/**
 * SQL表达式工具类
 *
 * @author Woody
 * @date 2024/5/16
 */
@Slf4j
final class Expressions {
    /**
     * 枚举选项限定配置
     */
    private static final String ENUM_OPTION_LIMIT = "enum.option.limit";

    private Expressions() {
    }

    /**
     * 零值
     */
    public static final LongValue ZERO_VALUE = new LongValue(0) {
        @Override
        public long getValue() {
            return 0;
        }

        @Override
        public BigInteger getBigIntegerValue() {
            return BigInteger.ZERO;
        }
    };

    /**
     * 环境字段
     */
    public static final Column ENV_COLUMN = new Column("env");

    /**
     * 创建人字段
     */
    public static final Column CREATOR_COLUMN = new Column("creator");

    /**
     * 更新人字段
     */
    public static final Column UPDATER_COLUMN = new Column("updater");

    /**
     * 逻辑删除字段
     */
    public static final Column DELETED_COLUMN = new Column("deleted");

    /**
     * 创建时间字段
     */
    public static final Column CREATE_TIME_COLUMN = new Column("create_time");

    /**
     * 更新时间字段
     */
    public static final Column UPDATE_TIME_COLUMN = new Column("update_time");

    /**
     * 零值/表达式映射
     */
    public static final Pair<Integer, Expression> ZERO = Pair.of(0, ZERO_VALUE);

    /**
     * 假值/表达式映射
     */
    public static final Pair<Boolean, Expression> FALSE = Pair.of(false, ZERO_VALUE);

    /**
     * 环境标识/表达式映射
     */
    private static volatile Pair<String, Expression> ENV;

    /**
     * 表达式提供器接口
     */
    interface Provider {
        /**
         * 获取表达式
         *
         * @param table 表信息
         * @return 字段名/表达式值映射表
         */
        Map<String, ItemsList> get(Tables.Table table);
    }

    /**
     * 表达式提供器抽象实现
     */
    static abstract class AbstractProvider implements Provider {
        protected final Set<String> columns;
        private volatile Map<String, ItemsList> expressions;

        public AbstractProvider(@NonNull Collection<String> columns) {
            this.columns = Collections.unmodifiableSet(Sets.newHashSet(columns));
            if (this.columns.isEmpty()) {
                this.expressions = Collections.emptyMap();
            }
        }

        /**
         * 获取字段值表达式
         *
         * @param table  表对象
         * @param column 字段名
         * @return 表达式实例
         */
        protected abstract ItemsList expression(Tables.Table table, String column);

        @Override
        public Map<String, ItemsList> get(@NonNull Tables.Table table) {
            if (this.expressions == null) {
                synchronized (this) {
                    if (this.expressions == null) {
                        this.expressions = Maps.newHashMapWithExpectedSize(this.columns.size());
                        this.columns.forEach(column -> {
                            ItemsList expression = this.expression(table, column);
                            if (expression != null) {
                                this.expressions.put(column, expression);
                            }
                        });
                    }
                }
            }
            return this.expressions;
        }
    }

    /**
     * 枚举字段值表达式提供器实现
     */
    static class EnumerateProvider extends AbstractProvider {
        public EnumerateProvider(@NonNull Set<String> columns) {
            super(columns);
        }

        @Override
        protected ItemsList expression(@NonNull Tables.Table table, @NonNull String column) {
            Tables.Field field = table.getField(column);
            if (field != null && field.getType().isEnum()) {
                Field[] options = field.getType().getDeclaredFields();
                if (ObjectUtils.notEmpty(options)) {
                    ArrayList<Expression> values = Lists.newArrayListWithCapacity(options.length);
                    for (Field option : options) {
                        if (!option.isSynthetic() && option.isEnumConstant()) {
                            values.add(new StringValue(option.getName()));
                        }
                    }
                    if (!values.isEmpty()) {
                        values.trimToSize();
                        return new ExpressionList(values);
                    }
                }
            }
            return null;
        }
    }

    /**
     * 获取环境信息
     *
     * @return 环境标识/表达式
     */
    public static Pair<String, Expression> env() {
        if (ENV == null) {
            synchronized (Expressions.class) {
                if (ENV == null) {
                    String profile = ApplicationContextHolder.getProfile();
                    ENV = profile == null ? Pair.of(null, null) : Pair.of(profile, new StringValue(profile));
                }
            }
        }
        return ENV;
    }

    /**
     * 获取当前操作人
     *
     * @return 操作人/表达式
     */
    public static Pair<Long, Expression> operator() {
        Long operator = ObjectUtils.ifNull(SessionContextHolder.getId(), 1L);
        return Pair.of(operator, new LongValue(operator));
    }

    /**
     * 获取当前时间
     *
     * @return 时间/表达式
     */
    public static Pair<Date, Expression> datetime() {
        Date datetime = new Date();
        LocalDateTime now = DateUtils.convert(datetime);
        String value = DateUtils.DEFAULT_TIMESTAMP_FORMATTER.format(now);
        return Pair.of(datetime, new StringValue(value));
    }

    /**
     * 获取字段值表达式提供器
     *
     * @return 表名称/字段值表达式提供器映射表
     */
    public static Map<String, Provider> getExpressionProviders() {
        return ApplicationContextHolder.getProperty(ENUM_OPTION_LIMIT, property -> {
            if (StringUtils.isEmpty(property) || (property = property.trim()).isEmpty()) {
                return Collections.emptyMap();
            }

            Map<String, Set<String>> tables = Maps.newHashMap();
            for (String expression : property.split(",")) {
                if ((expression = expression.trim()).isEmpty()) {
                    continue;
                }

                String table, column;
                int delimiter = expression.lastIndexOf('.');
                if (delimiter < 1 || StringUtils.isEmpty(table = expression.substring(0, delimiter).trim())
                        || StringUtils.isEmpty(column = expression.substring(delimiter + 1).trim())) {
                    log.warn("Invalid column expression: {}", expression);
                    continue;
                }
                tables.computeIfAbsent(table, k -> Sets.newHashSet()).add(column);
            }

            if (ObjectUtils.isEmpty(tables)) {
                return Collections.emptyMap();
            }

            Map<String, Provider> providers = Maps.newHashMapWithExpectedSize(tables.size());
            tables.forEach((table, columns) -> providers.put(table, new EnumerateProvider(columns)));
            return providers;
        });
    }
}
