package com.chelvc.framework.database.interceptor;

import java.io.Serializable;
import java.util.ArrayDeque;
import java.util.Collections;
import java.util.Deque;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.core.metadata.TableFieldInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfoHelper;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.annotation.Sensitive;
import com.chelvc.framework.database.util.SQLUtils;
import com.google.common.collect.Maps;
import lombok.Getter;
import lombok.NonNull;
import org.apache.ibatis.type.TypeHandler;
import org.apache.ibatis.type.UnknownTypeHandler;

/**
 * 数据表工具类
 *
 * @author Woody
 * @date 2025/3/16
 */
final class Tables {
    /**
     * 上下文
     */
    private static final ThreadLocal<Deque<Context>> CONTEXT = ThreadLocal.withInitial(ArrayDeque::new);

    /**
     * 表名称/对象映射表
     */
    private static final Map<String, Table> TABLE_NAME_MAPPING = Maps.newConcurrentMap();

    /**
     * 类对象/类型处理器实例映射表
     */
    private static final Map<Class<?>, TypeHandler<?>> TYPE_HANDLER_MAPPING = Maps.newConcurrentMap();

    private Tables() {
    }

    /**
     * SQL执行上下文
     */
    @Getter
    static class Context implements Serializable {
        /**
         * 数据库Schema
         */
        private final String schema;

        /**
         * 数据源标识
         */
        private final String datasource;

        public Context(@NonNull String schema, String datasource) {
            this.schema = schema;
            this.datasource = datasource;
        }
    }

    /**
     * 表信息
     */
    @Getter
    static class Table implements Serializable {
        /**
         * 空表信息
         */
        public static final Table EMPTY = new Table();

        private final TableInfo info;
        private final Map<String, Field> fields;

        private Table() {
            this.info = null;
            this.fields = null;
        }

        public Table(@NonNull TableInfo info) {
            this.info = info;
            if (ObjectUtils.isEmpty(info.getFieldList())) {
                this.fields = Collections.emptyMap();
            } else {
                this.fields = Collections.unmodifiableMap(info.getFieldList().stream()
                        .collect(Collectors.toMap(field -> SQLUtils.unquote(field.getColumn()), Field::new)));
            }
        }

        /**
         * 获取表类型
         *
         * @return 对象类型
         */
        public Class<?> getType() {
            return this.info == null ? null : this.info.getEntityType();
        }

        /**
         * 获取表字段
         *
         * @param column 字段名
         * @return 表字段信息
         */
        public Field getField(@NonNull String column) {
            return this.fields == null ? null : this.fields.get(SQLUtils.unquote(column));
        }
    }

    /**
     * 字段信息
     */
    @Getter
    static class Field implements Serializable {
        private final boolean sensitive;
        private final TableFieldInfo info;

        public Field(@NonNull TableFieldInfo info) {
            this.info = info;
            this.sensitive = info.getField().isAnnotationPresent(Sensitive.class);
        }

        /**
         * 获取字段类型
         *
         * @return 对象类型
         */
        public Class<?> getType() {
            return this.info.getPropertyType();
        }

        /**
         * 获取字段对象
         *
         * @return 字段对象实例
         */
        public java.lang.reflect.Field getField() {
            return this.info.getField();
        }

        /**
         * 获取字段名称
         *
         * @return 字段名称
         */
        public String getColumn() {
            return this.info.getColumn();
        }

        /**
         * 获取属性名称
         *
         * @return 属性名称
         */
        public String getProperty() {
            return this.info.getProperty();
        }

        /**
         * 获取类型处理器实例
         *
         * @return 类型处理器实例
         */
        public TypeHandler<?> getHandler() {
            Class<? extends TypeHandler<?>> type = this.info.getTypeHandler();
            if (type == null || type == UnknownTypeHandler.class) {
                return null;
            }
            TypeHandler<?> handler = TYPE_HANDLER_MAPPING.get(type);
            if (handler == null) {
                handler = TYPE_HANDLER_MAPPING.computeIfAbsent(type, k -> ObjectUtils.instance(type));
            }
            return handler;
        }
    }

    /**
     * 获取SQL执行上下文
     *
     * @return 上下文实例
     */
    public static Context getContext() {
        return Objects.requireNonNull(CONTEXT.get().peek());
    }

    /**
     * 设置SQL执行上下文
     *
     * @param context 上下文实例
     */
    public static void setContext(@NonNull Context context) {
        CONTEXT.get().push(context);
    }

    /**
     * 移除SQL执行上下文
     */
    public static void removeContext() {
        Deque<Context> deque = CONTEXT.get();
        deque.poll();
        if (deque.isEmpty()) {
            CONTEXT.remove();
        }
    }

    /**
     * 获取简单表名，去掉schema前缀
     *
     * @param name 表名
     * @return 表名
     */
    public static String getSimpleName(String name) {
        if (StringUtils.isEmpty(name)) {
            return name;
        }
        int delimiter = name.lastIndexOf('.');
        return delimiter > 0 ? name.substring(delimiter + 1) : name;
    }

    /**
     * 获取表信息
     *
     * @param name 表名称
     * @return 表信息
     */
    public static Table getTable(@NonNull String name) {
        Context context = getContext();
        String unquote = SQLUtils.unquote(name);
        String key = context.getSchema() + StringPool.DOT + unquote;
        Table table = TABLE_NAME_MAPPING.get(key);
        if (table == null) {
            table = TABLE_NAME_MAPPING.computeIfAbsent(key, k -> {
                boolean quoted = SQLUtils.isQuoted(name);
                String datasource = context.getDatasource();
                for (TableInfo info : TableInfoHelper.getTableInfos()) {
                    String n = getSimpleName(info.getTableName());
                    if ((StringUtils.isEmpty(datasource) || info.getCurrentNamespace().startsWith(datasource))
                            && (Objects.equals(n, name) || (quoted && Objects.equals(n, unquote)))) {
                        return new Table(info);
                    }
                }
                return Table.EMPTY;
            });
        }
        return table == Table.EMPTY ? null : table;
    }

    /**
     * 获取表类型
     *
     * @param table 表名
     * @return 对象类型
     */
    public static Class<?> getTableClass(@NonNull String table) {
        return ObjectUtils.ifNull(getTable(table), Table::getType);
    }

    /**
     * 获取表字段信息
     *
     * @param table  表名称
     * @param column 字段名
     * @return 表字段信息
     */
    public static Field getTableField(@NonNull String table, @NonNull String column) {
        return ObjectUtils.ifNull(getTable(table), tb -> tb.getField(column));
    }
}
