package com.chelvc.framework.database.support;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.core.enums.SqlMethod;
import com.baomidou.mybatisplus.core.injector.AbstractMethod;
import com.baomidou.mybatisplus.core.metadata.TableFieldInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfoHelper;
import com.baomidou.mybatisplus.core.toolkit.sql.SqlScriptUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import lombok.NonNull;
import org.apache.ibatis.executor.keygen.Jdbc3KeyGenerator;
import org.apache.ibatis.executor.keygen.KeyGenerator;
import org.apache.ibatis.executor.keygen.NoKeyGenerator;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.mapping.SqlSource;

/**
 * 数据创建方法抽象实现
 *
 * @author Woody
 * @date 2024/4/3
 */
public abstract class AbstractCreateMethod extends AbstractMethod {
    /**
     * 方法标识
     */
    protected final String id;

    public AbstractCreateMethod(@NonNull String id) {
        this.id = id;
    }

    /**
     * 获取表新增字段信息
     *
     * @param table 表信息
     * @return 字段信息
     */
    protected String getColumns(TableInfo table) {
        StringBuilder buffer = new StringBuilder();
        if (table.havePK() && table.getIdType() != IdType.AUTO) {
            buffer.append(table.getKeyColumn());
        }
        for (TableFieldInfo field : table.getFieldList()) {
            if (buffer.length() > 0) {
                buffer.append(COMMA);
            }
            buffer.append(field.getColumn());
        }
        return buffer.toString();
    }

    /**
     * 获取表新增属性
     *
     * @param table 表信息
     * @return 属性信息
     */
    protected String getProperties(TableInfo table) {
        return this.getProperties(table, null);
    }

    /**
     * 获取表新增属性
     *
     * @param table  表信息
     * @param prefix 属性前缀
     * @return 属性信息
     */
    protected String getProperties(TableInfo table, String prefix) {
        StringBuilder buffer = new StringBuilder();
        prefix = ObjectUtils.ifNull(prefix, StringUtils.EMPTY);
        if (table.havePK() && table.getIdType() != IdType.AUTO) {
            buffer.append(SqlScriptUtils.safeParam(prefix + table.getKeyProperty()));
        }
        for (TableFieldInfo field : table.getFieldList()) {
            if (buffer.length() > 0) {
                buffer.append(COMMA);
            }
            buffer.append(SqlScriptUtils.safeParam(prefix + field.getEl()));
        }
        return buffer.toString();
    }

    /**
     * 初始化方法脚本
     *
     * @param table 表信息
     * @return 脚本信息
     */
    protected abstract String initializeMethodScript(TableInfo table);

    @Override
    public MappedStatement injectMappedStatement(Class<?> mapper, Class<?> model, TableInfo table) {
        String keyColumn = null;
        String keyProperty = null;
        KeyGenerator keyGenerator;
        if (StringUtils.isEmpty(table.getKeyProperty())) {
            keyGenerator = new NoKeyGenerator();
        } else if (table.getIdType() == IdType.AUTO) {
            keyColumn = table.getKeyColumn();
            keyProperty = table.getKeyProperty();
            keyGenerator = new Jdbc3KeyGenerator();
        } else if (table.getKeySequence() != null) {
            keyColumn = table.getKeyColumn();
            keyProperty = table.getKeyProperty();
            String method = this.getMethod(SqlMethod.INSERT_ONE);
            keyGenerator = TableInfoHelper.genKeyGenerator(method, table, this.builderAssistant);
        } else {
            keyGenerator = new NoKeyGenerator();
        }
        String script = this.initializeMethodScript(table);
        SqlSource source = this.languageDriver.createSqlSource(this.configuration, script, model);
        return this.addInsertMappedStatement(
                mapper, model, this.id, source, keyGenerator, keyProperty, keyColumn
        );
    }
}
