package com.chelvc.framework.database.support;

import java.util.Collection;
import java.util.List;
import java.util.function.Function;

import com.baomidou.mybatisplus.core.toolkit.support.SFunction;
import com.baomidou.mybatisplus.extension.conditions.query.ChainQuery;
import com.baomidou.mybatisplus.extension.service.IService;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import lombok.NonNull;

/**
 * 业务增强操作接口
 *
 * @param <T> 数据模型类型
 * @author Woody
 * @date 2024/4/3
 */
public interface EnhanceService<T> extends IService<T> {
    /**
     * 获取增强Mapper
     *
     * @return Mapper实例
     */
    default EnhanceMapper<T> getEnhanceMapper() {
        return (EnhanceMapper<T>) this.getBaseMapper();
    }

    /**
     * 创建或忽略实体
     *
     * @param entity 实体对象实例
     * @return true/false
     */
    default boolean createIgnore(@NonNull T entity) {
        return this.getEnhanceMapper().createIgnore(entity) > 0;
    }

    /**
     * 创建或修改实体
     *
     * @param entity 实体对象实例
     * @return true/false
     */
    default boolean createUpdate(@NonNull T entity) {
        return this.createUpdate(entity, Updates.all(this.getEntityClass()));
    }

    /**
     * 创建或修改实体
     *
     * @param entity 实体对象实例
     * @param update 数据更新对象实例
     * @return true/false
     */
    default boolean createUpdate(@NonNull T entity, @NonNull Updates.Update<T> update) {
        return this.getEnhanceMapper().createUpdate(entity, update) > 0;
    }

    /**
     * 批量创建或忽略实体
     *
     * @param entities 实体对象实例集合
     * @return true/false
     */
    default boolean batchCreateIgnore(@NonNull Collection<T> entities) {
        return !entities.isEmpty() && this.getEnhanceMapper().batchCreateIgnore(entities) > 0;
    }

    /**
     * 批量创建或修改实体
     *
     * @param entities 实体对象实例集合
     * @return true/false
     */
    default boolean batchCreateUpdate(@NonNull Collection<T> entities) {
        return this.batchCreateUpdate(entities, Updates.all(this.getEntityClass()));
    }

    /**
     * 批量创建或修改实体
     *
     * @param entities 实体对象实例集合
     * @param update   数据更新对象实例
     * @return true/false
     */
    default boolean batchCreateUpdate(@NonNull Collection<T> entities, @NonNull Updates.Update<T> update) {
        return !entities.isEmpty() && this.getEnhanceMapper().batchCreateUpdate(entities, update) > 0;
    }

    /**
     * 判断指定字段值记录是否存在
     *
     * @param column 字段函数
     * @param value  字段值
     * @return true/false
     */
    default boolean exists(@NonNull SFunction<T, ?> column, @NonNull Object value) {
        Integer count = this.lambdaQuery().eq(column, value).count();
        return count != null && count > 0;
    }

    /**
     * 分页查询
     *
     * @param paging 查询分页信息
     * @return 对象分页信息
     */
    default Pagination<T> paging(@NonNull Paging paging) {
        return this.paging(this.lambdaQuery(), paging);
    }

    /**
     * 分页查询
     *
     * @param paging  查询分页信息
     * @param adapter 数据转换适配器
     * @param <R>     目标数据类型
     * @return 对象分页信息
     */
    default <R> Pagination<R> paging(@NonNull Paging paging, @NonNull Function<List<T>, List<R>> adapter) {
        return this.paging(this.lambdaQuery(), paging, adapter);
    }

    /**
     * 分页查询
     *
     * @param query  查询对象
     * @param paging 查询分页信息
     * @return 对象分页信息
     */
    default Pagination<T> paging(@NonNull ChainQuery<T> query, @NonNull Paging paging) {
        return DatabaseContextHolder.paging(paging, query::page);
    }

    /**
     * 分页查询
     *
     * @param query   查询对象
     * @param paging  查询分页信息
     * @param adapter 数据转换适配器
     * @param <R>     目标数据类型
     * @return 对象分页信息
     */
    default <R> Pagination<R> paging(@NonNull ChainQuery<T> query, @NonNull Paging paging,
                                     @NonNull Function<List<T>, List<R>> adapter) {
        return DatabaseContextHolder.paging(paging, query::page, adapter);
    }
}
