package com.chelvc.framework.database.support;

import java.io.Serializable;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.TreeUtils;
import com.chelvc.framework.database.entity.TreeEntity;
import lombok.NonNull;

/**
 * 树形模型业务增强处理接口
 *
 * @param <T> 数据模型类型
 * @author Woody
 * @date 2024/4/29
 */
public interface TreeService<T extends TreeEntity<? extends Serializable, T>> extends EnhanceService<T> {
    /**
     * 根据ID获取整颗树
     *
     * @param id 数据ID
     * @return 树实例
     */
    T tree(Serializable id);

    /**
     * 获取所有树
     *
     * @return 树列表
     */
    List<T> trees();

    /**
     * 根据ID获取对应树列表
     *
     * @param ids 数据ID集合
     * @return 树列表
     */
    List<T> trees(Collection<? extends Serializable> ids);

    /**
     * 根据ID获取对应树列表
     *
     * @param ids      数据ID集合
     * @param drilling 是否钻取子节点
     * @return 树列表
     */
    default List<T> trees(@NonNull Collection<? extends Serializable> ids, boolean drilling) {
        if (ObjectUtils.isEmpty(ids)) {
            return Collections.emptyList();
        }
        return drilling ? TreeUtils.assemble(this.families(ids)) : this.trees(ids);
    }

    /**
     * 根据ID获取子节点
     *
     * @param id 数据ID
     * @return 子节点列表
     */
    List<T> children(Serializable id);

    /**
     * 批量根据ID获取子节点
     *
     * @param ids 数据ID集合
     * @return 子节点列表
     */
    List<T> children(Collection<? extends Serializable> ids);

    /**
     * 根据ID获取整个家族节点
     *
     * @param id 数据ID
     * @return 树节点列表
     */
    List<T> families(Serializable id);

    /**
     * 批量根据ID获取整个家族节点
     *
     * @param ids 数据ID集合
     * @return 树节点列表
     */
    List<T> families(Collection<? extends Serializable> ids);
}
