package com.chelvc.framework.database.util;

import com.chelvc.framework.common.util.StringUtils;

/**
 * SQL工具类
 *
 * @author Woody
 * @date 2025/3/16
 */
public final class SQLUtils {
    /**
     * DB2 JDBC连接前缀
     */
    private static final String JDBC_DB2_PREFIX = "jdbc:db2://";

    /**
     * MySQL JDBC连接前缀
     */
    private static final String JDBC_MYSQL_PREFIX = "jdbc:mysql://";

    /**
     * Oracle JDBC连接前缀
     */
    private static final String JDBC_ORACLE_PREFIX = "jdbc:oracle:thin:@";

    /**
     * SqlServer JDBC连接前缀
     */
    private static final String JDBC_SQLSERVER_PREFIX = "jdbc:sqlserver://";

    /**
     * Postgresql JDBC连接前缀
     */
    private static final String JDBC_POSTGRESQL_PREFIX = "jdbc:postgresql://";


    private SQLUtils() {
    }

    /**
     * 字符串参数特殊字符转义
     *
     * @param value 参数值
     * @return 参数值
     */
    public static String escape(String value) {
        if (StringUtils.isEmpty(value)) {
            return value;
        }

        // 判断参数是否需要转义
        boolean needs = false;
        int size = value.length();
        for (int i = 0; i < size; i++) {
            char c = value.charAt(i);
            if (c == '\u0000' || c == '\n' || c == '\r' || c == '\u001a' || c == '"' || c == '\'' || c == '\\') {
                needs = true;
                break;
            }
        }
        if (!needs) {
            return value;
        }

        // 特殊字符转义
        StringBuilder buffer = new StringBuilder((int) ((double) size * 1.1D));
        for (int i = 0; i < size; i++) {
            char c = value.charAt(i);
            switch (c) {
                case '\u0000':
                    buffer.append('\\');
                    buffer.append('0');
                    break;
                case '\n':
                    buffer.append('\\');
                    buffer.append('n');
                    break;
                case '\r':
                    buffer.append('\\');
                    buffer.append('r');
                    break;
                case '\u001a':
                    buffer.append('\\');
                    buffer.append('Z');
                    break;
                case '\'':
                    buffer.append('\'');
                    buffer.append('\'');
                    break;
                case '\\':
                    buffer.append('\\');
                    buffer.append('\\');
                    break;
                default:
                    buffer.append(c);
            }
        }
        return buffer.toString();
    }

    /**
     * 去掉表、字段名称反引号
     *
     * @param name 表、字段名称
     * @return 表、字段名称
     */
    public static String unquote(String name) {
        return isQuoted(name) ? name.substring(1, name.length() - 1) : name;
    }

    /**
     * 判断表、字段名称是否包含反引号
     *
     * @param name 表、字段名称
     * @return true/false
     */
    public static boolean isQuoted(String name) {
        return StringUtils.notEmpty(name) && name.charAt(0) == '`' && name.charAt(name.length() - 1) == '`';
    }

    /**
     * 获取JDBC连接schema
     *
     * @param url JDBC连接
     * @return schema
     */
    public static String getSchema(String url) {
        if (StringUtils.isEmpty(url) || (url = url.trim()).isEmpty()) {
            return null;
        } else if (url.startsWith(JDBC_DB2_PREFIX)) {
            int begin = url.indexOf('/', JDBC_DB2_PREFIX.length()), end = url.indexOf('?', begin);
            return end < 0 ? url.substring(begin + 1).trim() : url.substring(begin + 1, end).trim();
        } else if (url.startsWith(JDBC_MYSQL_PREFIX)) {
            int begin = url.indexOf('/', JDBC_MYSQL_PREFIX.length()), end = url.indexOf('?', begin);
            return end < 0 ? url.substring(begin + 1).trim() : url.substring(begin + 1, end).trim();
        } else if (url.startsWith(JDBC_ORACLE_PREFIX)) {
            if (url.charAt(JDBC_ORACLE_PREFIX.length()) == ':') {
                int begin = url.lastIndexOf(':'), end = url.indexOf(';', begin);
                return end < 0 ? url.substring(begin + 1).trim() : url.substring(begin + 1, end).trim();
            }
            int begin = url.indexOf('/', JDBC_ORACLE_PREFIX.length() + 2), end = url.indexOf(';', begin);
            return end < 0 ? url.substring(begin + 1).trim() : url.substring(begin + 1, end).trim();
        } else if (url.startsWith(JDBC_SQLSERVER_PREFIX)) {
            String parameters = url.substring(url.indexOf(';', JDBC_SQLSERVER_PREFIX.length()) + 1);
            for (String parameter : parameters.split(";")) {
                int delimiter = parameter.indexOf('=');
                if (delimiter > 0 && parameter.substring(0, delimiter).trim().equalsIgnoreCase("databaseName")) {
                    return parameter.substring(delimiter + 1).trim();
                }
            }
        } else if (url.startsWith(JDBC_POSTGRESQL_PREFIX)) {
            int begin = url.indexOf('/', JDBC_POSTGRESQL_PREFIX.length()), end = url.indexOf('?', begin);
            return end < 0 ? url.substring(begin + 1).trim() : url.substring(begin + 1, end).trim();
        }
        throw new IllegalArgumentException("Unknown jdbc url: " + url);
    }
}
