package com.chelvc.framework.download;

import java.io.IOException;
import java.util.Collection;
import java.util.function.BiConsumer;
import java.util.function.BiFunction;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.common.function.Adapter;
import com.chelvc.framework.common.function.MultipleConsumer;
import com.chelvc.framework.common.function.MultipleFunction;
import com.chelvc.framework.common.function.OutputAcceptor;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Workbook;

/**
 * 数据下载处理器接口
 *
 * @author Woody
 * @date 2024/6/26
 */
public interface DownloadHandler {
    /**
     * 初始化处理器
     */
    default void initialize() {
    }

    /**
     * 获取下载任务状态
     *
     * @param id 下载任务ID
     * @return 下载任务状态
     */
    Download.Status status(long id);

    /**
     * 写入Excel下载数据
     *
     * @param id       下载任务ID
     * @param workbook Excel工作薄
     * @throws IOException I/O异常
     */
    void write(long id, Workbook workbook) throws IOException;

    /**
     * 写入下载数据
     *
     * @param id       下载任务ID
     * @param acceptor 输出流回调函数
     * @throws IOException I/O异常
     */
    void write(long id, OutputAcceptor acceptor) throws IOException;

    /**
     * 下载数据
     *
     * @param id       下载任务ID
     * @param response Http响应对象
     * @throws IOException I/O操作异常
     */
    void download(long id, HttpServletResponse response) throws IOException;

    /**
     * 异步下载数据
     *
     * @param filename 文件名称
     * @param adapter  数据处理回调函数
     * @return 下载任务ID
     */
    default long async(String filename, Adapter<Long, Integer> adapter) {
        return this.async(filename, adapter, (id, volume) -> {
        });
    }

    /**
     * 异步下载数据
     *
     * @param filename 文件名称
     * @param adapter  数据处理回调函数
     * @param listener 处理完成监听器
     * @return 下载任务ID
     */
    long async(String filename, Adapter<Long, Integer> adapter, BiConsumer<Long, Integer> listener);

    /**
     * 异步下载数据
     *
     * @param filename 文件名称
     * @param provider 数据生产者
     * @param writer   Excel对象实例写入接口
     * @param titles   表头数组
     * @param <T>      数据类型
     * @return 下载任务ID
     */
    default <T> long async(String filename, BiFunction<Integer, Integer, Collection<T>> provider,
                           MultipleConsumer<Row, T, Integer> writer, String... titles) {
        return this.async(filename, provider, (row, object, count) -> {
            writer.accept(row, object, count);
            return row.getRowNum();
        }, titles);
    }

    /**
     * 异步下载数据
     *
     * @param filename 文件名称
     * @param provider 数据生产者
     * @param writer   Excel对象实例写入接口
     * @param titles   表头数组
     * @param <T>      数据类型
     * @return 下载任务ID
     */
    default <T> long async(String filename, BiFunction<Integer, Integer, Collection<T>> provider,
                           MultipleFunction<Row, T, Integer, Integer> writer, String... titles) {
        return this.async(
                filename,
                (id, volume) -> {
                },
                provider, writer, titles
        );
    }

    /**
     * 异步下载数据
     *
     * @param filename 文件名称
     * @param listener 处理完成监听器
     * @param provider 数据生产者
     * @param writer   Excel对象实例写入接口
     * @param titles   表头数组
     * @param <T>      数据类型
     * @return 下载任务ID
     */
    default <T> long async(String filename, BiConsumer<Long, Integer> listener,
                           BiFunction<Integer, Integer, Collection<T>> provider,
                           MultipleConsumer<Row, T, Integer> writer, String... titles) {
        return this.async(filename, listener, provider, (row, object, count) -> {
            writer.accept(row, object, count);
            return row.getRowNum();
        }, titles);
    }

    /**
     * 异步下载数据
     *
     * @param filename 文件名称
     * @param listener 处理完成监听器
     * @param provider 数据生产者
     * @param writer   Excel对象实例写入接口
     * @param titles   表头数组
     * @param <T>      数据类型
     * @return 下载任务ID
     */
    <T> long async(String filename, BiConsumer<Long, Integer> listener,
                   BiFunction<Integer, Integer, Collection<T>> provider,
                   MultipleFunction<Row, T, Integer, Integer> writer, String... titles);

    /**
     * 销毁处理器
     */
    default void destroy() {
    }
}
