package com.chelvc.framework.download.support;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.Serializable;
import java.time.Duration;
import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.function.BiConsumer;
import java.util.function.BiFunction;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.ThreadContextHolder;
import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.common.function.Adapter;
import com.chelvc.framework.common.function.Executor;
import com.chelvc.framework.common.function.MultipleFunction;
import com.chelvc.framework.common.function.OutputAcceptor;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ExcelUtils;
import com.chelvc.framework.common.util.FileUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.ThreadUtils;
import com.chelvc.framework.download.Download;
import com.chelvc.framework.download.DownloadHandler;
import com.chelvc.framework.download.config.DownloadProperties;
import com.chelvc.framework.redis.context.RedisContextHolder;
import com.google.common.collect.Lists;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.springframework.boot.context.event.ApplicationStartedEvent;
import org.springframework.context.ApplicationListener;

/**
 * 数据下载处理器默认实现
 *
 * @author Woody
 * @date 2024/5/9
 */
@Slf4j
public class DefaultDownloadHandler implements DownloadHandler, ApplicationListener<ApplicationStartedEvent> {
    private final String directory;
    private final Duration duration;

    public DefaultDownloadHandler(@NonNull DownloadProperties properties) {
        this.directory = properties.getDirectory();
        this.duration = Duration.ofSeconds(properties.getExpiration());
    }

    /**
     * 清理下载文件
     */
    private void clear() {
        int batch = 100;
        List<File> files = Lists.newArrayListWithCapacity(batch);
        FileUtils.files(this.directory, file -> {
            files.add(file);
            if (files.size() == batch) {
                this.clear(files);
                files.clear();
            }
        });
        if (ObjectUtils.notEmpty(files)) {
            this.clear(files);
        }
    }

    /**
     * 清理下载文件
     *
     * @param files 文件列表
     */
    private void clear(List<File> files) {
        List<String> keys = files.stream().map(File::getName).map(this::key).collect(Collectors.toList());
        List<?> values = RedisContextHolder.getRedisTemplate().opsForValue().multiGet(keys);
        if (ObjectUtils.notEmpty(values)) {
            for (int i = 0; i < values.size(); i++) {
                if (Objects.isNull(values.get(i))) {
                    FileUtils.delete(files.get(i));
                }
            }
        }
    }

    /**
     * 构建任务标识
     *
     * @param id 任务ID
     * @return 任务标识
     */
    private String key(Serializable id) {
        return "download:" + id;
    }

    /**
     * 获取下载任务信息
     *
     * @param id 下载任务ID
     * @return 下载任务信息
     */
    private Download download(Serializable id) {
        return RedisContextHolder.get(this.key(id));
    }

    /**
     * 处理下载任务信息
     *
     * @param id       任务ID
     * @param filename 文件名称
     * @param status   任务状态
     * @param duration 任务有效期
     */
    private void processing(Serializable id, String filename, Download.Status status, Duration duration) {
        Download download = Download.builder().filename(filename).status(status).build();
        RedisContextHolder.set(this.key(id), download, duration);
    }

    @Override
    public void onApplicationEvent(ApplicationStartedEvent event) {
        // 初始化下载文件清理线程
        ThreadUtils.run(() -> {
            while (!Thread.currentThread().isInterrupted()) {
                try {
                    RedisContextHolder.tryLockAround("download:clear:lock", (Executor) this::clear);
                } catch (Exception e) {
                    log.error("Download temp file clear failed", e);
                }
                ThreadUtils.sleep(this.duration.toMillis());
            }
        });
    }

    @Override
    public Download.Status status(long id) {
        return ObjectUtils.ifNull(this.download(id), Download::getStatus);
    }

    @Override
    public void write(long id, @NonNull Workbook workbook) throws IOException {
        this.write(id, output -> {
            workbook.write(output);
            workbook.close();
        });
    }

    @Override
    public void write(long id, @NonNull OutputAcceptor acceptor) throws IOException {
        File file = new File(this.directory, String.valueOf(id));
        try (OutputStream output = new FileOutputStream(file)) {
            acceptor.accept(output);
        }
    }

    @Override
    public void download(long id, @NonNull HttpServletResponse response) throws IOException {
        Download download = this.download(id);
        AssertUtils.nonnull(download, () -> ApplicationContextHolder.getMessage("Download.Expired"));
        boolean completed = ObjectUtils.ifNull(download, Download::getStatus) == Download.Status.COMPLETED;
        AssertUtils.available(completed, () -> ApplicationContextHolder.getMessage("Download.Uncompleted"));
        String filename = ObjectUtils.ifNull(download, Download::getFilename);
        HttpUtils.write(response, new File(this.directory, String.valueOf(id)), filename);
    }

    @Override
    public long async(@NonNull String filename, @NonNull Adapter<Long, Integer> adapter,
                      @NonNull BiConsumer<Long, Integer> listener) {
        // 生成任务ID
        long id = RedisContextHolder.identity();

        // 初始化任务状态
        this.processing(id, filename, Download.Status.RUNNING, Duration.ofDays(1));

        // 异步执行任务处理
        ThreadContextHolder.run(() -> {
            int volume;
            try {
                // 任务处理回调
                volume = adapter.apply(id);

                // 更新任务完成状态
                this.processing(id, filename, Download.Status.COMPLETED, this.duration);
            } catch (Exception e) {
                log.error("Download execute failed", e);

                // 更新任务失败状态
                this.processing(id, filename, Download.Status.FAILED, this.duration);
                return;
            }

            // 任务完成监听回调
            try {
                listener.accept(id, volume);
            } catch (Exception e) {
                log.error("Download listening failed", e);
            }
        });
        return id;
    }

    @Override
    public <T> long async(@NonNull String filename, @NonNull BiConsumer<Long, Integer> listener,
                          @NonNull BiFunction<Integer, Integer, Collection<T>> provider,
                          @NonNull MultipleFunction<Row, T, Integer, Integer> writer, @NonNull String... titles) {
        return this.async(filename, id -> {
            Workbook workbook = new SXSSFWorkbook();
            int volume = ExcelUtils.write(workbook, provider, writer, titles);
            this.write(id, workbook);
            return volume;
        }, listener);
    }
}
