package com.chelvc.framework.dubbo.adapter;

import java.util.Collections;
import java.util.Map;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.ObjectUtils;
import com.google.common.collect.Maps;
import org.springframework.aop.framework.AopProxyUtils;
import org.springframework.core.ResolvableType;

/**
 * Dubbo对象适配器工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public class DubboObjectAdapterHolder {
    /**
     * 对象适配器映射表
     */
    private static volatile Map<Class<?>, DubboObjectAdapterWrapper<?, ?>> CLASS_WRAPPER_MAPPING;

    /**
     * 将对象适配器转换成包装对象实例
     *
     * @param adapter 对象适配器
     * @param <O>     原始对象类型
     * @param <T>     目标对象类型
     * @return 对象适配器包装对象实例
     */
    @SuppressWarnings("unchecked")
    private static <O, T> DubboObjectAdapterWrapper<O, T> adapter2wrapper(DubboObjectAdapter<O, T> adapter) {
        Class<?> clazz = AopProxyUtils.ultimateTargetClass(adapter);
        ResolvableType type = ResolvableType.forClass(clazz);
        ResolvableType[] interfaces = type.getInterfaces();
        Class<O> original = (Class<O>) interfaces[0].getGeneric(0).resolve();
        Class<T> target = (Class<T>) interfaces[0].getGeneric(1).resolve();
        return DubboObjectAdapterWrapper.<O, T>builder().original(original).target(target).adapter(adapter).build();
    }

    /**
     * 获取对象适配包装对象映射表
     *
     * @return 对象适配包装对象映射表
     */
    @SuppressWarnings({"rawtypes", "unchecked"})
    public static Map<Class<?>, DubboObjectAdapterWrapper<?, ?>> getAdapterWrappers() {
        if (CLASS_WRAPPER_MAPPING == null) {
            synchronized (DubboObjectAdapterWrapper.class) {
                if (CLASS_WRAPPER_MAPPING == null) {
                    Map<String, DubboObjectAdapter> adapters =
                            ApplicationContextHolder.getBeans(DubboObjectAdapter.class);
                    if (ObjectUtils.isEmpty(adapters)) {
                        CLASS_WRAPPER_MAPPING = Collections.emptyMap();
                    } else {
                        Map<Class<?>, DubboObjectAdapterWrapper<?, ?>> wrappers =
                                Maps.newHashMapWithExpectedSize(adapters.size());
                        adapters.forEach((key, adapter) -> {
                            DubboObjectAdapterWrapper<?, ?> wrapper = adapter2wrapper(adapter);
                            wrappers.put(wrapper.getOriginal(), wrapper);
                        });
                        CLASS_WRAPPER_MAPPING = Collections.unmodifiableMap(wrappers);
                    }
                }
            }
        }
        return CLASS_WRAPPER_MAPPING;
    }

    /**
     * 查找对象适配器
     *
     * @param type 对象类型
     * @param <O>  原始对象类型
     * @param <T>  目标对象类型
     * @return 对象适配器实例
     */
    @SuppressWarnings("unchecked")
    public static <O, T> DubboObjectAdapterWrapper<O, T> lookupObjectAdapter(Class<?> type) {
        return (DubboObjectAdapterWrapper<O, T>) getAdapterWrappers().get(type);
    }
}
