package com.chelvc.framework.dubbo.interceptor;

import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.dubbo.adapter.DubboObjectAdapterHolder;
import com.chelvc.framework.dubbo.adapter.DubboObjectAdapterWrapper;
import org.apache.dubbo.common.constants.CommonConstants;
import org.apache.dubbo.common.extension.Activate;
import org.apache.dubbo.rpc.AsyncRpcResult;
import org.apache.dubbo.rpc.Filter;
import org.apache.dubbo.rpc.Invocation;
import org.apache.dubbo.rpc.Invoker;
import org.apache.dubbo.rpc.Result;
import org.apache.dubbo.rpc.RpcContext;
import org.apache.dubbo.rpc.RpcException;
import org.apache.dubbo.rpc.RpcInvocation;

/**
 * Dubbo 消费者处理拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Activate(group = CommonConstants.CONSUMER)
public class DubboConsumerInterceptor implements Filter {
    /**
     * 查找接口对象适配器
     *
     * @param invocation Dubbo接口调用信息
     * @return Dubbo接口对象适配器
     */
    private DubboObjectAdapterWrapper<?, Object> lookupObjectAdapter(Invocation invocation) {
        if (!(invocation instanceof RpcInvocation)) {
            return null;
        }

        // 根据接口返回类型查找对象适配器
        Class<?> returnType = ((RpcInvocation) invocation).getReturnType();
        if (returnType == void.class) {
            return null;
        }
        DubboObjectAdapterWrapper<?, Object> wrapper = DubboObjectAdapterHolder.lookupObjectAdapter(returnType);
        if (wrapper != null) {
            // 设置适配目标对象类型
            ((RpcInvocation) invocation).setReturnType(wrapper.getTarget());
            ((RpcInvocation) invocation).getReturnTypes()[0] = wrapper.getTarget();
        }
        return wrapper;
    }

    /**
     * 接口调用处理
     *
     * @param invoker    接口调用器
     * @param invocation 调用信息
     * @return 调用结果
     * @throws RpcException 接口调用异常
     */
    private Result processing(Invoker<?> invoker, Invocation invocation) throws RpcException {
        // 获取接口调用结果
        Result result = invoker.invoke(invocation);
        Object value = result.getValue();
        if (value == null) {
            return result;
        }

        // 自定义对象解包
        DubboObjectAdapterWrapper<?, Object> wrapper = this.lookupObjectAdapter(invocation);
        if (wrapper == null) {
            return result;
        }
        value = wrapper.getAdapter().unwrap(result.getValue());
        return AsyncRpcResult.newDefaultAsyncResult(value, invocation);
    }

    @Override
    public Result invoke(Invoker<?> invoker, Invocation invocation) throws RpcException {
        Session session = SessionContextHolder.getSession(false);
        RpcContext.getClientAttachment().setObjectAttachment(Session.NAMING, session);
        try {
            return this.processing(invoker, invocation);
        } finally {
            RpcContext.getClientAttachment().clearAttachments();
        }
    }
}
