package com.chelvc.framework.elasticsearch;

import java.io.IOException;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.function.Function;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch._types.Conflicts;
import co.elastic.clients.elasticsearch._types.Script;
import co.elastic.clients.elasticsearch._types.ScriptLanguage;
import co.elastic.clients.elasticsearch._types.query_dsl.Query;
import co.elastic.clients.elasticsearch.core.BulkRequest;
import co.elastic.clients.elasticsearch.core.BulkResponse;
import co.elastic.clients.elasticsearch.core.CountRequest;
import co.elastic.clients.elasticsearch.core.CountResponse;
import co.elastic.clients.elasticsearch.core.CreateRequest;
import co.elastic.clients.elasticsearch.core.CreateResponse;
import co.elastic.clients.elasticsearch.core.DeleteByQueryRequest;
import co.elastic.clients.elasticsearch.core.DeleteByQueryResponse;
import co.elastic.clients.elasticsearch.core.DeleteRequest;
import co.elastic.clients.elasticsearch.core.DeleteResponse;
import co.elastic.clients.elasticsearch.core.ExistsRequest;
import co.elastic.clients.elasticsearch.core.GetRequest;
import co.elastic.clients.elasticsearch.core.GetResponse;
import co.elastic.clients.elasticsearch.core.IndexRequest;
import co.elastic.clients.elasticsearch.core.IndexResponse;
import co.elastic.clients.elasticsearch.core.MgetRequest;
import co.elastic.clients.elasticsearch.core.MgetResponse;
import co.elastic.clients.elasticsearch.core.ScrollRequest;
import co.elastic.clients.elasticsearch.core.ScrollResponse;
import co.elastic.clients.elasticsearch.core.SearchRequest;
import co.elastic.clients.elasticsearch.core.SearchResponse;
import co.elastic.clients.elasticsearch.core.UpdateByQueryRequest;
import co.elastic.clients.elasticsearch.core.UpdateByQueryResponse;
import co.elastic.clients.elasticsearch.core.UpdateRequest;
import co.elastic.clients.elasticsearch.core.UpdateResponse;
import co.elastic.clients.elasticsearch.core.bulk.BulkOperation;
import co.elastic.clients.elasticsearch.core.bulk.CreateOperation;
import co.elastic.clients.elasticsearch.core.bulk.DeleteOperation;
import co.elastic.clients.elasticsearch.core.bulk.IndexOperation;
import co.elastic.clients.elasticsearch.core.bulk.UpdateOperation;
import co.elastic.clients.json.JsonData;
import co.elastic.clients.transport.ElasticsearchTransport;
import co.elastic.clients.transport.endpoints.BooleanResponse;
import co.elastic.clients.util.ObjectBuilder;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.elasticsearch.annotation.Index;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.NonNull;

/**
 * ES操作接口默认实现
 *
 * @author Woody
 * @date 2024/10/17
 */
public class DefaultElasticsearchHandler implements ElasticsearchHandler {
    private final ElasticsearchClient client;
    private final Map<Class<?>, String> indexes = Maps.newConcurrentMap();

    public DefaultElasticsearchHandler(@NonNull ElasticsearchTransport transport) {
        this.client = new ElasticsearchClient(transport);
    }

    @Override
    public ElasticsearchClient getClient() {
        return this.client;
    }

    @Override
    public String getIndex(@NonNull Class<?> model) {
        String name = this.indexes.get(model);
        return name != null ? name : this.indexes.computeIfAbsent(model, k -> {
            Index index = model.getAnnotation(Index.class);
            String value = ObjectUtils.ifNull(index, Index::value);
            return StringUtils.notEmpty(value) ? value : StringUtils.hump2underscore(model.getSimpleName());
        });
    }

    @Override
    public CreateResponse create(@NonNull Document<?> document) {
        String index = this.getIndex(document.getClass());
        String id = ObjectUtils.ifNull(document.getId(), String::valueOf);
        CreateRequest<?> request = new CreateRequest.Builder<>().index(index).id(id).document(document).build();
        try {
            return this.client.create(request);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public <T extends Document<?>> BulkResponse create(@NonNull Collection<T> documents) {
        List<BulkOperation> operations = Lists.newArrayListWithCapacity(documents.size());
        for (T document : documents) {
            String index = this.getIndex(document.getClass());
            String id = ObjectUtils.ifNull(document.getId(), String::valueOf);
            CreateOperation<?> create = new CreateOperation.Builder<>().index(index).id(id).document(document).build();
            BulkOperation operation = new BulkOperation.Builder().create(create).build();
            operations.add(operation);
        }
        BulkRequest request = new BulkRequest.Builder().operations(operations).build();
        try {
            return this.client.bulk(request);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public DeleteResponse delete(@NonNull Class<?> model, @NonNull String id) {
        DeleteRequest request = new DeleteRequest.Builder().index(this.getIndex(model)).id(id).build();
        try {
            return this.client.delete(request);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public BulkResponse delete(@NonNull Class<?> model, @NonNull Collection<String> ids) {
        String index = this.getIndex(model);
        List<BulkOperation> operations = Lists.newArrayListWithCapacity(ids.size());
        ids.forEach(id -> {
            DeleteOperation delete = new DeleteOperation.Builder().index(index).id(id).build();
            BulkOperation operation = new BulkOperation.Builder().delete(delete).build();
            operations.add(operation);
        });
        BulkRequest request = new BulkRequest.Builder().operations(operations).build();
        try {
            return this.client.bulk(request);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public DeleteByQueryResponse delete(@NonNull Class<?> model, @NonNull Query query) {
        DeleteByQueryRequest request = new DeleteByQueryRequest.Builder().index(this.getIndex(model)).query(query)
                .conflicts(Conflicts.Proceed).build();
        try {
            return this.client.deleteByQuery(request);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public <T extends Document<?>> UpdateResponse<T> update(@NonNull T document) {
        return this.update(document, null, null);
    }

    @Override
    public <T extends Document<?>> BulkResponse update(@NonNull Collection<T> documents) {
        List<BulkOperation> operations = Lists.newArrayListWithCapacity(documents.size());
        for (T document : documents) {
            String index = this.getIndex(document.getClass());
            String id = ObjectUtils.ifNull(document.getId(), String::valueOf);
            UpdateOperation<T, Object> update = new UpdateOperation.Builder<T, Object>().index(index).id(id)
                    .action(a -> a.doc(document)).retryOnConflict(3).build();
            BulkOperation operation = new BulkOperation.Builder().update(update).build();
            operations.add(operation);
        }
        BulkRequest request = new BulkRequest.Builder().operations(operations).build();
        try {
            return this.client.bulk(request);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    @SuppressWarnings("unchecked")
    public <T extends Document<?>> UpdateResponse<T> update(@NonNull T document, @NonNull Long primary,
                                                            @NonNull Long sequence) {
        Class<T> model = (Class<T>) document.getClass();
        String id = ObjectUtils.ifNull(document.getId(), String::valueOf);
        return this.update(model, id, document, primary, sequence);
    }

    @Override
    public <T> UpdateResponse<T> update(@NonNull Class<T> model, @NonNull String id, @NonNull Object document) {
        return this.update(model, id, document, null, null);
    }

    @Override
    public UpdateByQueryResponse update(@NonNull Class<?> model, @NonNull Query query, @NonNull String field,
                                        Object value) {
        String source = "ctx._source." + field + (value == null ? "=null" : ("=params." + field));
        Map<String, JsonData> params = value == null ? Collections.emptyMap() :
                ImmutableMap.of(field, JsonData.of(value));
        Script script = new Script.Builder().lang(ScriptLanguage.Painless).source(source).params(params).build();
        UpdateByQueryRequest request = new UpdateByQueryRequest.Builder().index(this.getIndex(model)).script(script)
                .query(query).conflicts(Conflicts.Proceed).build();
        try {
            return this.client.updateByQuery(request);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public UpdateByQueryResponse update(@NonNull Class<?> model, @NonNull Query query,
                                        @NonNull Map<String, ?> parameters) {
        StringBuilder source = new StringBuilder();
        Map<String, JsonData> params = Maps.newHashMapWithExpectedSize(parameters.size());
        parameters.forEach((key, value) -> {
            if (source.length() > 0) {
                source.append(";");
            }
            if (value == null) {
                source.append("ctx._source.").append(key).append("=null");
            } else {
                source.append("ctx._source.").append(key).append("=params.").append(key);
                params.put(key, JsonData.of(value));
            }
        });
        Script script = new Script.Builder().lang(ScriptLanguage.Painless).source(source.toString())
                .params(params).build();
        UpdateByQueryRequest request = new UpdateByQueryRequest.Builder().index(this.getIndex(model)).script(script)
                .query(query).conflicts(Conflicts.Proceed).build();
        try {
            return this.client.updateByQuery(request);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public <T> UpdateResponse<T> update(@NonNull Class<T> model, @NonNull String id, @NonNull Object document,
                                        Long primary, Long sequence) {
        UpdateRequest<T, ?> request = new UpdateRequest.Builder<T, Object>().index(this.getIndex(model)).id(id)
                .doc(document).ifPrimaryTerm(primary).ifSeqNo(sequence).retryOnConflict(3).build();
        try {
            return this.client.update(request, model);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public IndexResponse index(@NonNull Document<?> document) {
        return this.index(document, null, null);
    }

    @Override
    public IndexResponse index(@NonNull Class<?> model, @NonNull String id, @NonNull Object document) {
        return this.index(model, id, document, null, null);
    }

    @Override
    public IndexResponse index(@NonNull Document<?> document, Long primary, Long sequence) {
        Class<?> model = document.getClass();
        String id = ObjectUtils.ifNull(document.getId(), String::valueOf);
        return this.index(model, id, document, primary, sequence);
    }

    @Override
    public IndexResponse index(@NonNull Class<?> model, @NonNull String id, @NonNull Object document, Long primary,
                               Long sequence) {
        IndexRequest<?> request = new IndexRequest.Builder<>().index(this.getIndex(model))
                .id(id).document(document).ifPrimaryTerm(primary).ifSeqNo(sequence).build();
        try {
            return this.client.index(request);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public <T extends Document<?>> BulkResponse index(@NonNull Collection<T> documents) {
        return this.index(documents, null, null);
    }

    @Override
    public <T extends Document<?>> BulkResponse index(@NonNull Collection<T> documents, Long primary, Long sequence) {
        List<BulkOperation> operations = Lists.newArrayListWithCapacity(documents.size());
        for (T document : documents) {
            String index = this.getIndex(document.getClass());
            String id = ObjectUtils.ifNull(document.getId(), String::valueOf);
            IndexOperation<?> operation = new IndexOperation.Builder<>().index(index).id(id).document(document)
                    .ifPrimaryTerm(primary).ifSeqNo(sequence).build();
            operations.add(new BulkOperation.Builder().index(operation).build());
        }
        BulkRequest request = new BulkRequest.Builder().operations(operations).build();
        try {
            return this.client.bulk(request);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public CountResponse count(@NonNull Class<?> model, @NonNull Query query) {
        CountRequest request = new CountRequest.Builder().index(this.getIndex(model)).query(query).build();
        try {
            return this.client.count(request);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public BooleanResponse exists(@NonNull Class<?> model, @NonNull String id) {
        ExistsRequest request = new ExistsRequest.Builder().index(this.getIndex(model)).id(id).build();
        try {
            return this.client.exists(request);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public <T> GetResponse<T> get(@NonNull Class<T> model, @NonNull String id) {
        GetRequest request = new GetRequest.Builder().index(this.getIndex(model)).id(id).build();
        try {
            return this.client.get(request, model);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public <T> MgetResponse<T> get(@NonNull Class<T> model, @NonNull List<String> ids) {
        MgetRequest request = new MgetRequest.Builder().index(this.getIndex(model)).ids(ids).build();
        try {
            return this.client.mget(request, model);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public <T> SearchResponse<T> search(@NonNull Class<T> model, @NonNull Query query) {
        SearchRequest request = new SearchRequest.Builder().index(this.getIndex(model)).query(query).build();
        return this.search(model, request);
    }

    @Override
    public <T> SearchResponse<T> search(@NonNull Class<T> model, @NonNull Query query,
                                        @NonNull Function<SearchRequest.Builder, ObjectBuilder<SearchRequest>> function) {
        SearchRequest.Builder builder = new SearchRequest.Builder().index(this.getIndex(model)).query(query);
        SearchRequest request = function.apply(builder).build();
        return this.search(model, request);
    }

    @Override
    public <T> SearchResponse<T> search(@NonNull Class<T> model, @NonNull SearchRequest request) {
        try {
            return this.client.search(request, model);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public <T> ScrollResponse<T> scroll(@NonNull Class<T> model, @NonNull String id, @NonNull String time) {
        ScrollRequest request = new ScrollRequest.Builder().scrollId(id).scroll(s -> s.time(time)).build();
        try {
            return this.client.scroll(request, model);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
