package com.chelvc.framework.elasticsearch;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.function.Function;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch._types.query_dsl.Query;
import co.elastic.clients.elasticsearch.core.BulkResponse;
import co.elastic.clients.elasticsearch.core.CountResponse;
import co.elastic.clients.elasticsearch.core.CreateResponse;
import co.elastic.clients.elasticsearch.core.DeleteByQueryResponse;
import co.elastic.clients.elasticsearch.core.DeleteResponse;
import co.elastic.clients.elasticsearch.core.GetResponse;
import co.elastic.clients.elasticsearch.core.IndexResponse;
import co.elastic.clients.elasticsearch.core.MgetResponse;
import co.elastic.clients.elasticsearch.core.ScrollResponse;
import co.elastic.clients.elasticsearch.core.SearchRequest;
import co.elastic.clients.elasticsearch.core.SearchResponse;
import co.elastic.clients.elasticsearch.core.UpdateByQueryResponse;
import co.elastic.clients.elasticsearch.core.UpdateResponse;
import co.elastic.clients.transport.endpoints.BooleanResponse;
import co.elastic.clients.util.ObjectBuilder;

/**
 * ES操作接口
 *
 * @author Woody
 * @date 2024/10/16
 */
public interface ElasticsearchHandler {
    /**
     * 获取ES客户端
     *
     * @return ES客户端实例
     */
    ElasticsearchClient getClient();

    /**
     * 获取文档索引名称
     *
     * @param model 文档模型
     * @return 索引名称
     */
    String getIndex(Class<?> model);

    /**
     * 创建文档
     *
     * @param document 文档实例
     * @return 响应结果
     */
    CreateResponse create(Document<?> document);

    /**
     * 创建文档
     *
     * @param documents 文档实例集合
     * @param <T>       数据类型
     * @return 响应结果
     */
    <T extends Document<?>> BulkResponse create(Collection<T> documents);

    /**
     * 删除文档
     *
     * @param model 数据模型
     * @param id    文档ID
     * @return 响应结果
     */
    DeleteResponse delete(Class<?> model, String id);

    /**
     * 删除文档
     *
     * @param model 数据模型
     * @param ids   文档ID集合
     * @return 响应结果
     */
    BulkResponse delete(Class<?> model, Collection<String> ids);

    /**
     * 删除文档
     *
     * @param model 数据模型
     * @param query 过滤条件
     * @return 响应结果
     */
    DeleteByQueryResponse delete(Class<?> model, Query query);

    /**
     * 更新文档
     *
     * @param document 文档实例
     * @param <T>      数据类型
     * @return 响应结果
     */
    <T extends Document<?>> UpdateResponse<T> update(T document);

    /**
     * 更新文档
     *
     * @param documents 文档实例集合
     * @param <T>       数据类型
     * @return 响应结果
     */
    <T extends Document<?>> BulkResponse update(Collection<T> documents);

    /**
     * 更新文档
     *
     * @param document 文档实例
     * @param primary  主版本
     * @param sequence 版本号
     * @param <T>      数据类型
     * @return 响应结果
     */
    <T extends Document<?>> UpdateResponse<T> update(T document, Long primary, Long sequence);

    /**
     * 更新文档
     *
     * @param model    数据模型
     * @param id       文档ID
     * @param document 文档信息
     * @param <T>      数据类型
     * @return 响应结果
     */
    <T> UpdateResponse<T> update(Class<T> model, String id, Object document);

    /**
     * 更新文档
     *
     * @param model 数据模型
     * @param query 过滤条件
     * @param field 字段名
     * @param value 字段值
     * @return 响应结果
     */
    UpdateByQueryResponse update(Class<?> model, Query query, String field, Object value);

    /**
     * 更新文档
     *
     * @param model      数据模型
     * @param query      过滤条件
     * @param parameters 更新参数
     * @return 响应结果
     */
    UpdateByQueryResponse update(Class<?> model, Query query, Map<String, ?> parameters);

    /**
     * 更新文档
     *
     * @param model    数据模型
     * @param id       文档ID
     * @param document 文档信息
     * @param primary  主版本
     * @param sequence 版本号
     * @param <T>      数据类型
     * @return 响应结果
     */
    <T> UpdateResponse<T> update(Class<T> model, String id, Object document, Long primary, Long sequence);

    /**
     * 创建或更新文档
     *
     * @param document 文档信息
     * @return 响应结果
     */
    IndexResponse index(Document<?> document);

    /**
     * 创建或更新文档
     *
     * @param model    数据模型
     * @param id       文档ID
     * @param document 文档信息
     * @return 响应结果
     */
    IndexResponse index(Class<?> model, String id, Object document);

    /**
     * 创建或更新文档
     *
     * @param document 文档信息
     * @param primary  主版本
     * @param sequence 版本号
     * @return 响应结果
     */
    IndexResponse index(Document<?> document, Long primary, Long sequence);

    /**
     * 创建或更新文档
     *
     * @param model    数据模型
     * @param id       文档ID
     * @param document 文档信息
     * @param primary  主版本
     * @param sequence 版本号
     * @return 响应结果
     */
    IndexResponse index(Class<?> model, String id, Object document, Long primary, Long sequence);

    /**
     * 创建或更新文档
     *
     * @param documents 文档信息集合
     * @param <T>       数据类型
     * @return 响应结果
     */
    <T extends Document<?>> BulkResponse index(Collection<T> documents);

    /**
     * 创建或更新文档
     *
     * @param documents 文档信息集合
     * @param primary   主版本
     * @param sequence  版本号
     * @param <T>       数据类型
     * @return 响应结果
     */
    <T extends Document<?>> BulkResponse index(Collection<T> documents, Long primary, Long sequence);

    /**
     * 获取文档数量
     *
     * @param model 数据模型
     * @param query 过滤条件
     * @return 响应结果
     */
    CountResponse count(Class<?> model, Query query);

    /**
     * 判断文档是否存在
     *
     * @param model 数据模型
     * @param id    文档ID
     * @return 响应结果
     */
    BooleanResponse exists(Class<?> model, String id);

    /**
     * 获取文档信息
     *
     * @param model 数据模型
     * @param id    文档ID
     * @param <T>   数据类型
     * @return 响应结果
     */
    <T> GetResponse<T> get(Class<T> model, String id);

    /**
     * 获取文档信息
     *
     * @param model 数据模型
     * @param ids   文档ID列表
     * @param <T>   数据类型
     * @return 响应结果
     */
    <T> MgetResponse<T> get(Class<T> model, List<String> ids);

    /**
     * 检索文档
     *
     * @param model 数据模型
     * @param query 过滤条件
     * @param <T>   数据类型
     * @return 响应结果
     */
    <T> SearchResponse<T> search(Class<T> model, Query query);

    /**
     * 检索文档
     *
     * @param model    数据模型
     * @param query    过滤条件
     * @param function 查询请求回调函数
     * @param <T>      数据类型
     * @return 响应结果
     */
    <T> SearchResponse<T> search(Class<T> model, Query query,
                                 Function<SearchRequest.Builder, ObjectBuilder<SearchRequest>> function);

    /**
     * 检索文档
     *
     * @param model   数据模型
     * @param request 查询请求
     * @param <T>     数据类型
     * @return 响应结果
     */
    <T> SearchResponse<T> search(Class<T> model, SearchRequest request);

    /**
     * 滚动获取文档列表
     *
     * @param model 数据模型
     * @param id    滚动标识
     * @param <T>   数据类型
     * @return 响应结果
     */
    default <T> ScrollResponse<T> scroll(Class<T> model, String id) {
        return this.scroll(model, id, "1m");
    }

    /**
     * 滚动获取文档列表
     *
     * @param model 数据模型
     * @param id    滚动标识
     * @param time  标识有效时间
     * @param <T>   数据类型
     * @return 响应结果
     */
    <T> ScrollResponse<T> scroll(Class<T> model, String id, String time);
}
