package com.chelvc.framework.email;

import java.util.Map;
import java.util.Objects;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import javax.mail.Session;

import ch.qos.logback.classic.filter.ThresholdFilter;
import ch.qos.logback.classic.spi.ILoggingEvent;
import ch.qos.logback.classic.spi.IThrowableProxy;
import ch.qos.logback.core.AppenderBase;
import ch.qos.logback.core.Layout;
import ch.qos.logback.core.spi.FilterReply;
import com.chelvc.framework.base.util.SpringUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.email.config.EmailProperties;
import com.chelvc.framework.email.context.EmailContextHolder;
import com.google.common.collect.Maps;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.Setter;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;

/**
 * Logback日志邮件发送器
 *
 * @author Woody
 * @date 2024/5/20
 */
@Slf4j
public class LogbackEmailAppender extends AppenderBase<ILoggingEvent> {
    @Setter
    private EmailConfig email;

    @Setter
    private Layout<ILoggingEvent> layout;

    private final long idle = 60000;
    private final Lock lock = new ReentrantLock();
    private final String name = this.getClass().getName();
    private final Map<Integer, Long> caches = Maps.newConcurrentMap();

    private volatile EmailHandler handler;
    private volatile long cleaning = System.currentTimeMillis();

    /**
     * 判断日志邮件发送器是否已准备就绪
     *
     * @return true/false
     */
    private boolean isReady() {
        return this.email != null && this.layout != null;
    }

    /**
     * 获取邮件处理器
     *
     * @return 邮件处理器
     */
    private EmailHandler getEmailHandler() {
        if (this.handler == null) {
            synchronized (this) {
                if (this.handler == null) {
                    Session session = EmailContextHolder.session(this.email);
                    this.handler = new DefaultEmailHandler(session, this.email);
                }
            }
        }
        return this.handler;
    }

    /**
     * 获取日志事件hash值
     *
     * @param event 日志事件对象实例
     * @return hash值
     */
    private int hash(ILoggingEvent event) {
        IThrowableProxy throwable = event.getThrowableProxy();
        int hash = throwable == null ? 0 : Objects.hash((Object[]) throwable.getStackTraceElementProxyArray());
        return 31 * Math.max(hash, 1) + Objects.hashCode(event.getLoggerName());
    }

    /**
     * 判断是否是重复日志事件
     *
     * @param event 日志事件对象实例
     * @return true/false
     */
    private boolean isDuplicated(ILoggingEvent event) {
        int hash = this.hash(event);
        long now = System.currentTimeMillis();
        Long timestamp = this.caches.put(hash, now);
        return timestamp != null && now - timestamp <= this.idle;
    }

    /**
     * 发送日志邮件
     *
     * @param event 日志事件对象实例
     */
    private void send(ILoggingEvent event) {
        String type = this.layout.getContentType();
        String content = this.layout.doLayout(event);
        Body body = Body.builder().to(this.email.to).cc(this.email.cc).bcc(this.email.bcc)
                .type(type).subject(this.email.subject).content(content).build();
        this.getEmailHandler().send(body);
    }

    @Override
    protected void append(ILoggingEvent event) {
        if (!this.isReady() || this.name.equals(event.getLoggerName()) || this.isDuplicated(event)) {
            return;
        }

        // 异步发送邮件
        this.context.getScheduledExecutorService().execute(() -> {
            try {
                this.send(event);
            } catch (Exception e) {
                log.warn("Email send failed: {}", e.getMessage());
            } finally {
                // 清理日志缓存
                long now = System.currentTimeMillis();
                if (now - this.cleaning > this.idle && this.lock.tryLock()) {
                    try {
                        this.caches.entrySet().removeIf(entry -> now - entry.getValue() > this.idle);
                        this.cleaning = now;
                    } finally {
                        this.lock.unlock();
                    }
                }
            }
        });
    }

    /**
     * 日志邮件配置
     */
    @Data
    @ToString(callSuper = true)
    @EqualsAndHashCode(callSuper = true)
    public static class EmailConfig extends EmailProperties {
        /**
         * 收件人
         */
        private String to;

        /**
         * 抄送人
         */
        private String cc;

        /**
         * 暗送人
         */
        private String bcc;

        /**
         * 邮件主题
         */
        private String subject;
    }

    /**
     * 日志邮件过滤器实现
     */
    public static class Filter extends ThresholdFilter {
        private String[] includes;
        private String[] excludes;

        /**
         * 判断日志事件是否满足邮件发送条件
         *
         * @param event 日志事件对象实例
         * @return true/false
         */
        protected boolean matches(ILoggingEvent event) {
            String logger = event.getLoggerName();
            return (StringUtils.isEmpty(this.excludes) || !SpringUtils.isPath(logger, this.excludes))
                    && (StringUtils.isEmpty(this.includes) || SpringUtils.isPath(logger, this.includes));
        }

        /**
         * 设置包含的日志包路径
         *
         * @param includes 包路径
         */
        public void setIncludes(String includes) {
            this.includes = StringUtils.ifEmpty(includes, StringUtils::splitActives);
        }

        /**
         * 设置排除的日志包路径
         *
         * @param excludes 包路径
         */
        public void setExcludes(String excludes) {
            this.excludes = StringUtils.ifEmpty(excludes, StringUtils::splitActives);
        }

        @Override
        public FilterReply decide(ILoggingEvent event) {
            if (!this.isStarted()) {
                return FilterReply.NEUTRAL;
            }

            FilterReply reply = super.decide(event);
            if (reply == FilterReply.NEUTRAL && this.matches(event)) {
                return FilterReply.NEUTRAL;
            }
            return FilterReply.DENY;
        }
    }
}
