package com.chelvc.framework.feign.encoder;

import java.lang.reflect.Field;
import java.lang.reflect.Type;
import java.nio.charset.Charset;
import java.util.Collection;
import java.util.Collections;
import java.util.Map;

import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Maps;
import feign.RequestTemplate;
import feign.codec.EncodeException;
import feign.codec.Encoder;
import feign.form.ContentProcessor;
import feign.form.ContentType;
import feign.form.FormProperty;
import feign.form.MultipartFormContentProcessor;
import feign.form.UrlencodedFormContentProcessor;
import feign.form.util.PojoUtil;
import lombok.NonNull;
import org.springframework.http.HttpHeaders;

/**
 * 自定义编码器
 *
 * @author Woody
 * @date 2024/1/30
 */
public class CustomizeEncoder implements Encoder {
    private final Encoder delegate;
    private final Map<ContentType, ContentProcessor> processors;

    public CustomizeEncoder() {
        this(new Encoder.Default());
    }

    public CustomizeEncoder(@NonNull Encoder delegate) {
        this.delegate = delegate;
        this.processors = ImmutableMap.of(
                ContentType.MULTIPART, new MultipartFormContentProcessor(delegate),
                ContentType.URLENCODED, new UrlencodedFormContentProcessor()
        );
    }

    /**
     * 获取Content-Type请求头
     *
     * @param template Feign请求模版实例
     * @return Content-Type内容
     */
    private String getContentType(RequestTemplate template) {
        for (Map.Entry<String, Collection<String>> entry : template.headers().entrySet()) {
            if (!entry.getKey().equalsIgnoreCase(HttpHeaders.CONTENT_TYPE)) {
                continue;
            }
            for (String value : entry.getValue()) {
                if (StringUtils.notEmpty(value)) {
                    return value;
                }
            }
        }
        return null;
    }

    /**
     * 将对象转换成属性名/值映射表
     *
     * @param object 对象实例
     * @return 对象属性名/值映射表
     */
    private Map<String, Object> object2map(Object object) {
        if (object == null) {
            return Collections.emptyMap();
        }
        Map<String, Field> fields = ObjectUtils.getClassFields(object.getClass());
        if (ObjectUtils.isEmpty(fields)) {
            return Collections.emptyMap();
        }
        Map<String, Object> values = Maps.newHashMapWithExpectedSize(fields.size());
        fields.forEach((name, field) -> {
            Object value = ObjectUtils.getObjectValue(object, field);
            FormProperty property = field.getAnnotation(FormProperty.class);
            if (property != null && StringUtils.notEmpty(property.value())) {
                values.put(property.value(), value);
            } else {
                values.put(field.getName(), value);
            }
        });
        return values;
    }

    @Override
    @SuppressWarnings("unchecked")
    public void encode(Object object, Type type, RequestTemplate template) throws EncodeException {
        String contentTypeValue = this.getContentType(template);
        ContentType contentType = ContentType.of(contentTypeValue);
        if (!this.processors.containsKey(contentType)) {
            this.delegate.encode(object, type, template);
            return;
        }

        Map<String, Object> data;
        if (Encoder.MAP_STRING_WILDCARD.equals(type)) {
            data = (Map<String, Object>) object;
        } else if (PojoUtil.isUserPojo(type)) {
            data = this.object2map(object);
        } else {
            this.delegate.encode(object, type, template);
            return;
        }

        Charset charset = HttpUtils.getContentTypeCharset(contentTypeValue);
        this.processors.get(contentType).process(template, charset, data);
    }
}
