package com.chelvc.framework.group;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.context.ThreadContextHolder;
import com.chelvc.framework.common.model.Caps;
import com.chelvc.framework.common.model.Invoking;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.group.annotation.Group;
import com.chelvc.framework.group.annotation.Groups;
import com.chelvc.framework.group.context.GroupContextHolder;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 场景分组处理拦截器
 *
 * @author Woody
 * @date 2024/11/24
 */
@Slf4j
@Aspect
@Component
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class GroupProcessInterceptor {
    private final GroupStore store;
    private final GroupHandlerFactory factory;

    /**
     * 判断请求是否匹配分组
     *
     * @param group 分组注解实例
     * @return true/false
     */
    private boolean matches(Group group) {
        return StringUtils.notEmpty(group.scene())
                && (ObjectUtils.isEmpty(group.channels()) || SessionContextHolder.isChannel(group.channels()))
                && (ObjectUtils.isEmpty(group.versions()) || SessionContextHolder.isVersion(group.versions()))
                && (ObjectUtils.isEmpty(group.platforms()) || SessionContextHolder.isPlatform(group.platforms()))
                && (ObjectUtils.isEmpty(group.terminals()) || SessionContextHolder.isTerminal(group.terminals()));
    }

    /**
     * 获取匹配的场景分组注解
     *
     * @param groups 多场景分组注解实例
     * @return 分组注解实例列表
     */
    private List<Group> matches(Groups groups) {
        if (Objects.isNull(SessionContextHolder.getId())) {
            return Collections.emptyList();
        }

        List<Group> matches = Lists.newLinkedList();
        for (Group group : groups.value()) {
            if (this.matches(group)) {
                matches.add(group);
            }
        }
        return matches.isEmpty() ? Collections.emptyList() : matches;
    }

    /**
     * 执行分组逻辑
     *
     * @param groups 分组注解列表
     * @param args   分组执行参数
     */
    private void execute(List<Group> groups, Object... args) {
        // 执行场景分组
        Map<String, Caps> values = Maps.newHashMapWithExpectedSize(groups.size());
        for (Group group : groups) {
            try {
                GroupHandler handler = this.factory.getHandler(group.handler());
                Caps value = handler.execute(group.scene(), args);
                if (value != null) {
                    values.put(group.scene(), value);
                }
            } catch (Exception e) {
                log.error("Group execute failed: {}", group.scene(), e);
            }
        }
        if (ObjectUtils.isEmpty(values)) {
            return;
        }

        // 保存场景分组，以最终结果为准
        Map<String, Caps> stores;
        try {
            stores = this.store.set(values);
            stores.forEach(GroupContextHolder::setGroup);
        } catch (Exception e) {
            log.error("Group storing failed", e);
            return;
        }

        // 执行分组完成回调方法
        for (Group group : groups) {
            String scene = group.scene();
            Caps value = values.get(scene);
            if (value != null && value == stores.get(scene)) {
                try {
                    GroupHandler handler = this.factory.getHandler(group.handler());
                    handler.complete(scene, value, args);
                } catch (Exception e) {
                    log.error("Group complete failed: {}", scene, e);
                }
            }
        }
    }

    /**
     * 方法调用分组拦截
     *
     * @param point  方法拦截点
     * @param groups 分组注解实例
     * @return 方法调用结果
     * @throws Throwable 方法调用异常
     */
    @Around("@annotation(groups)")
    public Object intercept(ProceedingJoinPoint point, Groups groups) throws Throwable {
        // 获取匹配的分组注解列表
        List<Group> matches = this.matches(groups);
        if (ObjectUtils.isEmpty(matches)) {
            return point.proceed();
        }

        // 获取已存储分组结果
        Set<String> scenes = matches.stream().map(Group::scene).collect(Collectors.toSet());
        try {
            Map<String, Caps> values = this.store.get(scenes);
            if (ObjectUtils.notEmpty(values)) {
                values.forEach((scene, value) -> {
                    if (value != null) {
                        GroupContextHolder.setGroup(scene, value);
                        matches.removeIf(group -> Objects.equals(group.scene(), scene));
                    }
                });
            }
        } catch (Exception e) {
            log.error("Group loading failed", e);
            return point.proceed();
        }

        if (ObjectUtils.isEmpty(matches)) {
            return point.proceed();
        }

        // 执行方法前置分组
        List<Group> sync = Lists.newLinkedList();
        List<Group> async = Lists.newLinkedList();
        Iterator<Group> iterator = matches.iterator();
        while (iterator.hasNext()) {
            Group group = iterator.next();
            if (group.invoking() == Invoking.REQUEST) {
                if (group.async()) {
                    async.add(group);
                } else {
                    sync.add(group);
                }
                iterator.remove();
            }
        }
        Object[] args = point.getArgs();
        if (ObjectUtils.notEmpty(async)) {
            ThreadContextHolder.run(() -> this.execute(async, args));
        }
        if (ObjectUtils.notEmpty(sync)) {
            this.execute(sync, args);
        }

        Object value = point.proceed();

        // 执行方法后置分组
        if (ObjectUtils.notEmpty(matches)) {
            async.clear();
            iterator = matches.iterator();
            while (iterator.hasNext()) {
                Group group = iterator.next();
                if (group.async()) {
                    async.add(group);
                    iterator.remove();
                }
            }
            Object[] vars = new Object[args.length + 1];
            System.arraycopy(args, 0, vars, 0, args.length);
            vars[args.length] = value;
            if (ObjectUtils.notEmpty(async)) {
                ThreadContextHolder.run(() -> this.execute(async, vars));
            }
            if (ObjectUtils.notEmpty(matches)) {
                this.execute(matches, vars);
            }
        }
        return value;
    }
}
