package com.chelvc.framework.group;

import java.time.Duration;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Caps;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.group.config.GroupProperties;
import com.chelvc.framework.group.context.GroupContextHolder;
import com.chelvc.framework.redis.context.RedisContextHolder;
import com.chelvc.framework.redis.context.RedisHashHolder;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.NonNull;
import org.springframework.dao.DataAccessException;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.RedisOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.SessionCallback;

/**
 * 场景分组存储Redis缓存实现
 *
 * @author Woody
 * @date 2024/11/28
 */
public class RedisGroupStore implements GroupStore {
    private final Duration timeout;
    private final GroupClient client;

    public RedisGroupStore(@NonNull GroupClient client, @NonNull GroupProperties properties) {
        this.client = client;
        this.timeout = Duration.ofSeconds(properties.getCache().getDuration());
    }

    /**
     * 获取Redis用户分组标识
     *
     * @return Redis标识
     */
    private String key() {
        Long id = SessionContextHolder.getId();
        return id == null ? null : ("group:" + id);
    }

    /**
     * 设置场景分组，如果存在则使用原值，否则更新
     *
     * @param key   缓存标识
     * @param scene 场景标识
     * @param group 分组标识
     * @return 分组标识
     */
    private Caps setnx(String key, String scene, Caps group) {
        // 更新场景分组缓存
        RedisTemplate<String, Object> template = RedisContextHolder.getDefaultTemplate();
        List<Object> results = template.executePipelined(new SessionCallback<Object>() {
            @Override
            @SuppressWarnings("unchecked")
            public <K, V> Object execute(RedisOperations<K, V> operations) throws DataAccessException {
                // 设置分组信息，如果已存在则忽略
                operations.opsForHash().putIfAbsent((K) key, scene, group.name());

                // 设置分组过期时间
                operations.expire((K) key, timeout);
                return null;
            }
        });

        // 如果缓存更新成功则持久化分组信息
        if (Boolean.TRUE.equals(results.get(0))) {
            this.client.set(scene, group);
            return group;
        }

        // 如果分组已存在则获取已有分组
        HashOperations<String, String, String> operations = template.opsForHash();
        return StringUtils.ifEmpty(operations.get(key, scene), Caps::valueOf);
    }

    /**
     * 批量设置场景分组，如果存在则使用原值，否则更新
     *
     * @param key    缓存标识
     * @param groups 场景/分组映射表
     * @return 场景/分组映射表
     */
    private Map<String, Caps> setnx(String key, Map<String, Caps> groups) {
        // 批量更新场景分组缓存
        Map<String, Caps> stored = Maps.newLinkedHashMap(groups);
        RedisTemplate<String, Object> template = RedisContextHolder.getDefaultTemplate();
        List<Object> results = template.executePipelined(new SessionCallback<Object>() {
            @Override
            @SuppressWarnings("unchecked")
            public <K, V> Object execute(RedisOperations<K, V> operations) throws DataAccessException {
                // 设置分组信息，如果已存在则忽略
                stored.forEach((scene, group) -> operations.opsForHash().putIfAbsent((K) key, scene, group.name()));

                // 设置分组过期时间
                operations.expire((K) key, timeout);
                return null;
            }
        });

        // 获取缓存已存在的场景标识
        List<String> exists = Lists.newLinkedList();
        Iterator<String> iterator = stored.keySet().iterator();
        for (int i = 0, size = results.size(); i < size && iterator.hasNext(); i++) {
            String scene = iterator.next();
            if (!Boolean.TRUE.equals(results.get(i))) {
                exists.add(scene);

                // 移除更新失败（已存在）场景分组
                iterator.remove();
            }
        }

        // 持久化缓存更新成功的场景分组
        if (ObjectUtils.notEmpty(stored)) {
            this.client.set(stored);
        }

        // 重新加载已存在缓存的场景分组
        if (ObjectUtils.notEmpty(exists)) {
            HashOperations<String, String, String> operations = template.opsForHash();
            List<String> values = operations.multiGet(key, exists);
            for (int i = 0, size = exists.size(); i < size; i++) {
                stored.put(exists.get(i), StringUtils.ifEmpty(values.get(i), Caps::valueOf));
            }
        }
        return stored;
    }

    @Override
    public Caps get(@NonNull String scene) {
        String key = this.key();
        if (StringUtils.isEmpty(key)) {
            return null;
        }

        // 优先使用配置分组
        Caps group = GroupContextHolder.getConfig(scene);
        if (group != null) {
            return group;
        }

        // 获取分组标识，如果缓存不存在则重新加载分组并更新缓存
        RedisTemplate<String, Object> template = RedisContextHolder.getDefaultTemplate();
        HashOperations<String, String, String> operations = template.opsForHash();
        group = StringUtils.ifEmpty(operations.get(key, scene), Caps::valueOf);
        if (group == null && (group = this.client.get(scene)) != null) {
            RedisHashHolder.set(template, key, scene, group, this.timeout);
        }
        return group;
    }

    @Override
    public Map<String, Caps> get(@NonNull Collection<String> scenes) {
        String key = this.key();
        if (StringUtils.isEmpty(key) || ObjectUtils.isEmpty(scenes)) {
            return Collections.emptyMap();
        }

        // 优先获取配置分组
        List<String> nones = Lists.newLinkedList();
        Map<String, Caps> groups = Maps.newHashMapWithExpectedSize(scenes.size());
        for (String scene : scenes) {
            Caps group = GroupContextHolder.getConfig(scene);
            if (group == null) {
                nones.add(scene);
            } else {
                groups.put(scene, group);
            }
        }

        // 批量获取缓存分组映射
        RedisTemplate<String, Object> template = RedisContextHolder.getDefaultTemplate();
        if (ObjectUtils.notEmpty(nones)) {
            HashOperations<String, String, String> operations = template.opsForHash();
            List<String> values = operations.multiGet(key, nones);

            int i = 0;
            Iterator<String> iterator = nones.iterator();
            while (iterator.hasNext()) {
                String group = values.get(i++);
                String scene = iterator.next();
                if (StringUtils.notEmpty(group)) {
                    groups.put(scene, Caps.valueOf(group));
                    iterator.remove();
                }
            }
        }

        // 重新加载场景分组为null的数据
        if (ObjectUtils.notEmpty(nones)) {
            Map<String, Caps> reloads = this.client.get(nones);
            if (ObjectUtils.notEmpty(reloads)) {
                groups.putAll(reloads);
                RedisHashHolder.set(template, key, reloads, this.timeout);
            }
        }
        return groups;
    }

    @Override
    public Caps set(@NonNull String scene, @NonNull Caps group) {
        String key = this.key();
        return StringUtils.isEmpty(key) ? group : this.setnx(key, scene, group);
    }

    @Override
    public Map<String, Caps> set(@NonNull Map<String, Caps> groups) {
        String key = this.key();
        if (StringUtils.isEmpty(key) || ObjectUtils.isEmpty(groups)) {
            return groups;
        } else if (groups.size() == 1) {
            Map.Entry<String, Caps> entry = groups.entrySet().iterator().next();
            String scene = entry.getKey();
            Caps group = this.setnx(key, scene, entry.getValue());
            return group == entry.getValue() ? groups : ImmutableMap.of(scene, group);
        }
        return this.setnx(key, groups);
    }
}
