package com.chelvc.framework.jpush;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import cn.jiguang.common.ServiceHelper;
import cn.jiguang.common.resp.APIRequestException;
import cn.jmessage.api.JMessageClient;
import cn.jmessage.api.common.model.RegisterInfo;
import cn.jmessage.api.user.UserInfoResult;
import cn.jmessage.api.user.UserStateListResult;
import cn.jmessage.api.user.UserStateResult;
import cn.jpush.api.JPushClient;
import cn.jpush.api.push.PushResult;
import cn.jpush.api.push.model.BatchPushResult;
import cn.jpush.api.push.model.Message;
import cn.jpush.api.push.model.PushPayload;
import cn.jpush.api.push.model.notification.Notification;
import cn.jpush.api.report.ReceivedsResult;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.LoggingContextHolder;
import com.chelvc.framework.base.context.RestContextHolder;
import com.chelvc.framework.common.model.Period;
import com.chelvc.framework.common.model.Platform;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.DateUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.jpush.config.JPushProperties;
import com.chelvc.framework.jpush.context.JPushContextHolder;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;

/**
 * 极光登录操作默认实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class DefaultJPushHandler implements JPushHandler {
    /**
     * 获取消息地址
     */
    private static final String GET_MESSAGE_URL =
            "https://report.im.jpush.cn/v2/users/%s/messages?count=%d&begin_time=%s&end_time=%s";

    /**
     * 获取汇总消息地址(首次)
     */
    private static final String GET_INITIAL_MESSAGE_URL =
            "https://report.im.jpush.cn/v2/messages?count=%d&begin_time=%s&end_time=%s";

    /**
     * 获取汇总消息地址(非首次)
     */
    private static final String GET_SUBSEQUENT_MESSAGE_URL = "https://report.im.jpush.cn/v2/messages?cursor=";

    /**
     * 获取历史消息地址
     */
    private static final String GET_HISTORY_MESSAGE_URL = "https://report.im.jpush.cn/v2/users/%s/messages?cursor=%s";

    private final String authorization;
    private final JPushClient pushClient;
    private final JMessageClient messageClient;
    private final JPushProperties properties;

    public DefaultJPushHandler(@NonNull JPushClient pushClient, @NonNull JMessageClient messageClient,
                               @NonNull JPushProperties properties) {
        this.pushClient = pushClient;
        this.messageClient = messageClient;
        this.properties = properties;
        this.authorization = ServiceHelper.getBasicAuthorization(properties.getAppid(), properties.getSecret());
    }

    /**
     * 获取聊天消息
     *
     * @param url 接口地址
     * @return 聊天消息
     */
    private ChatMessage getChatMessages(String url) {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", this.authorization);
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<?> entity = new HttpEntity<>(headers);
        try {
            return RestContextHolder.execute(rest -> rest.exchange(
                    url, HttpMethod.GET, entity, ChatMessage.class
            ).getBody());
        } catch (Exception e) {
            log.warn("JPush message get failed: {}, {}", url, e.getMessage());
            return null;
        }
    }

    @Override
    public String getAppid() {
        return this.properties.getAppid();
    }

    @Override
    public String getAuthorization() {
        return this.authorization;
    }

    @Override
    public boolean register(@NonNull RegisterInfo... registrations) {
        if (registrations.length == 0) {
            return false;
        }
        try {
            this.messageClient.registerUsers(registrations);
        } catch (Exception e) {
            int code;
            if (e instanceof APIRequestException
                    && ((code = ((APIRequestException) e).getErrorCode()) == 1011 || code == 899001)) {
                return true;
            }
            log.warn("JPush register failed: {}", e.getMessage());
            return false;
        }
        return true;
    }

    @Override
    public boolean register(@NonNull Collection<RegisterInfo> registrations) {
        return !registrations.isEmpty() && this.register(registrations.toArray(new RegisterInfo[0]));
    }

    @Override
    public long push(@NonNull PushPayload payload) {
        // 针对生产环境IOS推送，需要设置apnsProduction为true
        if (ApplicationContextHolder.isProduction()) {
            payload.resetOptionsApnsProduction(true);
        }
        PushResult result;
        try {
            result = this.pushClient.sendPush(payload);
        } catch (Exception e) {
            log.warn("JPush push failed: {}, {}", payload.getTarget(), e.getMessage());
            return 0;
        }
        if (result != null && result.statusCode == 0) {
            return result.msg_id;
        }
        log.warn("JPush push failed: {}, {}", payload.getTarget(), result);
        return 0;
    }

    @Override
    public long push(@NonNull Platform platform, @NonNull Message message, @NonNull String... aliases) {
        return this.push(JPushContextHolder.payload(platform, message, aliases));
    }

    @Override
    public long push(@NonNull Platform platform, @NonNull Message message, @NonNull Collection<String> aliases) {
        return this.push(JPushContextHolder.payload(platform, message, aliases));
    }

    @Override
    public long push(@NonNull Platform platform, @NonNull Notification notification, @NonNull String... aliases) {
        return this.push(JPushContextHolder.payload(platform, notification, aliases));
    }

    @Override
    public long push(@NonNull Platform platform, @NonNull Notification notification,
                     @NonNull Collection<String> aliases) {
        return this.push(JPushContextHolder.payload(platform, notification, aliases));
    }

    @Override
    public long push(@NonNull Platform platform, @NonNull String title, @NonNull String content,
                     @NonNull String... aliases) {
        return this.push(platform, title, content, Collections.emptyMap(), aliases);
    }

    @Override
    public long push(@NonNull Platform platform, @NonNull String title, @NonNull String content,
                     @NonNull Collection<String> aliases) {
        return this.push(platform, title, content, Collections.emptyMap(), aliases);
    }

    @Override
    public long push(@NonNull Platform platform, @NonNull String title, @NonNull String content,
                     @NonNull Map<String, String> extras, @NonNull String... aliases) {
        Notification notification = JPushContextHolder.notification(platform, title, content, extras);
        return this.push(platform, notification, aliases);
    }

    @Override
    public long push(@NonNull Platform platform, @NonNull String title, @NonNull String content,
                     @NonNull Map<String, String> extras, @NonNull Collection<String> aliases) {
        Notification notification = JPushContextHolder.notification(platform, title, content, extras);
        return this.push(platform, notification, aliases);
    }

    @Override
    public BatchPushResult push(@NonNull PushPayload... payloads) {
        AssertUtils.arg(payloads.length > 0, () -> "payloads must not be empty");
        return this.push(Arrays.asList(payloads));
    }

    @Override
    public BatchPushResult push(@NonNull List<PushPayload> payloads) {
        AssertUtils.arg(!payloads.isEmpty(), () -> "payloads must not be empty");
        // 针对生产环境IOS推送，需要设置apnsProduction为true
        if (ApplicationContextHolder.isProduction()) {
            payloads.forEach(payload -> payload.resetOptionsApnsProduction(true));
        }

        // 批量推送消息
        try {
            return this.pushClient.batchSendPushByAlias(payloads);
        } catch (Exception e) {
            log.warn("JPush push failed: {}", e.getMessage());
            return null;
        }
    }

    /**
     * 获取消息投递结果
     *
     * @param ids 消息ID（多个ID之间使用","号隔开，最多最多支持 100 个消息ID）
     * @return 投递结果列表
     */
    private List<ReceivedsResult.Received> getDeliveries(String ids) {
        if (StringUtils.isEmpty(ids)) {
            return Collections.emptyList();
        }
        ReceivedsResult result;
        try {
            result = this.pushClient.getReceivedsDetail(ids);
        } catch (Exception e) {
            log.warn("JPush delivery get failed: {}", e.getMessage());
            return Collections.emptyList();
        }
        return ObjectUtils.ifNull(result.received_list, Collections::emptyList);
    }

    @Override
    public ReceivedsResult.Received getDelivery(@NonNull Long id) {
        List<ReceivedsResult.Received> deliveries = this.getDeliveries(String.valueOf(id));
        return ObjectUtils.isEmpty(deliveries) ? null : deliveries.get(0);
    }

    @Override
    public List<ReceivedsResult.Received> getDeliveries(@NonNull Long... ids) {
        return ids.length == 0 ? Collections.emptyList() : this.getDeliveries(Arrays.asList(ids));
    }

    @Override
    public List<ReceivedsResult.Received> getDeliveries(@NonNull Collection<Long> ids) {
        return this.getDeliveries(
                ids.stream().filter(Objects::nonNull).map(Object::toString).collect(Collectors.joining(","))
        );
    }

    @Override
    public UserInfoResult getUserInfo(@NonNull String username) {
        try {
            return this.messageClient.getUserInfo(username);
        } catch (Exception e) {
            log.warn("JPush user info get failed: {}, {}", username, e.getMessage());
            return null;
        }
    }

    @Override
    public UserStateResult getUserState(@NonNull String username) {
        try {
            return this.messageClient.getUserState(username);
        } catch (Exception e) {
            log.warn("JPush user state get failed: {}, {}", username, e.getMessage());
            return null;
        }
    }

    @Override
    public UserStateListResult[] getUserStates(@NonNull String... usernames) {
        if (usernames.length == 0) {
            return new UserStateListResult[0];
        }
        try {
            return this.messageClient.getUsersState(usernames);
        } catch (Exception e) {
            LoggingContextHolder.warn(log, e);
            return new UserStateListResult[0];
        }
    }

    @Override
    public ChatMessage getInitialMessages(@NonNull Period period, int count) {
        return this.getChatMessages(String.format(
                GET_INITIAL_MESSAGE_URL, count,
                DateUtils.format(period.getBegin(), true), DateUtils.format(period.getEnd(), true)
        ));
    }

    @Override
    public ChatMessage getSubsequentMessages(@NonNull String cursor) {
        return this.getChatMessages(GET_SUBSEQUENT_MESSAGE_URL + cursor);
    }

    @Override
    public ChatMessage getUserMessages(@NonNull String username, @NonNull Period period, int count) {
        return this.getChatMessages(String.format(
                GET_MESSAGE_URL, username, count, DateUtils.format(period.getBegin(), true),
                DateUtils.format(period.getEnd(), true)
        ));
    }

    @Override
    public ChatMessage getUserHistoryMessages(@NonNull String username, @NonNull String cursor) {
        return this.getChatMessages(String.format(GET_HISTORY_MESSAGE_URL, username, cursor));
    }
}
