package com.chelvc.framework.jpush;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import cn.jmessage.api.common.model.RegisterInfo;
import cn.jmessage.api.user.UserInfoResult;
import cn.jmessage.api.user.UserStateListResult;
import cn.jmessage.api.user.UserStateResult;
import cn.jpush.api.push.model.BatchPushResult;
import cn.jpush.api.push.model.Message;
import cn.jpush.api.push.model.PushPayload;
import cn.jpush.api.push.model.notification.Notification;
import cn.jpush.api.report.ReceivedsResult;
import com.chelvc.framework.common.model.Period;
import com.chelvc.framework.common.model.Platform;

/**
 * JPush操作接口
 *
 * @author Woody
 * @date 2024/1/30
 */
public interface JPushHandler {
    /**
     * 获取应用标识
     *
     * @return 应用标识
     */
    String getAppid();

    /**
     * 获取认证信息
     *
     * @return 认证信息
     */
    String getAuthorization();

    /**
     * 账号注册
     *
     * @param registrations 注册信息数组
     * @return true/false
     */
    boolean register(RegisterInfo... registrations);

    /**
     * 账号注册
     *
     * @param registrations 注册信息集合
     * @return true/false
     */
    boolean register(Collection<RegisterInfo> registrations);

    /**
     * 推送消息
     *
     * @param payload 消息体
     * @return 消息ID
     */
    long push(PushPayload payload);

    /**
     * 推送消息
     *
     * @param platform 目标平台
     * @param message  透传消息
     * @param aliases  用户别名数组
     * @return 消息ID
     */
    long push(Platform platform, Message message, String... aliases);

    /**
     * 推送消息
     *
     * @param platform 目标平台
     * @param message  透传消息
     * @param aliases  用户别名集合
     * @return 消息ID
     */
    long push(Platform platform, Message message, Collection<String> aliases);

    /**
     * 推送消息
     *
     * @param platform     目标平台
     * @param notification 推送通知
     * @param aliases      用户别名数组
     * @return 消息ID
     */
    long push(Platform platform, Notification notification, String... aliases);

    /**
     * 推送消息
     *
     * @param platform     目标平台
     * @param notification 推送通知
     * @param aliases      用户别名集合
     * @return 消息ID
     */
    long push(Platform platform, Notification notification, Collection<String> aliases);

    /**
     * 推送消息
     *
     * @param platform 目标平台
     * @param title    消息标题
     * @param content  消息内容
     * @param aliases  用户别名数组
     * @return 消息ID
     */
    long push(Platform platform, String title, String content, String... aliases);

    /**
     * 推送消息
     *
     * @param platform 目标平台
     * @param title    消息标题
     * @param content  消息内容
     * @param aliases  用户别名集合
     * @return 消息ID
     */
    long push(Platform platform, String title, String content, Collection<String> aliases);

    /**
     * 推送消息
     *
     * @param platform 目标平台
     * @param title    消息标题
     * @param content  消息内容
     * @param extras   扩展信息
     * @param aliases  用户别名数组
     * @return 消息ID
     */
    long push(Platform platform, String title, String content, Map<String, String> extras, String... aliases);

    /**
     * 推送消息
     *
     * @param platform 目标平台
     * @param title    消息标题
     * @param content  消息内容
     * @param extras   扩展信息
     * @param aliases  用户别名集合
     * @return 消息ID
     */
    long push(Platform platform, String title, String content, Map<String, String> extras, Collection<String> aliases);

    /**
     * 推送消息
     *
     * @param payloads 消息体数组
     * @return 推送结果
     */
    BatchPushResult push(PushPayload... payloads);

    /**
     * 推送消息
     *
     * @param payloads 消息体列表
     * @return 推送结果
     */
    BatchPushResult push(List<PushPayload> payloads);

    /**
     * 获取消息投递结果
     *
     * @param id 消息ID
     * @return 投递结果
     */
    ReceivedsResult.Received getDelivery(Long id);

    /**
     * 获取消息投递结果
     *
     * @param ids 消息ID数组
     * @return 投递结果列表
     */
    List<ReceivedsResult.Received> getDeliveries(Long... ids);

    /**
     * 获取消息投递结果
     *
     * @param ids 消息ID集合
     * @return 投递结果列表
     */
    List<ReceivedsResult.Received> getDeliveries(Collection<Long> ids);

    /**
     * 获取用户信息
     *
     * @param username 用户名
     * @return 用户基本信息
     */
    UserInfoResult getUserInfo(String username);

    /**
     * 获取用户状态
     *
     * @param username 用户名
     * @return 用户状态信息
     */
    UserStateResult getUserState(String username);

    /**
     * 获取用户状态
     *
     * @param usernames 用户名数组
     * @return 用户状态信息数组
     */
    UserStateListResult[] getUserStates(String... usernames);

    /**
     * 获取用户聊天消息列表(首次获取)
     *
     * @param period 时间周期
     * @param count  获取消息数量
     * @return 消息列表
     */
    ChatMessage getInitialMessages(Period period, int count);

    /**
     * 获取用户聊天消息列表(非首次获取)
     *
     * @param cursor 游标
     * @return 消息列表
     */
    ChatMessage getSubsequentMessages(String cursor);

    /**
     * 获取用户聊天消息
     *
     * @param username 用户名
     * @param period   时间周期
     * @param count    获取消息数量
     * @return 聊天消息
     */
    ChatMessage getUserMessages(String username, Period period, int count);

    /**
     * 获取用户历史聊天消息
     *
     * @param username 用户名
     * @param cursor   消息记录游标
     * @return 聊天消息
     */
    ChatMessage getUserHistoryMessages(String username, String cursor);
}
