package com.chelvc.framework.kafka.config;

import java.lang.reflect.Type;
import java.util.List;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.kafka.context.KafkaContextHolder;
import com.chelvc.framework.kafka.fallback.KafkaFallbackPolicy;
import com.chelvc.framework.kafka.fallback.KafkaMemorySender;
import com.chelvc.framework.kafka.fallback.KafkaPersistentSender;
import com.chelvc.framework.kafka.fallback.KafkaStoreClient;
import com.chelvc.framework.kafka.interceptor.KafkaSessionInterceptor;
import com.chelvc.framework.kafka.producer.KafkaMessageSender;
import com.chelvc.framework.kafka.producer.KafkaSenderWrapper;
import com.chelvc.framework.kafka.producer.KafkaTransactionChecker;
import com.chelvc.framework.kafka.producer.StandardTransactionChecker;
import com.chelvc.framework.kafka.producer.TransactionCheckProcessor;
import lombok.RequiredArgsConstructor;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.springframework.beans.factory.ObjectProvider;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.kafka.ConcurrentKafkaListenerContainerFactoryConfigurer;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.kafka.annotation.KafkaListenerConfigurer;
import org.springframework.kafka.config.ConcurrentKafkaListenerContainerFactory;
import org.springframework.kafka.config.KafkaListenerConfigUtils;
import org.springframework.kafka.config.KafkaListenerContainerFactory;
import org.springframework.kafka.config.KafkaListenerEndpointRegistrar;
import org.springframework.kafka.config.KafkaListenerEndpointRegistry;
import org.springframework.kafka.core.ConsumerFactory;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.core.ProducerFactory;
import org.springframework.kafka.listener.RecordInterceptor;
import org.springframework.kafka.support.ProducerListener;
import org.springframework.kafka.support.SendResult;
import org.springframework.kafka.support.converter.BatchMessagingMessageConverter;
import org.springframework.kafka.support.converter.MessagingMessageConverter;
import org.springframework.kafka.support.converter.RecordMessageConverter;
import org.springframework.util.concurrent.ListenableFuture;

/**
 * Kafka配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Configuration
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class KafkaConfigurer implements KafkaListenerConfigurer {
    private final KafkaProperties properties;
    private final ApplicationContext applicationContext;

    @Bean
    public RecordMessageConverter recordMessageConverter() {
        return new MessagingMessageConverter() {
            @Override
            protected Object extractAndConvertValue(ConsumerRecord<?, ?> record, Type type) {
                return KafkaContextHolder.deserialize(record, type);
            }
        };
    }

    @Bean
    public RecordInterceptor<Object, Object> recordInterceptor() {
        return new KafkaSessionInterceptor();
    }

    @Bean
    public KafkaListenerContainerFactory<?> kafkaListenerContainerFactory(
            ConcurrentKafkaListenerContainerFactoryConfigurer configurer,
            ConsumerFactory<Object, Object> consumerFactory) {
        ConcurrentKafkaListenerContainerFactory<Object, Object> containerFactory =
                new ConcurrentKafkaListenerContainerFactory<>();
        configurer.configure(containerFactory, consumerFactory);
        containerFactory.setRecordInterceptor(this.recordInterceptor());
        containerFactory.getContainerProperties().setDeliveryAttemptHeader(true);
        return containerFactory;
    }

    @Bean
    public KafkaListenerContainerFactory<?> batchListenerContainerFactory(
            ConcurrentKafkaListenerContainerFactoryConfigurer configurer,
            ConsumerFactory<Object, Object> consumerFactory) {
        ConcurrentKafkaListenerContainerFactory<Object, Object> containerFactory =
                new ConcurrentKafkaListenerContainerFactory<>();
        configurer.configure(containerFactory, consumerFactory);
        containerFactory.setBatchListener(true);
        containerFactory.setRecordInterceptor(this.recordInterceptor());
        containerFactory.getContainerProperties().setDeliveryAttemptHeader(true);
        containerFactory.setMessageConverter(new BatchMessagingMessageConverter(this.recordMessageConverter()));
        return containerFactory;
    }

    @Override
    public void configureKafkaListeners(KafkaListenerEndpointRegistrar registrar) {
        KafkaListenerEndpointRegistry registry = this.applicationContext.getBean(
                KafkaListenerConfigUtils.KAFKA_LISTENER_ENDPOINT_REGISTRY_BEAN_NAME,
                KafkaListenerEndpointRegistry.class
        );
        registrar.setEndpointRegistry(new KafkaListenerRegistryWrapper(registry));
    }

    @Bean
    @ConditionalOnMissingBean(KafkaTransactionChecker.class)
    public KafkaTransactionChecker kafkaTransactionChecker(List<TransactionCheckProcessor<?>> processors) {
        return new StandardTransactionChecker(processors);
    }

    @Bean
    @ConditionalOnExpression("'${spring.kafka.fallbacks}'.contains('MEMORY')")
    public KafkaSenderWrapper memorySenderWrapper() {
        return new KafkaSenderWrapper() {
            private final int order = properties.getFallbacks().indexOf(KafkaFallbackPolicy.MEMORY);

            @Override
            public int getOrder() {
                return this.order;
            }

            @Override
            public <K, V> KafkaMessageSender<K, V> wrap(KafkaMessageSender<K, V> sender) {
                return new KafkaMemorySender<K, V>(sender);
            }
        };
    }

    @Bean
    @ConditionalOnExpression("'${spring.kafka.fallbacks}'.contains('PERSISTENT')")
    public KafkaSenderWrapper persistentSenderWrapper(KafkaStoreClient client) {
        return new KafkaSenderWrapper() {
            private final int order = properties.getFallbacks().indexOf(KafkaFallbackPolicy.PERSISTENT);

            @Override
            public int getOrder() {
                return this.order;
            }

            @Override
            public <K, V> KafkaMessageSender<K, V> wrap(KafkaMessageSender<K, V> sender) {
                return new KafkaPersistentSender<K, V>(client, sender);
            }
        };
    }

    @Bean
    @ConditionalOnBean(KafkaSenderWrapper.class)
    public KafkaTemplate<?, ?> kafkaTemplate(ProducerFactory<Object, Object> producerFactory,
                                             ProducerListener<Object, Object> producerListener,
                                             ObjectProvider<RecordMessageConverter> messageConverter) {
        KafkaTemplate<Object, Object> template = new KafkaTemplate<Object, Object>(producerFactory) {
            private final KafkaMessageSender<Object, Object> sender;

            {
                // 初始化消息发送处理器
                KafkaMessageSender<Object, Object> sender = super::doSend;
                List<KafkaSenderWrapper> wrappers =
                        ApplicationContextHolder.getSortBeans(applicationContext, KafkaSenderWrapper.class);
                for (KafkaSenderWrapper wrapper : wrappers) {
                    sender = wrapper.wrap(sender);
                }
                this.sender = sender;
            }

            @Override
            protected ListenableFuture<SendResult<Object, Object>> doSend(ProducerRecord<Object, Object> record) {
                return this.sender.send(record);
            }

            @Override
            public void destroy() {
                try {
                    this.sender.destroy();
                } catch (Exception e) {
                    throw new RuntimeException(e);
                } finally {
                    super.destroy();
                }
            }
        };
        messageConverter.ifUnique(template::setMessageConverter);
        template.setProducerListener(producerListener);
        template.setDefaultTopic(this.properties.getTemplate().getDefaultTopic());
        return template;
    }
}
