package com.chelvc.framework.kafka.config;

import java.util.Collection;
import java.util.Objects;
import java.util.Set;
import java.util.regex.Pattern;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.kafka.context.KafkaContextHolder;
import lombok.NonNull;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.core.env.Environment;
import org.springframework.kafka.config.KafkaListenerContainerFactory;
import org.springframework.kafka.config.KafkaListenerEndpoint;
import org.springframework.kafka.config.KafkaListenerEndpointRegistry;
import org.springframework.kafka.config.MethodKafkaListenerEndpoint;
import org.springframework.kafka.listener.MessageListenerContainer;

/**
 * Kafka消息监听器注册包装器
 *
 * @author Woody
 * @date 2024/7/13
 */
public class KafkaListenerRegistryWrapper extends KafkaListenerEndpointRegistry {
    private final KafkaListenerEndpointRegistry delegate;

    public KafkaListenerRegistryWrapper(@NonNull KafkaListenerEndpointRegistry delegate) {
        this.delegate = delegate;
    }

    /**
     * 环境隔离
     *
     * @param original 原始标记
     * @return 环境隔离标记
     */
    private String isolate(String original) {
        Environment environment = ApplicationContextHolder.getEnvironment();
        return KafkaContextHolder.isolate(environment.resolvePlaceholders(original));
    }

    /**
     * 隔离消息监听器端点环境
     *
     * @param endpoint 消息监听器端点实例
     */
    private void isolateListenerEndpointEnvironment(MethodKafkaListenerEndpoint<?, ?> endpoint) {
        if (StringUtils.notEmpty(endpoint.getGroup())) {
            endpoint.setGroup(this.isolate(endpoint.getGroup()));
        }
        if (StringUtils.notEmpty(endpoint.getGroupId())) {
            endpoint.setGroupId(this.isolate(endpoint.getGroupId()));
        }
        if (Objects.nonNull(endpoint.getTopicPattern())) {
            String pattern = endpoint.getTopicPattern().pattern();
            if (StringUtils.notEmpty(pattern)) {
                endpoint.setTopicPattern(Pattern.compile(this.isolate(pattern)));
            }
        }
        Collection<String> topics = endpoint.getTopics();
        if (ObjectUtils.notEmpty(topics)) {
            endpoint.setTopics(topics.stream().map(this::isolate).toArray(String[]::new));
        }
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.delegate.setApplicationContext(applicationContext);
    }

    @Override
    public MessageListenerContainer getListenerContainer(String id) {
        return this.delegate.getListenerContainer(id);
    }

    @Override
    public Set<String> getListenerContainerIds() {
        return this.delegate.getListenerContainerIds();
    }

    @Override
    public Collection<MessageListenerContainer> getListenerContainers() {
        return this.delegate.getListenerContainers();
    }

    @Override
    public Collection<MessageListenerContainer> getAllListenerContainers() {
        return this.delegate.getAllListenerContainers();
    }

    @Override
    public void registerListenerContainer(KafkaListenerEndpoint endpoint, KafkaListenerContainerFactory<?> factory) {
        this.registerListenerContainer(endpoint, factory, false);
    }

    @Override
    public void registerListenerContainer(KafkaListenerEndpoint endpoint, KafkaListenerContainerFactory<?> factory,
                                          boolean startImmediately) {
        this.isolateListenerEndpointEnvironment((MethodKafkaListenerEndpoint<?, ?>) endpoint);
        this.delegate.registerListenerContainer(endpoint, factory, startImmediately);
    }

    @Override
    public void destroy() {
        this.delegate.destroy();
    }

    @Override
    public int getPhase() {
        return this.delegate.getPhase();
    }

    @Override
    public boolean isAutoStartup() {
        return this.delegate.isAutoStartup();
    }

    @Override
    public void start() {
        this.delegate.start();
    }

    @Override
    public void stop() {
        this.delegate.stop();
    }

    @Override
    public void stop(Runnable callback) {
        this.delegate.stop(callback);
    }

    @Override
    public boolean isRunning() {
        return this.delegate.isRunning();
    }

    @Override
    public void onApplicationEvent(ContextRefreshedEvent event) {
        this.delegate.onApplicationEvent(event);
    }
}
