package com.chelvc.framework.kafka.fallback;

import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;

import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ThreadUtils;
import com.chelvc.framework.kafka.producer.KafkaMessageSender;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.tuple.Pair;
import org.apache.kafka.clients.producer.ProducerRecord;

/**
 * Kafka降级内存发送处理器实现
 *
 * @param <K> 顺序标识类型
 * @param <V> 消息载体类型
 * @author Woody
 * @date 2024/7/18
 */
@Slf4j
public class KafkaMemorySender<K, V> extends KafkaFallbackSender<K, V> {
    private final int capacity;
    private volatile Thread consumer;
    private volatile BlockingQueue<Pair<ProducerRecord<K, V>, Session>> queue;

    public KafkaMemorySender(@NonNull KafkaMessageSender<K, V> delegate) {
        this(10000, delegate);
    }

    public KafkaMemorySender(int capacity, @NonNull KafkaMessageSender<K, V> delegate) {
        super(delegate);
        AssertUtils.arg(capacity > 0, () -> "Kafka fallback queue capacity must be greater than 0");
        this.capacity = capacity;
    }

    /**
     * 初始化消息队列
     */
    private synchronized void initializeMessageQueue() {
        if (this.queue != null) {
            return;
        }

        this.queue = new ArrayBlockingQueue<>(this.capacity);
        this.consumer = ThreadUtils.consume(this.queue, pair -> {
            ProducerRecord<K, V> message = pair.getLeft();
            SessionContextHolder.setSession(pair.getRight());
            try {
                this.delegate.send(message);
                return true;
            } catch (Exception e) {
                log.error("Kafka message send failed: {}", message.topic(), e);
            } finally {
                SessionContextHolder.removeSessionContext();
            }
            ThreadUtils.sleep(5000);
            return false;
        });
    }

    @Override
    protected void fallback(ProducerRecord<K, V> record) {
        if (log.isDebugEnabled()) {
            log.debug("Kafka fallback message: {}, {}", record.topic(), record.value());
        }

        // 初始化消息队列
        if (this.queue == null) {
            this.initializeMessageQueue();
        }

        // 将消息放入内存消息队列
        Session session = SessionContextHolder.getSession(false);
        if (!this.queue.offer(Pair.of(record, session))) {
            throw new RuntimeException("Kafka fallback queue is full: " + record.topic() + ", " + record.value());
        }
    }

    @Override
    public void destroy() throws Exception {
        if (this.consumer != null) {
            this.consumer.interrupt();
            ThreadUtils.join(this.consumer);
        }
    }
}
