package com.chelvc.framework.kafka.producer;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.kafka.context.KafkaContextHolder;
import com.chelvc.framework.redis.context.RedisContextHolder;
import com.chelvc.framework.redis.queue.DelayRedisQueue;
import com.chelvc.framework.redis.queue.RedisQueues;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.stereotype.Component;

/**
 * Kafka事务延时消息处理器
 *
 * @author Woody
 * @date 2025/5/7
 */
@Slf4j
@Component
@ConditionalOnClass(RedisContextHolder.class)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class TransactionMessageProcessor implements ApplicationRunner {
    private final KafkaTransactionChecker checker;

    /**
     * 处理事务延时消息
     *
     * @param queue   延时队列
     * @param message 延时消息
     */
    private void processing(DelayRedisQueue<String> queue, String message) {
        // 获取事务处理决策
        TransactionResolution resolution;
        TransactionMessage instance = JacksonUtils.deserialize(message, TransactionMessage.class);
        SessionContextHolder.setSession(instance.getSession());
        try {
            resolution = ObjectUtils.ifNull(this.checker.check(instance), TransactionResolution.UNKNOWN);
            if (resolution == TransactionResolution.COMMIT) {
                // 本地事务处理成功则重新发送消息
                String topic = instance.getTopic();
                Object payload = instance.getPayload();
                Object ordering = instance.getOrdering();
                Integer partition = instance.getPartition();
                KafkaContextHolder.send(topic, payload, ordering, partition);
            }
        } finally {
            SessionContextHolder.removeSessionContext();
        }

        // 如果事务处理决策是非未知则删除事务延时消息
        if (resolution != TransactionResolution.UNKNOWN) {
            queue.remove(message);
        }
    }

    @Override
    public void run(ApplicationArguments args) throws Exception {
        DelayRedisQueue<String> queue = KafkaContextHolder.getTransactionQueue();
        RedisQueues.consume(queue, 100, messages -> messages.forEach(message -> {
            try {
                this.processing(queue, message);
            } catch (Exception e) {
                log.error("Transaction message processing failed: {}", message);
            }
        }));
    }
}
