package com.chelvc.framework.kubernetes.context;

import java.io.IOException;
import java.util.Collections;
import java.util.Map;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.common.util.ThreadUtils;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Maps;
import io.kubernetes.client.openapi.ApiClient;
import io.kubernetes.client.openapi.ApiException;
import io.kubernetes.client.openapi.Configuration;
import io.kubernetes.client.openapi.apis.CoreV1Api;
import io.kubernetes.client.openapi.models.V1ConfigMap;
import io.kubernetes.client.util.Config;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * Kubernetes上下文工具
 *
 * @author Woody
 * @date 2025/3/19
 */
@Slf4j
public final class KubernetesContextHolder {
    /**
     * 核心API对象实例
     */
    private static final CoreV1Api CORE_API;

    private KubernetesContextHolder() {
    }

    static {
        // 初始化默认客户端
        ApiClient client;
        try {
            client = Config.defaultClient();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        Configuration.setDefaultApiClient(client);

        // 初始化核心API对象实例
        CORE_API = new CoreV1Api();
    }

    /**
     * 获取核心API对象实例
     *
     * @return 核心API对象实例
     */
    public static CoreV1Api getCoreApi() {
        return CORE_API;
    }

    /**
     * 获取配置
     *
     * @return 配置键/值映射表
     */
    public static Map<String, String> getConfig() {
        return getConfig(ApplicationContextHolder.getApplicationName(), ApplicationContextHolder.getProfile());
    }

    /**
     * 获取配置
     *
     * @param name      配置名称
     * @param namespace 命名空间
     * @return 配置键/值映射表
     */
    public static Map<String, String> getConfig(@NonNull String name, @NonNull String namespace) {
        V1ConfigMap response;
        CoreV1Api api = getCoreApi();
        try {
            response = api.readNamespacedConfigMap(name, namespace, null);
        } catch (ApiException e) {
            throw new RuntimeException(e);
        }
        Map<String, String> config = ObjectUtils.ifNull(response, V1ConfigMap::getData);
        return ObjectUtils.isEmpty(config) ? Collections.emptyMap() : config;
    }

    /**
     * 更新配置
     *
     * @param key   配置键
     * @param value 配置值
     */
    public static void updateConfig(@NonNull String key, String value) {
        updateConfig(ApplicationContextHolder.getApplicationName(), ApplicationContextHolder.getProfile(), key, value);
    }

    /**
     * 更新配置
     *
     * @param name      配置名称
     * @param namespace 命名空间
     * @param key       配置键
     * @param value     配置值
     */
    public static void updateConfig(@NonNull String name, @NonNull String namespace, @NonNull String key,
                                    String value) {
        updateConfig(name, namespace, ImmutableMap.of(key, ObjectUtils.ifNull(value, StringUtils.EMPTY)));
    }

    /**
     * 更新配置
     *
     * @param config 配置键/值映射表
     */
    public static void updateConfig(@NonNull Map<String, String> config) {
        updateConfig(ApplicationContextHolder.getApplicationName(), ApplicationContextHolder.getProfile(), config);
    }

    /**
     * 更新配置
     *
     * @param name      配置名称
     * @param namespace 命名空间
     * @param config    配置键/值映射表
     */
    public static void updateConfig(@NonNull String name, @NonNull String namespace,
                                    @NonNull Map<String, String> config) {
        if (ObjectUtils.isEmpty(config)) {
            return;
        }

        int retry = 2;
        CoreV1Api api = getCoreApi();
        do {
            try {
                V1ConfigMap configmap = api.readNamespacedConfigMap(name, namespace, null);
                Map<String, String> original = ObjectUtils.ifNull(configmap, V1ConfigMap::getData);
                Map<String, String> update =
                        Maps.newHashMapWithExpectedSize(ObjectUtils.size(original) + config.size());
                if (ObjectUtils.notEmpty(original)) {
                    update.putAll(original);
                }
                config.forEach((key, value) -> {
                    if (StringUtils.isEmpty(value)) {
                        update.remove(key);
                    } else {
                        update.put(key, value);
                    }
                });
                configmap.setData(update);
                api.replaceNamespacedConfigMap(name, namespace, configmap, null, null, null, null);
            } catch (ApiException e) {
                if (retry == 0) {
                    throw new RuntimeException(e);
                }
                log.warn("Kubernetes configmap replace failed: {}, {}", name, namespace, e);
                ThreadUtils.sleep(200);
            }
        } while (retry-- > 0);
    }

    /**
     * 删除配置
     */
    public static void removeConfig() {
        removeConfig(ApplicationContextHolder.getApplicationName(), ApplicationContextHolder.getProfile());
    }

    /**
     * 删除配置
     *
     * @param name      配置名称
     * @param namespace 命名空间
     */
    public static void removeConfig(@NonNull String name, @NonNull String namespace) {
        CoreV1Api api = getCoreApi();
        try {
            api.deleteNamespacedConfigMap(name, namespace, null, null, null, null, null, null);
        } catch (ApiException e) {
            throw new RuntimeException(e);
        }
    }
}
