package com.chelvc.framework.location.support;

import java.time.Duration;
import java.util.List;
import java.util.concurrent.ThreadLocalRandom;
import java.util.function.Supplier;

import com.chelvc.framework.location.Address;
import com.chelvc.framework.location.LocationHandler;
import com.chelvc.framework.redis.context.RedisContextHolder;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Hex;

/**
 * 可缓存的地址定位实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class CacheableLocationHandler implements LocationHandler {
    private final int expiration;
    private final LocationHandler handler;

    public CacheableLocationHandler(@NonNull LocationHandler handler, int expiration) {
        this.handler = handler;
        this.expiration = expiration;
    }

    /**
     * 地址定位缓存处理
     *
     * @param code     缓存标识
     * @param supplier 地址定位处理回调函数
     * @param <T>      数据类型
     * @return 结果数据
     */
    @SuppressWarnings("unchecked")
    private <T> T execute(String code, Supplier<T> supplier) {
        String key = "location:" + code;
        T t = null;
        try {
            t = (T) RedisContextHolder.getDefaultTemplate().opsForValue().get(key);
        } catch (Exception e) {
            log.warn("Location cache handle failed: {}", e.getMessage());
        }
        if (t == null && (t = supplier.get()) != null) {
            // 为避免缓存穿透，缓存过期时间增加随机数
            int random = ThreadLocalRandom.current().nextInt(60);
            Duration duration = Duration.ofSeconds(this.expiration + random);
            try {
                RedisContextHolder.getDefaultTemplate().opsForValue().set(key, t, duration);
            } catch (Exception e) {
                log.warn("Location cache handle failed: {}", e.getMessage());
            }
        }
        return t;
    }

    @Override
    public List<Address> addresses() {
        return this.handler.addresses();
    }

    @Override
    public Address analyse(@NonNull String address) {
        return this.execute(Hex.encodeHexString(address.getBytes()), () -> this.handler.analyse(address));
    }

    @Override
    public Address ip2address(@NonNull String ip) {
        return this.execute(ip, () -> this.handler.ip2address(ip));
    }

    @Override
    public Address mobile2address(@NonNull String mobile) {
        return this.execute(mobile, () -> this.handler.mobile2address(mobile));
    }

    @Override
    public Address location2address(@NonNull String location) {
        return this.execute(location, () -> this.handler.location2address(location));
    }

    @Override
    public Address location2address(double longitude, double latitude) {
        String location = longitude + "," + latitude;
        return this.execute(location, () -> this.handler.location2address(longitude, latitude));
    }
}
