package com.chelvc.framework.location.support;

import java.util.List;
import java.util.function.Function;

import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.location.Address;
import com.chelvc.framework.location.LocationHandler;
import com.google.common.collect.Lists;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 地址定位处理器委派实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class DelegatingLocationHandler implements LocationHandler {
    private final List<LocationHandler> handlers;

    public DelegatingLocationHandler(@NonNull List<LocationHandler> handlers) {
        AssertUtils.nonempty(handlers, () -> "Delegating handlers must not be empty");
        this.handlers = Lists.newArrayList(handlers);
    }

    /**
     * 地址定位处理
     *
     * @param callback 地址定位处理回调函数
     * @param <T>      数据类型
     * @return 结果数据
     */
    private <T> T execute(Function<LocationHandler, T> callback) {
        Exception unsupported = null;
        for (LocationHandler handler : this.handlers) {
            try {
                T t = callback.apply(handler);
                if (t != null) {
                    return t;
                }
            } catch (Exception e) {
                if (e instanceof UnsupportedOperationException) {
                    unsupported = e;
                } else {
                    log.warn("Location handle failed: {}", e.getMessage());
                }
            }
        }
        if (unsupported != null) {
            log.warn("Location handle failed: {}", unsupported.getMessage());
        }
        return null;
    }

    @Override
    public List<Address> addresses() {
        return this.execute(LocationHandler::addresses);
    }

    @Override
    public Address analyse(@NonNull String address) {
        return this.execute(handler -> handler.analyse(address));
    }

    @Override
    public Address ip2address(@NonNull String ip) {
        return this.execute(handler -> handler.ip2address(ip));
    }

    @Override
    public Address mobile2address(@NonNull String mobile) {
        return this.execute(handler -> handler.mobile2address(mobile));
    }

    @Override
    public Address location2address(@NonNull String location) {
        return this.execute(handler -> handler.location2address(location));
    }

    @Override
    public Address location2address(double longitude, double latitude) {
        return this.execute(handler -> handler.location2address(longitude, latitude));
    }
}
