package com.chelvc.framework.location.support;

import java.util.List;
import java.util.Objects;

import com.chelvc.framework.base.context.RestContextHolder;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.location.Address;
import com.chelvc.framework.location.Area;
import com.chelvc.framework.location.LocationHandler;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 基于聚合平台地址定位实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class JuheLocationHandler implements LocationHandler {
    /**
     * 手机号定位接口地址
     */
    private static final String MOBILE2ADDRESS_URL = "http://apis.juhe.cn/mobile/get?phone=%s&key=%s";

    private final String key;

    public JuheLocationHandler(@NonNull String key) {
        this.key = key;
    }

    /**
     * 执行定位请求
     *
     * @param url       定位接口地址
     * @param parameter 定位参数
     * @return 地址信息
     */
    private Address locate(String url, String parameter) {
        url = String.format(url, parameter, this.key);
        boolean debug = log.isDebugEnabled();
        if (debug) {
            log.debug("Juhe locate request: {}, {}", url, parameter);
        }
        JuheLocationResponse response;
        try {
            response = RestContextHolder.get(url, JuheLocationResponse.class);
        } catch (Exception e) {
            log.warn("Juhe locate failed: {}, {}", parameter, e.getMessage());
            return null;
        }
        if (debug) {
            log.debug("Juhe locate response: {}, {}", parameter, response);
        }
        if (response == null || response.getStatus() == null || response.getStatus() != 0) {
            log.warn("Juhe locate failed: {}, {}", parameter, response);
            return null;
        }
        String province = ObjectUtils.ifNull(response.getResult(), JuheLocationResponse.Position::getProvince);
        String city = ObjectUtils.ifNull(response.getResult(), JuheLocationResponse.Position::getCity);
        if (StringUtils.isEmpty(province) && StringUtils.isEmpty(city)) {
            return null;
        } else if (Objects.equals(province, city) || StringUtils.isEmpty(city)) {
            // 针对直辖市省、市名称相同的地址只返回市名称
            return Address.builder().province(Area.builder().name(province).build()).fullname(province).build();
        } else if (StringUtils.isEmpty(province)) {
            return Address.builder().city(Area.builder().name(city).build()).fullname(city).build();
        }
        return Address.builder().province(Area.builder().name(province).build())
                .city(Area.builder().name(city).build()).fullname(province + city).build();
    }

    @Override
    public List<Address> addresses() {
        throw new UnsupportedOperationException();
    }

    @Override
    public Address analyse(String address) {
        throw new UnsupportedOperationException();
    }

    @Override
    public Address ip2address(String ip) {
        throw new UnsupportedOperationException();
    }

    @Override
    public Address mobile2address(String mobile) {
        return this.locate(MOBILE2ADDRESS_URL, mobile);
    }

    @Override
    public Address location2address(String location) {
        throw new UnsupportedOperationException();
    }

    @Override
    public Address location2address(double longitude, double latitude) {
        throw new UnsupportedOperationException();
    }
}
