package com.chelvc.framework.location.support;

import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.chelvc.framework.base.context.RestContextHolder;
import com.chelvc.framework.common.model.Region;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.location.Address;
import com.chelvc.framework.location.Area;
import com.chelvc.framework.location.LocationHandler;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 腾讯地址定位处理实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class TencentLocationHandler implements LocationHandler {
    /**
     * 获取行政区划接口地址
     */
    private static final String GET_ADDRESS_URL = "https://apis.map.qq.com/ws/district/v1/list?key=%s&output=JSON";

    /**
     * IP定位接口地址
     */
    private static final String IP2ADDRESS_URL = "https://apis.map.qq.com/ws/location/v1/ip?ip=%s&key=%s";

    /**
     * 经纬度定位接口地址
     */
    private static final String LOCATION2ADDRESS_URL = "https://apis.map.qq.com/ws/geocoder/v1/?location=%s&key=%s";

    /**
     * 地址解析接口地址
     */
    private static final String ADDRESS_ANALYSE_URL = "https://apis.map.qq.com/ws/geocoder/v1/?address=%s&key=%s";

    private final String key;

    public TencentLocationHandler(@NonNull String key) {
        this.key = key;
    }

    /**
     * 执行定位请求
     *
     * @param url       定位接口地址
     * @param parameter 定位参数
     * @return 地址信息
     */
    private Address locate(String url, String parameter) {
        url = String.format(url, parameter, this.key);
        boolean debug = log.isDebugEnabled();
        if (debug) {
            log.debug("Tencent locate request: {}, {}", url, parameter);
        }
        TencentLocationResponse response;
        try {
            response = RestContextHolder.get(url, TencentLocationResponse.class);
        } catch (Exception e) {
            log.warn("Tencent locate failed: {}, {}", parameter, e.getMessage());
            return null;
        }
        if (debug) {
            log.debug("Tencent locate response: {}, {}", parameter, response);
        }
        if (response == null || response.getStatus() == null || response.getStatus() != 0) {
            log.warn("Tencent locate failed: {}, {}", parameter, response);
            return null;
        }
        TencentLocationResponse.Position position = response.getResult().getPosition();
        if (!Region.isRegion(position.getAdcode())) {
            log.warn("Tencent invalid location: {}, {}", parameter, position.getAdcode());
            return null;
        }

        // 构建地址信息
        Region region = Objects.requireNonNull(Region.of(position.getAdcode()));
        TencentLocationResponse.Location location = response.getResult().getLocation();
        Double longitude = ObjectUtils.ifNull(location, TencentLocationResponse.Location::getLongitude);
        Double latitude = ObjectUtils.ifNull(location, TencentLocationResponse.Location::getLatitude);
        Address address = Address.builder().id(region.getCode()).longitude(longitude).latitude(latitude)
                .fullname(response.getResult().getAddress()).build();
        address.setName(Stream.of(position.getProvince(), position.getCity(), position.getDistrict())
                .filter(StringUtils::notEmpty).collect(Collectors.joining("·")
                ));
        address.setProvince(Area.builder().id(Region.code2province(position.getAdcode()))
                .name(position.getProvince()).build());
        if (!region.isProvincial()) {
            address.setCity(Area.builder().id(Region.code2city(region.getCode())).name(position.getCity()).build());
        }
        if (region.isDistrict()) {
            address.setDistrict(Area.builder().id(region.getCode()).name(position.getDistrict()).build());
        }
        return address;
    }

    @Override
    public List<Address> addresses() {
        String url = String.format(GET_ADDRESS_URL, this.key);
        boolean debug = log.isDebugEnabled();
        if (debug) {
            log.debug("Tencent addressing request: {}", url);
        }
        TencentAddressResponse response;
        try {
            response = RestContextHolder.get(url, TencentAddressResponse.class);
        } catch (Exception e) {
            log.warn("Tencent addressing failed: {}", e.getMessage());
            return Collections.emptyList();
        }
        if (debug) {
            log.debug("Tencent addressing response: {}", response);
        }
        if (response == null || response.getStatus() == null || response.getStatus() != 0) {
            log.warn("Tencent addressing failed: {}", response);
            return Collections.emptyList();
        }
        if (ObjectUtils.isEmpty(response.getResult())) {
            return Collections.emptyList();
        }
        return response.getResult().stream().filter(ObjectUtils::notEmpty).flatMap(List::stream)
                .collect(Collectors.toList());
    }

    @Override
    public Address analyse(@NonNull String address) {
        return this.locate(ADDRESS_ANALYSE_URL, address);
    }

    @Override
    public Address ip2address(@NonNull String ip) {
        return this.locate(IP2ADDRESS_URL, ip);
    }

    @Override
    public Address mobile2address(String mobile) {
        throw new UnsupportedOperationException();
    }

    @Override
    public Address location2address(@NonNull String location) {
        return this.locate(LOCATION2ADDRESS_URL, location);
    }

    @Override
    public Address location2address(double longitude, double latitude) {
        return this.location2address(latitude + "," + longitude);
    }
}
