package com.chelvc.framework.redis.config;

import java.time.Duration;

import com.chelvc.framework.common.util.JacksonUtils;
import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.PropertyAccessor;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.MapperFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.databind.jsontype.impl.LaissezFaireSubTypeValidator;
import lombok.RequiredArgsConstructor;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.AutoConfigureAfter;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.data.redis.RedisAutoConfiguration;
import org.springframework.cache.CacheManager;
import org.springframework.cache.annotation.CachingConfigurerSupport;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.cache.interceptor.KeyGenerator;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Primary;
import org.springframework.core.task.TaskExecutor;
import org.springframework.data.redis.cache.RedisCacheConfiguration;
import org.springframework.data.redis.cache.RedisCacheManager;
import org.springframework.data.redis.cache.RedisCacheWriter;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.listener.RedisMessageListenerContainer;
import org.springframework.data.redis.serializer.Jackson2JsonRedisSerializer;
import org.springframework.data.redis.serializer.RedisSerializationContext;
import org.springframework.data.redis.serializer.RedisSerializer;
import org.springframework.stereotype.Component;

/**
 * Redis缓存配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Component
@EnableCaching
@AutoConfigureAfter(RedisAutoConfiguration.class)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class RedisConfigurer extends CachingConfigurerSupport {
    private final TaskExecutor executor;
    private final LettuceConnectionFactory connectionFactory;

    @Bean
    public KeyGenerator keyGenerator() {
        return (target, method, params) -> {
            StringBuilder builder = new StringBuilder();
            builder.append(target.getClass().getName());
            builder.append(method.getName());
            builder.append("&");
            for (Object param : params) {
                if (param != null) {
                    builder.append(param.getClass().getName());
                    builder.append("&");
                    builder.append(JacksonUtils.serialize(param));
                    builder.append("&");
                }
            }
            return DigestUtils.sha256Hex(builder.toString());
        };
    }

    @Bean
    public CacheManager cacheManager() {
        RedisCacheWriter writer = RedisCacheWriter.nonLockingRedisCacheWriter(this.connectionFactory);
        RedisSerializationContext.SerializationPair<?> serialization =
                RedisSerializationContext.SerializationPair.fromSerializer(this.jackson2JsonRedisSerializer());
        RedisCacheConfiguration redisCacheConfiguration = RedisCacheConfiguration.defaultCacheConfig()
                .disableCachingNullValues().entryTtl(Duration.ofMinutes(10)).serializeValuesWith(serialization);
        return RedisCacheManager.builder(writer).cacheDefaults(redisCacheConfiguration).build();
    }

    @Bean
    @Primary
    public RedisMessageListenerContainer redisMessageListenerContainer(RedisProperties properties) {
        RedisProperties.Listener config = properties.getListener();
        RedisMessageListenerContainer container = new RedisMessageListenerContainer();
        container.setTaskExecutor(this.executor);
        container.setConnectionFactory(this.connectionFactory);
        container.setRecoveryInterval(config.getRecoveryInterval());
        container.setMaxSubscriptionRegistrationWaitingTime(config.getMaxSubscriptionRegistrationWaitingTime());
        return container;
    }

    @Bean
    public RedisTemplate<String, Object> redisTemplate() {
        RedisTemplate<String, Object> template = new RedisTemplate<>();
        template.setConnectionFactory(this.connectionFactory);
        template.setKeySerializer(RedisSerializer.string());
        template.setValueSerializer(this.jackson2JsonRedisSerializer());
        template.setHashKeySerializer(RedisSerializer.string());
        template.setHashValueSerializer(this.jackson2JsonRedisSerializer());
        template.afterPropertiesSet();
        return template;
    }

    @Bean
    public Jackson2JsonRedisSerializer<Object> jackson2JsonRedisSerializer() {
        // objectMapper for redis serialization
        ObjectMapper mapper = new ObjectMapper();
        mapper.setVisibility(PropertyAccessor.ALL, JsonAutoDetect.Visibility.ANY);
        mapper.setSerializationInclusion(JsonInclude.Include.ALWAYS);
        mapper.activateDefaultTyping(LaissezFaireSubTypeValidator.instance, ObjectMapper.DefaultTyping.NON_FINAL);
        mapper.configure(MapperFeature.USE_ANNOTATIONS, true);
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        mapper.configure(SerializationFeature.FAIL_ON_EMPTY_BEANS, false);
        // jackson2JsonRedisSerializer
        Jackson2JsonRedisSerializer<Object> serializer = new Jackson2JsonRedisSerializer<>(Object.class);
        serializer.setObjectMapper(mapper);
        return serializer;
    }

    @Bean
    @ConditionalOnMissingBean(RedisMQListenerRegistry.class)
    public RedisMQListenerRegistry redisMQListenerRegistry() {
        return new RedisMQListenerRegistry();
    }
}
