package com.chelvc.framework.redis.config;

import java.lang.reflect.Type;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.redis.annotation.RedisMQConsumer;
import com.chelvc.framework.redis.queue.DefaultRedisMQListenerContainer;
import com.chelvc.framework.redis.queue.RedisMQListener;
import com.chelvc.framework.redis.queue.RedisMQListenerContainer;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.aop.framework.AopProxyUtils;
import org.springframework.boot.context.event.ApplicationStartedEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.context.support.GenericApplicationContext;

/**
 * Redis消息监听器注册器
 *
 * @author Woody
 * @date 2024/7/13
 */
@Slf4j
public class RedisMQListenerRegistry implements ApplicationListener<ApplicationStartedEvent> {
    /**
     * 创建监听器容器
     *
     * @param <T> 消息对象类型
     * @return 容器实例
     */
    @SuppressWarnings("unchecked")
    protected <T> RedisMQListenerContainer<T> createListenerContainer() {
        String name = StringUtils.uuid();
        GenericApplicationContext applicationContext =
                (GenericApplicationContext) ApplicationContextHolder.getApplicationContext();
        applicationContext.registerBean(name, RedisMQListenerContainer.class, DefaultRedisMQListenerContainer::new);
        return applicationContext.getBean(name, RedisMQListenerContainer.class);
    }

    /**
     * 注册消息监听器容器
     *
     * @param name     消息监听器名称
     * @param listener 消息监听器实例
     * @param <T>      消息对象类型
     */
    protected <T> void registerListenerContainer(@NonNull String name, @NonNull RedisMQListener<T> listener) {
        Class<?> clazz = AopProxyUtils.ultimateTargetClass(listener);
        RedisMQConsumer annotation = clazz.getAnnotation(RedisMQConsumer.class);
        AssertUtils.state(annotation != null, () -> "@RedisMQConsumer annotation is missing: " + clazz.getName());
        RedisMQListenerContainer<T> container = this.createListenerContainer();
        Type type = ObjectUtils.lookupSuperclassParameterized(clazz, RedisMQListener.class, Object.class);
        try {
            container.initialize(type, listener, annotation);
        } catch (Exception e) {
            log.error("RedisMQListenerContainer initialize failed", e);
        }
    }

    @Override
    public void onApplicationEvent(ApplicationStartedEvent event) {
        ApplicationContextHolder.getBeans(RedisMQListener.class).forEach(this::registerListenerContainer);
    }
}
