package com.chelvc.framework.redis.context;

import java.io.Serializable;
import java.time.Duration;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import com.chelvc.framework.common.util.DateUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import lombok.NonNull;
import org.springframework.data.redis.core.RedisTemplate;

/**
 * Redis Hash结构当日数据操作工具类
 *
 * @author Woody
 * @date 2024/6/29
 */
public final class RedisDailyHashHolder {
    private RedisDailyHashHolder() {
    }

    /**
     * 获取过期时间倒计时
     *
     * @return 倒计时长
     */
    public static Duration countdown() {
        return DateUtils.countdown(Calendar.DATE);
    }

    /**
     * 获取Redis Hash结构名称
     *
     * @param type 数据类型
     * @param id   数据标识
     * @return Hash结构名称
     */
    public static String name(@NonNull String type, @NonNull Serializable id) {
        return type + ":" + id + ":" + DateUtils.date2number(DateUtils.today());
    }

    /**
     * 获取Redis Hash结构当日数据
     *
     * @param type 数据类型
     * @param id   数据标识
     * @param key  Hash键
     * @return Hash值
     */
    public static Object get(@NonNull String type, @NonNull Serializable id, @NonNull String key) {
        return RedisHashHolder.get(name(type, id), key);
    }

    /**
     * 获取Redis Hash结构当日数据
     *
     * @param template Redis操作模版实例
     * @param type     数据类型
     * @param id       数据标识
     * @param key      Hash键
     * @return Hash值
     */
    public static Object get(@NonNull RedisTemplate<String, ?> template, @NonNull String type, @NonNull Serializable id,
                             @NonNull String key) {
        return RedisHashHolder.get(template, name(type, id), key);
    }

    /**
     * 获取Redis Hash结构当日数据
     *
     * @param type 数据类型
     * @param id   数据标识
     * @param keys Hash键数组
     * @return Hash值列表
     */
    public static List<?> get(@NonNull String type, @NonNull Serializable id, @NonNull String... keys) {
        return ObjectUtils.isEmpty(keys) ? Collections.emptyList() : RedisHashHolder.get(name(type, id), keys);
    }

    /**
     * 获取Redis Hash结构当日数据
     *
     * @param template Redis操作模版实例
     * @param type     数据类型
     * @param id       数据标识
     * @param keys     Hash键数组
     * @return Hash值列表
     */
    public static List<?> get(@NonNull RedisTemplate<String, ?> template, @NonNull String type,
                              @NonNull Serializable id, @NonNull String... keys) {
        if (ObjectUtils.isEmpty(keys)) {
            return Collections.emptyList();
        }
        return RedisHashHolder.get(template, name(type, id), keys);
    }

    /**
     * 获取Redis Hash结构当日数据
     *
     * @param type 数据类型
     * @param id   数据标识
     * @param keys Hash键集合
     * @return Hash值列表
     */
    public static List<?> get(@NonNull String type, @NonNull Serializable id, @NonNull Collection<String> keys) {
        return ObjectUtils.isEmpty(keys) ? Collections.emptyList() : RedisHashHolder.get(name(type, id), keys);
    }

    /**
     * 获取Redis Hash结构当日数据
     *
     * @param template Redis操作模版实例
     * @param type     数据类型
     * @param id       数据标识
     * @param keys     Hash键集合
     * @return Hash值列表
     */
    public static List<?> get(@NonNull RedisTemplate<String, ?> template, @NonNull String type,
                              @NonNull Serializable id, @NonNull Collection<String> keys) {
        if (ObjectUtils.isEmpty(keys)) {
            return Collections.emptyList();
        }
        return RedisHashHolder.get(template, name(type, id), keys);
    }

    /**
     * 设置Redis Hash结构当日数据
     *
     * @param type  数据类型
     * @param id    数据标识
     * @param array Hash键/值数组
     * @return true/false
     */
    public static boolean set(@NonNull String type, @NonNull Serializable id, @NonNull Object... array) {
        return ObjectUtils.notEmpty(array) && RedisHashHolder.set(name(type, id), array, countdown());
    }

    /**
     * 设置Redis Hash结构当日数据
     *
     * @param type 数据类型
     * @param id   数据标识
     * @param map  Hash键/值映射表
     * @return true/false
     */
    public static boolean set(@NonNull String type, @NonNull Serializable id, @NonNull Map<String, ?> map) {
        return ObjectUtils.notEmpty(map) && RedisHashHolder.set(name(type, id), map, countdown());
    }

    /**
     * 设置Redis Hash结构当日数据
     *
     * @param template Redis操作模版实例
     * @param type     数据类型
     * @param id       数据标识
     * @param array    Hash键/值数组
     * @return true/false
     */
    public static boolean set(@NonNull RedisTemplate<String, ?> template, @NonNull String type,
                              @NonNull Serializable id, @NonNull Object... array) {
        return ObjectUtils.notEmpty(array) && RedisHashHolder.set(template, name(type, id), array, countdown());
    }

    /**
     * 设置Redis Hash结构当日数据
     *
     * @param template Redis操作模版实例
     * @param type     数据类型
     * @param id       数据标识
     * @param map      Hash键/值映射表
     * @return true/false
     */
    public static boolean set(@NonNull RedisTemplate<String, ?> template, @NonNull String type,
                              @NonNull Serializable id, @NonNull Map<String, ?> map) {
        return ObjectUtils.notEmpty(map) && RedisHashHolder.set(template, name(type, id), map, countdown());
    }

    /**
     * Redis Hash结构当日数字自增1
     *
     * @param type 数据类型
     * @param id   数据标识
     * @param key  Hash键
     * @return 增/减后数值
     */
    public static long increment(@NonNull String type, @NonNull Serializable id, @NonNull String key) {
        return RedisHashHolder.increment(name(type, id), key, countdown());
    }

    /**
     * Redis Hash结构当日数字自增/减，delta < 0 时自减
     *
     * @param type  数据类型
     * @param id    数据标识
     * @param key   Hash键
     * @param delta 增/减值
     * @return 增/减后数值
     */
    public static long increment(@NonNull String type, @NonNull Serializable id, @NonNull String key, long delta) {
        return RedisHashHolder.increment(name(type, id), key, delta, countdown());
    }

    /**
     * Redis Hash结构当日数字自增1
     *
     * @param template Redis操作模版实例
     * @param type     数据类型
     * @param id       数据标识
     * @param key      Hash键
     * @return 增/减后数值
     */
    public static long increment(@NonNull RedisTemplate<String, ?> template, @NonNull String type,
                                 @NonNull Serializable id, @NonNull String key) {
        return RedisHashHolder.increment(template, name(type, id), key, countdown());
    }

    /**
     * Redis Hash结构当日数字自增/减，delta < 0 时自减
     *
     * @param template Redis操作模版实例
     * @param type     数据类型
     * @param id       数据标识
     * @param key      Hash键
     * @param delta    增/减值
     * @return 增/减后数值
     */
    public static long increment(@NonNull RedisTemplate<String, ?> template, @NonNull String type,
                                 @NonNull Serializable id, @NonNull String key, long delta) {
        return RedisHashHolder.increment(template, name(type, id), key, delta, countdown());
    }

    /**
     * Redis Hash结构删除当日数据
     *
     * @param type 数据类型
     * @param id   数据标识
     * @param keys Hash键数组
     * @return 成功删除数量
     */
    public static long delete(@NonNull String type, @NonNull Serializable id, @NonNull String... keys) {
        return ObjectUtils.isEmpty(keys) ? 0 : RedisHashHolder.delete(name(type, id), keys);
    }

    /**
     * Redis Hash结构删除当日数据
     *
     * @param template Redis操作模版实例
     * @param type     数据类型
     * @param id       数据标识
     * @param keys     Hash键数组
     * @return 成功删除数量
     */
    public static long delete(@NonNull RedisTemplate<String, ?> template, @NonNull String type,
                              @NonNull Serializable id, @NonNull String... keys) {
        return ObjectUtils.isEmpty(keys) ? 0 : RedisHashHolder.delete(template, name(type, id), keys);
    }

    /**
     * Redis Hash结构删除当日数据
     *
     * @param type 数据类型
     * @param id   数据标识
     * @param keys Hash键集合
     * @return 成功删除数量
     */
    public static long delete(@NonNull String type, @NonNull Serializable id, @NonNull Collection<String> keys) {
        return ObjectUtils.isEmpty(keys) ? 0 : RedisHashHolder.delete(name(type, id), keys);
    }

    /**
     * Redis Hash结构删除当日数据
     *
     * @param template Redis操作模版实例
     * @param type     数据类型
     * @param id       数据标识
     * @param keys     Hash键集合
     * @return 成功删除数量
     */
    public static long delete(@NonNull RedisTemplate<String, ?> template, @NonNull String type,
                              @NonNull Serializable id, @NonNull Collection<String> keys) {
        return ObjectUtils.isEmpty(keys) ? 0 : RedisHashHolder.delete(template, name(type, id), keys);
    }
}
