package com.chelvc.framework.redis.context;

import java.lang.reflect.Type;
import java.nio.charset.StandardCharsets;
import java.time.Duration;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.Executor;
import java.util.function.Consumer;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.redis.config.RedisProperties;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Lists;
import lombok.NonNull;
import org.springframework.data.redis.RedisSystemException;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.connection.RedisStreamCommands;
import org.springframework.data.redis.connection.stream.ByteRecord;
import org.springframework.data.redis.connection.stream.MapRecord;
import org.springframework.data.redis.connection.stream.RecordId;
import org.springframework.data.redis.connection.stream.StreamInfo;
import org.springframework.data.redis.connection.stream.StreamOffset;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.script.DefaultRedisScript;
import org.springframework.data.redis.core.script.RedisScript;
import org.springframework.data.redis.serializer.RedisSerializer;
import org.springframework.data.redis.serializer.SerializationException;
import org.springframework.data.redis.stream.StreamListener;
import org.springframework.data.redis.stream.StreamMessageListenerContainer;

/**
 * Redis Stream操作工具类
 *
 * @author Woody
 * @date 2024/6/28
 */
public final class RedisStreamHolder {
    /**
     * 消息ID标识
     */
    private static final String ID = "id";

    /**
     * 消息主题标识
     */
    private static final String TOPIC = "topic";

    /**
     * 消息载体标识
     */
    private static final String PAYLOAD = "payload";

    /**
     * 延时时间标识
     */
    private static final String DELAYING = "delaying";

    /**
     * 新增并移除过期消息脚本
     */
    private static final RedisScript<String> XADD_MINID_SCRIPT = new DefaultRedisScript<>(
            "return redis.call('XADD', KEYS[1], 'MINID', '~', ARGV[1], '*', unpack(ARGV, 2))", String.class
    );

    /**
     * 对象字符串序列化处理器
     */
    private static final RedisSerializer<Object> OBJECT_STRING_SERIALIZER = new RedisSerializer<Object>() {
        @Override
        public byte[] serialize(Object o) throws SerializationException {
            return o == null ? null : o.toString().getBytes(StandardCharsets.UTF_8);
        }

        @Override
        public Object deserialize(byte[] bytes) throws SerializationException {
            return bytes == null ? null : new String(bytes, StandardCharsets.UTF_8);
        }
    };

    /**
     * 字符串RedisTemplate实例
     */
    private static volatile RedisTemplate<String, ?> STRING_TEMPLATE;

    private RedisStreamHolder() {
    }

    /**
     * 获取消息最小ID
     *
     * @return 消息ID
     */
    public static String getMinid() {
        RedisProperties properties = ApplicationContextHolder.getBean(RedisProperties.class);
        // 消息过期时间最短1分钟，最长24小时
        int expiration = Math.min(Math.max(properties.getStream().getExpiration(), 60), 86400);
        return (System.currentTimeMillis() - expiration * 1000) + "-0";
    }

    /**
     * 获取消息流处理RedisTemplate实例
     *
     * @param <T> 数据类型
     * @return RedisTemplate实例
     */
    @SuppressWarnings("unchecked")
    public static <T> RedisTemplate<String, T> getStreamTemplate() {
        if (STRING_TEMPLATE == null) {
            synchronized (RedisTemplate.class) {
                if (STRING_TEMPLATE == null) {
                    RedisTemplate<String, Object> template = new RedisTemplate<>();
                    template.setConnectionFactory(RedisContextHolder.getDefaultConnectionFactory());
                    template.setKeySerializer(RedisSerializer.string());
                    template.setValueSerializer(OBJECT_STRING_SERIALIZER);
                    template.setHashKeySerializer(RedisSerializer.string());
                    template.setHashValueSerializer(OBJECT_STRING_SERIALIZER);
                    template.afterPropertiesSet();

                    STRING_TEMPLATE = template;
                }
            }
        }
        return (RedisTemplate<String, T>) STRING_TEMPLATE;
    }

    /**
     * 环境隔离
     *
     * @param original 原始标记
     * @return 环境隔离标记
     */
    public static String isolate(@NonNull String original) {
        String namespace = ApplicationContextHolder.getProfile();
        return StringUtils.isEmpty(namespace) ? original : (namespace + "-" + original);
    }

    /**
     * 序列化消息记录
     *
     * @param record 消息记录
     * @return 消息内容
     */
    public static String serialize(@NonNull MapRecord<String, String, String> record) {
        Map<String, String> value = record.getValue();
        ImmutableMap.Builder<String, String> builder = ImmutableMap.builder();
        builder.put(ID, record.getId().getValue());
        builder.put(TOPIC, Objects.requireNonNull(record.getStream()));
        String payload = value.get(PAYLOAD);
        if (StringUtils.notEmpty(payload)) {
            builder.put(PAYLOAD, payload);
        }
        String session = value.get(Session.NAMING);
        if (StringUtils.notEmpty(session)) {
            builder.put(Session.NAMING, session);
        }
        String delaying = value.get(DELAYING);
        if (StringUtils.notEmpty(delaying)) {
            builder.put(DELAYING, delaying);
        }
        return JacksonUtils.serialize(builder.build());
    }

    /**
     * 反序列化消息记录
     *
     * @param message 消息内容
     * @return 消息记录实例
     */
    @SuppressWarnings("unchecked")
    public static MapRecord<String, String, String> deserialize(@NonNull String message) {
        Map<String, String> content = JacksonUtils.deserialize(message, Map.class);
        String id = content.remove(ID);
        String topic = content.remove(TOPIC);
        return MapRecord.create(topic, content).withId(RecordId.of(id));
    }

    /**
     * 发送消息
     *
     * @param topic   消息主题
     * @param payload 消息内容
     */
    public static void send(@NonNull String topic, @NonNull Object payload) {
        send(topic, payload, (Long) null);
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     */
    public static void send(@NonNull String topic, @NonNull Object payload, Duration delaying) {
        send(topic, payload, delaying == null ? null : System.currentTimeMillis() + delaying.toMillis());
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     */
    public static void send(@NonNull String topic, @NonNull Object payload, Date delaying) {
        send(topic, payload, ObjectUtils.ifNull(delaying, Date::getTime));
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     */
    public static void send(@NonNull String topic, @NonNull Object payload, Long delaying) {
        List<String> args = Lists.newLinkedList();
        args.add(getMinid());
        args.add(PAYLOAD);
        args.add(JacksonUtils.serialize(payload));
        if (delaying != null) {
            args.add(DELAYING);
            args.add(String.valueOf(delaying));
        }
        Session session = SessionContextHolder.getSession(false);
        if (session != null) {
            args.add(Session.NAMING);
            args.add(JacksonUtils.serialize(session));
        }
        List<String> keys = Collections.singletonList(isolate(topic));
        getStreamTemplate().execute(XADD_MINID_SCRIPT, keys, args.toArray());
    }

    /**
     * 转移消息
     *
     * @param topic    消息主题
     * @param consumer 目标消费者
     * @param ids      目标记录ID数组
     * @return 转移成功消息记录列表
     */
    public static List<ByteRecord> claim(@NonNull String topic,
                                         @NonNull org.springframework.data.redis.connection.stream.Consumer consumer,
                                         @NonNull RecordId... ids) {
        return claim(topic, consumer.getGroup(), consumer.getName(), ids);
    }

    /**
     * 转移消息
     *
     * @param topic    消息主题
     * @param group    消费者组
     * @param consumer 目标消费者名称
     * @param ids      目标记录ID数组
     * @return 转移成功消息记录列表
     */
    public static List<ByteRecord> claim(@NonNull String topic, @NonNull String group, @NonNull String consumer,
                                         @NonNull RecordId... ids) {
        return claim(topic, group, consumer, Duration.ZERO, ids);
    }

    /**
     * 转移消息
     *
     * @param topic    消息主题
     * @param consumer 目标消费者
     * @param idle     消息空闲时间
     * @param ids      目标ID数组
     * @return 转移成功消息记录列表
     */
    public static List<ByteRecord> claim(@NonNull String topic,
                                         @NonNull org.springframework.data.redis.connection.stream.Consumer consumer,
                                         @NonNull Duration idle, @NonNull RecordId... ids) {
        return claim(topic, consumer.getGroup(), consumer.getName(), idle, ids);
    }

    /**
     * 转移消息
     *
     * @param topic    消息主题
     * @param group    消费者组
     * @param consumer 目标消费者名称
     * @param idle     消息空闲时间
     * @param ids      目标ID数组
     * @return 转移成功消息记录列表
     */
    public static List<ByteRecord> claim(@NonNull String topic, @NonNull String group, @NonNull String consumer,
                                         @NonNull Duration idle, @NonNull RecordId... ids) {
        if (ObjectUtils.isEmpty(ids)) {
            return Collections.emptyList();
        }
        RedisStreamCommands.XClaimOptions options = RedisStreamCommands.XClaimOptions.minIdle(idle).ids(ids);
        return RedisContextHolder.execute(
                RedisContextHolder.getDefaultConnectionFactory(),
                connection -> connection.streamCommands().xClaim(topic.getBytes(), group, consumer, options)
        );
    }

    /**
     * 消息消费确认
     *
     * @param topic 消息主题
     * @param group 消费者组
     * @param ids   消息记录ID数组
     */
    public static void ack(@NonNull String topic, @NonNull String group, @NonNull RecordId... ids) {
        if (ObjectUtils.notEmpty(ids)) {
            getStreamTemplate().opsForStream().acknowledge(topic, group, ids);
        }
    }

    /**
     * 删除消费者
     *
     * @param topic 消息主题
     * @param ids   消息记录ID数组
     */
    public static void delete(@NonNull String topic, @NonNull RecordId... ids) {
        if (ObjectUtils.notEmpty(ids)) {
            getStreamTemplate().opsForStream().delete(topic, ids);
        }
    }

    /**
     * 删除消费者
     *
     * @param topic    消息主题
     * @param consumer 消费者信息
     */
    public static void delete(@NonNull String topic,
                              @NonNull org.springframework.data.redis.connection.stream.Consumer consumer) {
        getStreamTemplate().opsForStream().deleteConsumer(topic, consumer);
    }

    /**
     * 发送心跳包
     *
     * @param topic 消息主题
     */
    public static void heartbeat(@NonNull String topic) {
        List<String> keys = Collections.singletonList(topic);
        getStreamTemplate().execute(XADD_MINID_SCRIPT, keys, getMinid(), PAYLOAD, StringUtils.EMPTY);
    }

    /**
     * 判断消息是否是心跳包
     *
     * @param record 消息记录
     * @return true/false
     */
    public static boolean isHeartbeat(MapRecord<String, String, String> record) {
        Map<String, String> value = ObjectUtils.ifNull(record, MapRecord::getValue);
        return ObjectUtils.isEmpty(value) || StringUtils.isEmpty(value.get(PAYLOAD));
    }

    /**
     * 消费消息
     *
     * @param record   消息记录
     * @param type     消息类型
     * @param consumer 消息消费者
     */
    @SuppressWarnings("unchecked")
    public static <T> void consume(@NonNull MapRecord<String, String, String> record, @NonNull Type type,
                                   @NonNull Consumer<T> consumer) {
        Map<String, String> value = record.getValue();
        T payload = (T) JacksonUtils.deserialize(value.get(PAYLOAD), type);
        Session session = JacksonUtils.deserialize(value.get(Session.NAMING), Session.class);
        SessionContextHolder.setSession(session);
        try {
            consumer.accept(payload);
        } finally {
            SessionContextHolder.removeSessionContext();
        }
    }

    /**
     * 获取消费者信息
     *
     * @param topic 消息主题
     * @param group 消费者组
     * @return 消费者信息
     */
    public static StreamInfo.XInfoConsumers consumers(@NonNull String topic, @NonNull String group) {
        return getStreamTemplate().opsForStream().consumers(topic, group);
    }

    /**
     * 获取消息延时时间戳
     *
     * @param record 消息记录
     * @return 时间戳
     */
    public static Long getMessageDelaying(@NonNull MapRecord<String, String, String> record) {
        String delaying = ObjectUtils.ifNull(record.getValue(), value -> value.get(DELAYING));
        return StringUtils.ifEmpty(delaying, Long::parseLong);
    }

    /**
     * 初始化消费者组
     *
     * @param topic 消息主题
     * @param group 消费者组
     */
    public static void initializeConsumerGroup(@NonNull String topic, @NonNull String group) {
        try {
            getStreamTemplate().opsForStream().createGroup(topic, group);
        } catch (RedisSystemException e) {
            if (StringUtils.isEmpty(e.getMessage()) || !e.getMessage().contains("BUSYGROUP")) {
                throw e;
            }
        }
    }

    /**
     * 初始化消息监听器容器
     *
     * @param listener 消息监听器
     * @param consumer 消费者信息
     * @param offset   消息偏移信息
     * @param batch    批量获取消息数量
     * @param executor 消息获取执行器
     * @return 消息监听器容器
     */
    public static StreamMessageListenerContainer<String, MapRecord<String, String, String>>
    initializeMessageListenerContainer(@NonNull StreamListener<String, MapRecord<String, String, String>> listener,
                                       @NonNull org.springframework.data.redis.connection.stream.Consumer consumer,
                                       @NonNull StreamOffset<String> offset, int batch, @NonNull Executor executor) {
        StreamMessageListenerContainer.
                StreamMessageListenerContainerOptionsBuilder<String, MapRecord<String, String, String>>
                builder = StreamMessageListenerContainer.StreamMessageListenerContainerOptions.builder()
                .batchSize(batch).executor(executor).serializer(RedisSerializer.string());
        RedisConnectionFactory connectionFactory = RedisContextHolder.getDefaultConnectionFactory();
        StreamMessageListenerContainer<String, MapRecord<String, String, String>> container =
                StreamMessageListenerContainer.create(connectionFactory, builder.build());
        StreamMessageListenerContainer.StreamReadRequest<String> request =
                StreamMessageListenerContainer.StreamReadRequest.builder(offset).consumer(consumer)
                        .cancelOnError(t -> false).build();
        container.register(request, listener);
        return container;
    }
}
