package com.chelvc.framework.redis.queue;

import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.redis.context.RedisContextHolder;
import com.chelvc.framework.redis.context.RedisStreamHolder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.connection.stream.Consumer;
import org.springframework.data.redis.connection.stream.MapRecord;
import org.springframework.data.redis.connection.stream.ReadOffset;
import org.springframework.data.redis.connection.stream.StreamOffset;
import org.springframework.data.redis.stream.StreamMessageListenerContainer;

/**
 * 消费者获取未确认消息清理处理器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class ConsumerPendingCleaner<T> {
    private final String topic;
    private final Consumer source;
    private final Consumer target;
    private final MessageStreamListener<T> listener;
    private final Executor executor;
    private ScheduledExecutorService monitor;
    private StreamMessageListenerContainer<String, MapRecord<String, String, String>> container;
    private volatile long timestamp;

    public ConsumerPendingCleaner(String topic, Consumer source, Consumer target, MessageStreamListener<T> listener,
                                  Executor executor) {
        this.topic = topic;
        this.source = source;
        this.target = target;
        this.listener = listener;
        this.executor = executor;
    }

    /**
     * 开始清理
     */
    public void start() {
        // 锁定消费者，确保只有一个消费者对目标消费者数据进行转移
        String lock = RedisContextHolder.tryLock(this.source.getName());
        if (StringUtils.isEmpty(lock)) {
            return;
        }

        // 初始化并启动PEL数据转移消息监听器容器
        this.timestamp = System.currentTimeMillis();
        this.container = RedisStreamHolder.initializeMessageListenerContainer(
                record -> {
                    RedisStreamHolder.claim(this.topic, this.target, record.getId());
                    listener.onMessage(record);
                    timestamp = System.currentTimeMillis();
                },
                this.source,
                StreamOffset.create(this.topic, ReadOffset.from("0")),
                100,
                this.executor
        );
        this.container.start();

        // 启动容器销毁监视器
        this.monitor = Executors.newScheduledThreadPool(1);
        this.monitor.scheduleAtFixedRate(() -> {
            // 如果超过3秒未接收到新消息则销毁容器
            if (System.currentTimeMillis() - this.timestamp > 3000) {
                log.info("RedisMQ consumer pending claimed: {}", this.source);
                try {
                    this.container.stop();
                } finally {
                    try {
                        this.monitor.shutdown();
                    } finally {
                        RedisContextHolder.unlock(this.source.getName(), lock);
                    }
                }
            }
        }, 0, 1, TimeUnit.SECONDS);
    }
}
