package com.chelvc.framework.redis.queue;

import java.time.Duration;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Maps;
import lombok.Getter;
import lombok.NonNull;
import org.springframework.dao.DataAccessException;
import org.springframework.data.redis.core.RedisOperations;
import org.springframework.data.redis.core.SessionCallback;
import org.springframework.data.redis.core.script.DefaultRedisScript;
import org.springframework.data.redis.core.script.RedisScript;

/**
 * Redis延时队列实现
 *
 * @param <E> 元素类型
 * @author Woody
 * @date 2024/8/29
 */
@Getter
public class DelayRedisQueue<E> extends PriorityRedisQueue<E> {
    /**
     * 弹出并延迟脚本
     */
    @SuppressWarnings("rawtypes")
    private static final RedisScript<List> POLL_DELAY_SCRIPT = new DefaultRedisScript<>(
            "local values = redis.call('ZRANGEBYSCORE', KEYS[1], ARGV[1], ARGV[2], 'LIMIT', 0, ARGV[3]) " +
                    "if #values > 0 then for i = 1, #values do redis.call('ZINCRBY', KEYS[1], ARGV[4], values[i]) " +
                    "end end return values", List.class
    );

    private final long timeout;
    private final long expiration;

    public DelayRedisQueue(@NonNull String name) {
        this(name, Duration.ZERO, Duration.ZERO);
    }

    public DelayRedisQueue(@NonNull String name, @NonNull Duration timeout, @NonNull Duration expiration) {
        this(ApplicationContextHolder.getProfile(), name, timeout, expiration);
    }

    DelayRedisQueue(String namespace, @NonNull String name, @NonNull Duration timeout, @NonNull Duration expiration) {
        super(namespace, name);
        this.timeout = timeout.toMillis();
        this.expiration = expiration.toMillis();
    }

    /**
     * 判断元素是否处于延时状态
     *
     * @param e 元素对象
     * @return true/false
     */
    public boolean isDelaying(E e) {
        if (e == null) {
            return false;
        }

        Double score = this.template().opsForZSet().score(this.getName(), e);
        return score != null && score > System.currentTimeMillis();
    }

    /**
     * 判断元素是否处于延时状态
     *
     * @param collection 元素集合
     * @return 元素/是否延时映射表
     */
    public Map<E, Boolean> isDelaying(Collection<E> collection) {
        if (ObjectUtils.isEmpty(collection)) {
            return Collections.emptyMap();
        }

        // 处理单个元素
        if (collection.size() == 1) {
            E e = collection.iterator().next();
            return ImmutableMap.of(e, this.isDelaying(e));
        }

        // 批量处理多个元素
        List<Object> scores = this.template().executePipelined(new SessionCallback<Object>() {
            @Override
            @SuppressWarnings("unchecked")
            public <K, V> Object execute(RedisOperations<K, V> operations) throws DataAccessException {
                for (E e : collection) {
                    operations.opsForZSet().score((K) getName(), e);
                }
                return null;
            }
        });
        if (ObjectUtils.isEmpty(scores)) {
            return Collections.emptyMap();
        }
        int i = 0;
        long timestamp = System.currentTimeMillis();
        Map<E, Boolean> consumings = Maps.newHashMapWithExpectedSize(collection.size());
        for (E e : collection) {
            Double score = (Double) scores.get(i++);
            consumings.put(e, score != null && score > timestamp);
        }
        return consumings;
    }

    /**
     * 将元素放入队列
     *
     * @param e        元素
     * @param duration 延时周期
     * @return true/false
     */
    public boolean offer(@NonNull E e, @NonNull Duration duration) {
        return this.offer(e, duration, true);
    }

    /**
     * 将元素放入队列
     *
     * @param e        元素
     * @param duration 延时周期
     * @param override 是否可覆盖
     * @return true/false
     */
    public boolean offer(@NonNull E e, @NonNull Duration duration, boolean override) {
        return super.offer(e, System.currentTimeMillis() + duration.toMillis(), override);
    }

    @Override
    protected Long offer(RedisOperations<String, E> operations, Collection<? extends E> objects, double priority,
                         boolean override) {
        if (this.expiration > 0) {
            long max = System.currentTimeMillis() - this.expiration - 1000;
            List<Object> values = this.template().executePipelined(new SessionCallback<Object>() {
                @Override
                @SuppressWarnings("unchecked")
                public <K, V> Object execute(RedisOperations<K, V> operations) throws DataAccessException {
                    DelayRedisQueue.super.offer((RedisOperations<String, E>) operations, objects, priority, override);
                    operations.opsForZSet().removeRangeByScore((K) getName(), 0, max);
                    return null;
                }
            });
            return ObjectUtils.isEmpty(values) ? null : (Long) values.get(0);
        }
        return super.offer(operations, objects, priority, override);
    }

    @Override
    public E peek() {
        long now = System.currentTimeMillis();
        if (this.expiration > 0) {
            return super.peek(now - this.expiration, now);
        }
        return super.peek(Long.MIN_VALUE, now);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<E> poll(int size) {
        AssertUtils.arg(size > 0, () -> "size must be greater than 0");

        long now = System.currentTimeMillis();
        long min = this.expiration > 0 ? now - this.expiration : Long.MIN_VALUE;
        if (this.timeout > 0) {
            List<String> keys = Collections.singletonList(this.getName());
            List<E> values = this.template().execute(POLL_DELAY_SCRIPT, keys, min, now, size, this.timeout);
            return ObjectUtils.ifEmpty(values, Collections::emptyList);
        }
        return super.poll(min, now, size);
    }
}
