package com.chelvc.framework.redis.queue;

import java.lang.reflect.Type;
import java.time.Duration;
import java.util.concurrent.Executor;

import com.chelvc.framework.redis.context.RedisStreamHolder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.connection.stream.Consumer;
import org.springframework.data.redis.connection.stream.MapRecord;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.stream.StreamListener;

/**
 * 消息流监听器实现
 *
 * @author Woody
 * @date 2024/4/3
 */
@Slf4j
public class MessageStreamListener<T> implements StreamListener<String, MapRecord<String, String, String>> {
    private final Type type;
    private final String topic;
    private final Executor executor;
    private final Consumer consumer;
    private final RedisMQListener<T> listener;
    private final DelayRedisQueue<String> delayQueue;

    public MessageStreamListener(Type type, String topic, Consumer consumer, RedisMQListener<T> listener,
                                 Executor executor) {
        this.type = type;
        this.topic = topic;
        this.executor = executor;
        this.consumer = consumer;
        this.listener = listener;

        // 初始化延时消息队列
        String name = this.topic + ":" + this.consumer.getGroup();
        this.delayQueue = new DelayRedisQueue<String>(null, name, Duration.ofSeconds(60), Duration.ZERO) {
            @Override
            protected RedisTemplate<String, String> template() {
                return RedisStreamHolder.getStreamTemplate();
            }
        };
        RedisQueues.consume(this.delayQueue, 100, messages -> messages.forEach(this::processing));
    }

    /**
     * 消息消费处理
     *
     * @param message 消息内容（JSON格式）
     */
    private void processing(String message) {
        this.executor.execute(() -> {
            try {
                MapRecord<String, String, String> record = RedisStreamHolder.deserialize(message);
                if (RedisStreamHolder.isHeartbeat(record)) {
                    // 如果是心跳消息则删除相关数据
                    this.delayQueue.remove(message);
                    RedisStreamHolder.delete(this.topic, record.getId());
                } else if (this.processing(record)) {
                    this.delayQueue.remove(message);
                }
            } catch (Throwable t) {
                log.error("RedisMQ message consume failed: {}, {}", this.consumer, message, t);
            }
        });
    }

    /**
     * 消息消费处理
     *
     * @param record 消息记录实例
     * @return true/false
     */
    private boolean processing(MapRecord<String, String, String> record) {
        try {
            RedisStreamHolder.consume(record, this.type, this.listener::consume);
        } catch (Throwable t) {
            log.error("RedisMQ message consume failed: {}, {}", this.consumer, record, t);
            return false;
        }
        return true;
    }

    /**
     * 消息延时处理
     *
     * @param record    消息记录实例
     * @param timestamp 延时时间戳
     * @return true/false
     */
    private boolean delaying(MapRecord<String, String, String> record, long timestamp) {
        try {
            this.delayQueue.offer(RedisStreamHolder.serialize(record), timestamp);
        } catch (Throwable t) {
            log.error("RedisMQ message delaying failed: {}, {}", this.consumer, record, t);
            return false;
        }
        return true;
    }

    @Override
    public void onMessage(MapRecord<String, String, String> record) {
        this.executor.execute(() -> {
            // 如果是心跳消息则延时3秒后删除该消息，以防止同一topic的心跳消息立即删除后其他消费组无法消费到心跳消息的问题
            if (RedisStreamHolder.isHeartbeat(record)) {
                this.delaying(record, System.currentTimeMillis() + 3000);
                RedisStreamHolder.ack(this.topic, this.consumer.getGroup(), record.getId());
                return;
            }

            boolean success;
            Long delaying = RedisStreamHolder.getMessageDelaying(record);
            if (delaying != null && delaying - System.currentTimeMillis() >= 1000) {
                // 消息延时处理
                success = this.delaying(record, delaying);
            } else if (!(success = this.processing(record))) {
                // 消息处理异常延迟10秒后重试
                success = this.delaying(record, System.currentTimeMillis() + 10000);
            }
            if (success) {
                // 消息消费确认
                RedisStreamHolder.ack(this.topic, this.consumer.getGroup(), record.getId());
            }
        });
    }
}
