package com.chelvc.framework.redis.queue;

import java.util.AbstractQueue;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Objects;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.redis.context.RedisContextHolder;
import lombok.Getter;
import lombok.NonNull;
import org.springframework.data.redis.core.RedisTemplate;

/**
 * Redis普通队列实现
 *
 * @param <E> 元素类型
 * @author Woody
 * @date 2024/8/19
 */
@Getter
public class NormalRedisQueue<E> extends AbstractQueue<E> {
    private final String name;

    public NormalRedisQueue(@NonNull String name) {
        this(ApplicationContextHolder.getProfile(), name);
    }

    NormalRedisQueue(String namespace, @NonNull String name) {
        this.name = StringUtils.isEmpty(namespace) ? name : (namespace + "-" + name);
    }

    /**
     * 集合迭代器实现
     */
    private class Iter implements Iterator<E> {
        private E element;
        private int index = 0;
        private final int size;

        public Iter(int size) {
            this.size = size;
        }

        @Override
        public boolean hasNext() {
            return this.index < this.size;
        }

        @Override
        public E next() {
            return this.element = template().opsForList().index(name, this.index++);
        }

        @Override
        public void remove() {
            if (this.element != null) {
                template().opsForList().remove(name, 1, this.element);
                this.element = null;
            }
        }
    }

    /**
     * 获取RedisTemplate实例
     *
     * @return RedisTemplate实例
     */
    protected RedisTemplate<String, E> template() {
        return RedisContextHolder.getDefaultTemplate();
    }

    @Override
    public Iterator<E> iterator() {
        return new Iter(this.size());
    }

    @Override
    public int size() {
        Long size = this.template().opsForList().size(this.name);
        return size == null ? 0 : size.intValue();
    }

    @Override
    public boolean add(E e) {
        return this.offer(e);
    }

    @Override
    public boolean offer(E e) {
        Long count = this.template().opsForList().rightPush(this.name, Objects.requireNonNull(e));
        return count != null && count > 0;
    }

    @Override
    public E poll() {
        return this.template().opsForList().leftPop(this.name);
    }

    @Override
    public E peek() {
        return this.template().opsForList().index(this.name, 0);
    }

    @Override
    public void clear() {
        this.template().delete(this.name);
    }

    @Override
    @SuppressWarnings("unchecked")
    public boolean addAll(Collection<? extends E> c) {
        if (ObjectUtils.isEmpty(c)) {
            return false;
        }

        c.forEach(Objects::requireNonNull);
        Long count = this.template().opsForList().rightPushAll(this.name, (Collection<E>) c);
        return count != null && count > 0;
    }

    @Override
    public Object[] toArray() {
        List<E> values = this.template().opsForList().range(this.name, 0, -1);
        return ObjectUtils.ifNull(values, Collections::emptyList).toArray();
    }

    @Override
    public <T> T[] toArray(T[] a) {
        List<E> values = this.template().opsForList().range(this.name, 0, -1);
        return ObjectUtils.ifNull(values, Collections::emptyList).toArray(a);
    }

    @Override
    public String toString() {
        List<E> values = this.template().opsForList().range(this.name, 0, -1);
        return ObjectUtils.ifNull(values, Collections::emptyList).toString();
    }
}
