package com.chelvc.framework.redis.queue;

import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Objects;
import java.util.Set;

import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.google.common.collect.Sets;
import lombok.NonNull;
import org.springframework.dao.DataAccessException;
import org.springframework.data.redis.core.DefaultTypedTuple;
import org.springframework.data.redis.core.RedisOperations;
import org.springframework.data.redis.core.SessionCallback;
import org.springframework.data.redis.core.ZSetOperations;
import org.springframework.data.redis.core.script.DefaultRedisScript;
import org.springframework.data.redis.core.script.RedisScript;

/**
 * Redis优先级队列实现
 *
 * @author Woody
 * @date 2025/4/28
 */
public class PriorityRedisQueue<E> extends NormalRedisQueue<E> {
    /**
     * 添加或忽略脚本
     */
    private static final RedisScript<Long> ADD_IGNORE_SCRIPT = new DefaultRedisScript<>(
            "return redis.call('ZADD', KEYS[1], 'NX', 'CH', unpack(ARGV))", Long.class
    );

    /**
     * 弹出并移除脚本
     */
    @SuppressWarnings("rawtypes")
    private static final RedisScript<List> POLL_REMOVE_SCRIPT = new DefaultRedisScript<>(
            "local values = redis.call('ZRANGEBYSCORE', KEYS[1], ARGV[1], ARGV[2], 'LIMIT', 0, ARGV[3]) " +
                    "if #values > 0 then redis.call('ZREM', KEYS[1], unpack(values)) end return values", List.class
    );

    public PriorityRedisQueue(@NonNull String name) {
        super(name);
    }

    PriorityRedisQueue(String namespace, @NonNull String name) {
        super(namespace, name);
    }

    /**
     * 集合迭代器实现
     */
    private class Iter implements Iterator<E> {
        private E element;
        private int index = 0;
        private final int size;

        public Iter(int size) {
            this.size = size;
        }

        @Override
        public boolean hasNext() {
            return this.index < this.size;
        }

        @Override
        public E next() {
            Set<E> values = template().opsForZSet().range(getName(), this.index, this.index++);
            return ObjectUtils.isEmpty(values) ? null : (this.element = values.iterator().next());
        }

        @Override
        public void remove() {
            if (this.element != null) {
                template().opsForZSet().remove(getName(), this.element);
                this.element = null;
            }
        }
    }

    /**
     * 将元素放入队列
     *
     * @param operations Redis操作接口
     * @param objects    元素列表
     * @param priority   优先级数字
     * @param override   是否可覆盖
     * @return 元素写入成功个数
     */
    protected Long offer(RedisOperations<String, E> operations, Collection<? extends E> objects, double priority,
                         boolean override) {
        if (ObjectUtils.isEmpty(objects)) {
            return 0L;
        }

        // 可覆盖模式
        if (override) {
            Set<ZSetOperations.TypedTuple<E>> tuples = Sets.newHashSetWithExpectedSize(objects.size());
            for (E e : objects) {
                tuples.add(new DefaultTypedTuple<>(e, priority++));
            }
            return operations.opsForZSet().add(this.getName(), tuples);
        }

        // 不可覆盖模式
        int i = 0;
        Object[] args = new Object[objects.size() * 2];
        for (E e : objects) {
            args[i++] = priority++;
            args[i++] = Objects.requireNonNull(e);
        }
        return operations.execute(ADD_IGNORE_SCRIPT, Collections.singletonList(this.getName()), args);
    }

    /**
     * 弹出元素
     *
     * @param size 元素个数
     * @return 元素列表
     */
    public List<E> poll(int size) {
        return this.poll(Long.MIN_VALUE, Long.MAX_VALUE, size);
    }

    /**
     * 弹出元素
     *
     * @param min  最小优先级
     * @param max  最大优先级
     * @param size 元素个数
     * @return 元素列表
     */
    @SuppressWarnings("unchecked")
    public List<E> poll(double min, double max, int size) {
        AssertUtils.arg(size > 0, () -> "size must be greater than 0");

        List<String> keys = Collections.singletonList(this.getName());
        List<E> values = this.template().execute(POLL_REMOVE_SCRIPT, keys, min, max, size);
        return ObjectUtils.ifEmpty(values, Collections::emptyList);
    }

    /**
     * 查看元素
     *
     * @param min 最小优先级
     * @param max 最大优先级
     * @return 元素
     */
    public E peek(double min, double max) {
        Set<E> values = this.template().opsForZSet().rangeByScore(this.getName(), min, max, 0, 1);
        return ObjectUtils.isEmpty(values) ? null : values.iterator().next();
    }

    /**
     * 将元素放入队列
     *
     * @param e        元素
     * @param priority 优先级数字
     * @return true/false
     */
    public boolean offer(@NonNull E e, double priority) {
        return this.offer(e, priority, true);
    }

    /**
     * 将元素放入队列
     *
     * @param e        元素
     * @param override 是否可覆盖
     * @return true/false
     */
    public boolean offer(@NonNull E e, boolean override) {
        return this.offer(e, System.currentTimeMillis(), override);
    }

    /**
     * 将元素放入队列
     *
     * @param e        元素
     * @param priority 优先级数字
     * @param override 是否可覆盖
     * @return true/false
     */
    public boolean offer(@NonNull E e, double priority, boolean override) {
        Long count = this.offer(this.template(), Collections.singleton(e), priority, override);
        return count != null && count > 0;
    }

    /**
     * 将元素放入队列
     *
     * @param objects  元素列表
     * @param override 是否可覆盖
     * @return true/false
     */
    public boolean offer(@NonNull Collection<? extends E> objects, boolean override) {
        if (ObjectUtils.isEmpty(objects)) {
            return false;
        }

        Long count = this.offer(this.template(), objects, System.currentTimeMillis(), override);
        return count != null && count > 0;
    }

    @Override
    public Iterator<E> iterator() {
        return new Iter(this.size());
    }

    @Override
    public int size() {
        Long size = this.template().opsForZSet().size(this.getName());
        return size == null ? 0 : size.intValue();
    }

    @Override
    public boolean offer(E e) {
        return this.offer(e, true);
    }

    @Override
    public E poll() {
        List<E> values = this.poll(1);
        return ObjectUtils.isEmpty(values) ? null : values.get(0);
    }

    @Override
    public E peek() {
        return this.peek(Long.MIN_VALUE, Long.MAX_VALUE);
    }

    @Override
    public boolean addAll(Collection<? extends E> c) {
        return this.offer(c, true);
    }

    @Override
    public boolean contains(Object o) {
        if (o == null) {
            return false;
        }

        Long index = this.template().opsForZSet().rank(this.getName(), o);
        return index != null && index >= 0;
    }

    @Override
    public boolean containsAll(Collection<?> c) {
        if (ObjectUtils.isEmpty(c)) {
            return false;
        }

        // 处理单个元素
        if (c.size() == 1) {
            return this.contains(c.iterator().next());
        }

        // 批量处理多个元素
        List<Object> indexes = this.template().executePipelined(new SessionCallback<Object>() {
            @Override
            @SuppressWarnings("unchecked")
            public <K, V> Object execute(RedisOperations<K, V> operations) throws DataAccessException {
                for (Object o : c) {
                    operations.opsForZSet().rank((K) getName(), o);
                }
                return null;
            }
        });
        return ObjectUtils.notEmpty(indexes) && indexes.size() == c.size()
                && indexes.stream().allMatch(index -> index != null && ((Long) index) >= 0);
    }

    @Override
    public boolean remove(Object o) {
        if (o == null) {
            return false;
        }

        Long count = this.template().opsForZSet().remove(this.getName(), o);
        return count != null && count > 0;
    }

    @Override
    public boolean removeAll(Collection<?> c) {
        if (ObjectUtils.isEmpty(c)) {
            return false;
        }

        Long count = this.template().opsForZSet().remove(this.getName(), c.toArray());
        return count != null && count > 0;
    }

    @Override
    public Object[] toArray() {
        Set<E> values = this.template().opsForZSet().range(this.getName(), 0, -1);
        return ObjectUtils.ifNull(values, Collections::emptySet).toArray();
    }

    @Override
    public <T> T[] toArray(T[] a) {
        Set<E> values = this.template().opsForZSet().range(this.getName(), 0, -1);
        return ObjectUtils.ifNull(values, Collections::emptySet).toArray(a);
    }

    @Override
    public String toString() {
        Set<E> values = this.template().opsForZSet().range(this.getName(), 0, -1);
        return ObjectUtils.ifNull(values, Collections::emptySet).toString();
    }
}
