package com.chelvc.framework.redis.queue;

import java.util.List;
import java.util.Map;
import java.util.function.Consumer;
import java.util.function.Function;

import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.ThreadUtils;
import com.google.common.collect.Maps;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * Redis队列处理工具类
 *
 * @author Woody
 * @date 2025/4/28
 */
@Slf4j
public final class RedisQueues {
    /**
     * 普通队列名称/实例映射表
     */
    @SuppressWarnings("rawtypes")
    private static final Map<String, NormalRedisQueue> NORMAL_REDIS_QUEUE_INSTANCES = Maps.newConcurrentMap();

    /**
     * 优先级队列名称/实例映射表
     */
    @SuppressWarnings("rawtypes")
    private static final Map<String, PriorityRedisQueue> PRIORITY_REDIS_QUEUE_INSTANCES = Maps.newConcurrentMap();

    /**
     * 延时队列名称/实例映射表
     */
    @SuppressWarnings("rawtypes")
    private static final Map<String, DelayRedisQueue> DELAY_REDIS_QUEUE_INSTANCES = Maps.newConcurrentMap();

    private RedisQueues() {
    }

    /**
     * 获取普通消息队列
     *
     * @param name 队列名称
     * @param <E>  消息类型
     * @return 队列实例
     */
    public static <E> NormalRedisQueue<E> getNormalRedisQueue(@NonNull String name) {
        return getNormalRedisQueue(name, NormalRedisQueue::new);
    }

    /**
     * 获取普通消息队列
     *
     * @param name    队列名称
     * @param builder 队列构建函数
     * @param <E>     消息类型
     * @return 队列实例
     */
    @SuppressWarnings("unchecked")
    public static <E> NormalRedisQueue<E> getNormalRedisQueue(@NonNull String name,
                                                              @NonNull Function<String, NormalRedisQueue<E>> builder) {
        NormalRedisQueue<E> queue = NORMAL_REDIS_QUEUE_INSTANCES.get(name);
        return queue == null ? NORMAL_REDIS_QUEUE_INSTANCES.computeIfAbsent(name, builder) : queue;
    }

    /**
     * 获取优先级消息队列
     *
     * @param name 队列名称
     * @param <E>  消息类型
     * @return 队列实例
     */
    public static <E> PriorityRedisQueue<E> getPriorityRedisQueue(@NonNull String name) {
        return getPriorityRedisQueue(name, PriorityRedisQueue::new);
    }

    /**
     * 获取优先级消息队列
     *
     * @param name    队列名称
     * @param builder 队列构建函数
     * @param <E>     消息类型
     * @return 队列实例
     */
    @SuppressWarnings("unchecked")
    public static <E> PriorityRedisQueue<E> getPriorityRedisQueue(@NonNull String name,
                                                                  @NonNull Function<String, PriorityRedisQueue<E>> builder) {
        PriorityRedisQueue<E> queue = PRIORITY_REDIS_QUEUE_INSTANCES.get(name);
        return queue == null ? PRIORITY_REDIS_QUEUE_INSTANCES.computeIfAbsent(name, builder) : queue;
    }

    /**
     * 获取延时消息队列
     *
     * @param name 队列名称
     * @param <E>  消息类型
     * @return 队列实例
     */
    public static <E> DelayRedisQueue<E> getDelayRedisQueue(@NonNull String name) {
        return getDelayRedisQueue(name, DelayRedisQueue::new);
    }

    /**
     * 获取延时消息队列
     *
     * @param name    队列名称
     * @param builder 队列构建函数
     * @param <E>     消息类型
     * @return 队列实例
     */
    @SuppressWarnings("unchecked")
    public static <E> DelayRedisQueue<E> getDelayRedisQueue(@NonNull String name,
                                                            @NonNull Function<String, DelayRedisQueue<E>> builder) {
        DelayRedisQueue<E> queue = DELAY_REDIS_QUEUE_INSTANCES.get(name);
        return queue == null ? DELAY_REDIS_QUEUE_INSTANCES.computeIfAbsent(name, builder) : queue;
    }

    /**
     * 消费队列消息
     *
     * @param queue    消息队列
     * @param consumer 消息消费者
     * @param <T>      消息类型
     */
    public static <T> void consume(@NonNull NormalRedisQueue<T> queue, @NonNull Consumer<T> consumer) {
        ThreadUtils.run(() -> {
            while (!Thread.currentThread().isInterrupted()) {
                // 拉取消息
                T message = null;
                try {
                    message = queue.poll();
                } catch (Throwable t) {
                    log.error("RedisMQ message poll failed: {}", queue.getName(), t);
                }

                // 如果当前没有消息则暂停1秒后继续
                if (message == null) {
                    ThreadUtils.sleep(1000);
                    continue;
                }

                // 消费消息
                try {
                    consumer.accept(message);
                } catch (Throwable t) {
                    log.error("RedisMQ message consume failed: {}, {}", queue.getName(), message, t);
                }
            }
        });
    }

    /**
     * 消费队列消息
     *
     * @param queue    消息队列
     * @param size     拉取数量
     * @param consumer 消息消费者
     * @param <T>      消息类型
     */
    public static <T> void consume(@NonNull PriorityRedisQueue<T> queue, int size,
                                   @NonNull Consumer<List<T>> consumer) {
        AssertUtils.arg(size > 0, () -> "size must be greater than 0");
        ThreadUtils.run(() -> {
            while (!Thread.currentThread().isInterrupted()) {
                // 拉取消息
                List<T> messages = null;
                try {
                    messages = queue.poll(size);
                } catch (Throwable t) {
                    log.error("RedisMQ message poll failed: {}", queue.getName(), t);
                }

                // 如果当前没有消息则暂停1秒后继续
                if (ObjectUtils.isEmpty(messages)) {
                    ThreadUtils.sleep(1000);
                    continue;
                }

                // 消费消息
                try {
                    consumer.accept(messages);
                } catch (Throwable t) {
                    log.error("RedisMQ message consume failed: {}, {}", queue.getName(), messages, t);
                }
            }
        });
    }
}
