package com.chelvc.framework.rocketmq.config;

import java.util.List;

import com.chelvc.framework.rocketmq.fallback.RocketMQFallbackPolicy;
import com.chelvc.framework.rocketmq.fallback.RocketMQMemoryProducer;
import com.chelvc.framework.rocketmq.fallback.RocketMQPersistentProducer;
import com.chelvc.framework.rocketmq.fallback.RocketMQStoreClient;
import com.chelvc.framework.rocketmq.producer.RocketMQProducerWrapper;
import com.chelvc.framework.rocketmq.producer.StandardTransactionChecker;
import com.chelvc.framework.rocketmq.producer.TransactionCheckProcessor;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.apis.ClientConfiguration;
import org.apache.rocketmq.client.apis.ClientServiceProvider;
import org.apache.rocketmq.client.apis.SessionCredentialsProvider;
import org.apache.rocketmq.client.apis.StaticSessionCredentialsProvider;
import org.apache.rocketmq.client.apis.producer.Producer;
import org.apache.rocketmq.client.apis.producer.TransactionChecker;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * RocketMQ配置类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@Configuration
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class RocketMQConfigurer {
    private final RocketMQProperties properties;

    @Bean
    @ConditionalOnMissingBean(ClientServiceProvider.class)
    public ClientServiceProvider provider() {
        return ClientServiceProvider.loadService();
    }

    @Bean
    @ConditionalOnMissingBean(ClientConfiguration.class)
    public ClientConfiguration configuration() {
        SessionCredentialsProvider sessionCredentialsProvider =
                new StaticSessionCredentialsProvider(this.properties.getAppid(), this.properties.getSecret());
        return ClientConfiguration.newBuilder().setEndpoints(this.properties.getEndpoints())
                .setCredentialProvider(sessionCredentialsProvider).build();
    }

    @Bean
    @ConditionalOnMissingBean(TransactionChecker.class)
    public TransactionChecker transactionChecker(List<TransactionCheckProcessor<?>> processors) {
        return new StandardTransactionChecker(processors);
    }

    @Bean
    @ConditionalOnMissingBean(RocketMQListenerRegistry.class)
    public RocketMQListenerRegistry listenerRegistry(ClientServiceProvider provider,
                                                     ClientConfiguration configuration) {
        return new RocketMQListenerRegistry(provider, configuration);
    }

    @Bean
    @ConditionalOnExpression("'${rocketmq.fallbacks}'.contains('MEMORY')")
    public RocketMQProducerWrapper memoryProducerWrapper() {
        return new RocketMQProducerWrapper() {
            private final int order = properties.getFallbacks().indexOf(RocketMQFallbackPolicy.MEMORY);

            @Override
            public int getOrder() {
                return this.order;
            }

            @Override
            public Producer wrap(Producer producer) {
                return new RocketMQMemoryProducer(producer);
            }
        };
    }

    @Bean
    @ConditionalOnExpression("'${rocketmq.fallbacks}'.contains('PERSISTENT')")
    public RocketMQProducerWrapper persistentProducerWrapper(RocketMQStoreClient client) {
        return new RocketMQProducerWrapper() {
            private final int order = properties.getFallbacks().indexOf(RocketMQFallbackPolicy.PERSISTENT);

            @Override
            public int getOrder() {
                return this.order;
            }

            @Override
            public Producer wrap(Producer producer) {
                return new RocketMQPersistentProducer(client, producer);
            }
        };
    }
}
