package com.chelvc.framework.rocketmq.fallback;

import java.util.Objects;
import java.util.function.BiConsumer;

import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.apis.ClientException;
import org.apache.rocketmq.client.apis.message.Message;
import org.apache.rocketmq.client.apis.producer.Transaction;

/**
 * RocketMQ降级事务实现
 *
 * @author Woody
 * @date 2024/7/14
 */
@Slf4j
public class FallbackTransaction implements Transaction {
    private volatile Message message;
    private volatile boolean degraded;
    private final Transaction delegate;
    private final BiConsumer<Message, Transaction> callback;

    public FallbackTransaction(Transaction delegate, @NonNull BiConsumer<Message, Transaction> callback) {
        this.delegate = delegate;
        this.callback = callback;
        this.degraded = Objects.isNull(delegate);
    }

    /**
     * 判断是否已降级
     *
     * @return true/false
     */
    boolean isDegraded() {
        return this.degraded;
    }

    /**
     * 设置降级处理消息
     *
     * @param message 消息实例
     */
    void delegate(@NonNull Message message) {
        this.message = message;
        this.degraded = true;
    }

    /**
     * 获取代理事务处理器
     *
     * @return 事务处理器
     */
    Transaction getTransaction() {
        return this.delegate;
    }

    @Override
    public void commit() throws ClientException {
        if (!this.degraded) {
            try {
                this.delegate.commit();
            } catch (Exception e) {
                log.error("RocketMQ transaction commit failed", e);
            }
        } else if (this.message != null) {
            this.callback.accept(this.message, this.delegate);
        }
    }

    @Override
    public void rollback() throws ClientException {
        if (!this.degraded) {
            try {
                this.delegate.rollback();
            } catch (Exception e) {
                log.error("RocketMQ transaction rollback failed", e);
            }
        }
    }
}
