package com.chelvc.framework.rocketmq.fallback;

import java.io.IOException;
import java.util.concurrent.CompletableFuture;

import com.chelvc.framework.rocketmq.context.RocketMQContextHolder;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.apis.ClientException;
import org.apache.rocketmq.client.apis.message.Message;
import org.apache.rocketmq.client.apis.producer.Producer;
import org.apache.rocketmq.client.apis.producer.SendReceipt;
import org.apache.rocketmq.client.apis.producer.Transaction;

/**
 * RocketMQ降级生产者抽象实现
 *
 * @author Woody
 * @date 2024/7/23
 */
@Slf4j
public abstract class RocketMQFallbackProducer implements Producer {
    protected final Producer delegate;

    public RocketMQFallbackProducer(@NonNull Producer delegate) {
        this.delegate = delegate;
    }

    /**
     * 消息发送降级处理
     *
     * @param message     消息实例
     * @param transaction 消息处理事务
     */
    protected abstract void fallback(Message message, Transaction transaction);

    @Override
    public SendReceipt send(Message message) throws ClientException {
        try {
            return this.delegate.send(message);
        } catch (Exception e) {
            log.error("RocketMQ message send failed: {}", RocketMQContextHolder.topic(message), e);
        }
        this.fallback(message, null);
        return FallbackSendReceipt.INSTANCE;
    }

    @Override
    public SendReceipt send(Message message, Transaction transaction) throws ClientException {
        // 执行原生逻辑
        if (!(transaction instanceof FallbackTransaction)) {
            return this.delegate.send(message, transaction);
        }

        // 降级处理
        FallbackTransaction proxy = (FallbackTransaction) transaction;
        if (!proxy.isDegraded()) {
            try {
                return this.delegate.send(message, proxy.getTransaction());
            } catch (Exception e) {
                log.error("RocketMQ transactional message send failed: {}", RocketMQContextHolder.topic(message), e);
            }
        }
        proxy.delegate(message);
        return FallbackSendReceipt.INSTANCE;
    }

    @Override
    public CompletableFuture<SendReceipt> sendAsync(Message message) {
        return this.delegate.sendAsync(message).exceptionally(e -> {
            log.error("RocketMQ message async send failed: {}", RocketMQContextHolder.topic(message), e);
            this.fallback(message, null);
            return FallbackSendReceipt.INSTANCE;
        });
    }

    @Override
    public Transaction beginTransaction() throws ClientException {
        Transaction transaction = null;
        try {
            transaction = this.delegate.beginTransaction();
        } catch (Exception e) {
            log.error("RocketMQ transaction open failed", e);
        }
        return new FallbackTransaction(transaction, this::fallback);
    }

    @Override
    public void close() throws IOException {
        this.delegate.close();
    }
}
