package com.chelvc.framework.rocketmq.fallback;

import java.util.Objects;

import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.rocketmq.context.RocketMQContextHolder;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.apis.ClientException;
import org.apache.rocketmq.client.apis.message.Message;
import org.apache.rocketmq.client.apis.producer.Producer;
import org.apache.rocketmq.client.apis.producer.Transaction;

/**
 * RocketMQ持久化降级消息生产者实现
 *
 * @author Woody
 * @date 2024/12/20
 */
@Slf4j
public class RocketMQPersistentProducer extends RocketMQFallbackProducer {
    private final RocketMQStoreClient client;

    public RocketMQPersistentProducer(@NonNull RocketMQStoreClient client, @NonNull Producer delegate) {
        super(delegate);
        this.client = client;
    }

    /**
     * 保存消息
     *
     * @param message     消息实例
     * @param transaction 消息处理事务
     */
    private void save(Message message, Transaction transaction) {
        String topic = RocketMQContextHolder.topic(message);
        Object payload = JacksonUtils.deserialize(RocketMQContextHolder.body(message));
        String ordering = message.getMessageGroup().orElse(null);
        RocketMQFallbackMessage fallback =
                new RocketMQFallbackMessage(topic, payload, ordering, Objects.nonNull(transaction));
        this.client.save(fallback);
        if (transaction != null) {
            try {
                transaction.commit();
            } catch (ClientException e) {
                throw new RuntimeException(e);
            }
        }
    }

    @Override
    protected void fallback(Message message, Transaction transaction) {
        if (log.isDebugEnabled()) {
            String topic = RocketMQContextHolder.topic(message), body = RocketMQContextHolder.body(message);
            log.debug("RocketMQ fallback message: {}, {}", topic, body);
        }

        Session session = SessionContextHolder.getSession(false);
        if (session == null && (session = RocketMQContextHolder.getSession(message)) != null) {
            SessionContextHolder.setSession(session);
            try {
                this.save(message, transaction);
            } finally {
                SessionContextHolder.removeSessionContext();
            }
        } else {
            this.save(message, transaction);
        }
    }
}
