package com.chelvc.framework.rocketmq.producer;

import java.util.List;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.rocketmq.context.RocketMQContextHolder;
import lombok.RequiredArgsConstructor;
import org.apache.rocketmq.client.apis.ClientConfiguration;
import org.apache.rocketmq.client.apis.ClientException;
import org.apache.rocketmq.client.apis.ClientServiceProvider;
import org.apache.rocketmq.client.apis.producer.Producer;
import org.apache.rocketmq.client.apis.producer.ProducerBuilder;
import org.apache.rocketmq.client.apis.producer.TransactionChecker;
import org.apache.rocketmq.client.apis.producer.TransactionResolution;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.FactoryBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

/**
 * RocketMQ消息生产者工厂对象
 *
 * @author Woody
 * @date 2024/7/16
 */
@Component
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class RocketMQProducerFactory implements FactoryBean<Producer>, DisposableBean {
    private Producer producer;
    private final ApplicationContext applicationContext;

    @Override
    public boolean isSingleton() {
        return true;
    }

    @Override
    public Class<?> getObjectType() {
        return Producer.class;
    }

    @Override
    public Producer getObject() throws Exception {
        if (this.producer != null) {
            return this.producer;
        }

        TransactionChecker checker = this.applicationContext.getBean(TransactionChecker.class);
        ClientServiceProvider provider = this.applicationContext.getBean(ClientServiceProvider.class);
        ClientConfiguration configuration = this.applicationContext.getBean(ClientConfiguration.class);
        ProducerBuilder builder = provider.newProducerBuilder().setClientConfiguration(configuration);

        // 设置事务消息本地事务检测回调方法
        builder.setTransactionChecker(message -> {
            TransactionResolution resolution = RocketMQContextHolder.deserialize(message, checker::check);
            return ObjectUtils.ifNull(resolution, TransactionResolution.UNKNOWN);
        });

        // 构建消息生产者实例
        Producer producer;
        try {
            producer = builder.build();
        } catch (ClientException e) {
            throw new RuntimeException(e);
        }

        // 消息生产者包装处理
        List<RocketMQProducerWrapper> wrappers = ApplicationContextHolder.getSortBeans(RocketMQProducerWrapper.class);
        for (RocketMQProducerWrapper wrapper : wrappers) {
            producer = wrapper.wrap(producer);
        }
        return this.producer = producer;
    }

    @Override
    public void destroy() throws Exception {
        if (this.producer != null) {
            this.producer.close();
        }
    }
}
