package com.chelvc.framework.rocketmq.producer;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.rocketmq.context.RocketMQContextHolder;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.apis.message.MessageView;
import org.apache.rocketmq.client.apis.producer.TransactionChecker;
import org.apache.rocketmq.client.apis.producer.TransactionResolution;

/**
 * RocketMQ事务消息检测器标准实现
 *
 * @author Woody
 * @date 2024/10/29
 */
@Slf4j
public class StandardTransactionChecker implements TransactionChecker {
    private final Map<String, TransactionCheckProcessor<?>> processors;

    public StandardTransactionChecker(@NonNull List<TransactionCheckProcessor<?>> processors) {
        this.processors = ObjectUtils.isEmpty(processors) ? Collections.emptyMap() :
                processors.stream().collect(Collectors.toMap(TransactionCheckProcessor::topic, processor -> processor));
    }

    @Override
    @SuppressWarnings({"unchecked", "rawtypes"})
    public TransactionResolution check(MessageView message) {
        String topic = RocketMQContextHolder.topic(message);
        TransactionCheckProcessor processor = this.processors.get(topic);
        if (processor == null) {
            log.error("Transaction message check processor is missing: {}", topic);
            return TransactionResolution.UNKNOWN;
        }

        String body = RocketMQContextHolder.body(message);
        Object payload = JacksonUtils.deserialize(body, processor.model());
        SessionContextHolder.setSession(RocketMQContextHolder.getSession(message));
        try {
            return ObjectUtils.ifNull(processor.check(payload), TransactionResolution.UNKNOWN);
        } finally {
            SessionContextHolder.removeSessionContext();
        }
    }
}
