package com.chelvc.framework.security.config;

import java.time.Duration;
import java.util.List;
import java.util.Set;

import com.chelvc.framework.base.logic.ContextVariableParser;
import com.chelvc.framework.base.logic.VariableParser;
import com.chelvc.framework.security.annotation.Crypto;
import com.chelvc.framework.security.crypto.CacheableSecurityCipherHandler;
import com.chelvc.framework.security.crypto.DefaultSecurityCipherHandler;
import com.chelvc.framework.security.crypto.SecurityCipherHandler;
import com.chelvc.framework.security.firewall.Action;
import com.chelvc.framework.security.firewall.ActionExecutor;
import com.chelvc.framework.security.firewall.DefaultActionExecutor;
import com.chelvc.framework.security.firewall.DefaultFirewallProcessor;
import com.chelvc.framework.security.firewall.FirewallProcessor;
import com.chelvc.framework.security.interceptor.MethodSecurityExpression;
import com.chelvc.framework.security.jackson.JacksonArrayDecrypter;
import com.chelvc.framework.security.jackson.JacksonArrayEncryptor;
import com.chelvc.framework.security.jackson.JacksonListDecrypter;
import com.chelvc.framework.security.jackson.JacksonListEncryptor;
import com.chelvc.framework.security.jackson.JacksonSetDecrypter;
import com.chelvc.framework.security.jackson.JacksonSetEncryptor;
import com.chelvc.framework.security.jackson.JacksonStringDecrypter;
import com.chelvc.framework.security.jackson.JacksonStringEncryptor;
import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.introspect.Annotated;
import com.fasterxml.jackson.databind.introspect.JacksonAnnotationIntrospector;
import com.fasterxml.jackson.databind.type.CollectionType;
import org.aopalliance.intercept.MethodInvocation;
import org.springframework.beans.BeansException;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.access.expression.method.DefaultMethodSecurityExpressionHandler;
import org.springframework.security.access.expression.method.MethodSecurityExpressionHandler;
import org.springframework.security.access.expression.method.MethodSecurityExpressionOperations;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.method.configuration.GlobalMethodSecurityConfiguration;
import org.springframework.security.core.Authentication;

/**
 * 安全配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Configuration
@EnableGlobalMethodSecurity(prePostEnabled = true)
public class SecurityConfigurer extends GlobalMethodSecurityConfiguration implements ApplicationContextAware {
    @Override
    protected MethodSecurityExpressionHandler createExpressionHandler() {
        return new DefaultMethodSecurityExpressionHandler() {
            @Override
            protected MethodSecurityExpressionOperations createSecurityExpressionRoot(
                    Authentication authentication, MethodInvocation invocation) {
                MethodSecurityExpression root = new MethodSecurityExpression(authentication);
                root.setThis(invocation.getThis());
                root.setPermissionEvaluator(this.getPermissionEvaluator());
                root.setTrustResolver(this.getTrustResolver());
                root.setRoleHierarchy(this.getRoleHierarchy());
                root.setDefaultRolePrefix(this.getDefaultRolePrefix());
                return root;
            }
        };
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        // 设置数据加解密处理器
        ObjectMapper mapper = applicationContext.getBean(ObjectMapper.class);
        mapper.setAnnotationIntrospector(new JacksonAnnotationIntrospector() {
            @Override
            public Object findSerializer(Annotated am) {
                Crypto crypto = am.getAnnotation(Crypto.class);
                if (crypto == null || !crypto.response()) {
                    return super.findSerializer(am);
                }

                JavaType type = am.getType();
                Class<?> raw = am.getRawType();
                if (raw == String.class) {
                    return JacksonStringEncryptor.class;
                } else if (raw.isArray() && raw.getComponentType() == String.class) {
                    return JacksonArrayEncryptor.class;
                } else if (type instanceof CollectionType && type.getContentType().getRawClass() == String.class) {
                    if (Set.class.isAssignableFrom(type.getRawClass())) {
                        return JacksonSetEncryptor.class;
                    }
                    return JacksonListEncryptor.class;
                }
                return super.findSerializer(am);
            }

            @Override
            public Object findDeserializer(Annotated am) {
                Crypto crypto = am.getAnnotation(Crypto.class);
                if (crypto == null || !crypto.request()) {
                    return super.findDeserializer(am);
                }

                JavaType type = am.getType();
                Class<?> raw = am.getRawType();
                if (raw == String.class) {
                    return JacksonStringDecrypter.class;
                } else if (raw.isArray() && raw.getComponentType() == String.class) {
                    return JacksonArrayDecrypter.class;
                } else if (type instanceof CollectionType && type.getContentType().getRawClass() == String.class) {
                    if (Set.class.isAssignableFrom(type.getRawClass())) {
                        return JacksonSetDecrypter.class;
                    }
                    return JacksonListDecrypter.class;
                }
                return super.findDeserializer(am);
            }
        });
    }

    @Bean
    @ConditionalOnMissingBean(VariableParser.class)
    public VariableParser variableParser() {
        return new ContextVariableParser();
    }

    @Bean
    @ConditionalOnMissingBean(ActionExecutor.class)
    public ActionExecutor actionExecutor(List<Action> actions) {
        return new DefaultActionExecutor(actions);
    }

    @Bean
    @ConditionalOnMissingBean(FirewallProcessor.class)
    public FirewallProcessor firewallProcessor(VariableParser parser, ActionExecutor executor) {
        return new DefaultFirewallProcessor(parser, executor);
    }

    @Bean
    @ConditionalOnProperty(name = "security.crypto.secret")
    @ConditionalOnMissingBean(SecurityCipherHandler.class)
    public SecurityCipherHandler securityCipherHandler(SecurityProperties properties) {
        SecurityProperties.Crypto crypto = properties.getCrypto();
        SecurityCipherHandler handler = new DefaultSecurityCipherHandler(crypto);
        if (crypto.getCache().isEnabled()) {
            Duration duration = Duration.ofSeconds(crypto.getCache().getDuration());
            return new CacheableSecurityCipherHandler(handler, crypto.getCache().getCapacity(), duration);
        }
        return handler;
    }
}
