package com.chelvc.framework.security.context;

import java.lang.reflect.Method;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.base.util.SpringUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.security.annotation.Security;
import com.chelvc.framework.security.config.SecurityProperties;
import com.chelvc.framework.security.crypto.SecurityCipherHandler;
import com.google.common.collect.Sets;
import lombok.NonNull;
import org.springframework.aop.framework.AopProxyUtils;
import org.springframework.beans.factory.ObjectProvider;
import org.springframework.context.ApplicationContext;
import org.springframework.core.env.Environment;
import org.springframework.core.io.Resource;
import org.springframework.security.oauth2.jwt.Jwt;

/**
 * 应用安全相关操作上下文工具类
 *
 * @author Woody
 * @date 2024/5/24
 */
public final class SecurityContextHolder {
    /**
     * 应用范围标识
     */
    public static final String SCOPE = "scope";

    /**
     * 客户端登陆标识
     */
    public static final String CLIENT = "client";

    /**
     * 手机号码标识
     */
    public static final String MOBILE = "mobile";

    /**
     * 用户账号标识
     */
    public static final String USERNAME = "user_name";

    /**
     * 主体创建时间戳
     */
    public static final String CREATING = "creating";

    /**
     * 数字签名标识
     */
    public static final String SIGNATURE = "signature";

    /**
     * 主体注册时间戳
     */
    public static final String REGISTERING = "registering";

    /**
     * 用户权限标识
     */
    public static final String AUTHORITIES = "authorities";

    /**
     * 数据安全加解密处理器
     */
    private static volatile SecurityCipherHandler SECURITY_CIPHER_HANDLER;

    private SecurityContextHolder() {
    }

    /**
     * 获取主体身份
     *
     * @param jwt JWT对象
     * @return 主体身份标识
     */
    public static Long getId(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(USERNAME), Long::parseLong);
    }

    /**
     * 获取应用范围
     *
     * @param jwt Jwt对象
     * @return 应用范围
     */
    public static String getScope(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(SCOPE), (String) null);
    }

    /**
     * 是否是客户端登陆
     *
     * @param jwt Jwt对象
     * @return true/false
     */
    public static boolean isClient(Jwt jwt) {
        return jwt != null && Boolean.TRUE.equals(jwt.getClaimAsBoolean(CLIENT));
    }

    /**
     * 获取用户授权信息
     *
     * @param jwt JWT对象
     * @return 授权信息集合
     */
    public static Set<String> getAuthorities(Jwt jwt) {
        if (jwt == null) {
            return Collections.emptySet();
        }
        List<String> authorities = jwt.getClaimAsStringList(AUTHORITIES);
        return ObjectUtils.isEmpty(authorities) ? Collections.emptySet() : Sets.newHashSet(authorities);
    }

    /**
     * 获取数据安全加解密处理器
     *
     * @return 数据安全加解密处理器实例
     */
    public static SecurityCipherHandler getSecurityCipherHandler() {
        if (SECURITY_CIPHER_HANDLER == null) {
            synchronized (SecurityCipherHandler.class) {
                if (SECURITY_CIPHER_HANDLER == null) {
                    SECURITY_CIPHER_HANDLER = ApplicationContextHolder.getBean(SecurityCipherHandler.class);
                }
            }
        }
        return SECURITY_CIPHER_HANDLER;
    }

    /**
     * 获取排除安全校验接口地址
     *
     * @return 接口地址集合
     */
    public static Set<String> getSecurityIgnores() {
        return getSecurityIgnores(ApplicationContextHolder.getApplicationContext());
    }

    /**
     * 获取排除安全校验接口地址
     *
     * @param applicationContext 应用上下文
     * @return 接口地址集合
     */
    public static Set<String> getSecurityIgnores(@NonNull ApplicationContext applicationContext) {
        SecurityProperties properties = applicationContext.getBean(SecurityProperties.class);
        Set<String> ignores = Sets.newHashSet(properties.getIgnores());
        Environment environment = applicationContext.getEnvironment();
        ignores.add(environment.resolveRequiredPlaceholders("${server.error.path:${error.path:/error}}"));
        try {
            Class<?> clazz =
                    Class.forName("org.springframework.boot.actuate.autoconfigure.endpoint.web.WebEndpointProperties");
            ObjectProvider<?> actuator = applicationContext.getBeanProvider(clazz);
            String basePath = (String) ObjectUtils.getObjectValue(actuator.getIfAvailable(), "basePath");
            if (StringUtils.notEmpty(basePath)) {
                ignores.add(HttpUtils.uri(basePath, "/**"));
            }
        } catch (ClassNotFoundException ignore) {
        }
        return ignores;
    }

    /**
     * 获取排除认证接口地址
     *
     * @return 接口地址集合
     */
    public static Set<String> getAuthorizeIgnores() {
        return getAuthorizeIgnores(ApplicationContextHolder.getApplicationContext());
    }

    /**
     * 获取排除认证接口地址
     *
     * @param applicationContext 应用上下文
     * @return 接口地址集合
     */
    public static Set<String> getAuthorizeIgnores(@NonNull ApplicationContext applicationContext) {
        Set<String> ignores = Sets.newHashSet();
        boolean multiple = SpringUtils.isMultipleServer();
        List<Resource> resources = multiple ?
                SpringUtils.getPropertyResources(applicationContext.getEnvironment()) : Collections.emptyList();
        SpringUtils.getControllers(applicationContext).forEach(controller -> {
            Class<?> clazz = AopProxyUtils.ultimateTargetClass(controller);

            // 如果启用多服务MVC配置则将服务名作为接口地址前缀
            String prefix = null;
            if (multiple) {
                Resource resource = SpringUtils.getClassResource(clazz, resources);
                prefix = resource == null ? null : ApplicationContextHolder.getApplicationName(resource);
            }

            // 遍历所有接口方法
            for (Method method : clazz.getDeclaredMethods()) {
                Security security = method.getAnnotation(Security.class);
                if ((security != null || (security = clazz.getAnnotation(Security.class)) != null)
                        && (!security.enabled() || !security.authorize())) {
                    for (String api : SpringUtils.getApis(method)) {
                        ignores.add(HttpUtils.uri(prefix, api));
                    }
                }
            }
        });
        return ObjectUtils.isEmpty(ignores) ? Collections.emptySet() : ignores;
    }
}
