package com.chelvc.framework.security.firewall;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

import com.chelvc.framework.base.context.ThreadContextHolder;
import com.chelvc.framework.base.logic.VariableParser;
import com.chelvc.framework.common.model.Pair;
import com.chelvc.framework.common.util.IdentityUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.redis.context.RedisContextHolder;
import com.google.common.collect.Lists;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.dao.DataAccessException;
import org.springframework.data.redis.core.RedisOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.SessionCallback;

/**
 * 防火墙处理器默认实现
 *
 * @author Woody
 * @date 2025/2/26
 */
@Slf4j
public class DefaultFirewallProcessor implements FirewallProcessor {
    private final VariableParser parser;
    private final ActionExecutor executor;
    private volatile IdentityUtils.Generator sequenceGenerator;

    public DefaultFirewallProcessor(@NonNull VariableParser parser, @NonNull ActionExecutor executor) {
        this.parser = parser;
        this.executor = executor;
    }

    /**
     * 获取序列号
     *
     * @return 序列号数字
     */
    private Long getSequence() {
        if (this.sequenceGenerator == null) {
            synchronized (this) {
                if (this.sequenceGenerator == null) {
                    this.sequenceGenerator = RedisContextHolder.getIdentityGenerator().clone();
                }
            }
        }
        return this.sequenceGenerator.next();
    }

    /**
     * 获取匹配规则
     *
     * @param rules 规则列表
     * @return 主体/规则列表
     */
    private List<Pair<String, Rule>> getMatchRules(List<Rule> rules) {
        List<Pair<String, Rule>> matches = Lists.newLinkedList();
        for (Rule rule : rules) {
            CharSequence principal = null;
            if (rule.getCondition().eval(this.parser) && (rule.getCount() < 1
                    || StringUtils.notEmpty(principal = rule.getPrincipal().parse(this.parser)))) {
                matches.add(Pair.of(StringUtils.toString(principal), rule));
            }
        }
        return ObjectUtils.isEmpty(matches) ? Collections.emptyList() : matches;
    }

    /**
     * 获取匹配规则动作列表
     *
     * @param pairs 主体/规则列表
     * @return 动作列表
     */
    protected List<String> getAccessControls(List<Pair<String, Rule>> pairs) {
        List<String> keys = Lists.newArrayListWithCapacity(pairs.size());
        pairs.forEach(pair -> keys.add(pair.getValue().getAclKey(pair.getKey())));
        RedisTemplate<String, String> template = RedisContextHolder.getDefaultTemplate();
        return Objects.requireNonNull(template.opsForValue().multiGet(keys));
    }

    /**
     * 刷新匹配规则访问控制数据
     *
     * @param pairs 主体/规则列表
     */
    protected void refreshAccessControls(List<Pair<String, Rule>> pairs) {
        // 批量刷新时间窗口数据
        Long sequence = this.getSequence(), timestamp = System.currentTimeMillis();
        RedisTemplate<String, Object> template = RedisContextHolder.getDefaultTemplate();
        List<Object> results = template.executePipelined(new SessionCallback<Object>() {
            @Override
            @SuppressWarnings("unchecked")
            public <K, V> Object execute(RedisOperations<K, V> operations) throws DataAccessException {
                pairs.forEach(pair -> {
                    Rule rule = pair.getValue();
                    K key = (K) rule.getAccessKey(pair.getKey());
                    long mark = timestamp - rule.getCycle() * 1000;

                    // 添加访问记录
                    operations.opsForZSet().add(key, (V) sequence, timestamp);

                    // 移除时间窗口外记录
                    operations.opsForZSet().removeRangeByScore(key, 0, mark - 1000);

                    // 设置时间窗口过期时间
                    operations.expire(key, Math.max(rule.getCycle(), 60), TimeUnit.SECONDS);

                    // 统计时间窗口内访问次数
                    operations.opsForZSet().count(key, mark, timestamp);
                });
                return null;
            }
        });

        // 批量更新满足规则条件的访问控制动作
        List<Pair<String, Rule>> matches = Lists.newLinkedList();
        for (int i = 0, size = pairs.size(); i < size; i++) {
            Pair<String, Rule> pair = pairs.get(i);
            Long count = (Long) results.get((i + 1) * 4 - 1);
            if (count != null && count > pair.getValue().getCount()) {
                matches.add(pair);
            }
        }
        if (ObjectUtils.notEmpty(matches)) {
            template.executePipelined(new SessionCallback<Object>() {
                @Override
                @SuppressWarnings("unchecked")
                public <K, V> Object execute(RedisOperations<K, V> operations) throws DataAccessException {
                    matches.forEach(pair -> {
                        Rule rule = pair.getValue();
                        K key = (K) rule.getAclKey(pair.getKey());
                        V value = (V) rule.getAction();
                        operations.opsForValue().setIfAbsent(key, value, rule.getExpiration(), TimeUnit.SECONDS);
                    });
                    return null;
                }
            });
        }
    }

    @Override
    public boolean processing(List<Rule> rules) {
        if (ObjectUtils.isEmpty(rules)) {
            return true;
        }

        // 获取匹配规则
        List<Pair<String, Rule>> matches;
        try {
            matches = this.getMatchRules(rules);
        } catch (Exception e) {
            log.error("Firewall processing failed", e);
            return true;
        }

        // 处理直接拦截规则动作
        if (ObjectUtils.notEmpty(matches)) {
            matches.removeIf(pair -> {
                if (pair.getValue().getCount() > 0) {
                    return false;
                }
                this.executor.execute(pair.getKey(), pair.getValue());
                return true;
            });
        }
        if (ObjectUtils.isEmpty(matches)) {
            return true;
        }

        // 处理匹配规则访问控制动作
        List<String> actions;
        try {
            actions = this.getAccessControls(matches);
        } catch (Exception e) {
            log.error("Firewall processing failed", e);
            return true;
        }
        Iterator<Pair<String, Rule>> iterator = matches.iterator();
        for (int i = 0, size = actions.size(); i < size && iterator.hasNext(); i++) {
            Pair<String, Rule> pair = iterator.next();
            if (StringUtils.notEmpty(actions.get(i++))) {
                this.executor.execute(pair.getKey(), pair.getValue());
                iterator.remove();
            }
        }

        // 异步刷新访问控制数据
        if (ObjectUtils.notEmpty(matches)) {
            ThreadContextHolder.run(() -> {
                try {
                    this.refreshAccessControls(matches);
                } catch (Exception e) {
                    log.error("Firewall processing failed", e);
                }
            });
        }
        return true;
    }
}
