package com.chelvc.framework.security.interceptor;

import java.util.Objects;
import java.util.Set;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Platform;
import com.chelvc.framework.common.model.Terminal;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import lombok.NonNull;
import org.springframework.security.access.PermissionEvaluator;
import org.springframework.security.access.expression.SecurityExpressionRoot;
import org.springframework.security.access.expression.method.MethodSecurityExpressionOperations;
import org.springframework.security.access.hierarchicalroles.RoleHierarchy;
import org.springframework.security.authentication.AuthenticationTrustResolver;
import org.springframework.security.core.Authentication;

/**
 * 方法权限表达式处理包装类
 *
 * @author Woody
 * @date 2024/1/30
 */
public class MethodSecurityExpression implements MethodSecurityExpressionOperations {
    private final SecurityExpressionRoot root;
    private Object filterObject;
    private Object returnObject;
    private Object target;

    public MethodSecurityExpression(Authentication authentication) {
        this.root = new SecurityExpressionRoot(authentication) {
        };
    }

    @Override
    public Object getFilterObject() {
        return this.filterObject;
    }

    @Override
    public void setFilterObject(Object filterObject) {
        this.filterObject = filterObject;
    }

    @Override
    public Object getReturnObject() {
        return this.returnObject;
    }

    @Override
    public void setReturnObject(Object returnObject) {
        this.returnObject = returnObject;
    }

    @Override
    public Object getThis() {
        return this.target;
    }

    public void setThis(Object target) {
        this.target = target;
    }

    @Override
    public Authentication getAuthentication() {
        return this.root.getAuthentication();
    }

    @Override
    public boolean hasAuthority(String authority) {
        return StringUtils.notEmpty(authority) && this.hasAnyAuthority(authority);
    }

    @Override
    public boolean hasAnyAuthority(String... authorities) {
        if (ObjectUtils.isEmpty(authorities)) {
            return false;
        }

        Session session = SessionContextHolder.getSession(false);
        Set<String> owns = ObjectUtils.ifNull(session, Session::getAuthorities);
        if (ObjectUtils.isEmpty(owns)) {
            return false;
        }

        for (String authority : authorities) {
            if (StringUtils.notEmpty(authority) && owns.contains(authority)) {
                return true;
            }
        }

        for (String own : owns) {
            Set<?> properties = ApplicationContextHolder.getProperty(own, Set.class);
            if (ObjectUtils.notEmpty(properties)) {
                for (String authority : authorities) {
                    if (StringUtils.notEmpty(authority) && properties.contains(authority)) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    @Override
    public boolean hasRole(String role) {
        return this.root.hasRole(role);
    }

    @Override
    public boolean hasAnyRole(String... roles) {
        return this.root.hasAnyRole(roles);
    }

    @Override
    public boolean permitAll() {
        return this.root.permitAll();
    }

    @Override
    public boolean denyAll() {
        return this.root.denyAll();
    }

    @Override
    public boolean isAnonymous() {
        return SessionContextHolder.isAnonymous();
    }

    @Override
    public boolean isAuthenticated() {
        return Objects.nonNull(SessionContextHolder.getId());
    }

    @Override
    public boolean isRememberMe() {
        return this.root.isRememberMe();
    }

    @Override
    public boolean isFullyAuthenticated() {
        return this.root.isFullyAuthenticated();
    }

    @Override
    public boolean hasPermission(Object target, Object permission) {
        return this.root.hasPermission(target, permission);
    }

    @Override
    public boolean hasPermission(Object targetId, String targetType, Object permission) {
        return this.root.hasPermission(targetId, targetType, permission);
    }

    public void setPermissionEvaluator(PermissionEvaluator permissionEvaluator) {
        this.root.setPermissionEvaluator(permissionEvaluator);
    }

    public void setTrustResolver(AuthenticationTrustResolver trustResolver) {
        this.root.setTrustResolver(trustResolver);
    }

    public void setRoleHierarchy(RoleHierarchy roleHierarchy) {
        this.root.setRoleHierarchy(roleHierarchy);
    }

    public void setDefaultRolePrefix(String defaultRolePrefix) {
        this.root.setDefaultRolePrefix(defaultRolePrefix);
    }

    /**
     * 判断是否是审核用户
     *
     * @return true/false
     */
    public boolean isAuditor() {
        return SessionContextHolder.isAuditor();
    }

    /**
     * 判断是否是指定应用范围
     *
     * @param scopes 应用范围数组
     * @return true/false
     */
    public boolean isScope(@NonNull String... scopes) {
        return SessionContextHolder.isScope(scopes);
    }

    /**
     * 判断是否是指定渠道
     *
     * @param channels 渠道数组
     * @return true/false
     */
    public boolean isChannel(@NonNull String... channels) {
        return SessionContextHolder.isChannel(channels);
    }

    /**
     * 判断是否是指定平台
     *
     * @param platforms 平台数组
     * @return true/false
     */
    public boolean isPlatform(@NonNull Platform... platforms) {
        return SessionContextHolder.isPlatform(platforms);
    }

    /**
     * 判断是否是指定终端
     *
     * @param terminals 终端数组
     * @return true/false
     */
    public boolean isTerminal(@NonNull Terminal... terminals) {
        return SessionContextHolder.isTerminal(terminals);
    }

    /**
     * 判断是否在指定版本之前
     *
     * @param version  指定版本
     * @param contains 是否包含当前版本号
     * @return true/false
     */
    public boolean isBeforeVersion(@NonNull String version, boolean contains) {
        Terminal terminal = SessionContextHolder.getTerminal();
        return terminal != null && SessionContextHolder.isBeforeVersion(terminal, version, contains);
    }

    /**
     * 判断是否在指定版本之后
     *
     * @param version  指定版本
     * @param contains 是否包含当前版本号
     * @return true/false
     */
    public boolean isAfterVersion(@NonNull String version, boolean contains) {
        Terminal terminal = SessionContextHolder.getTerminal();
        return terminal != null && SessionContextHolder.isAfterVersion(terminal, version, contains);
    }
}
