package com.chelvc.framework.security.interceptor;

import java.util.Collections;
import java.util.List;
import java.util.Objects;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.exception.FrameworkException;
import com.chelvc.framework.common.model.Ip;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.redis.context.RedisContextHolder;
import com.chelvc.framework.security.firewall.FirewallProcessor;
import com.chelvc.framework.security.firewall.Rule;
import com.fasterxml.jackson.core.type.TypeReference;
import com.google.common.collect.Lists;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.core.Ordered;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

/**
 * 安全防火墙拦截器
 *
 * @author Woody
 * @date 2025/2/23
 */
@Slf4j
@Component
@ConditionalOnClass(RedisContextHolder.class)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class SecurityFirewallInterceptor implements HandlerInterceptor, WebMvcConfigurer {
    /**
     * 规则列表类型定义
     */
    private static final TypeReference<List<Rule>> RULE_LIST_TYPE = new TypeReference<List<Rule>>() {
    };

    private final FirewallProcessor processor;

    /**
     * 判断功能是否启用
     *
     * @return true/false
     */
    private boolean isEnabled() {
        return ApplicationContextHolder.getProperty("security.firewall.enabled", boolean.class, false);
    }

    /**
     * 获取规则配置
     *
     * @return 规则列表
     */
    private List<Rule> getRules() {
        return ApplicationContextHolder.getProperty("security.firewall.rules", json -> {
            List<Rule> rules = JacksonUtils.deserialize(json, RULE_LIST_TYPE);
            if (ObjectUtils.notEmpty(rules)) {
                rules.removeIf(rule -> {
                    if (this.matches(rule)) {
                        log.info("Applying firewall rule: {}", rule);
                        return false;
                    }
                    return true;
                });
            }
            return ObjectUtils.isEmpty(rules) ? Collections.emptyList() : rules;
        });
    }

    /**
     * 获取主体白名单
     *
     * @return 主体列表
     */
    private List<CharSequence> getWhitelists() {
        return ApplicationContextHolder.getProperty("security.firewall.whitelists", this::property2principals);
    }

    /**
     * 获取主体黑名单
     *
     * @return 主体列表
     */
    private List<CharSequence> getBlacklists() {
        return ApplicationContextHolder.getProperty("security.firewall.blacklists", this::property2principals);
    }

    /**
     * 获取客户端IP数字
     *
     * @return IP数字
     */
    private long getHostNumber() {
        String host = SessionContextHolder.getHost();
        if (StringUtils.notEmpty(host)) {
            try {
                return Ip.getAddressNumber(host);
            } catch (Exception e) {
                log.error("Address number convert failed", e);
            }
        }
        return 0;
    }

    /**
     * 判断规则是否匹配
     *
     * @param rule 规则信息
     * @return true/false
     */
    private boolean matches(Rule rule) {
        return (StringUtils.isEmpty(rule.getApp())
                || rule.getApp().equalsIgnoreCase(ApplicationContextHolder.getApplicationName()))
                && StringUtils.notEmpty(rule.getAction()) && rule.getCondition() != null
                && (rule.getCount() < 1 || (rule.getPrincipal() != null
                && rule.getCycle() > 0 && rule.getCycle() <= 3600
                && rule.getExpiration() >= 60 && rule.getExpiration() <= 1800));
    }

    /**
     * 判断主体是否匹配
     *
     * @param principal 主体信息
     * @param host      IP数字
     * @return true/false
     */
    private boolean matches(CharSequence principal, long host) {
        if (principal instanceof Ip) {
            return host > 0 && ((Ip) principal).matches(host);
        }
        Session session = SessionContextHolder.getSession(false);
        return session != null && (Objects.equals(principal, session.getMobile())
                || Objects.equals(principal, session.getDevice()));
    }

    /**
     * 将属性值转换成主体信息
     *
     * @param property 属性值
     * @return 主体信息列表
     */
    private List<CharSequence> property2principals(String property) {
        if (StringUtils.isEmpty(property)) {
            return Collections.emptyList();
        }

        List<CharSequence> principals = Lists.newLinkedList();
        for (String principal : property.split(",")) {
            if (!(principal = principal.trim()).isEmpty()) {
                if (StringUtils.isIp(principal)) {
                    principals.add(new Ip(principal));
                } else {
                    principals.add(principal);
                }
            }
        }
        return principals.isEmpty() ? Collections.emptyList() : principals;
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
            throws Exception {
        if (!(handler instanceof HandlerMethod) || !this.isEnabled()) {
            return true;
        }

        // 主体黑/白名单校验
        List<CharSequence> whitelists = this.getWhitelists();
        List<CharSequence> blacklists = this.getBlacklists();
        if (ObjectUtils.notEmpty(whitelists) || ObjectUtils.notEmpty(blacklists)) {
            long host = this.getHostNumber();

            // 白名单放行
            for (CharSequence principal : whitelists) {
                if (this.matches(principal, host)) {
                    return true;
                }
            }

            // 黑名单拒绝访问
            for (CharSequence principal : blacklists) {
                if (this.matches(principal, host)) {
                    String message = ApplicationContextHolder.getMessage("Forbidden");
                    throw new FrameworkException(HttpStatus.FORBIDDEN.name(), null, message);
                }
            }
        }

        // 规则防护处理
        List<Rule> rules = this.getRules();
        return ObjectUtils.isEmpty(rules) || this.processor.processing(rules);
    }

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(this).addPathPatterns("/**").order(Ordered.HIGHEST_PRECEDENCE);
    }
}
