package com.chelvc.framework.security.interceptor;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Type;
import java.util.Objects;
import java.util.Set;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.JacksonContextHolder;
import com.chelvc.framework.base.context.LoggingContextHolder;
import com.chelvc.framework.base.context.Result;
import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.interceptor.BufferedRequestWrapper;
import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.base.util.SpringUtils;
import com.chelvc.framework.common.exception.FrameworkException;
import com.chelvc.framework.common.util.CodecUtils;
import com.chelvc.framework.common.util.FileUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.security.annotation.Crypto;
import com.chelvc.framework.security.annotation.Security;
import com.chelvc.framework.security.config.SecurityProperties;
import com.chelvc.framework.security.context.SecurityContextHolder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;
import org.springframework.core.MethodParameter;
import org.springframework.core.Ordered;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpInputMessage;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;
import org.springframework.web.servlet.mvc.method.annotation.RequestBodyAdvice;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;

/**
 * 请求安全拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@ControllerAdvice
public class SecurityValidateInterceptor implements HandlerInterceptor, WebMvcConfigurer, RequestBodyAdvice,
        ResponseBodyAdvice<Object> {
    private final Set<String> ignores;
    private final SecurityProperties properties;

    public SecurityValidateInterceptor(ApplicationContext applicationContext) {
        this.ignores = SecurityContextHolder.getSecurityIgnores(applicationContext);
        this.properties = applicationContext.getBean(SecurityProperties.class);
    }

    /**
     * 是否开启校验观察
     *
     * @return true/false
     */
    private boolean isObservable() {
        return ApplicationContextHolder.getProperty("security.validate.observable", boolean.class, false);
    }

    /**
     * 数据加密
     *
     * @param original 原始数据
     * @return 数据密文
     */
    private String encrypt(Object original) {
        String plaintext = JacksonContextHolder.serialize(original);
        return SecurityContextHolder.getSecurityCipherHandler().encrypt(plaintext);
    }

    /**
     * 数据解密
     *
     * @param message 原始消息
     * @return 明文消息
     * @throws IOException I/O异常
     */
    private HttpInputMessage decrypt(HttpInputMessage message) throws IOException {
        byte[] bytes;
        try (InputStream input = message.getBody()) {
            if (input instanceof BufferedRequestWrapper.BufferedServletInputStream) {
                bytes = ((BufferedRequestWrapper.BufferedServletInputStream) input).bytes();
            } else {
                bytes = FileUtils.getBytes(input);
            }
        }
        byte[] ciphertext = CodecUtils.decodeBase64(bytes);
        byte[] plaintext = SecurityContextHolder.getSecurityCipherHandler().decrypt(ciphertext);
        return new HttpInputMessage() {
            @Override
            public HttpHeaders getHeaders() {
                return message.getHeaders();
            }

            @Override
            public InputStream getBody() throws IOException {
                return new ByteArrayInputStream(plaintext);
            }
        };
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
            throws Exception {
        if (!(handler instanceof HandlerMethod) || SpringUtils.isPath(request.getRequestURI(), this.ignores)) {
            return true;
        }

        // 获取安全注解实例
        HandlerMethod method = (HandlerMethod) handler;
        Security security = method.getMethodAnnotation(Security.class);
        if (security == null) {
            security = method.getBeanType().getAnnotation(Security.class);
        }
        if (security != null && !security.enabled()) {
            return true;
        }

        Session session = SessionContextHolder.getSession(false);
        if (session == null) {
            return true;
        }

        if (security == null || security.header()) {
            // 请求头非空校验
            Long timestamp = session.getTimestamp();
            if (timestamp == null || session.getPlatform() == null || session.getTerminal() == null
                    || StringUtils.isEmpty(session.getVersion())) {
                String message = ApplicationContextHolder.getMessage("Header.Missing");
                if (this.isObservable()) {
                    LoggingContextHolder.warn(log, request, message);
                } else {
                    throw new FrameworkException(HttpStatus.BAD_REQUEST.name(), null, message);
                }
            }

            // 请求时间戳校验
            long expiration = this.properties.getRequest().getExpiration();
            if (timestamp != null && expiration > 0 && Math.abs(System.currentTimeMillis() - timestamp) > expiration) {
                String message = ApplicationContextHolder.getMessage("Time.Deviated");
                if (this.isObservable()) {
                    LoggingContextHolder.warn(log, request, message, timestamp);
                } else {
                    throw new FrameworkException(HttpStatus.BAD_REQUEST.name(), null, message);
                }
            }
        }

        // 数字签名校验
        if (security != null && security.sign()) {
            String payload = HttpUtils.serialize(request, true);
            String signature = request.getHeader(SecurityContextHolder.SIGNATURE);
            String ciphertext = SecurityContextHolder.getSecurityCipherHandler().sign(session, payload);
            if (!Objects.equals(ciphertext, signature)) {
                String message = ApplicationContextHolder.getMessage("Signature.Invalid");
                if (this.isObservable()) {
                    LoggingContextHolder.warn(log, request, message, signature, ciphertext, payload);
                } else {
                    throw new FrameworkException(HttpStatus.BAD_REQUEST.name(), null, message);
                }
            }
        }
        return true;
    }

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(this).addPathPatterns("/**").order(Ordered.HIGHEST_PRECEDENCE + 1);
    }

    @Override
    public boolean supports(MethodParameter method, Type type, Class<? extends HttpMessageConverter<?>> clazz) {
        Crypto crypto = method.getMethodAnnotation(Crypto.class);
        return crypto != null && crypto.request();
    }

    @Override
    public HttpInputMessage beforeBodyRead(HttpInputMessage message, MethodParameter method, Type type,
                                           Class<? extends HttpMessageConverter<?>> clazz) throws IOException {
        try {
            return this.decrypt(message);
        } catch (Exception e) {
            log.warn("Request parameter decrypt failed: {}", e.getMessage());
            throw new FrameworkException(HttpStatus.BAD_REQUEST.name(), null,
                    ApplicationContextHolder.getMessage("Request.Invalid"));
        }
    }

    @Override
    public Object afterBodyRead(Object body, HttpInputMessage message, MethodParameter method, Type type,
                                Class<? extends HttpMessageConverter<?>> clazz) {
        return body;
    }

    @Override
    public Object handleEmptyBody(Object body, HttpInputMessage message, MethodParameter method, Type type,
                                  Class<? extends HttpMessageConverter<?>> clazz) {
        return body;
    }

    @Override
    public boolean supports(MethodParameter method, Class<? extends HttpMessageConverter<?>> clazz) {
        Crypto crypto = method.getMethodAnnotation(Crypto.class);
        return crypto != null && crypto.response();
    }

    @Override
    public Object beforeBodyWrite(Object body, MethodParameter method, MediaType media,
                                  Class<? extends HttpMessageConverter<?>> clazz,
                                  ServerHttpRequest request, ServerHttpResponse response) {
        if (body == null) {
            return null;
        } else if (body instanceof Result) {
            Result<?> result = (Result<?>) body;
            if (result.getData() == null) {
                return result;
            }
            String ciphertext = this.encrypt(result.getData());
            return Result.of(result.getCode(), ciphertext, result.getMessage());
        }
        return this.encrypt(body);
    }
}
