package com.chelvc.framework.security.session;

import java.util.Map;
import java.util.Objects;
import java.util.Set;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.context.Using;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.redis.context.RedisContextHolder;
import com.chelvc.framework.redis.context.RedisHashHolder;
import com.chelvc.framework.redis.context.RedisUserDailyHashHolder;
import com.chelvc.framework.security.config.SecurityProperties;
import com.chelvc.framework.security.context.SecurityContextHolder;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.oauth2.core.OAuth2AuthenticationException;
import org.springframework.security.oauth2.core.OAuth2Error;
import org.springframework.security.oauth2.core.OAuth2TokenValidatorResult;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.stereotype.Component;

/**
 * 基于Redis的令牌验证器实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Component
@ConditionalOnClass(RedisContextHolder.class)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class RedisSessionValidator extends DefaultSessionValidator {
    private final SecurityProperties properties;

    @Override
    public OAuth2TokenValidatorResult validate(Jwt jwt) {
        if (SecurityContextHolder.isClient(jwt)) {
            return OAuth2TokenValidatorResult.success();
        }

        // 校验令牌有效性
        Long id = this.getId(jwt);
        RedisTemplate<String, Object> template = RedisContextHolder.getDefaultTemplate();
        Map<String, ?> context = RedisHashHolder.get(template, "token:" + id);
        String terminal = String.valueOf(SessionContextHolder.getTerminal());
        String token = (String) context.get(terminal), scope = this.getScope(jwt);
        if (StringUtils.isEmpty(token)) {
            throw new OAuth2AuthenticationException(new OAuth2Error(
                    "TOKEN_EXPIRED", ApplicationContextHolder.getMessage("Token.Expired"), null
            ));
        } else if (!Objects.equals(token, jwt.getTokenValue())) {
            // 判断令牌是否相同，如果不相同则表示令牌已经被重置（账号已在其他地方登录）
            throw new OAuth2AuthenticationException(new OAuth2Error(
                    "TOKEN_CHANGED", ApplicationContextHolder.getMessage("Token.Changed"), null
            ));
        } else if (this.properties.getOauth().isScoped()) {
            // 判断应用范围是否相同，如果不同则表示应用范围已被重置，需要刷新令牌
            String real = (String) context.get(SecurityContextHolder.SCOPE);
            if (!Objects.equals(scope, real)) {
                String arg = StringUtils.ifEmpty(real, ApplicationContextHolder::getMessage);
                String message = ApplicationContextHolder.getMessage("Scope.Changed", new Object[]{arg});
                throw new OAuth2AuthenticationException(new OAuth2Error("SCOPE_CHANGED", message, null));
            }
        }

        // 更新会话主体信息
        String mobile = (String) context.get(SecurityContextHolder.MOBILE);
        Long creating = (Long) context.get(SecurityContextHolder.CREATING);
        Long registering = (Long) context.get(SecurityContextHolder.REGISTERING);
        long interval = this.properties.getRequest().getUsingRefreshInterval();
        Using using = Using.from(RedisUserDailyHashHolder.using(template, id), creating, interval);
        Set<String> authorities = SecurityContextHolder.getAuthorities(jwt);
        SessionContextHolder.setSession(id, using, scope, mobile, registering, authorities);
        return OAuth2TokenValidatorResult.success();
    }
}
