package com.chelvc.framework.sms.support;

import java.util.Map;
import java.util.Objects;

import com.aliyun.dysmsapi20170525.Client;
import com.aliyun.dysmsapi20170525.models.SendSmsRequest;
import com.aliyun.dysmsapi20170525.models.SendSmsResponse;
import com.aliyun.dysmsapi20170525.models.SendSmsResponseBody;
import com.aliyun.teaopenapi.models.Config;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.sms.Captcha;
import com.chelvc.framework.sms.TemplateSmsHandler;
import com.chelvc.framework.sms.config.SmsProperties;
import com.google.common.collect.ImmutableMap;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 基于阿里云短信操作实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class AliyunSmsHandler implements TemplateSmsHandler {
    private final Client client;
    private final String template;
    private final String signature;

    public AliyunSmsHandler(@NonNull SmsProperties.Client properties) {
        Config config = new Config().setAccessKeyId(properties.getAppid()).setAccessKeySecret(properties.getSecret())
                .setEndpoint("dysmsapi.aliyuncs.com");
        try {
            this.client = new Client(config);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        this.template = properties.getTemplate();
        this.signature = properties.getSignature();
    }

    @Override
    public void send(@NonNull Captcha captcha) {
        this.send(this.template, captcha);
    }

    @Override
    public void send(@NonNull String template, @NonNull Captcha captcha) {
        this.send(captcha.getMobile(), template, ImmutableMap.of("code", captcha.getCode()));
    }

    @Override
    public void send(String mobile, String template, String... parameters) {
        throw new UnsupportedOperationException();
    }

    @Override
    public void send(@NonNull String mobile, @NonNull String template, @NonNull Map<String, ?> parameters) {
        boolean debug = log.isDebugEnabled();
        if (debug) {
            log.debug("Aliyun sms request: {}, {}", mobile, parameters);
        }
        SendSmsRequest request = new SendSmsRequest().setPhoneNumbers(mobile).setSignName(this.signature)
                .setTemplateCode(template).setTemplateParam(JacksonUtils.serialize(parameters));
        SendSmsResponse response;
        try {
            response = this.client.sendSms(request);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        if (debug) {
            log.debug("Aliyun sms response: {}, {}", mobile, response.toMap());
        }
        SendSmsResponseBody body = response.getBody();
        if (!Objects.equals(body.getCode(), "OK")) {
            if (Objects.equals(body.getCode(), "isv.DAY_LIMIT_CONTROL")
                    || Objects.equals(body.getCode(), "isv.MONTH_LIMIT_CONTROL")
                    || Objects.equals(body.getCode(), "isv.BUSINESS_LIMIT_CONTROL")) {
                throw new ResourceUnavailableException(ApplicationContextHolder.getMessage("SMS.Count.Limit"));
            }
            throw new ResourceUnavailableException(body.getMessage());
        }
    }
}
