package com.chelvc.framework.sms.support;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.function.Function;

import com.chelvc.framework.common.model.BiPair;
import com.chelvc.framework.common.model.Pair;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.sms.NormalSmsHandler;
import com.google.common.collect.Lists;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 普通短信发送处理器委派实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class DelegatingNormalSmsHandler implements NormalSmsHandler {
    private final List<NormalSmsHandler> handlers;

    public DelegatingNormalSmsHandler(@NonNull List<NormalSmsHandler> handlers) {
        AssertUtils.arg(!handlers.isEmpty(), () -> "Delegating handlers must not be empty");
        this.handlers = Lists.newArrayList(handlers);
    }

    /**
     * 执行短信发送处理
     *
     * @param callback 短信处理回调函数
     * @param <T>      结果类型
     * @return 发送结果
     */
    private <T> T execute(Function<NormalSmsHandler, T> callback) {
        Exception unsupported = null;
        for (NormalSmsHandler handler : this.handlers) {
            try {
                return callback.apply(handler);
            } catch (Exception e) {
                if (e instanceof UnsupportedOperationException) {
                    unsupported = e;
                } else {
                    log.warn("Normal SMS send failed: {}", e.getMessage());
                }
            }
        }
        if (unsupported != null) {
            log.warn("Normal SMS send failed: {}", unsupported.getMessage());
        }
        return null;
    }

    @Override
    public boolean send(@NonNull String content, @NonNull String... mobiles) {
        if (mobiles.length > 0) {
            Boolean success = this.execute(handler -> handler.send(content, mobiles));
            return Boolean.TRUE.equals(success);
        }
        return false;
    }

    @Override
    public Map<String, Long> send(@NonNull Collection<BiPair<String, String, String>> pairs) {
        if (ObjectUtils.isEmpty(pairs)) {
            return Collections.emptyMap();
        }
        Map<String, Long> result = this.execute(handler -> handler.send(pairs));
        return ObjectUtils.ifNull(result, Collections::emptyMap);
    }

    @Override
    public Map<String, Long> send(@NonNull String content, @NonNull Collection<Pair<String, String>> pairs) {
        if (StringUtils.isEmpty(content) || ObjectUtils.isEmpty(pairs)) {
            return Collections.emptyMap();
        }
        Map<String, Long> result = this.execute(handler -> handler.send(content, pairs));
        return ObjectUtils.ifNull(result, Collections::emptyMap);
    }
}
