package com.chelvc.framework.wechat;

import java.util.Map;
import java.util.Objects;

/**
 * 微信支付操作接口
 *
 * @author Woody
 * @date 2024/6/19
 */
public interface WechatPaymentHandler {
    /**
     * 判断是否支持指定应用
     *
     * @param name 应用名称
     * @return true/false
     */
    default boolean supports(String name) {
        return this.supports(name, null);
    }

    /**
     * 判断是否支持指定应用
     *
     * @param name    应用名称
     * @param channel 支付渠道
     * @return true/false
     */
    boolean supports(String name, PayChannel channel);

    /**
     * 获取微信支付处理器
     *
     * @param name 应用名称
     * @return 支付处理器实例
     */
    default WechatPaymentProcessor getProcessor(String name) {
        return this.getProcessor(name, null);
    }

    /**
     * 获取微信支付处理器
     *
     * @param name    应用名称
     * @param channel 支付渠道
     * @return 支付处理器实例
     */
    WechatPaymentProcessor getProcessor(String name, PayChannel channel);

    /**
     * 参数签名
     *
     * @param name       应用名称
     * @param parameters 签名参数
     * @return 签名信息
     */
    default String sign(String name, Map<String, String> parameters) {
        WechatPaymentProcessor processor = this.getProcessor(name);
        return Objects.requireNonNull(processor).sign(parameters);
    }

    /**
     * 支付回调参数解析
     *
     * @param name    应用名称
     * @param content 支付回调内容
     * @return 支付回调信息
     */
    default WechatPaymentCallback callback(String name, String content) {
        WechatPaymentProcessor processor = this.getProcessor(name);
        return Objects.requireNonNull(processor).callback(content);
    }

    /**
     * 微信统一下单
     *
     * @param name    应用名称
     * @param request 支付请求参数
     * @return 微信统一下单信息
     */
    default WechatUnifiedOrder unifiedorder(String name, WechatPayRequest request) {
        WechatPaymentProcessor processor = this.getProcessor(name);
        return Objects.requireNonNull(processor).unifiedorder(request);
    }
}
