package com.chelvc.framework.wechat.support;

import java.util.Map;

import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.wechat.WechatAppletHandler;
import com.chelvc.framework.wechat.WechatMobile;
import com.chelvc.framework.wechat.WechatScheme;
import com.chelvc.framework.wechat.WechatSession;
import com.chelvc.framework.wechat.WechatUser;
import com.chelvc.framework.wechat.config.WechatProperties;
import com.chelvc.framework.wechat.context.WechatContextHolder;
import com.google.common.collect.ImmutableMap;
import lombok.NonNull;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;

/**
 * 微信小程序操作默认实现
 *
 * @author Woody
 * @date 2024/6/19
 */
public class DefaultWechatAppletHandler implements WechatAppletHandler {
    /**
     * 获取用户信息接口地址
     */
    private static final String USER_URL =
            "https://api.weixin.qq.com/cgi-bin/user/info?access_token=%s&openid=%s&lang=zh_CN";

    /**
     * 微信获取手机号接口地址
     */
    private static final String CODE2MOBILE_URL =
            "https://api.weixin.qq.com/wxa/business/getuserphonenumber?access_token=";

    /**
     * 微信登录接口地址
     */
    private static final String JSCODE2SESSION_URL =
            "https://api.weixin.qq.com/sns/jscode2session?appid=%s&secret=%s&js_code=%s&grant_type=authorization_code";

    /**
     * 获取小程序schemeCode接口地址
     */
    private static final String GENERATE_SCHEME_URL = "https://api.weixin.qq.com/wxa/generatescheme?access_token=";

    private final WechatProperties.Applet properties;

    public DefaultWechatAppletHandler(@NonNull WechatProperties.Applet properties) {
        this.properties = properties;
    }

    @Override
    public String getName() {
        return this.properties.getName();
    }

    @Override
    public String getAppid() {
        return this.properties.getAppid();
    }

    @Override
    public String getScheme(String path, String query, String environment) {
        Map<?, ?> body = ImmutableMap.of(
                "jump_wxa", ImmutableMap.of(
                        "path", StringUtils.ifEmpty(path, StringUtils.EMPTY),
                        "query", StringUtils.ifEmpty(query, StringUtils.EMPTY),
                        "env_version", StringUtils.ifEmpty(environment, StringUtils.EMPTY)
                ),
                "expire_type", 1,
                "expire_interval", 30
        );
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<?> entity = new HttpEntity<>(body, headers);
        String url = GENERATE_SCHEME_URL + this.getAccessToken();
        return WechatContextHolder.exchange(url, HttpMethod.POST, entity, WechatScheme.class).getOpenlink();
    }

    @Override
    public String getAccessToken() {
        return WechatContextHolder.getAccessToken(this.properties.getAppid());
    }

    @Override
    public WechatUser getUser(@NonNull String openid) {
        String url = String.format(USER_URL, this.getAccessToken(), openid);
        return WechatContextHolder.get(url, WechatUser.class);
    }

    @Override
    public String code2mobile(@NonNull String code) {
        String url = CODE2MOBILE_URL + this.getAccessToken();
        Map<String, String> request = ImmutableMap.of("code", code);
        WechatMobile.Mobile mobile = WechatContextHolder.post(url, request, WechatMobile.class).getMobile();
        return StringUtils.ifEmpty(mobile.getNumber(), mobile.getPureNumber());
    }

    @Override
    public WechatSession code2session(@NonNull String code) {
        String url = String.format(JSCODE2SESSION_URL, this.properties.getAppid(), this.properties.getSecret(), code);
        return WechatContextHolder.get(url, WechatSession.class);
    }
}
