package com.chelvc.framework.wechat.support;

import java.security.MessageDigest;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.wechat.WechatPublicHandler;
import com.chelvc.framework.wechat.WechatPublicMessage;
import com.chelvc.framework.wechat.WechatPublicOpenids;
import com.chelvc.framework.wechat.WechatPublicSubscriber;
import com.chelvc.framework.wechat.WechatPublicSubscribers;
import com.chelvc.framework.wechat.WechatUser;
import com.chelvc.framework.wechat.WechatWebToken;
import com.chelvc.framework.wechat.config.WechatProperties;
import com.chelvc.framework.wechat.context.WechatContextHolder;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Maps;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;

/**
 * 微信公众号操作默认实现
 *
 * @author Woody
 * @date 2024/4/3
 */
@Slf4j
public class DefaultWechatPublicHandler implements WechatPublicHandler {
    /**
     * 获取用户信息接口地址
     */
    private static final String USER_URL =
            "https://api.weixin.qq.com/cgi-bin/user/info?access_token=%s&openid=%s&lang=zh_CN";

    /**
     * 获取公众号粉丝unionId信息地址
     */
    private static final String FANS_URL = "https://api.weixin.qq.com/cgi-bin/user/info/batchget?access_token=";

    /**
     * 批量获取关注用户openid地址
     */
    private static final String OPENID_URL =
            "https://api.weixin.qq.com/cgi-bin/user/get?access_token=%s&next_openid=%s";

    /**
     * 获取微信网页令牌接口地址
     */
    private static final String CODE2TOKEN_URL = "https://api.weixin.qq.com/sns/oauth2/access_token" +
            "?appid=%s&secret=%s&code=%s&grant_type=authorization_code";

    /**
     * 发送微信服务号模板消息接口地址
     */
    private static final String SEND_TEMPLATE_MESSAGE_URL =
            "https://api.weixin.qq.com/cgi-bin/message/template/send?access_token=";

    private final String applet;
    private final WechatProperties.Public properties;

    public DefaultWechatPublicHandler(@NonNull WechatProperties.Push push,
                                      @NonNull WechatProperties.Public properties) {
        String applet = properties.getPush().getApplet();
        this.applet = ObjectUtils.ifNull(ObjectUtils.ifNull(applet, push.getApplet()), StringUtils.EMPTY);
        this.properties = properties;
    }

    @Override
    public String getName() {
        return StringUtils.ifEmpty(this.properties.getName(), this.properties.getAppid());
    }

    @Override
    public String getAppid() {
        return this.properties.getAppid();
    }

    @Override
    public String getAccessToken() {
        return WechatContextHolder.getAccessToken(this.properties.getAppid());
    }

    @Override
    public String decrypt(@NonNull String ciphertext) {
        String secret = this.properties.getKey();
        AssertUtils.state(StringUtils.notEmpty(secret), () -> "Wechat secret is missing");
        return WechatContextHolder.decrypt(ciphertext, secret);
    }

    @Override
    public WechatWebToken code2token(@NonNull String code) {
        String url = String.format(CODE2TOKEN_URL, this.properties.getAppid(), this.properties.getSecret(), code);
        return WechatContextHolder.get(url, WechatWebToken.class);
    }

    @Override
    public WechatUser getUser(@NonNull String openid) {
        String url = String.format(USER_URL, this.getAccessToken(), openid);
        return WechatContextHolder.get(url, WechatUser.class);
    }

    @Override
    public String sign(long timestamp, @NonNull String... elements) {
        MessageDigest digest = DigestUtils.getDigest("SHA-1");
        String original = Stream.concat(
                Stream.of(this.properties.getToken(), String.valueOf(timestamp)), Stream.of(elements)
        ).sorted().collect(Collectors.joining());
        return Hex.encodeHexString(digest.digest(original.getBytes()));
    }

    @Override
    public void openids(@NonNull Consumer<List<String>> consumer) {
        this.openids((openids, total) -> consumer.accept(openids));
    }

    @Override
    public void openids(@NonNull BiConsumer<List<String>, Integer> consumer) {
        int total = 0;
        String next = null;
        do {
            String url = String.format(OPENID_URL, this.getAccessToken(), ObjectUtils.ifNull(next, StringUtils.EMPTY));
            WechatPublicOpenids response = WechatContextHolder.get(url, WechatPublicOpenids.class);
            List<String> openids = ObjectUtils.ifNull(response.getData(), WechatPublicOpenids.Openids::getOpenid);
            if (ObjectUtils.isEmpty(openids)) {
                break;
            }
            consumer.accept(openids, total += openids.size());
            next = response.getNextOpenid();
        } while (StringUtils.notEmpty(next));
    }

    @Override
    public List<WechatPublicSubscriber> getSubscribers(@NonNull Collection<String> openids) {
        if (ObjectUtils.isEmpty(openids)) {
            return Collections.emptyList();
        }
        Map<?, ?> body = ImmutableMap.of(
                "user_list",
                openids.stream().map(openid -> ImmutableMap.of("openid", openid)).collect(Collectors.toList())
        );
        String url = FANS_URL + this.getAccessToken();
        WechatPublicSubscribers response = WechatContextHolder.post(url, body, WechatPublicSubscribers.class);
        return ObjectUtils.ifNull(response.getSubscribers(), Collections::emptyList);
    }

    @Override
    public long push(@NonNull String openid, @NonNull String template, @NonNull Map<?, ?> parameters) {
        return this.push(openid, template, StringUtils.EMPTY, parameters);
    }

    @Override
    public long push(@NonNull String openid, @NonNull String template, @NonNull String redirect,
                     @NonNull Map<?, ?> parameters) {
        return this.push(openid, template, redirect, false, parameters);
    }

    @Override
    public long push(@NonNull String openid, @NonNull String template, @NonNull String redirect, boolean external,
                     @NonNull Map<?, ?> parameters) {
        boolean debug = log.isDebugEnabled();
        if (debug) {
            log.debug("Wechat public push request: {}, {}, {}, {}", openid, template, redirect, parameters);
        }

        Map<String, Object> body = Maps.newHashMapWithExpectedSize(5);
        body.put("touser", openid);
        body.put("template_id", template);
        body.put("data", parameters);
        if (external) {
            body.put("url", redirect);
        } else {
            body.put("url", StringUtils.EMPTY);
            String applet = (redirect = redirect.trim()).isEmpty() ? StringUtils.EMPTY : this.applet;
            body.put("miniprogram", ImmutableMap.of("appid", applet, "pagepath", redirect));
        }
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<?> entity = new HttpEntity<>(body, headers);
        String url = SEND_TEMPLATE_MESSAGE_URL + this.getAccessToken();
        WechatPublicMessage response;
        try {
            response = WechatContextHolder.exchange(url, HttpMethod.POST, entity, WechatPublicMessage.class);
        } catch (Exception e) {
            log.warn("Wechat public push failed: {}, {}, {}, {}", openid, template, redirect, e.getMessage());
            return 0;
        }
        if (debug) {
            log.debug("Wechat public push response: {}", response);
        }
        return response.getId();
    }
}
